@@ -85,6 +85,14 @@
 # define XINE_DEPRECATED
 #endif
 
+#if defined(SUPPORT_ATTRIBUTE_DEPRECATED) && (!defined(XINE_LIBRARY_COMPILE)) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 5 ))
+# define XINE_PRIVATE_FIELD __attribute__((__deprecated__("this is xine-engine private field")))
+#elif defined(SUPPORT_ATTRIBUTE_DEPRECATED) && (!defined(XINE_LIBRARY_COMPILE))
+# define XINE_PRIVATE_FIELD __attribute__((__deprecated__))
+#else
+# define XINE_PRIVATE_FIELD
+#endif
+
 #ifdef SUPPORT_ATTRIBUTE_WEAK
 # define XINE_WEAK __attribute__((weak))
 #else
@@ -101,7 +109,19 @@
 
 /* Format attributes */
 #ifdef SUPPORT_ATTRIBUTE_FORMAT
-# define XINE_FORMAT_PRINTF(fmt,var) __attribute__((__format__(__printf__, fmt, var)))
+# if defined(__MINGW32__)
+#  if defined(__USE_MINGW_ANSI_STDIO) && __USE_MINGW_ANSI_STDIO
+#    ifndef __MINGW_PRINTF_FORMAT
+#     define __MINGW_PRINTF_FORMAT gnu_printf
+#   endif
+#  endif
+#  ifndef __MINGW_PRINTF_FORMAT
+#   define __MINGW_PRINTF_FORMAT __printf__
+#  endif
+#  define XINE_FORMAT_PRINTF(fmt,var) __attribute__((__format__(__MINGW_PRINTF_FORMAT, fmt, var)))
+# else
+#  define XINE_FORMAT_PRINTF(fmt,var) __attribute__((__format__(__printf__, fmt, var)))
+# endif
 # define XINE_FORMAT_SCANF(fmt,var) __attribute__((__format__(__scanf__, fmt, var)))
 #else
 # define XINE_FORMAT_PRINTF(fmt,var)
