@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -35,13 +35,12 @@
 extern "C" {
 #endif
 
-#include <string.h>
-#include <stdio.h>
-#include <pthread.h>
-#include <sys/types.h>
+#include <string.h>    /* memcmp */
+#include <sys/types.h> /* pthread_* */
 
 #include <xine/os_types.h>
 #include <xine/attributes.h>
+#include <xine/tickets.h>
 
 #define BUF_MAX_CALLBACKS 5
 
@@ -194,6 +193,9 @@ extern "C" {
 #define BUF_VIDEO_VP8		0x02680000
 #define BUF_VIDEO_VP9		0x02690000
 #define BUF_VIDEO_HEVC		0x026A0000
+#define BUF_VIDEO_AV1		0x026B0000
+#define BUF_VIDEO_PNG		0x026C0000
+
 /*@}*/
 
 /**
@@ -290,6 +292,7 @@ extern "C" {
 #define BUF_SPU_OGM             0x04060000
 #define BUF_SPU_CMML            0x04070000
 #define BUF_SPU_HDMV            0x04080000
+#define BUF_SPU_HDMV_TEXT       0x04090000
 /*@}*/
 
 /**
@@ -658,6 +661,22 @@ struct fifo_buffer_s
   int              buffer_pool_num_waiters;
   int              buffer_pool_large_wait;
   int              fifo_num_waiters;
+
+  /* Same as get but if ticket is not NULL
+   * - release it while waiting for a buf, or
+   * - renew it when revoked.
+   */
+  buf_element_t *(*tget) (fifo_buffer_t *fifo, xine_ticket_t *ticket);
+
+  /* Try to enlarge an allocated buf.
+   * There are 2 possible results:
+   * a) The buffer has been enlarged, and NULL is returned.
+   * b) The buffer cannot be enlarged due to internal memory fragmentation,
+   *    and a new buffer for the extra data is returned.
+   *    No need for an extra buffer_pool_size_alloc ().
+   * Any result may still be smaller, do check buf->max_size.
+   */
+  buf_element_t *(*buffer_pool_realloc) (buf_element_t *buf, size_t new_size);
 } ;
 
 /**
@@ -676,6 +695,12 @@ fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_PROT
  */
 fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size);
 
+/**
+ * @brief Free chained list of buffer elements.
+ * @param head List to free. May be NULL.
+ */
+void _x_free_buf_elements (buf_element_t *head) XINE_PROTECTED;
+
 
 /**
  * @brief Returns the \ref buffer_video "BUF_VIDEO_xxx" for the given fourcc.
@@ -752,6 +777,14 @@ static __inline int _x_is_fourcc(const void *ptr, const void *tag) {
   return memcmp(ptr, tag, 4) == 0;
 }
 
+/**
+ * @brief Make string from machine endian 32bit tag, eg a fourcc or a chunk ID.
+ * @param s   Pointer to string buffer of at least 20 bytes.
+ * @param tag The 32bit tag value in native byte order.
+ * @return    The length of target string written to s, without the trailing \0.
+ */
+size_t _x_tag32_me2str (char *s, uint32_t tag) XINE_PROTECTED;
+
 #ifdef __cplusplus
 }
 #endif
