@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -27,6 +27,7 @@ extern "C" {
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <stddef.h>
 #include <pthread.h>
 
 #ifdef WIN32
@@ -56,13 +57,36 @@ extern "C" {
 #endif
 
 
-/* Amiga style doubly linked lists, taken from TJtools. */
-/* Why does this work with aliasing? */
+/* Amiga style doubly linked lists, taken from TJtools.
+ * Most compilers will support the straightforward aliasing safe version.
+ * For others, try that "volatile" hack. */
 
-#define DLIST_IS_EMPTY(l) ((void *)((l)->head) == (void *)(&(l)->null))
+typedef struct dnode_st {
+  struct dnode_st *next, *prev;
+} dnode_t;
+
+#ifdef HAVE_NAMELESS_STRUCT_IN_UNION
+#  define DLIST_H(l) (&(l)->h)
+#  define DLIST_T(l) (&(l)->t)
+typedef union {
+  struct { dnode_t *head, *null, *tail; };
+  struct { dnode_t h; dnode_t *dummy1;  };
+  struct { dnode_t *dummy2; dnode_t t;  };
+} dlist_t;
+#else
+#  define DLIST_H(l) ((void *)(&(l)->head))
+#  define DLIST_T(l) ((void *)(&(l)->null))
+typedef struct {
+  dnode_t * volatile head;
+  dnode_t *null;
+  dnode_t * volatile tail;
+} dlist_t;
+#endif
+
+#define DLIST_IS_EMPTY(l) ((l)->head == DLIST_T(l))
 
 #define DLIST_REMOVE(n) { \
-  dnode_t *dl_rm_this = (dnode_t *)(n); \
+  dnode_t *dl_rm_this = n; \
   dnode_t *dl_rm_prev = dl_rm_this->prev; \
   dnode_t *dl_rm_next = dl_rm_this->next; \
   dl_rm_next->prev = dl_rm_prev; \
@@ -70,38 +94,40 @@ extern "C" {
 }
 
 #define DLIST_ADD_HEAD(n,l) { \
-  dlist_t *dl_ah_list = (dlist_t *)(l); \
-  dnode_t *dl_ah_node = (dnode_t *)(n); \
+  dlist_t *dl_ah_list = l; \
+  dnode_t *dl_ah_node = n; \
   dnode_t *dl_ah_head = dl_ah_list->head; \
   dl_ah_node->next = dl_ah_head; \
-  dl_ah_node->prev = (void *)dl_ah_list; \
+  dl_ah_node->prev = DLIST_H(dl_ah_list); \
   dl_ah_list->head = dl_ah_node; \
   dl_ah_head->prev = dl_ah_node; \
 }
 
 #define DLIST_ADD_TAIL(n,l) { \
-  dlist_t *dl_at_list = (dlist_t *)(l); \
-  dnode_t *dl_at_node = (dnode_t *)(n); \
+  dlist_t *dl_at_list = l; \
+  dnode_t *dl_at_node = n; \
   dnode_t *dl_at_tail = dl_at_list->tail; \
-  dl_at_node->next = (void *)(&dl_at_list->null); \
+  dl_at_node->next = DLIST_T(dl_at_list); \
   dl_at_node->prev = dl_at_tail; \
   dl_at_tail->next = dl_at_node; \
   dl_at_list->tail = dl_at_node; \
 }
 
+#define DLIST_INSERT(n,h) { \
+  dnode_t *dl_i_node = n; \
+  dnode_t *dl_i_here = h; \
+  dnode_t *dl_i_prev = dl_i_here->prev; \
+  dl_i_prev->next = dl_i_node; \
+  dl_i_here->prev = dl_i_node; \
+  dl_i_node->next = dl_i_here; \
+  dl_i_node->prev = dl_i_prev; \
+}
+
 #define DLIST_INIT(l) { \
-  dlist_t *dl_in_list = (dlist_t *)(l); \
-  dl_in_list->head = (void *)(&dl_in_list->null); \
+  dlist_t *dl_in_list = l; \
+  dl_in_list->head = DLIST_T(dl_in_list); \
   dl_in_list->null = NULL; \
-  dl_in_list->tail = (void *)(&dl_in_list->head); }
-
-typedef struct dnode_st {
-  struct dnode_st *next, *prev;
-} dnode_t;
-
-typedef struct {
-  dnode_t *head, *null, *tail;
-} dlist_t;
+  dl_in_list->tail = DLIST_H(dl_in_list); }
 
 
   /*
@@ -164,12 +190,36 @@ typedef struct {
 
 uint32_t xine_mm_accel (void) XINE_CONST XINE_PROTECTED;
 
+int xine_cpu_count(void) XINE_CONST XINE_PROTECTED;
 
 
 		     /* Optimized/fast memcpy */
 
 extern void *(* xine_fast_memcpy)(void *to, const void *from, size_t len) XINE_PROTECTED;
 
+/* len (usually) < 500, but not a build time constant. */
+#define xine_small_memcpy(xsm_to,xsm_from,xsm_len) memcpy (xsm_to, xsm_from, xsm_len)
+
+#if (defined(__GNUC__) && (__GNUC__ >= 4)) || defined(__clang__)
+#  if defined(ARCH_X86)
+#    undef xine_small_memcpy
+static inline void *xine_small_memcpy (void *to, const void *from, size_t len) {
+  void *t2 = to;
+  size_t l2 = len;
+  __asm__ __volatile__ (
+    "cld\n\trep movsb"
+    : "=S" (from), "=D" (t2), "=c" (l2), "=m" (*(struct {char foo[len];} *)to)
+    : "0"  (from), "1"  (t2), "2"  (l2)
+    : "cc"
+  );
+  (void)from;
+  (void)t2;
+  (void)l2;
+  return to;
+}
+#  endif
+#endif
+
 /*
  * Debug stuff
  */
@@ -182,6 +232,22 @@ void xine_profiler_start_count (int id) XINE_PROTECTED;
 void xine_profiler_stop_count (int id) XINE_PROTECTED;
 void xine_profiler_print_results (void) XINE_PROTECTED;
 
+/*
+ * xine_container_of()
+ * calculate struct pointer from field pointer
+ */
+
+#if defined(__GNUC__)
+#  define xine_container_of(ptr, type, member)           \
+  ({                                                     \
+     const typeof(((type *)0)->member) *__mptr = (ptr);  \
+     (type *)((char *)__mptr - offsetof(type, member));  \
+  })
+#else
+#  define xine_container_of(ptr, type, member) \
+  ((type *)((char *)(1 ? (ptr) : &((type *)0)->member) - offsetof(type, member)))
+#endif
+
 /*
  * Allocate and clean memory size_t 'size', then return the pointer
  * to the allocated memory.
@@ -200,6 +266,13 @@ static inline void _x_freep(void *ptr) {
   *p = NULL;
 }
 
+static inline void _x_freep_wipe_string(char **pp) {
+  char *p = *pp;
+  while (p && *p)
+    *p++ = 0;
+  _x_freep(pp);
+}
+
 /*
  * Copy blocks of memory.
  */
@@ -279,7 +352,7 @@ void xine_usec_sleep(unsigned usec) XINE_PROTECTED;
  * normally, updates & returns *dest
  * on error, *dest is unchanged & NULL is returned.
  */
-char *xine_strcat_realloc (char **dest, char *append) XINE_PROTECTED;
+char *xine_strcat_realloc (char **dest, const char *append) XINE_PROTECTED;
 
 /**
  * asprintf wrapper
@@ -354,6 +427,7 @@ extern void (*yuy2_to_yv12)
    unsigned char *v_dst, int v_dst_pitch,
    int width, int height) XINE_PROTECTED;
 
+
 /* convert full range rgb to mpeg range yuv */
 #define SCALESHIFT 16
 #define SCALEFACTOR (1<<SCALESHIFT)
@@ -418,13 +492,19 @@ extern int v_g_table[256] XINE_PROTECTED;
 extern int v_b_table[256] XINE_PROTECTED;
 
 /* TJ. direct sliced rgb -> yuy2 conversion */
-extern void *rgb2yuy2_alloc (int color_matrix, const char *format) XINE_PROTECTED;
-extern void  rgb2yuy2_free (void *rgb2yuy2) XINE_PROTECTED;
-extern void  rgb2yuy2_slice (void *rgb2yuy2, const uint8_t *in, int ipitch, uint8_t *out, int opitch,
+typedef struct rgb2yuy2_s rgb2yuy2_t;
+extern rgb2yuy2_t *rgb2yuy2_alloc (int color_matrix, const char *format) XINE_PROTECTED;
+extern void  rgb2yuy2_free (rgb2yuy2_t *rgb2yuy2) XINE_PROTECTED;
+extern void  rgb2yuy2_slice (rgb2yuy2_t *rgb2yuy2, const uint8_t *in, int ipitch, uint8_t *out, int opitch,
   int width, int height) XINE_PROTECTED;
-extern void  rgb2yuy2_palette (void *rgb2yuy2, const uint8_t *pal, int num_colors, int bits_per_pixel)
+extern void  rgb2yuy2_palette (rgb2yuy2_t *rgb2yuy2, const uint8_t *pal, int num_colors, int bits_per_pixel)
   XINE_PROTECTED;
 
+extern void rgb2yv12_slice (rgb2yuy2_t *rgb2yuy2, const uint8_t *src, int src_stride,
+                           uint8_t *y_dst, int y_pitch,
+                           uint8_t *u_dst, int u_pitch,
+                           uint8_t *v_dst, int v_pitch,
+                           int width, int height) XINE_PROTECTED;
 
 /* frame copying functions */
 extern void yv12_to_yv12
@@ -437,6 +517,13 @@ extern void yuy2_to_yuy2
    unsigned char *dst, int dst_pitch,
    int width, int height) XINE_PROTECTED;
 
+void _x_nv12_to_yv12(const uint8_t *y_src,  int y_src_pitch,
+                     const uint8_t *uv_src, int uv_src_pitch,
+                     uint8_t *y_dst, int y_dst_pitch,
+                     uint8_t *u_dst, int u_dst_pitch,
+                     uint8_t *v_dst, int v_dst_pitch,
+                     int width, int height) XINE_PROTECTED;
+
 /* print a hexdump of the given data */
 void xine_hexdump (const void *buf, int length) XINE_PROTECTED;
 
@@ -470,11 +557,13 @@ void xine_hexdump (const void *buf, int length) XINE_PROTECTED;
   } while(0)
 #endif
 
+XINE_DEPRECATED static inline void _x_abort_is_deprecated(void) {}
 #define _x_abort()                                                    \
   do {                                                                \
     fprintf(stderr, "abort: %s:%d: %s: Aborting.\n",                  \
             __FILE__, __LINE__, __XINE_FUNCTION__);                   \
     abort();                                                          \
+    _x_abort_is_deprecated();                                         \
   } while(0)
 
 
@@ -494,7 +583,7 @@ void xine_hexdump (const void *buf, int length) XINE_PROTECTED;
 #endif /* LOG_VERBOSE */
 
 #ifdef LOG
-  #ifdef __GNUC__
+  #if defined(__GNUC__) && (!defined(__STDC_VERSION__) || __STDC_VERSION__ < 199901L)
     #define lprintf(fmt, args...)                                           \
       do {                                                                  \
         LONG_LOG_MODULE_STRING                                              \
@@ -519,23 +608,20 @@ void xine_hexdump (const void *buf, int length) XINE_PROTECTED;
     #endif  /* _MSC_VER */
   #endif /* __GNUC__ */
 #else /* LOG */
-  #ifdef __GNUC__
-    #ifdef DEBUG
-      #define lprintf(fmt, args...)                                           \
-        do { if (0) { printf(fmt, ##args); } } while(0)  /* silence unused warnings, check format, ... */
-    #else
+  #if defined(DEBUG) && defined(XINE_COMPILE)
+XINE_FORMAT_PRINTF(1, 2) static inline void lprintf(const char * fmt, ...) { (void)fmt; }
+  #elif defined(__STDC_VERSION__) &&  __STDC_VERSION__ >= 199901L
+    #define lprintf(...)              do {} while(0)
+  #elif defined(__GNUC__)
       #define lprintf(fmt, args...)     do {} while(0)
-    #endif
-  #else
-  #ifdef _MSC_VER
+  #elif defined(_MSC_VER)
 void __inline lprintf(const char * fmt, ...) {}
   #else
     #define lprintf(...)              do {} while(0)
-  #endif /* _MSC_VER */
-  #endif /* __GNUC__ */
+  #endif
 #endif /* LOG */
 
-#ifdef __GNUC__
+#if defined(__GNUC__) && (!defined(__STDC_VERSION__) || __STDC_VERSION__ < 199901L)
   #define llprintf(cat, fmt, args...)                                       \
     do{                                                                     \
       if(cat){                                                              \
@@ -563,7 +649,7 @@ void __inline lprintf(const char * fmt, ...) {}
 #endif /* _MSC_VER */
 #endif /* __GNUC__ */
 
-#ifdef  __GNUC__
+#if defined(__GNUC__) && (!defined(__STDC_VERSION__) || __STDC_VERSION__ < 199901L)
   #define xprintf(xine, verbose, fmt, args...)                              \
     do {                                                                    \
       if((xine) && (xine)->verbosity >= verbose){                           \
@@ -630,6 +716,7 @@ const char *xine_guess_spu_encoding(void) XINE_PROTECTED;
  * use the best clock reference (API compatible with gettimeofday)
  * note: it will be a monotonic clock, if available.
  */
+struct timezone;
 int xine_monotonic_clock(struct timeval *tv, struct timezone *tz) XINE_PROTECTED;
 
 /**
