@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 the xine project
+ * Copyright (C) 2005-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -46,10 +46,6 @@
 #define MAXFRAMESIZE 131072
 
 
-typedef struct {
-  audio_decoder_class_t   decoder_class;
-} dvaudio_class_t;
-
 typedef struct dvaudio_decoder_s {
   audio_decoder_t   audio_decoder;
 
@@ -64,7 +60,7 @@ typedef struct dvaudio_decoder_s {
   int               bufsize;
   int               size;
 
-  char             *decode_buffer;
+  uint8_t          *decode_buffer;
   int               decoder_ok;
 
 } dvaudio_decoder_t;
@@ -321,6 +317,7 @@ static void dvaudio_reset (audio_decoder_t *this_gen) {
 }
 
 static void dvaudio_discontinuity (audio_decoder_t *this_gen) {
+  (void)this_gen;
 }
 
 static void dvaudio_dispose (audio_decoder_t *this_gen) {
@@ -341,7 +338,10 @@ static audio_decoder_t *dvaudio_open_plugin (audio_decoder_class_t *class_gen, x
 
   dvaudio_decoder_t *this ;
 
+  (void)class_gen;
   this = calloc(1, sizeof (dvaudio_decoder_t));
+  if (!this)
+    return NULL;
 
   this->audio_decoder.decode_data    = dvaudio_decode_data;
   this->audio_decoder.reset          = dvaudio_reset;
@@ -358,18 +358,17 @@ static audio_decoder_t *dvaudio_open_plugin (audio_decoder_class_t *class_gen, x
   return &this->audio_decoder;
 }
 
-static void *init_dvaudio_plugin (xine_t *xine, void *data) {
+static void *init_dvaudio_plugin (xine_t *xine, const void *data) {
 
-  dvaudio_class_t *this ;
-
-  this = calloc(1, sizeof (dvaudio_class_t));
-
-  this->decoder_class.open_plugin     = dvaudio_open_plugin;
-  this->decoder_class.identifier      = "dv audio";
-  this->decoder_class.description     = N_("dv audio decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
-
-  return this;
+  static const audio_decoder_class_t this = {
+    .open_plugin     = dvaudio_open_plugin,
+    .identifier      = "dv audio",
+    .description     = N_("dv audio decoder plugin"),
+    .dispose         = NULL
+  };
+  (void)xine;
+  (void)data;
+  return (void *)&this;
 }
 
 static uint32_t supported_audio_types[] = {
@@ -378,8 +377,8 @@ static uint32_t supported_audio_types[] = {
 };
 
 static const decoder_info_t dec_info_dvaudio = {
-  supported_audio_types,   /* supported types */
-  5                        /* priority        */
+  .supported_types = supported_audio_types,
+  .priority        = 5,
 };
 
 /*
@@ -389,5 +388,5 @@ static const decoder_info_t dec_info_dvaudio = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_DECODER, 16, "dvaudio", XINE_VERSION_CODE, &dec_info_dvaudio, init_dvaudio_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
