@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -26,6 +26,7 @@
  * @author James Courtier-Dutton (2001-12-09)
  */
 
+#define _DEFAULT_SOURCE 1
 #ifndef __sun
 /* required for swab() */
 #define _XOPEN_SOURCE 500
@@ -59,10 +60,6 @@
 
 #define MAX_AC5_FRAME 4096
 
-typedef struct {
-  audio_decoder_class_t   decoder_class;
-} dts_class_t;
-
 typedef struct {
   audio_decoder_t  audio_decoder;
 
@@ -93,9 +90,11 @@ typedef struct {
 } dts_decoder_t;
 
 static void dts_reset (audio_decoder_t *const this_gen) {
+  (void)this_gen;
 }
 
 static void dts_discontinuity (audio_decoder_t *const this_gen) {
+  (void)this_gen;
 }
 
 /**
@@ -445,6 +444,7 @@ static void dts_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 #else
           dts_decode_frame (this, this->pts);
 #endif
+          /* fall through */
     case 4:  /* Clear up ready for next frame */
           this->pts = 0;
 	  this->syncdword = 0;
@@ -559,19 +559,17 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   return &this->audio_decoder;
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
-  dts_class_t *this ;
+static void *init_plugin (xine_t *xine, const void *data) {
 
-  lprintf("init_plugin\n");
-
-  this = calloc(1, sizeof (dts_class_t));
-
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "DTS";
-  this->decoder_class.description     = N_("DTS passthru audio format decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
-
-  return this;
+  static const audio_decoder_class_t this = {
+    .open_plugin     = open_plugin,
+    .identifier      = "DTS",
+    .description     = N_("DTS passthru audio format decoder plugin"),
+    .dispose         = NULL
+  };
+  (void)xine;
+  (void)data;
+  return (void *)&this;
 }
 
 static const uint32_t audio_types[] = {
@@ -579,12 +577,12 @@ static const uint32_t audio_types[] = {
  };
 
 static const decoder_info_t dec_info_audio = {
-  audio_types,         /* supported types */
-  8                    /* priority        */
+  .supported_types = audio_types,
+  .priority        = 8,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_DECODER, 16, "dts", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
