@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -97,9 +97,6 @@ typedef struct faad_decoder_s {
   /* 1 (OK), 0 (closed), < 0 (# failed attempts) */
   int              output_open;
 
-  unsigned long    total_time;
-  unsigned long    total_data;
-
   int              in_channels, out_channels, out_used;
   int              in_mode, out_mode, out_flags;
 
@@ -164,7 +161,7 @@ static int faad_map_channels (faad_decoder_t *this) {
     "faad_audio_decoder: channel layout: %s -> %s\n",
     input_names[this->in_mode], out_names[this->out_mode]);
   return 1;
-};
+}
 
 
 static void faad_reset (audio_decoder_t *this_gen) {
@@ -201,6 +198,7 @@ static void faad_meta_info_set ( faad_decoder_t *this ) {
                                 "AAC 2.0 (libfaad)");
         break;
       }
+      /* fall through */
     case 6:
       if (this->faac_finfo.sbr == SBR_UPSAMPLED)
         _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
@@ -310,7 +308,7 @@ static int faad_apply_conf (faad_decoder_t *this, uint8_t *conf, int len) {
       break;
     memcpy (&bits, conf, 4);
     bits = bebf_ADJ32 (bits);
-    if ((bits & 0xf8787c00) != ((AOT_PS << (32 - 5)) | (1 << (32 - 5 - 4 - 4)) | (AOT_AAC_LC << (32 - 5 - 4 - 4 - 4 - 5))))
+    if ((bits & 0xf8787c00) != (uint32_t)((AOT_PS << (32 - 5)) | (1 << (32 - 5 - 4 - 4)) | (AOT_AAC_LC << (32 - 5 - 4 - 4 - 4 - 5))))
       break;
     if (double_samplerates[(bits >> (32 - 5 - 4)) & 15] != ((bits >> (32 - 5 - 4 - 4 - 4)) & 15))
       break;
@@ -580,19 +578,6 @@ static void faad_decode_audio ( faad_decoder_t *this, int end_frame ) {
         faad_meta_info_set( this );
       }
 
-      /* estimate bitrate */
-      this->total_time += (1000*this->faac_finfo.samples/(this->rate*this->num_channels));
-      this->total_data += 8*used;
-
-      if ((this->total_time > LONG_MAX) || (this->total_data > LONG_MAX)) {
-        this->total_time >>= 2;
-        this->total_data >>= 2;
-      }
-
-      if (this->total_time)
-        _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITRATE,
-                           1000*(this->total_data/this->total_time));
-
       decoded = this->faac_finfo.samples / this->num_channels;
 
       lprintf("decoded %d/%d output %ld\n",
@@ -961,6 +946,7 @@ static void faad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 }
 
 static void faad_discontinuity (audio_decoder_t *this_gen) {
+  (void)this_gen;
 }
 
 static void faad_dispose (audio_decoder_t *this_gen) {
@@ -1011,8 +997,6 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   this->max_audio_src_size = 0;
   this->dec_config         = NULL;
   this->dec_config_size    = 0;
-  this->total_time         = 0;
-  this->total_data         = 0;
   this->rate               = 0;
 
   this->class  = (faad_class_t *)class_gen;
@@ -1104,11 +1088,12 @@ static void faad_class_unref (audio_decoder_class_t *this_gen) {
   free (master);
 }
 
-static void *faad_init_plugin (xine_t *xine, void *data, const char *id) {
+static void *faad_init_plugin (xine_t *xine, const void *data, const char *id) {
 
   faad_class_t *this, *master;
   struct cfg_entry_s *entry;
 
+  (void)data;
   this = calloc(1, sizeof (faad_class_t));
   if (!this)
     return NULL;
@@ -1143,11 +1128,11 @@ static void *faad_init_plugin (xine_t *xine, void *data, const char *id) {
   return this;
 }
 
-static void *latm_init_class (xine_t *xine, void *data) {
+static void *latm_init_class (xine_t *xine, const void *data) {
   return faad_init_plugin (xine, data, "FAAD-LATM");
 }
 
-static void *faad_init_class (xine_t *xine, void *data) {
+static void *faad_init_class (xine_t *xine, const void *data) {
   return faad_init_plugin (xine, data, "FAAD");
 }
 
@@ -1156,8 +1141,8 @@ static const uint32_t audio_types[] = {
  };
 
 static const decoder_info_t dec_info_audio = {
-  audio_types,         /* supported types */
-  8                    /* priority        */
+  .supported_types = audio_types,
+  .priority        = 8,
 };
 
 static const uint32_t latm_audio_types[] = {
@@ -1165,14 +1150,14 @@ static const uint32_t latm_audio_types[] = {
  };
 
 static const decoder_info_t dec_info_latm_audio = {
-  latm_audio_types,    /* supported types */
-  1                    /* priority        */
+  .supported_types = latm_audio_types,
+  .priority        = 1,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_DECODER, 16, "faad", XINE_VERSION_CODE, &dec_info_audio, faad_init_class },
   { PLUGIN_AUDIO_DECODER, 16, "faad-latm", XINE_VERSION_CODE, &dec_info_latm_audio, latm_init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
