@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,6 +25,7 @@
  * @date 2001-08-31 Added LPCM rate sensing
  */
 
+#define _DEFAULT_SOURCE 1
 #ifndef __sun
 #define _XOPEN_SOURCE 500
 #endif
@@ -52,10 +53,6 @@
 /*#include <Winsock2.h>*/ /* htons */
 #endif
 
-typedef struct {
-  audio_decoder_class_t   decoder_class;
-} lpcm_class_t;
-
 typedef struct lpcm_decoder_s {
   audio_decoder_t  audio_decoder;
 
@@ -187,6 +184,14 @@ static void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
     format_changed++;
   }
 
+  /* bail out if format was not regonized (avoid division by zero) */
+  if (!this->bits_per_sample || !this->number_of_channels || !this->rate) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+            "lpcm_decoder: unregonized format, dropped %zu bytes\n",
+            this->buffered_bytes);
+    return;
+  }
+
   /*
    * (re-)open output device
    */
@@ -360,18 +365,22 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 
   lpcm_decoder_t *this ;
 
+  (void)class_gen;
   this = (lpcm_decoder_t *) calloc(1, sizeof(lpcm_decoder_t));
-
+  if (!this)
+    return NULL;
+#ifndef HAVE_ZERO_SAFE_MEM
+  this->output_open        = 0;
+  this->rate               = 0;
+  this->bits_per_sample    = 0;
+  this->number_of_channels = 0;
+  this->ao_cap_mode        = 0;
+#endif
   this->audio_decoder.decode_data         = lpcm_decode_data;
   this->audio_decoder.reset               = lpcm_reset;
   this->audio_decoder.discontinuity       = lpcm_discontinuity;
   this->audio_decoder.dispose             = lpcm_dispose;
 
-  this->output_open   = 0;
-  this->rate          = 0;
-  this->bits_per_sample=0;
-  this->number_of_channels=0;
-  this->ao_cap_mode=0;
   this->stream = stream;
 
   this->cpu_be        = ( htons(1) == 1 );
@@ -379,18 +388,17 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   return &this->audio_decoder;
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
+static void *init_plugin (xine_t *xine, const void *data) {
 
-  lpcm_class_t *this ;
-
-  this = (lpcm_class_t *) calloc(1, sizeof(lpcm_class_t));
-
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "Linear PCM";
-  this->decoder_class.description     = N_("Linear PCM audio decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
-
-  return this;
+  static const audio_decoder_class_t this = {
+    .open_plugin     = open_plugin,
+    .identifier      = "Linear PCM",
+    .description     = N_("Linear PCM audio decoder plugin"),
+    .dispose         = NULL
+  };
+  (void)xine;
+  (void)data;
+  return (void *)&this;
 }
 
 static const uint32_t audio_types[] = {
@@ -398,12 +406,12 @@ static const uint32_t audio_types[] = {
 };
 
 static const decoder_info_t dec_info_audio = {
-  audio_types,         /* supported types */
-  1                    /* priority        */
+  .supported_types = audio_types,
+  .priority        = 1,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_DECODER, 16, "pcm", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
