@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -62,10 +62,6 @@
  */
 #define MAD_MIN_SIZE 2889
 
-typedef struct {
-  audio_decoder_class_t   decoder_class;
-} mad_class_t;
-
 typedef struct mad_decoder_s {
   audio_decoder_t   audio_decoder;
 
@@ -77,15 +73,15 @@ typedef struct mad_decoder_s {
   struct mad_stream stream;
   struct mad_frame  frame;
 
-  int               output_sampling_rate;
+  unsigned int      output_sampling_rate;
   int               output_open;
   int               output_mode;
 
   uint8_t           buffer[INPUT_BUF_SIZE];
   int               bytes_in_buffer;
   int               preview_mode;
-  int               start_padding;
-  int               end_padding;
+  unsigned int      start_padding;
+  unsigned int      end_padding;
   int               needs_more_data;
 
 } mad_decoder_t;
@@ -235,7 +231,7 @@ static void mad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	int mode = (this->frame.header.mode == MAD_MODE_SINGLE_CHANNEL) ? AO_CAP_MODE_MONO : AO_CAP_MODE_STEREO;
 
 	if (!this->output_open
-	    || (this->output_sampling_rate != this->frame.header.samplerate)
+            || (this->output_sampling_rate != this->frame.header.samplerate)
 	    || (this->output_mode != mode)) {
 
 	  lprintf ("audio sample rate %d mode %08x\n", this->frame.header.samplerate, mode);
@@ -292,7 +288,7 @@ static void mad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	  mad_fixed_t const   *left_ch, *right_ch;
 	  struct mad_pcm      *pcm = &this->synth.pcm;
 	  audio_buffer_t      *audio_buffer;
-	  uint16_t            *output;
+          int16_t             *output;
           int                  bitrate;
           int                  pts_offset;
 
@@ -307,12 +303,12 @@ static void mad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	  /* padding */
 	  if (this->start_padding || this->end_padding) {
 	    /* check padding validity */
-	    if (nsamples < (this->start_padding + this->end_padding)) {
+            if (nsamples < this->start_padding + this->end_padding) {
 	      lprintf("invalid padding data");
 	      this->start_padding = 0;
 	      this->end_padding = 0;
 	    }
-	    lprintf("nsamples=%d, start_padding=%d, end_padding=%d\n",
+            lprintf("nsamples=%u, start_padding=%u, end_padding=%u\n",
 	            nsamples, this->start_padding, this->end_padding);
 	    nsamples -= this->start_padding + this->end_padding;
 	    left_ch  += this->start_padding;
@@ -389,6 +385,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 
   mad_decoder_t *this ;
 
+  (void)class_gen;
   this = (mad_decoder_t *) calloc(1, sizeof(mad_decoder_t));
   if (!this)
     return NULL;
@@ -422,18 +419,17 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 /*
  * mad plugin class
  */
-static void *init_plugin (xine_t *xine, void *data) {
+static void *init_plugin (xine_t *xine, const void *data) {
 
-  mad_class_t *this;
-
-  this = (mad_class_t *) calloc(1, sizeof(mad_class_t));
-
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "mad";
-  this->decoder_class.description     = N_("libmad based mpeg audio layer 1/2/3 decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
-
-  return this;
+  (void)xine;
+  (void)data;
+  static const audio_decoder_class_t this = {
+    .open_plugin     = open_plugin,
+    .identifier      = "mad",
+    .description     = N_("libmad based mpeg audio layer 1/2/3 decoder plugin"),
+    .dispose         = NULL
+  };
+  return (void *)&this;
 }
 
 static const uint32_t audio_types[] = {
@@ -441,12 +437,12 @@ static const uint32_t audio_types[] = {
 };
 
 static const decoder_info_t dec_info_audio = {
-  audio_types,         /* supported types */
-  8                    /* priority        */
+  .supported_types = audio_types,
+  .priority        = 8,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_DECODER, 16, "mad", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
