@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -44,7 +44,7 @@
 
 #define ALSA_PCM_NEW_HW_PARAMS_API
 #define ALSA_PCM_NEW_SW_PARAMS_API
-#include <asoundlib.h>
+#include <alsa/asoundlib.h>
 
 #include <sys/ioctl.h>
 #include <inttypes.h>
@@ -246,9 +246,8 @@ static void *ao_alsa_handle_event_thread(void *data) {
 	event.data_length = sizeof(data);
 
 	pthread_mutex_lock(&this->class->xine->streams_lock);
-	for(ite = xine_list_front(this->class->xine->streams);
-	    ite; ite = xine_list_next(this->class->xine->streams, ite)) {
-	  stream = xine_list_get_value(this->class->xine->streams, ite);
+        ite = NULL;
+        while ((stream = xine_list_next_value (this->class->xine->streams, &ite))) {
 	  event.stream = stream;
 	  xine_event_send(stream, &event);
 	}
@@ -292,7 +291,13 @@ static void error_callback(const char *file, int line,
     printf("%s: %s() %s.\n", file, function, buf);
     free(buf);
   }
+#else
+  (void)file;
+  (void)function;
+  (void)fmt;
 #endif
+  (void)err;
+  (void)line;
 }
 
 /*
@@ -576,6 +581,10 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
 			    && snd_pcm_hw_params_can_resume (params) );
   xprintf(this->class->xine, XINE_VERBOSITY_DEBUG,
 	  "audio_alsa_out:open pause_resume=%d\n", this->has_pause_resume);
+  if (this->has_pause_resume)
+    this->capabilities &= ~AO_CAP_NO_UNPAUSE;
+  else
+    this->capabilities |= AO_CAP_NO_UNPAUSE;
   this->sample_rate_factor = (double) this->output_sample_rate / (double) this->input_sample_rate;
   this->bytes_per_frame = snd_pcm_frames_to_bytes (this->audio_fd, 1);
   /*
@@ -664,6 +673,7 @@ static int ao_alsa_bytes_per_frame(ao_driver_t *this_gen) {
  * Return gap tolerance (in pts)
  */
 static int ao_alsa_get_gap_tolerance (ao_driver_t *this_gen) {
+  (void)this_gen;
   return GAP_TOLERANCE;
 }
 
@@ -830,7 +840,7 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
   }
 
   while( number_of_frames > 0) {
-    if ( (state == SND_PCM_STATE_RUNNING) ) {
+    if ( state == SND_PCM_STATE_RUNNING ) {
 #ifdef LOG_DEBUG
       printf("audio_alsa_out:write:loop:waiting for Godot\n");
 #endif
@@ -929,6 +939,9 @@ static uint32_t ao_alsa_get_capabilities (ao_driver_t *this_gen) {
 static void ao_alsa_exit(ao_driver_t *this_gen) {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
 
+  this->class->xine->config->unregister_callbacks (this->class->xine->config,
+    "audio.output.speaker_arrangement", NULL, this, sizeof (*this));
+
   /*
    * Destroy the mixer thread and cleanup the mixer, so that
    * any child processes (such as xscreensaver) cannot inherit
@@ -1387,6 +1400,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   AUDIO_DEVICE_SPEAKER_ARRANGEMENT_TYPES;
   int speakers;
 
+  (void)data;
   this = calloc(1, sizeof (alsa_driver_t));
   if (!this)
     return NULL;
@@ -1409,55 +1423,55 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
                                  "will increase performance."),
                                10, NULL,
                                NULL);
-  pcm_device = config->register_string(config,
-				       "audio.device.alsa_default_device",
-				       "default",
-				       _("device used for mono output"),
-				       _("xine will use this alsa device to output "
-				         "mono sound.\nSee the alsa documentation "
-				         "for information on alsa devices."),
-				       10, NULL,
-				       NULL);
-  pcm_device = config->register_string(config,
-				       "audio.device.alsa_front_device",
-				       "plug:front:default",
-				       _("device used for stereo output"),
-				       _("xine will use this alsa device to output "
-				         "stereo sound.\nSee the alsa documentation "
-				         "for information on alsa devices."),
-				       10, NULL,
-				       NULL);
-  pcm_device = config->register_string(config,
-				       "audio.device.alsa_surround40_device",
-				       "plug:surround40:0",
-				       _("device used for 4-channel output"),
-				       _("xine will use this alsa device to output "
-				         "4 channel (4.0) surround sound.\nSee the "
-				         "alsa documentation for information on alsa "
-				         "devices."),
-				       10, NULL,
-				       NULL);
-  pcm_device = config->register_string(config,
-				       "audio.device.alsa_surround51_device",
-				       "plug:surround51:0",
-				       _("device used for 5.1-channel output"),
-				       _("xine will use this alsa device to output "
-				         "5 channel plus LFE (5.1) surround sound.\n"
-				         "See the alsa documentation for information "
-				         "on alsa devices."),
-                                       10,  NULL,
-				       NULL);
-  pcm_device = config->register_string(config,
-				       "audio.device.alsa_passthrough_device",
-				       "iec958:AES0=0x6,AES1=0x82,AES2=0x0,AES3=0x2",
-				       _("device used for 5.1-channel output"),
-				       _("xine will use this alsa device to output "
-				         "undecoded digital surround sound. This can "
-				         "be used be external surround decoders.\nSee the "
-				         "alsa documentation for information on alsa "
-				         "devices."),
-				       10, NULL,
-				       NULL);
+  config->register_string(config,
+                          "audio.device.alsa_default_device",
+                          "default",
+                          _("device used for mono output"),
+                          _("xine will use this alsa device to output "
+                            "mono sound.\nSee the alsa documentation "
+                            "for information on alsa devices."),
+                          10, NULL,
+                          NULL);
+  config->register_string(config,
+                          "audio.device.alsa_front_device",
+                          "plug:front:default",
+                          _("device used for stereo output"),
+                          _("xine will use this alsa device to output "
+                            "stereo sound.\nSee the alsa documentation "
+                            "for information on alsa devices."),
+                          10, NULL,
+                          NULL);
+  config->register_string(config,
+                          "audio.device.alsa_surround40_device",
+                          "plug:surround40:0",
+                          _("device used for 4-channel output"),
+                          _("xine will use this alsa device to output "
+                            "4 channel (4.0) surround sound.\nSee the "
+                            "alsa documentation for information on alsa "
+                            "devices."),
+                          10, NULL,
+                          NULL);
+  config->register_string(config,
+                          "audio.device.alsa_surround51_device",
+                          "plug:surround51:0",
+                          _("device used for 5.1-channel output"),
+                          _("xine will use this alsa device to output "
+                            "5 channel plus LFE (5.1) surround sound.\n"
+                            "See the alsa documentation for information "
+                            "on alsa devices."),
+                          10,  NULL,
+                          NULL);
+  config->register_string(config,
+                          "audio.device.alsa_passthrough_device",
+                          "iec958:AES0=0x6,AES1=0x82,AES2=0x0,AES3=0x2",
+                          _("device used for 5.1-channel output"),
+                          _("xine will use this alsa device to output "
+                            "undecoded digital surround sound. This can "
+                            "be used be external surround decoders.\nSee the "
+                            "alsa documentation for information on alsa "
+                            "devices."),
+                          10, NULL,
+                          NULL);
 
   /* Use the default device to open first */
   pcm_device = config->lookup_entry(config, "audio.device.alsa_default_device")->str_value;
@@ -1507,102 +1521,117 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
                                    AUDIO_DEVICE_SPEAKER_ARRANGEMENT_HELP,
                                    0, alsa_speaker_arrangement_cb, this);
 
-  char *logmsg = strdup (_("audio_alsa_out : supported modes are"));
-
-  if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_U8))) {
-    this->capabilities |= AO_CAP_8BITS;
-    xine_strcat_realloc (&logmsg, _(" 8bit"));
-  }
-  /* ALSA automatically appends _LE or _BE depending on the CPU */
-  if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_S16))) {
-    this->capabilities |= AO_CAP_16BITS;
-    xine_strcat_realloc (&logmsg, _(" 16bit"));
-  }
-  if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_S24))) {
-    this->capabilities |= AO_CAP_24BITS;
-    xine_strcat_realloc (&logmsg, _(" 24bit"));
-  }
-  if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_FLOAT))) {
-    this->capabilities |= AO_CAP_FLOAT32;
-    xine_strcat_realloc (&logmsg, _(" 32bit"));
-  }
-  if (0 == (this->capabilities & (AO_CAP_FLOAT32 | AO_CAP_24BITS | AO_CAP_16BITS | AO_CAP_8BITS))) {
-    xprintf(class->xine, XINE_VERBOSITY_LOG, "%s\n", logmsg);
-    free (logmsg);
-    xprintf (class->xine, XINE_VERBOSITY_DEBUG,
-             "audio_alsa_out: no supported PCM format found\n");
-    snd_pcm_close(this->audio_fd);
-    free(this);
-    return NULL;
-  }
-  if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 1))) {
-    this->capabilities |= AO_CAP_MODE_MONO;
-    xine_strcat_realloc (&logmsg, _(" mono"));
-  }
-  if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 2))) {
-    this->capabilities |= AO_CAP_MODE_STEREO;
-    xine_strcat_realloc (&logmsg, _(" stereo"));
-  }
-  if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 4)) &&
-     ( speakers == SURROUND4 )) {
-    this->capabilities |= AO_CAP_MODE_4CHANNEL;
-    xine_strcat_realloc (&logmsg, _(" 4-channel"));
-  }
-  else
-    xine_strcat_realloc (&logmsg, _(" (4-channel not enabled in xine config)"));
-
-  if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 6)) &&
-     ( speakers == SURROUND41 )) {
-    this->capabilities |= AO_CAP_MODE_4_1CHANNEL;
-    xine_strcat_realloc (&logmsg, _(" 4.1-channel"));
-  }
-  else
-    xine_strcat_realloc (&logmsg, _(" (4.1-channel not enabled in xine config)"));
-
-  if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 6)) &&
-     ( speakers == SURROUND5 )) {
-    this->capabilities |= AO_CAP_MODE_5CHANNEL;
-    xine_strcat_realloc (&logmsg, _(" 5-channel"));
-  }
-  else
-    xine_strcat_realloc (&logmsg, _(" (5-channel not enabled in xine config)"));
-
-  if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 6)) &&
-     ( speakers >= SURROUND51 )) {
-    this->capabilities |= AO_CAP_MODE_5_1CHANNEL;
-    xine_strcat_realloc (&logmsg, _(" 5.1-channel"));
-  }
-  else
-    xine_strcat_realloc (&logmsg, _(" (5.1-channel not enabled in xine config)"));
-
   this->has_pause_resume = 0; /* This is checked at open time instead */
   this->is_paused = 0;
-
-  snd_pcm_close (this->audio_fd);
-  this->audio_fd=NULL;
-
-  /* Fallback to "default" if device "front" does not exist */
-  /* Needed for some very basic sound cards. */
-  pcm_device = config->lookup_entry(config, "audio.device.alsa_front_device")->str_value;
-  err=snd_pcm_open(&this->audio_fd, pcm_device, SND_PCM_STREAM_PLAYBACK, 1); /* NON-BLOCK mode */
-  if(err < 0) {
-    config->update_string(config, "audio.device.alsa_front_device", "default");
-  } else {
-    snd_pcm_close (this->audio_fd);
-    this->audio_fd=NULL;
-  }
-
   this->output_sample_rate = 0;
-  if ( speakers == A52_PASSTHRU ) {
-    this->capabilities |= AO_CAP_MODE_A52;
-    this->capabilities |= AO_CAP_MODE_AC5;
-    xine_strcat_realloc (&logmsg, _(" a/52 and DTS pass-through"));
-  }
-  else
-    xine_strcat_realloc (&logmsg, _(" (a/52 and DTS pass-through not enabled in xine config)"));
 
-  xprintf(class->xine, XINE_VERBOSITY_LOG, "%s\n", logmsg);
-  free (logmsg);
+  {
+    char logbuf[2048], *q, *logend = logbuf + sizeof (logbuf);
+    q = logbuf;
+    q += strlcpy (q, _("audio_alsa_out : supported modes are"), logend - q);
+    if (q >= logend)
+      q = logend;
+    if (!(snd_pcm_hw_params_test_format (this->audio_fd, params, SND_PCM_FORMAT_U8))) {
+      this->capabilities |= AO_CAP_8BITS;
+      q += strlcpy (q, _(" 8bit"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    /* ALSA automatically appends _LE or _BE depending on the CPU */
+    if (!(snd_pcm_hw_params_test_format (this->audio_fd, params, SND_PCM_FORMAT_S16))) {
+      this->capabilities |= AO_CAP_16BITS;
+      q += strlcpy (q, _(" 16bit"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    if (!(snd_pcm_hw_params_test_format (this->audio_fd, params, SND_PCM_FORMAT_S24))) {
+      this->capabilities |= AO_CAP_24BITS;
+      q += strlcpy (q, _(" 24bit"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    if (!(snd_pcm_hw_params_test_format (this->audio_fd, params, SND_PCM_FORMAT_FLOAT))) {
+      this->capabilities |= AO_CAP_FLOAT32;
+      q += strlcpy (q, _(" 32bit"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    if (0 == (this->capabilities & (AO_CAP_FLOAT32 | AO_CAP_24BITS | AO_CAP_16BITS | AO_CAP_8BITS))) {
+      xprintf (class->xine, XINE_VERBOSITY_LOG, "%s.\n", logbuf);
+      xprintf (class->xine, XINE_VERBOSITY_DEBUG, "audio_alsa_out: no supported PCM format found.\n");
+      snd_pcm_close (this->audio_fd);
+      free (this);
+      return NULL;
+    }
+    if (!(snd_pcm_hw_params_test_channels (this->audio_fd, params, 1))) {
+      this->capabilities |= AO_CAP_MODE_MONO;
+      q += strlcpy (q, _(" mono"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    if (!(snd_pcm_hw_params_test_channels (this->audio_fd, params, 2))) {
+      this->capabilities |= AO_CAP_MODE_STEREO;
+      q += strlcpy (q, _(" stereo"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    if (!(snd_pcm_hw_params_test_channels (this->audio_fd, params, 4))) {
+      if (speakers == SURROUND4) {
+        this->capabilities |= AO_CAP_MODE_4CHANNEL;
+        q += strlcpy (q, _(" 4-channel"), logend - q);
+      } else
+        q += strlcpy (q, _(" (4-channel not enabled in xine config)"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    if (!(snd_pcm_hw_params_test_channels (this->audio_fd, params, 6))) {
+      if (speakers == SURROUND41) {
+        this->capabilities |= AO_CAP_MODE_4_1CHANNEL;
+        q += strlcpy (q, _(" 4.1-channel"), logend - q);
+      } else
+        q += strlcpy (q, _(" (4.1-channel not enabled in xine config)"), logend - q);
+      if (q >= logend)
+        q = logend;
+      if (speakers == SURROUND5) {
+        this->capabilities |= AO_CAP_MODE_5CHANNEL;
+        q += strlcpy (q, _(" 5-channel"), logend - q);
+      } else
+        q += strlcpy (q, _(" (5-channel not enabled in xine config)"), logend - q);
+      if (q >= logend)
+        q = logend;
+      if (speakers >= SURROUND51) {
+        this->capabilities |= AO_CAP_MODE_5_1CHANNEL;
+        q += strlcpy (q, _(" 5.1-channel"), logend - q);
+      } else
+        q += strlcpy (q, _(" (5.1-channel not enabled in xine config)"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+
+    snd_pcm_close (this->audio_fd);
+    this->audio_fd = NULL;
+
+    /* Fallback to "default" if device "front" does not exist. */
+    /* Needed for some very basic sound cards. */
+    pcm_device = config->lookup_entry (config, "audio.device.alsa_front_device")->str_value;
+    err = snd_pcm_open (&this->audio_fd, pcm_device, SND_PCM_STREAM_PLAYBACK, 1); /* NON-BLOCK mode */
+    if (err < 0) {
+      config->update_string (config, "audio.device.alsa_front_device", "default");
+    } else {
+      snd_pcm_close (this->audio_fd);
+      this->audio_fd = NULL;
+    }
+
+    if (speakers == A52_PASSTHRU) {
+      this->capabilities |= AO_CAP_MODE_A52 | AO_CAP_MODE_AC5;
+      q += strlcpy (q, _(" a/52 and DTS pass-through"), logend - q);
+    } else
+      q += strlcpy (q, _(" (a/52 and DTS pass-through not enabled in xine config)"), logend - q);
+/*  if (q >= logend)
+      q = logend; */
+
+    xprintf (class->xine, XINE_VERBOSITY_LOG, "%s.\n", logbuf);
+  }
 
   /* printf("audio_alsa_out: capabilities 0x%X\n",this->capabilities); */
 
@@ -1678,10 +1707,11 @@ static void alsa_speaker_arrangement_cb (void *user_data,
 /*
  * class functions
  */
-static void *init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, const void *data) {
 
   alsa_class_t        *this;
 
+  (void)data;
   this = calloc(1, sizeof (alsa_class_t));
   if (!this)
     return NULL;
@@ -1697,7 +1727,7 @@ static void *init_class (xine_t *xine, void *data) {
  }
 
 static ao_info_t ao_info_alsa = {
-  10
+  .priority = 10,
 };
 
 /*
@@ -1707,6 +1737,6 @@ static ao_info_t ao_info_alsa = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_OUT, AO_OUT_ALSA_IFACE_VERSION, "alsa", XINE_VERSION_CODE, &ao_info_alsa, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
