@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2000-2018 the xine project
+ *
+ * This file is part of xine, a free video player.
+ *
+ * xine is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * xine is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -64,7 +84,6 @@ typedef struct jack_driver_s
 typedef struct
 {
   audio_driver_class_t	driver_class;
-  config_values_t	*config;
   xine_t		*xine;
 } jack_class_t;
 
@@ -519,6 +538,8 @@ static int ao_jack_bytes_per_frame (ao_driver_t *this_gen)
 
 static int ao_jack_get_gap_tolerance (ao_driver_t *this_gen)
 {
+  (void)this_gen;
+
   return GAP_TOLERANCE;
 }
 
@@ -557,9 +578,9 @@ static int ao_jack_write (ao_driver_t *this_gen, int16_t *frame_buffer,
   this->underrun = 0;
   /* TODO: In the future Xine should pass only floats to us, so no conversion needed */
   if (this->bits_per_sample == 16)
-    written = write_buffer_16 (this, (char *) frame_buffer, num_bytes);
+    written = write_buffer_16 (this, (unsigned char *) frame_buffer, num_bytes);
   else if (this->bits_per_sample == 32)
-    written = write_buffer_32 (this, (char *) frame_buffer, num_bytes);
+    written = write_buffer_32 (this, (unsigned char *) frame_buffer, num_bytes);
 
   /* If this fails then need to spin and keep trying until everything written */
   int spin_count = 0;
@@ -579,9 +600,9 @@ static int ao_jack_write (ao_driver_t *this_gen, int16_t *frame_buffer,
 		       100) * 1000.0 * 1000.0) / this->output_sample_rate);
 
     if (this->bits_per_sample == 16)
-      written = write_buffer_16 (this, (char *) frame_buffer, num_bytes);
+      written = write_buffer_16 (this, (unsigned char *) frame_buffer, num_bytes);
     else if (this->bits_per_sample == 32)
-      written = write_buffer_32 (this, (char *) frame_buffer, num_bytes);
+      written = write_buffer_32 (this, (unsigned char *) frame_buffer, num_bytes);
 
     if (written == 0)
       spin_count++;
@@ -610,6 +631,8 @@ static void ao_jack_exit (ao_driver_t *this_gen)
 {
   jack_driver_t *this = (jack_driver_t *) this_gen;
 
+  this->xine->config->unregister_callbacks (this->xine->config, "audio.output.speaker_arrangement", NULL, this, sizeof (*this));
+
   ao_jack_close (this_gen);
   if (this->buffer)
     free (this->buffer);
@@ -681,7 +704,7 @@ static ao_driver_t *open_jack_plugin (audio_driver_class_t *class_gen,
 				      const void *data)
 {
   jack_class_t *class = (jack_class_t *) class_gen;
-  config_values_t *config = class->config;
+  config_values_t *config = class->xine->config;
   jack_driver_t *this;
 
   jack_client_t *client;
@@ -689,6 +712,8 @@ static ao_driver_t *open_jack_plugin (audio_driver_class_t *class_gen,
   char *jack_device;
   const char **matching_ports = NULL;
 
+  (void)data;
+
   AUDIO_DEVICE_SPEAKER_ARRANGEMENT_TYPES;
   int speakers;
 
@@ -852,18 +877,21 @@ static void jack_speaker_arrangement_cb (void *user_data,
 /*
  * class functions
  */
-static void *init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, const void *data) {
 
     jack_class_t        *this;
 
+    (void)data;
+
     this = calloc(1, sizeof (jack_class_t));
+    if (!this)
+      return NULL;
 
     this->driver_class.open_plugin     = open_jack_plugin;
     this->driver_class.identifier      = "jack";
     this->driver_class.description     = N_("xine output plugin for JACK Audio Connection Kit");
     this->driver_class.dispose         = default_audio_driver_class_dispose;
 
-    this->config = xine->config;
     this->xine   = xine;
 
     fprintf(stderr, "jack init_class returning %p\n", (void *)this);
@@ -871,7 +899,9 @@ static void *init_class (xine_t *xine, void *data) {
     return this;
 }
 
-static ao_info_t ao_info_jack = { 6 };
+static ao_info_t ao_info_jack = {
+  .priority = 6,
+};
 
 /*
  * exported plugin catalog entry
@@ -880,5 +910,5 @@ static ao_info_t ao_info_jack = { 6 };
 const plugin_info_t xine_plugin_info[] EXPORTED = {
     /* type, API, "name", version, special_info, init_function */
     { PLUGIN_AUDIO_OUT, AO_OUT_JACK_IFACE_VERSION, "jack", XINE_VERSION_CODE /* XINE_VERSION_CODE */, &ao_info_jack, init_class },
-    { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+    { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
