@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -120,8 +120,6 @@ typedef struct oss_driver_s {
   int              capabilities;
   int              mode;
 
-  config_values_t *config;
-
   int32_t          output_sample_rate, input_sample_rate;
   int32_t          output_sample_k_rate;
   uint32_t         num_channels;
@@ -150,7 +148,6 @@ typedef struct oss_driver_s {
 typedef struct {
   audio_driver_class_t  driver_class;
 
-  config_values_t      *config;
   xine_t               *xine;
 } oss_class_t;
 
@@ -173,7 +170,7 @@ static int ao_oss_open(ao_driver_t *this_gen,
 
   if (this->audio_fd > -1) {
 
-    if ( (mode == this->mode) && (rate == this->input_sample_rate) ) {
+    if ( (mode == this->mode) && (rate == (uint32_t)this->input_sample_rate) ) {
       return this->output_sample_rate;
     }
 
@@ -199,7 +196,13 @@ static int ao_oss_open(ao_driver_t *this_gen,
   }
 
   /* We wanted non blocking open but now put it back to normal */
-  fcntl(this->audio_fd, F_SETFL, fcntl(this->audio_fd, F_GETFL)&~O_NONBLOCK);
+  {
+    int val = fcntl(this->audio_fd, F_GETFL);
+    if (val != -1)
+      val = fcntl(this->audio_fd, F_SETFL, val & ~O_NONBLOCK);
+    if (val == -1)
+      lprintf("error restoring blocking mode: %s\n", strerror(errno));
+  }
 
   /*
    * configure audio device
@@ -368,6 +371,7 @@ static int ao_oss_get_gap_tolerance (ao_driver_t *this_gen){
 
   /* oss_driver_t *this = (oss_driver_t *) this_gen; */
 
+  (void)this_gen;
   return GAP_TOLERANCE;
 }
 
@@ -381,7 +385,7 @@ static int ao_oss_delay(ao_driver_t *this_gen) {
 
   switch (this->sync_method) {
   case OSS_SYNC_PROBEBUFFER:
-    if( this->bytes_in_buffer < this->buffer_size )
+    if( (int)(this->bytes_in_buffer) < this->buffer_size )
       bytes_left = this->bytes_in_buffer;
     else
       bytes_left = this->buffer_size;
@@ -425,7 +429,7 @@ static int ao_oss_delay(ao_driver_t *this_gen) {
 
     lprintf ("%d bytes output\n", info.bytes);
 
-    if (this->bytes_in_buffer < info.bytes) {
+    if ((int)(this->bytes_in_buffer) < info.bytes) {
       this->bytes_in_buffer -= this->last_getoptr; /* GETOPTR wrapped */
     }
 
@@ -471,7 +475,7 @@ static int ao_oss_write(ao_driver_t *this_gen,
 
     simulated_bytes_in_buffer = frames * this->bytes_per_frame;
 
-    if (this->bytes_in_buffer < simulated_bytes_in_buffer)
+    if ((int)(this->bytes_in_buffer) < simulated_bytes_in_buffer)
       this->bytes_in_buffer = simulated_bytes_in_buffer;
   }
 
@@ -503,6 +507,8 @@ static void ao_oss_exit(ao_driver_t *this_gen) {
 
   oss_driver_t    *this   = (oss_driver_t *) this_gen;
 
+  this->xine->config->unregister_callbacks (this->xine->config, "audio.output.speaker_arrangement", NULL, this, sizeof (*this));
+
   if (this->mixer.fd != -1)
     close(this->mixer.fd);
   if (this->audio_fd != -1)
@@ -704,7 +710,7 @@ static void oss_speaker_arrangement_cb (void *user_data,
 static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *data) {
 
   oss_class_t     *class = (oss_class_t *) class_gen;
-  config_values_t *config = class->config;
+  config_values_t *config = class->xine->config;
   oss_driver_t    *this;
   int              caps;
   int              audio_fd;
@@ -717,6 +723,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   int speakers;
 
 
+  (void)data;
   this = calloc(1, sizeof (oss_driver_t));
   if (!this)
     return NULL;
@@ -746,7 +753,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
 				       "The range of this value is -1 or 0-15. This setting is "
 				       "ignored, when the OSS audio device name is set to \"auto\"."),
 				     10, NULL, NULL);
-  if (devname_val == 0) {
+  if (devname_val == 0 || devname_val > 15) {
     xprintf(class->xine, XINE_VERBOSITY_LOG,
 	    _("audio_oss_out: audio.device.oss_device_name = auto, probing devs\n"));
     if ( ! probe_audio_devices(this)) {  /* Returns zero on fail */
@@ -897,7 +904,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
     }
   }
 
-  this->capabilities = 0;
+  /* we always reset or reopen at pause for now. */
+  this->capabilities = AO_CAP_NO_UNPAUSE;
 
   arg = AFMT_U8;
   if( ioctl(audio_fd, SNDCTL_DSP_SETFMT, &arg) != -1  && arg == AFMT_U8)
@@ -918,72 +926,84 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
                                    AUDIO_DEVICE_SPEAKER_ARRANGEMENT_HELP,
                                    0, oss_speaker_arrangement_cb, this);
 
-
-  char *logmsg = strdup (_("audio_oss_out: supported modes are"));
-  num_channels = 1;
-  status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
-  if ( (status != -1) && (num_channels==1) ) {
-    this->capabilities |= AO_CAP_MODE_MONO;
-    xine_strcat_realloc (&logmsg, _(" mono"));
-  }
-  num_channels = 2;
-  status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
-  if ( (status != -1) && (num_channels==2) ) {
-    this->capabilities |= AO_CAP_MODE_STEREO;
-    xine_strcat_realloc (&logmsg, _(" stereo"));
-  }
-  num_channels = 4;
-  status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
-  if ( (status != -1) && (num_channels==4) )  {
-    if  ( speakers == SURROUND4 ) {
-      this->capabilities |= AO_CAP_MODE_4CHANNEL;
-      xine_strcat_realloc (&logmsg, _(" 4-channel"));
+  {
+    char logbuf[2048], *q, *logend = logbuf + sizeof (logbuf);
+    q = logbuf;
+    q += strlcpy (q, _("audio_oss_out: supported modes are"), logend - q);
+    if (q >= logend)
+      q = logend;
+    num_channels = 1;
+    status = ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
+    if ((status != -1) && (num_channels == 1)) {
+      this->capabilities |= AO_CAP_MODE_MONO;
+      q += strlcpy (q, _(" mono"), logend - q);
+      if (q >= logend)
+        q = logend;
     }
-    else
-      xine_strcat_realloc (&logmsg, _(" (4-channel not enabled in xine config)"));
-  }
-  num_channels = 5;
-  status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
-  if ( (status != -1) && (num_channels==5) ) {
-    if  ( speakers == SURROUND5 ) {
-      this->capabilities |= AO_CAP_MODE_5CHANNEL;
-      xine_strcat_realloc (&logmsg, _(" 5-channel"));
+    num_channels = 2;
+    status = ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
+    if ((status != -1) && (num_channels == 2)) {
+      this->capabilities |= AO_CAP_MODE_STEREO;
+      q += strlcpy (q, _(" stereo"), logend - q);
+      if (q >= logend)
+        q = logend;
     }
-    else
-      xine_strcat_realloc (&logmsg, _(" (5-channel not enabled in xine config)"));
-  }
-  num_channels = 6;
-  status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
-  if ( (status != -1) && (num_channels==6) ) {
-    if  ( speakers == SURROUND51 ) {
-      this->capabilities |= AO_CAP_MODE_5_1CHANNEL;
-      xine_strcat_realloc (&logmsg, _(" 5.1-channel"));
+    num_channels = 4;
+    status = ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
+    if ((status != -1) && (num_channels == 4)) {
+      if (speakers == SURROUND4) {
+        this->capabilities |= AO_CAP_MODE_4CHANNEL;
+        q += strlcpy (q, _(" 4-channel"), logend - q);
+      } else
+        q += strlcpy (q, _(" (4-channel not enabled in xine config)"), logend - q);
+      if (q >= logend)
+        q = logend;
     }
-    else
-      xine_strcat_realloc (&logmsg, _(" (5.1-channel not enabled in xine config)"));
+    num_channels = 5;
+    status = ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
+    if ((status != -1) && (num_channels == 5)) {
+      if (speakers == SURROUND5) {
+        this->capabilities |= AO_CAP_MODE_5CHANNEL;
+        q += strlcpy (q, _(" 5-channel"), logend - q);
+      } else
+        q += strlcpy (q, _(" (5-channel not enabled in xine config)"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+    num_channels = 6;
+    status = ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &num_channels);
+    if ((status != -1) && (num_channels == 6)) {
+      if (speakers == SURROUND51 ) {
+        this->capabilities |= AO_CAP_MODE_5_1CHANNEL;
+        q += strlcpy (q, _(" 5.1-channel"), logend - q);
+      } else
+        q += strlcpy (q, _(" (5.1-channel not enabled in xine config)"), logend - q);
+      if (q >= logend)
+        q = logend;
+    }
+
+    ioctl (audio_fd, SNDCTL_DSP_GETFMTS, &caps);
+
+    /* one would normally check for (caps & AFMT_AC3) before asking about passthrough,
+     * but some buggy OSS drivers do not report this properly, so we ask anyway. */
+    if (speakers == A52_PASSTHRU) {
+      this->capabilities |= AO_CAP_MODE_A52;
+      this->capabilities |= AO_CAP_MODE_AC5;
+      q += strlcpy (q, _(" a/52 pass-through"), logend - q);
+    } else
+      q += strlcpy (q, _(" (a/52 pass-through not enabled in xine config)"), logend - q);
+/*  if (q >= logend)
+      q = logend; */
+
+    xprintf (class->xine, XINE_VERBOSITY_DEBUG, "%s\n", logbuf);
   }
 
-  ioctl(audio_fd,SNDCTL_DSP_GETFMTS,&caps);
-
-  /* one would normally check for (caps & AFMT_AC3) before asking about passthrough,
-   * but some buggy OSS drivers do not report this properly, so we ask anyway */
-  if  ( speakers == A52_PASSTHRU ) {
-    this->capabilities |= AO_CAP_MODE_A52;
-    this->capabilities |= AO_CAP_MODE_AC5;
-    xine_strcat_realloc (&logmsg, _(" a/52 pass-through"));
-  }
-  else
-    xine_strcat_realloc (&logmsg, _(" (a/52 pass-through not enabled in xine config)"));
-
-  xprintf(class->xine, XINE_VERBOSITY_DEBUG, "%s\n", logmsg);
-  free (logmsg);
-
   /*
    * mixer initialisation.
    */
   {
     char mixer_name[32];
-    char mixer_dev[32];
+    char mixer_dev[48];
     int mixer_num;
     int audio_devs;
     char *parse;
@@ -1004,7 +1024,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
     if ((parse = strstr(mixer_name, "dsp"))) {
       parse[0] = '\0';
       parse += 3;
-      if (devname_val == 0)
+      if (devname_val == 0 || devname_val > 15)
 	snprintf(mixer_dev, sizeof(mixer_dev), "%smixer%s", mixer_name, parse);
       else if (mixer_num == -1)
 	snprintf(mixer_dev, sizeof(mixer_dev), "%smixer", mixer_name);
@@ -1053,7 +1073,6 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   this->audio_fd              = -1;
   this->xine                  = class->xine;
 
-  this->config                        = config;
   this->ao_driver.get_capabilities    = ao_oss_get_capabilities;
   this->ao_driver.get_property        = ao_oss_get_property;
   this->ao_driver.set_property        = ao_oss_set_property;
@@ -1107,10 +1126,11 @@ static void oss_speaker_arrangement_cb (void *user_data,
 /*
  * class functions
  */
-static void *init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, const void *data) {
 
   oss_class_t        *this;
 
+  (void)data;
   this = calloc(1, sizeof (oss_class_t));
   if (!this)
     return NULL;
@@ -1120,14 +1140,13 @@ static void *init_class (xine_t *xine, void *data) {
   this->driver_class.description     = N_("xine audio output plugin using oss-compliant audio devices/drivers");
   this->driver_class.dispose         = default_audio_driver_class_dispose;
 
-  this->config = xine->config;
   this->xine   = xine;
 
   return this;
 }
 
 static const ao_info_t ao_info_oss = {
-  9 /* less than alsa so xine will use alsa's native interface by default */
+  .priority = 9, /* less than alsa so xine will use alsa's native interface by default */
 };
 
 /*
@@ -1137,5 +1156,5 @@ static const ao_info_t ao_info_oss = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_OUT, AO_OUT_OSS_IFACE_VERSION, "oss", XINE_VERSION_CODE, &ao_info_oss, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
