@@ -365,7 +365,7 @@ static void dispose_class (audio_driver_class_t *this_gen)
   free(this);
 }
 
-static void *init_class (xine_t *xine, void *data)
+static void *init_class (xine_t *xine, const void *data)
 {
   sndio_class_t        *this;
 
@@ -386,7 +386,7 @@ static void *init_class (xine_t *xine, void *data)
 }
 
 static const ao_info_t ao_info_sndio = {
-  12
+  .priority = 12,
 };
 
 /*
@@ -396,5 +396,5 @@ static const ao_info_t ao_info_sndio = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_OUT, 9, "sndio", XINE_VERSION_CODE, &ao_info_sndio, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
