@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2017 the xine project
+ * Copyright (C) 2001-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -90,7 +90,7 @@ typedef struct ff_audio_decoder_s {
   int               ff_channels;
   int               ff_bits;
   int               ff_sample_rate;
-  int64_t           ff_map;
+  uint64_t          ff_map;
 
   /* channel mixer settings */
   /* map[ao_channel] = ff_channel */
@@ -148,7 +148,7 @@ static int ff_aac_mode_parse (ff_audio_decoder_t *this, uint8_t *buf, int size,
       /* also test the "layer" bits for 0 (mpeg layer 4 audio). */
       /* dont get fooled by 0xff padding bytes. */
       if ((v & 0xfff6) == 0xfff0) {
-        uint32_t s;
+        int s;
         /* test header size */
         if (size - i < 7 - 1)
           continue;
@@ -197,7 +197,7 @@ static int ff_aac_mode_parse (ff_audio_decoder_t *this, uint8_t *buf, int size,
       v <<= 8;
       v |= buf[i];
       if ((v & 0xfff6) == 0xfff0) {
-        uint32_t s;
+        int s;
         /* test header size */
         if (size - i < 7 - 1)
           return -1;
@@ -513,16 +513,16 @@ static void ff_map_channels (ff_audio_decoder_t *this) {
     ff_map = ((uint64_t)1 << this->context->channels) - 1;
 
   if ((caps != this->ao_caps) || (ff_map != this->ff_map)) {
-    int i, j;
+    unsigned int i, j;
     /* ff: see names[] below; xine: L R RL RR C LFE */
     const int8_t base_map[] = {0, 1, 4, 5, 2, 3, -1, -1, -1, 2, 3};
-    int8_t name_map[MAX_CHANNELS];
+    int8_t name_map[MAX_CHANNELS] = {0, 0, 0, 0, 0, 0};
     const int modes[] = {
       AO_CAP_MODE_MONO, AO_CAP_MODE_STEREO,
       AO_CAP_MODE_4CHANNEL, AO_CAP_MODE_4_1CHANNEL,
       AO_CAP_MODE_5CHANNEL, AO_CAP_MODE_5_1CHANNEL
     };
-    const int num_modes = sizeof (modes) / sizeof (modes[0]);
+    const unsigned int num_modes = sizeof (modes) / sizeof (modes[0]);
     const int8_t mode_channels[]   = {1, 2, 4, 6, 6, 6};
     const int8_t wishlist[] = {
       0, 1, 2, 3, 4, 5, /* mono */
@@ -620,13 +620,13 @@ static void ff_map_channels (ff_audio_decoder_t *this) {
       int8_t *indx = this->left;
       for (i = 0; i < 2; i++) {
         buf[p++] = '[';
-        for (j = 0; j < this->front_mixes; j++)
-          p += sprintf (buf + p, "%s%s", names[name_map[indx[j]]], (j < this->front_mixes - 1) ? " + " : "");
+        for (j = 0; (int)j < this->front_mixes; j++)
+          p += sprintf (buf + p, "%s%s", names[name_map[indx[j]]], ((int)j < this->front_mixes - 1) ? " + " : "");
         buf[p++] = ']';
         buf[p++] = ' ';
         indx = this->right;
       }
-      for (i = 2; i < this->ao_channels; i++)
+      for (i = 2; i < (unsigned int)this->ao_channels; i++)
         p += sprintf (buf + p, "[%s] ",
           ((this->map[i] < 0) || (this->map[i] > 5)) ? (const int8_t *)"-" : names[name_map[this->map[i]]]);
       buf[p++] = '\n';
@@ -691,8 +691,20 @@ static int ff_audio_decode (ff_audio_decoder_t *this,
   float gain = this->class->gain;
   if (!this->av_frame)
     this->av_frame = XFF_ALLOC_FRAME ();
-
+#   if XFF_AUDIO == 5
+  {
+    int err = avcodec_send_packet (this->context, &avpkt);
+    /* xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "ff_audio_dec: send (%d) = %d.\n", (int)size, err); */
+    /* multiple frames per packet */
+    consumed = (err >= 0) ? size : ((err == AVERROR (EAGAIN)) ? 0 : err);
+    /* that calls av_frame_unref () first. */
+    err = avcodec_receive_frame (this->context, this->av_frame);
+    /* xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "ff_audio_dec: recv () = %d.\n", err); */
+    got_frame = (err == 0);
+  }
+#   else
   consumed = avcodec_decode_audio4 (this->context, this->av_frame, &got_frame, &avpkt);
+#   endif
   if ((consumed >= 0) && got_frame) {
     /* setup may have altered while decoding */
     ff_map_channels (this);
@@ -1011,7 +1023,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
       /* pad input data */
       memset(this->buf + this->size, 0, AV_INPUT_BUFFER_PADDING_SIZE);
 
-      while (this->size>=0) {
+      while (this->size > 0) {
 
         decode_buffer_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;
 
@@ -1195,7 +1207,12 @@ static void ff_audio_reset (audio_decoder_t *this_gen) {
   /* try to reset the wma decoder */
   if( this->decoder_ok ) {
 #if XFF_AUDIO > 3
-    XFF_FREE_FRAME (this->av_frame);
+    if (this->av_frame) {
+# if XFF_AUDIO == 5
+      av_frame_unref (this->av_frame);
+# endif
+      XFF_FREE_FRAME (this->av_frame);
+    }
 #endif
     pthread_mutex_lock (&ffmpeg_lock);
     avcodec_close (this->context);
@@ -1231,7 +1248,12 @@ static void ff_audio_dispose (audio_decoder_t *this_gen) {
 
   if( this->decoder_ok ) {
 #if XFF_AUDIO > 3
-    XFF_FREE_FRAME (this->av_frame);
+    if (this->av_frame) {
+# if XFF_AUDIO == 5
+      av_frame_unref (this->av_frame);
+# endif
+      XFF_FREE_FRAME (this->av_frame);
+    }
 #endif
     pthread_mutex_lock (&ffmpeg_lock);
     avcodec_close (this->context);
@@ -1258,6 +1280,7 @@ static audio_decoder_t *ff_audio_open_plugin (audio_decoder_class_t *class_gen,
   if (!this)
     return NULL;
 
+#ifndef HAVE_ZERO_SAFE_MEM
   /* Do these first, when compiler still knows stream is all zeroed.
    * Let it optimize away this on most systems where clear mem
    * interpretes as 0, 0f or NULL safely.
@@ -1266,6 +1289,10 @@ static audio_decoder_t *ff_audio_open_plugin (audio_decoder_class_t *class_gen,
   this->ff_channels = 0;
   this->size        = 0;
   this->decoder_ok  = 0;
+# if XFF_AUDIO > 3
+  this->av_frame    = NULL;
+# endif
+#endif
 
   this->class  = (ff_audio_class_t *)class_gen;
   this->stream = stream;
@@ -1292,9 +1319,6 @@ static audio_decoder_t *ff_audio_open_plugin (audio_decoder_class_t *class_gen,
     return NULL;
   } while (0);
 
-#if XFF_AUDIO > 3
-  this->av_frame = NULL;
-#endif
   return &this->audio_decoder;
 }
 
@@ -1309,15 +1333,17 @@ static void dispose_audio_class (audio_decoder_class_t *this_gen) {
   ff_audio_class_t *this = (ff_audio_class_t *)this_gen;
   config_values_t *config = this->xine->config;
 
-  config->unregister_callback (config, "audio.processing.ffmpeg_gain_dB");
+  config->unregister_callbacks (config, NULL, NULL, this, sizeof (*this));
 
   free (this);
 }
 
-void *init_audio_plugin (xine_t *xine, void *data) {
+void *init_audio_plugin (xine_t *xine, const void *data) {
 
   ff_audio_class_t *this ;
 
+  (void)data;
+
   this = calloc(1, sizeof (ff_audio_class_t));
   if (!this) {
     return NULL;
@@ -1348,3 +1374,4 @@ const decoder_info_t dec_info_ffmpeg_audio = {
   supported_audio_types,   /* supported types */
   7                        /* priority        */
 };
+
