@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2017 the xine project
+ * Copyright (C) 2001-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -44,12 +44,14 @@
 #include "ffmpeg_decoder.h"
 #include "ff_mpeg_parser.h"
 
+#ifdef HAVE_POSTPROC
 #ifdef HAVE_FFMPEG_AVUTIL_H
 # include <postprocess.h>
 #else
 # include <libpostproc/postprocess.h>
 # include <libavutil/mem.h>
 #endif
+#endif
 
 #ifdef HAVE_VA_VA_X11_H
 # include <libavcodec/vaapi.h>
@@ -80,7 +82,9 @@ typedef struct ff_video_decoder_s ff_video_decoder_t;
 typedef struct ff_video_class_s {
   video_decoder_class_t   decoder_class;
 
+#ifdef HAVE_POSTPROC
   int                     pp_quality;
+#endif
   int                     thread_count;
   int8_t                  skip_loop_filter_enum;
   int8_t                  choose_speed_over_accuracy;
@@ -110,7 +114,9 @@ struct ff_video_decoder_s {
   uint8_t           decoder_ok:1;
   uint8_t           decoder_init_mode:1;
   uint8_t           is_mpeg12:1;
+#ifdef HAVE_POSTPROC
   uint8_t           pp_available:1;
+#endif
   uint8_t           is_direct_rendering_disabled:1;  /* used only to avoid flooding log */
   uint8_t           cs_convert_init:1;
   uint8_t           assume_bad_field_picture:1;
@@ -131,10 +137,12 @@ struct ff_video_decoder_s {
   AVCodecContext   *context;
   AVCodec          *codec;
 
+#ifdef HAVE_POSTPROC
   int               pp_quality;
   int               pp_flags;
   pp_context       *our_context;
   pp_mode          *our_mode;
+#endif /* HAVE_POSTPROC */
 
   /* mpeg-es parsing */
   mpeg_parser_t    *mpeg_parser;
@@ -163,7 +171,7 @@ struct ff_video_decoder_s {
   unsigned char     ytab[1024], ctab[1024];
 
   int               pix_fmt;
-  void             *rgb2yuy2;
+  rgb2yuy2_t       *rgb2yuy2;
 
 #ifdef LOG
   enum PixelFormat  debug_fmt;
@@ -189,6 +197,9 @@ struct ff_video_decoder_s {
     STATE_FLUSHED
   }                 state;
   int               decode_attempts;
+#if XFF_VIDEO == 3
+  int               flush_packet_sent;
+#endif
 
 #ifdef ENABLE_EMMS
   /* see get_buffer () */
@@ -344,20 +355,20 @@ static ff_saved_frame_t *ffsf_new (ff_video_decoder_t *this) {
     ffsf = calloc (1, sizeof (*ffsf));
     if (ffsf) {
       ffsf->this = this;
-      DLIST_ADD_TAIL (ffsf, &this->ffsf_used);
+      DLIST_ADD_TAIL (&ffsf->node, &this->ffsf_used);
       this->ffsf_num++;
       this->ffsf_total++;
     }
   } else {
     ffsf = (ff_saved_frame_t *)this->ffsf_free.head;
-    DLIST_REMOVE (ffsf);
+    DLIST_REMOVE (&ffsf->node);
     ffsf->refs = 0;
     ffsf->this = this;
     ffsf->vo_frame = NULL;
 # ifdef ENABLE_VAAPI
     ffsf->va_surface = NULL;
 # endif
-    DLIST_ADD_TAIL (ffsf, &this->ffsf_used);
+    DLIST_ADD_TAIL (&ffsf->node, &this->ffsf_used);
     this->ffsf_num++;
   }
   pthread_mutex_unlock (&this->ffsf_mutex);
@@ -369,8 +380,8 @@ static void ffsf_delete (ff_saved_frame_t *ffsf) {
     return;
 
   pthread_mutex_lock (&ffsf->this->ffsf_mutex);
-  DLIST_REMOVE (ffsf);
-  DLIST_ADD_TAIL (ffsf, &ffsf->this->ffsf_free);
+  DLIST_REMOVE (&ffsf->node);
+  DLIST_ADD_TAIL (&ffsf->node, &ffsf->this->ffsf_free);
   ffsf->this->ffsf_num--;
   pthread_mutex_unlock (&ffsf->this->ffsf_mutex);
 }
@@ -378,13 +389,14 @@ static void ffsf_delete (ff_saved_frame_t *ffsf) {
 # ifdef XFF_AV_BUFFER
 static void release_frame (void *saved_frame, uint8_t *data) {
   ff_saved_frame_t *ffsf = saved_frame;
+  (void)data;
   /* At this point in time, AVFrame may already be reused. So take our saved values instead. */
   if (ffsf) {
     if (--(ffsf->refs))
       return;
 #  ifdef ENABLE_VAAPI
     if (ffsf->va_surface)
-      ffsf->this->accel->release_vaapi_surface (ffsf->this->accel_img, ffsf->va_surface);
+      ffsf->this->accel->f->release_vaapi_surface (ffsf->this->accel_img, ffsf->va_surface);
 #  endif
     if (ffsf->vo_frame)
       ffsf->vo_frame->free (ffsf->vo_frame);
@@ -397,10 +409,10 @@ static void release_buffer(struct AVCodecContext *context, AVFrame *av_frame){
 
 #  ifdef ENABLE_VAAPI
   if( this->context->pix_fmt == PIX_FMT_VAAPI_VLD ) {
-    if(this->accel->guarded_render(this->accel_img)) {
+    if(this->accel->f->guarded_render(this->accel_img)) {
       ff_vaapi_surface_t *va_surface = (ff_vaapi_surface_t *)av_frame->data[0];
       if(va_surface != NULL) {
-        this->accel->release_vaapi_surface(this->accel_img, va_surface);
+        this->accel->f->release_vaapi_surface(this->accel_img, va_surface);
         lprintf("release_buffer: va_surface_id 0x%08x\n", (unsigned int)av_frame->data[3]);
       }
     }
@@ -458,10 +470,10 @@ static int get_buffer_vaapi_vld (AVCodecContext *context, AVFrame *av_frame)
 
     this->vaapi_width  = width;
     this->vaapi_height = height;
-    status = this->accel->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
+    status = this->accel->f->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
 
     if (status == VA_STATUS_SUCCESS) {
-      ff_vaapi_context_t *va_context = this->accel->get_context (this->accel_img);
+      ff_vaapi_context_t *va_context = this->accel->f->get_context (this->accel_img);
 
       if (va_context) {
         this->vaapi_context.config_id  = va_context->va_config_id;
@@ -471,7 +483,7 @@ static int get_buffer_vaapi_vld (AVCodecContext *context, AVFrame *av_frame)
     }
   }
 
-  if(!this->accel->guarded_render(this->accel_img)) {
+  if(!this->accel->f->guarded_render(this->accel_img)) {
     vo_frame_t *img;
     img = this->stream->video_out->get_frame (this->stream->video_out,
                                               width,
@@ -481,7 +493,7 @@ static int get_buffer_vaapi_vld (AVCodecContext *context, AVFrame *av_frame)
                                               VO_BOTH_FIELDS|this->frame_flags);
 
     vaapi_accel_t *accel = (vaapi_accel_t*)img->accel_data;
-    ff_vaapi_surface_t *va_surface = accel->get_vaapi_surface(img);
+    ff_vaapi_surface_t *va_surface = accel->f->get_vaapi_surface(img);
 
     if(va_surface) {
       av_frame->data[0] = (void *)va_surface;//(void *)(uintptr_t)va_surface->va_surface_id;
@@ -489,7 +501,7 @@ static int get_buffer_vaapi_vld (AVCodecContext *context, AVFrame *av_frame)
     }
     ffsf->vo_frame = img;
   } else {
-    ff_vaapi_surface_t *va_surface = this->accel->get_vaapi_surface(this->accel_img);
+    ff_vaapi_surface_t *va_surface = this->accel->f->get_vaapi_surface(this->accel_img);
 
     if(va_surface) {
       av_frame->data[0] = (void *)va_surface;//(void *)(uintptr_t)va_surface->va_surface_id;
@@ -592,7 +604,7 @@ static int get_buffer (AVCodecContext *context, AVFrame *av_frame)
   }
 
   if(this->accel)
-    guarded_render = this->accel->guarded_render(this->accel_img);
+    guarded_render = this->accel->f->guarded_render(this->accel_img);
 # endif /* ENABLE_VAAPI */
 
   /* The alignment rhapsody */
@@ -828,7 +840,7 @@ static enum PixelFormat get_format(struct AVCodecContext *context, const enum Pi
       continue;
     }
 
-    this->vaapi_profile = accel->profile_from_imgfmt (this->accel_img, format);
+    this->vaapi_profile = accel->f->profile_from_imgfmt (this->accel_img, format);
 
     if (this->vaapi_profile >= 0) {
       int width  = context->width;
@@ -841,10 +853,10 @@ static enum PixelFormat get_format(struct AVCodecContext *context, const enum Pi
       }
       this->vaapi_width  = width;
       this->vaapi_height = height;
-      status = accel->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
+      status = accel->f->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
 
       if( status == VA_STATUS_SUCCESS ) {
-        ff_vaapi_context_t *va_context = accel->get_context(this->accel_img);
+        ff_vaapi_context_t *va_context = accel->f->get_context(this->accel_img);
 
         if(!va_context)
           break;
@@ -874,30 +886,6 @@ static enum PixelFormat get_format(struct AVCodecContext *context, const enum Pi
 static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type) {
   int thread_count = this->class->thread_count;
   int use_vaapi = 0;
-  size_t i;
-
-  /* find the decoder */
-  this->codec = NULL;
-
-  for(i = 0; i < sizeof(ff_video_lookup)/sizeof(ff_codec_t); i++)
-    if(ff_video_lookup[i].type == codec_type) {
-      pthread_mutex_lock(&ffmpeg_lock);
-      this->codec = avcodec_find_decoder(ff_video_lookup[i].id);
-      pthread_mutex_unlock(&ffmpeg_lock);
-      _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC,
-                            ff_video_lookup[i].name);
-      break;
-    }
-
-  if (!this->codec) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-            _("ffmpeg_video_dec: couldn't find ffmpeg decoder for buf type 0x%X\n"),
-            codec_type);
-    _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HANDLED, 0);
-    return;
-  }
-
-  lprintf("lavc decoder found\n");
 
   this->context->width = this->bih.biWidth;
   this->context->height = this->bih.biHeight;
@@ -917,13 +905,14 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
          This will be slightly faster. And it is also a workaround for buggy
          v54 who likes to ignore EMU_EDGE for wmv2 and xvid. */
       this->edge = XFF_EDGE_WIDTH ();
+    }
 #ifdef CODEC_FLAG_EMU_EDGE
-    } else {
+    else {
       /* Some codecs (eg rv10) copy flags in init so it's necessary to set
        * this flag here in case we are going to use direct rendering */
       this->context->flags |= CODEC_FLAG_EMU_EDGE;
-#endif
     }
+#endif
   }
 
   /* TJ. without this, it wont work at all on my machine */
@@ -948,7 +937,7 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
 #ifdef ENABLE_VAAPI
   if( this->class->enable_vaapi ) {
     uint32_t format = vaapi_pixfmt2imgfmt(PIX_FMT_VAAPI_VLD, this->codec->id, -1);
-    if (format && this->accel->profile_from_imgfmt (this->accel_img, format) >= 0) {
+    if (format && this->accel->f->profile_from_imgfmt (this->accel_img, format) >= 0) {
       use_vaapi = 1;
     } else {
       xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
@@ -964,7 +953,9 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
 # ifdef XFF_AV_BUFFER
     this->context->get_buffer2 = get_buffer;
     this->context->thread_safe_callbacks = 1;
+#  if XFF_VIDEO != 3
     this->context->refcounted_frames = 1;
+#  endif
 # else
     this->context->get_buffer = get_buffer;
     this->context->release_buffer = release_buffer;
@@ -1125,11 +1116,13 @@ static void thread_count_cb(void *user_data, xine_cfg_entry_t *entry) {
     class->thread_count = 8;
 }
 
+#ifdef HAVE_POSTPROC
 static void pp_quality_cb(void *user_data, xine_cfg_entry_t *entry) {
   ff_video_class_t   *class = (ff_video_class_t *) user_data;
 
   class->pp_quality = entry->num_value;
 }
+#endif
 
 static void dri_cb(void *user_data, xine_cfg_entry_t *entry) {
   ff_video_class_t   *class = (ff_video_class_t *) user_data;
@@ -1137,6 +1130,7 @@ static void dri_cb(void *user_data, xine_cfg_entry_t *entry) {
   class->enable_dri = entry->num_value;
 }
 
+#ifdef HAVE_POSTPROC
 static void pp_change_quality (ff_video_decoder_t *this) {
   this->pp_quality = this->class->pp_quality;
 
@@ -1161,8 +1155,10 @@ static void pp_change_quality (ff_video_decoder_t *this) {
     }
   }
 }
+#endif /* HAVE_POSTPROC */
 
 static void init_postprocess (ff_video_decoder_t *this) {
+#ifdef HAVE_POSTPROC
 #if defined(ARCH_X86)
   uint32_t cpu_caps;
 #endif
@@ -1200,6 +1196,7 @@ static void init_postprocess (ff_video_decoder_t *this) {
 
   /* Set level */
   pp_change_quality(this);
+#endif /* HAVE_POSTPROC */
 }
 
 static int ff_handle_mpeg_sequence(ff_video_decoder_t *this, mpeg_parser_t *parser) {
@@ -1254,7 +1251,7 @@ static void ff_setup_rgb2yuy2 (ff_video_decoder_t *this, int pix_fmt) {
   }
   if (this->stream->video_out->get_capabilities (this->stream->video_out) & VO_CAP_FULLRANGE)
     cm = 11; /* full range */
-  free (this->rgb2yuy2);
+  rgb2yuy2_free (this->rgb2yuy2);
   this->rgb2yuy2 = rgb2yuy2_alloc (cm, fmt);
   this->pix_fmt = pix_fmt;
   VO_SET_FLAGS_CM (cm, this->frame_flags);
@@ -1292,9 +1289,9 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img, AVFrame
 
 #ifdef ENABLE_VAAPI
   if (this->context->pix_fmt == PIX_FMT_VAAPI_VLD) {
-    if (this->accel->guarded_render(this->accel_img)) {
+    if (this->accel->f->guarded_render(this->accel_img)) {
       ff_vaapi_surface_t *va_surface = (ff_vaapi_surface_t *)av_frame->data[0];
-      this->accel->render_vaapi_surface (img, va_surface);
+      this->accel->f->render_vaapi_surface (img, va_surface);
     }
     return;
   }
@@ -1650,7 +1647,7 @@ static void ff_handle_header_buffer (ff_video_decoder_t *this, buf_element_t *bu
       /* init package containing bih */
       memcpy ( &this->bih, this->buf, sizeof(xine_bmiheader) );
 
-      if (this->bih.biSize > sizeof(xine_bmiheader)) {
+      if (this->bih.biSize > (int)sizeof(xine_bmiheader)) {
       this->context->extradata_size = this->bih.biSize - sizeof(xine_bmiheader);
         this->context->extradata = malloc(this->context->extradata_size +
                                           AV_INPUT_BUFFER_PADDING_SIZE);
@@ -1842,17 +1839,17 @@ static void ff_check_pts_tagging(ff_video_decoder_t *this, uint64_t pts)
   }
 }
 
-static int decode_video_wrapper(ff_video_decoder_t *this, AVFrame *av_frame, int *got_picture,
-                                void *buf, size_t buf_size)
-{
+static int decode_video_wrapper (ff_video_decoder_t *this,
+  AVFrame *av_frame, int *err, void *buf, size_t buf_size) {
+  int len;
+
 #if ENABLE_VAAPI
   int locked = 0;
   if (this->accel) {
-    locked = this->accel->lock_vaapi(this->accel_img);
+    locked = this->accel->f->lock_vaapi(this->accel_img);
   }
 #endif /* ENABLE_VAAPI */
 
-  int len;
 #if XFF_VIDEO > 1
   AVPacket avpkt;
   av_init_packet(&avpkt);
@@ -1869,8 +1866,30 @@ static int decode_video_wrapper(ff_video_decoder_t *this, AVFrame *av_frame, int
 # endif /* XFF_PALETTE */
 
   this->decode_attempts++;
-  len = avcodec_decode_video2 (this->context, av_frame,
-                               got_picture, &avpkt);
+
+# if XFF_VIDEO == 3
+  {
+    int e = AVERROR (EAGAIN);
+    if (buf || !this->flush_packet_sent) {
+      e = avcodec_send_packet (this->context, &avpkt);
+      this->flush_packet_sent = (buf == NULL);
+    }
+    len = (e == AVERROR (EAGAIN)) ? 0 : buf_size;
+    /* that calls av_frame_unref () again but seems safe. */
+    *err = avcodec_receive_frame (this->context, av_frame);
+  }
+# else
+  {
+    int got_picture = 0;
+    len = avcodec_decode_video2 (this->context, av_frame, &got_picture, &avpkt);
+    if ((len < 0) || (len > (int)buf_size)) {
+      *err = got_picture ? 0 : len;
+      len = buf_size;
+    } else {
+      *err = got_picture ? 0 : AVERROR (EAGAIN);
+    }
+  }
+# endif
 
 # if XFF_PALETTE == 2 || XFF_PALETTE == 3
   if (buf && this->palette_changed) {
@@ -1891,15 +1910,22 @@ static int decode_video_wrapper(ff_video_decoder_t *this, AVFrame *av_frame, int
 # endif /* XFF_PALETTE */
 
 #else /* XFF_VIDEO */
-
   this->decode_attempts++;
-  len = avcodec_decode_video (this->context, av_frame,
-                              got_picture, buf, buf_size);
+  {
+    int got_picture = 0;
+    len = avcodec_decode_video (this->context, av_frame, got_picture, buf, buf_size);
+    if ((len < 0) || (len > (int)buf_size)) {
+      *err = got_picture ? 0 : len;
+      len = buf_size;
+    } else {
+      *err = got_picture ? 0 : AVERROR (EAGAIN);
+    }
+  }
 #endif /* XFF_VIDEO */
 
 #if ENABLE_VAAPI
   if (locked) {
-    this->accel->unlock_vaapi(this->accel_img);
+    this->accel->f->unlock_vaapi(this->accel_img);
   }
 #endif /* ENABLE_VAAPI */
 
@@ -1910,7 +1936,7 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
 
   vo_frame_t *img;
   int         free_img;
-  int         got_picture, len;
+  int         len;
   int         offset = 0;
   int         flush = 0;
   int         size = buf->size;
@@ -1922,10 +1948,14 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
     ff_init_mpeg12_mode(this);
   }
 
+#ifdef DEBUG_MPEG_PARSER
+  printf ("ff_mpeg_parser: buf %d bytes.\n", size);
+#endif
+
   while ((size > 0) || (flush == 1)) {
 
     uint8_t *current;
-    int next_flush;
+    int next_flush, err;
 #ifdef XFF_AV_BUFFER
     int need_unref = 0;
 #endif
@@ -1937,7 +1967,6 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
       this->pts = 0;
     }
 
-    got_picture = 0;
     if (!flush) {
       current = mpeg_parser_decode_data(this->mpeg_parser,
                                         buf->content + offset, buf->content + offset + size,
@@ -1973,28 +2002,26 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
 
     lprintf("avcodec_decode_video: size=%d\n", this->mpeg_parser->buffer_size);
 
-    len = decode_video_wrapper(this, this->av_frame, &got_picture,
-                               this->mpeg_parser->chunk_buffer, this->mpeg_parser->buffer_size);
+    err = 1;
+    len = decode_video_wrapper (this, this->av_frame, &err,
+      this->mpeg_parser->chunk_buffer, this->mpeg_parser->buffer_size);
 #ifdef XFF_AV_BUFFER
     need_unref = 1;
 #endif
-    lprintf("avcodec_decode_video: decoded_size=%d, got_picture=%d\n",
-            len, got_picture);
+    lprintf ("avcodec_decode_video: decoded_size=%d, err=%d\n", len, err);
     len = current - buf->content - offset;
     lprintf("avcodec_decode_video: consumed_size=%d\n", len);
 
     flush = next_flush;
 
-    if ((len < 0) || (len > buf->size)) {
+    if ((err < 0) && (err != AVERROR (EAGAIN))) {
       xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-                "ffmpeg_video_dec: error decompressing frame\n");
-      size = 0; /* draw a bad frame and exit */
-    } else {
-      size -= len;
-      offset += len;
+        "ffmpeg_video_dec: error decompressing frame (%d).\n", err);
     }
+    size -= len;
+    offset += len;
 
-    if (got_picture && this->class->enable_vaapi) {
+    if (!err && this->class->enable_vaapi) {
       this->bih.biWidth = this->context->width;
       this->bih.biHeight = this->context->height;
     }
@@ -2004,7 +2031,7 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
       this->set_stream_info = 0;
     }
 
-    if (got_picture && this->av_frame->data[0]) {
+    if (!err && this->av_frame->data[0]) {
       /* got a picture, draw it */
       img = NULL;
 #ifdef ENABLE_DIRECT_RENDERING
@@ -2083,6 +2110,7 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
   }
 }
 
+#ifdef HAVE_POSTPROC
 static void ff_postprocess (ff_video_decoder_t *this, AVFrame *av_frame, vo_frame_t *img) {
   int qstride, qtype;
   int8_t *qtable;
@@ -2106,6 +2134,7 @@ static void ff_postprocess (ff_video_decoder_t *this, AVFrame *av_frame, vo_fram
                   qtable, qstride, this->our_mode, this->our_context,
                   av_frame->pict_type | (qtype ? PP_PICT_TYPE_QP2 : 0));
 }
+#endif /* HAVE_POSTPROC */
 
 static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
   uint8_t *chunk_buf = this->buf;
@@ -2170,7 +2199,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 
     vo_frame_t *img;
     int         free_img;
-    int         got_picture, len;
+    int         err, len;
     int         got_one_picture = 0;
     int         offset = 0;
     int         codec_type = buf->type & 0xFFFF0000;
@@ -2188,9 +2217,9 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
     while (this->size > 0) {
 
       /* DV frames can be completely skipped */
-      if( codec_type == BUF_VIDEO_DV && this->skipframes ) {
+      if ((codec_type == BUF_VIDEO_DV) && this->skipframes) {
         this->size = 0;
-        got_picture = 0;
+        err = 1;
       } else {
         /* skip decoding b frames if too late */
 #if XFF_VIDEO > 1
@@ -2206,8 +2235,8 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         }
 #endif
 
-        len = decode_video_wrapper(this, this->av_frame, &got_picture,
-                                   &chunk_buf[offset], this->size);
+        len = decode_video_wrapper (this, this->av_frame, &err, &chunk_buf[offset], this->size);
+        lprintf ("consumed size: %d, err: %d\n", len, err);
 
 #ifdef XFF_AV_BUFFER
         need_unref = 1;
@@ -2215,10 +2244,15 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         /* reset consumed pts value */
         this->context->reordered_opaque = ff_tag_pts(this, 0);
 
-        lprintf("consumed size: %d, got_picture: %d\n", len, got_picture);
-        if ((len <= 0) || (len > this->size)) {
+        if (err) {
+
+          if (err == AVERROR (EAGAIN)) {
+            offset += len;
+            this->size -= len;
+            continue;
+          }
           xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-                    "ffmpeg_video_dec: error decompressing frame\n");
+            "ffmpeg_video_dec: error decompressing frame (%d).\n", err);
           this->size = 0;
 
         } else {
@@ -2267,7 +2301,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         this->set_stream_info = 0;
       }
 
-      if (got_picture && this->av_frame->data[0]) {
+      if (!err && this->av_frame->data[0]) {
         /* got a picture, draw it */
         got_one_picture = 1;
         img = NULL;
@@ -2328,6 +2362,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         }
 
         /* post processing */
+#ifdef HAVE_POSTPROC
         if(this->pp_quality != this->class->pp_quality && this->context->pix_fmt != PIX_FMT_VAAPI_VLD)
           pp_change_quality(this);
 
@@ -2347,7 +2382,9 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
             free_img = 1;
           }
           ff_postprocess (this, this->av_frame, img);
-        } else if (free_img) {
+        } else
+#endif /* HAVE_POSTPROC */
+        if (free_img) {
           /* colorspace conversion or copy */
           ff_convert_frame(this, img, this->av_frame);
         }
@@ -2478,7 +2515,7 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
 static void ff_flush_internal (ff_video_decoder_t *this, int display) {
   vo_frame_t *img;
-  int         free_img, got_picture, len, frames = 0;
+  int         free_img, frames = 0;
   int         video_step_to_use = this->video_step;
   AVRational  avr00 = {0, 1};
 
@@ -2492,9 +2529,9 @@ static void ff_flush_internal (ff_video_decoder_t *this, int display) {
   this->state = STATE_FLUSHED;
 
   while (1) {
-    got_picture = 0;
-    len = decode_video_wrapper(this, this->av_frame2, &got_picture, NULL, 0);
-    if (len < 0 || !got_picture || !this->av_frame2->data[0]) {
+    int err = 1;
+    decode_video_wrapper (this, this->av_frame2, &err, NULL, 0);
+    if (err || !this->av_frame2->data[0]) {
 #ifdef XFF_AV_BUFFER
       av_frame_unref (this->av_frame2);
 #endif
@@ -2559,6 +2596,7 @@ static void ff_flush_internal (ff_video_decoder_t *this, int display) {
     }
 
     /* post processing */
+#ifdef HAVE_POSTPROC
     if (this->pp_quality != this->class->pp_quality && this->context->pix_fmt != PIX_FMT_VAAPI_VLD)
       pp_change_quality (this);
     if (this->pp_available && this->pp_quality && this->context->pix_fmt != PIX_FMT_VAAPI_VLD) {
@@ -2572,7 +2610,9 @@ static void ff_flush_internal (ff_video_decoder_t *this, int display) {
         free_img = 1;
       }
       ff_postprocess (this, this->av_frame2, img);
-    } else if (free_img) {
+    } else
+#endif /* HAVE_POSTPROC */
+    if (free_img) {
       /* colorspace conversion or copy */
       ff_convert_frame (this, img, this->av_frame2);
     }
@@ -2640,8 +2680,8 @@ static void ff_free_dr1_frames (ff_video_decoder_t *this, int all) {
       ffsf->vo_frame->free (ffsf->vo_frame);
       frames++;
     }
-    DLIST_REMOVE (ffsf);
-    DLIST_ADD_TAIL (ffsf, &this->ffsf_free);
+    DLIST_REMOVE (&ffsf->node);
+    DLIST_ADD_TAIL (&ffsf->node, &this->ffsf_free);
     this->ffsf_num--;
   }
   pthread_mutex_unlock (&this->ffsf_mutex);
@@ -2771,18 +2811,20 @@ static void ff_dispose (video_decoder_t *this_gen) {
     free(this->buf);
   this->buf = NULL;
 
+#ifdef HAVE_POSTPROC
   if(this->our_context)
     pp_free_context(this->our_context);
 
   if(this->our_mode)
     pp_free_mode(this->our_mode);
+#endif /* HAVE_POSTPROC */
 
   mpeg_parser_dispose(this->mpeg_parser);
 
 #ifdef ENABLE_DIRECT_RENDERING
   while (!DLIST_IS_EMPTY (&this->ffsf_free)) {
     ff_saved_frame_t *ffsf = (ff_saved_frame_t *)this->ffsf_free.head;
-    DLIST_REMOVE (ffsf);
+    DLIST_REMOVE (&ffsf->node);
     free (ffsf);
   }
   if (this->ffsf_total)
@@ -2802,9 +2844,31 @@ static void ff_dispose (video_decoder_t *this_gen) {
 static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen, xine_stream_t *stream) {
 
   ff_video_decoder_t  *this ;
+  AVCodec             *codec = NULL;
+  uint32_t             video_type;
+  size_t               i;
 
   lprintf ("open_plugin\n");
 
+  /* check for codec support */
+  video_type = BUF_VIDEO_BASE | (_x_get_video_streamtype(stream) << 16);
+  for (i = 0; i < sizeof(ff_video_lookup)/sizeof(ff_codec_t); i++) {
+    if(ff_video_lookup[i].type == video_type) {
+      pthread_mutex_lock(&ffmpeg_lock);
+      codec = avcodec_find_decoder(ff_video_lookup[i].id);
+      pthread_mutex_unlock(&ffmpeg_lock);
+      _x_meta_info_set_utf8(stream, XINE_META_INFO_VIDEOCODEC, ff_video_lookup[i].name);
+      break;
+    }
+  }
+  if (!codec) {
+    xprintf (stream->xine, XINE_VERBOSITY_LOG,
+             _("ffmpeg_video_dec: couldn't find ffmpeg decoder for buf type 0x%X\n"),
+             video_type);
+    return NULL;
+  }
+  lprintf("lavc decoder found\n");
+
   this = calloc(1, sizeof (ff_video_decoder_t));
   if (!this)
     return NULL;
@@ -2817,9 +2881,11 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
   this->decoder_ok      = 0;
   this->is_mpeg12       = 0;
   this->aspect_ratio    = 0;
+#ifdef HAVE_POSTPROC
   this->pp_quality      = 0;
   this->our_context     = NULL;
   this->our_mode        = NULL;
+#endif
   this->mpeg_parser     = NULL;
   this->set_stream_info = 0;
   this->rgb2yuy2        = NULL;
@@ -2827,6 +2893,9 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
   this->accel           = NULL;
   this->accel_img       = NULL;
 #endif
+#if XFF_VIDEO == 3
+  this->flush_packet_sent = 0;
+#endif
 
   this->video_decoder.decode_data   = ff_decode_data;
   this->video_decoder.flush         = ff_flush;
@@ -2836,6 +2905,7 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
 
   this->stream = stream;
   this->class  = (ff_video_class_t *)class_gen;
+  this->codec  = codec;
 
   this->bufsize = VIDEOBUFSIZE;
   do {
@@ -2880,11 +2950,12 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
 #endif
 
 #ifdef ENABLE_VAAPI
-  if(this->class->enable_vaapi && (stream->video_driver->get_capabilities(stream->video_driver) & VO_CAP_VAAPI)) {
+  if (this->class->enable_vaapi && (stream->video_out->get_capabilities(stream->video_out) & VO_CAP_VAAPI)) {
     xprintf(this->class->xine, XINE_VERBOSITY_LOG, _("ffmpeg_video_dec: vaapi_mpeg_softdec %d\n"),
           this->class->vaapi_mpeg_softdec );
 
-    this->accel_img  = stream->video_out->get_frame( stream->video_out, 1920, 1080, 1, XINE_IMGFMT_VAAPI, VO_BOTH_FIELDS );
+    this->accel_img  = stream->video_out->get_frame( stream->video_out, 1920, 1080, 1, XINE_IMGFMT_VAAPI,
+                                                     VO_BOTH_FIELDS | VO_GET_FRAME_MAY_FAIL );
 
     if( this->accel_img ) {
       this->accel = (vaapi_accel_t*)this->accel_img->accel_data;
@@ -2908,24 +2979,18 @@ static void dispose_video_class (video_decoder_class_t *this_gen) {
   ff_video_class_t *this = (ff_video_class_t *)this_gen;
   config_values_t *config = this->xine->config;
 
-  config->unregister_callback (config, "video.processing.ffmpeg_pp_quality");
-  config->unregister_callback (config, "video.processing.ffmpeg_thread_count");
-  config->unregister_callback (config, "video.processing.ffmpeg_skip_loop_filter");
-  config->unregister_callback (config, "video.processing.ffmpeg_choose_speed_over_accuracy");
-  config->unregister_callback (config, "video.processing.ffmpeg_direct_rendering");
-#ifdef ENABLE_VAAPI
-  config->unregister_callback (config, "video.processing.vaapi_mpeg_softdec");
-  config->unregister_callback (config, "video.processing.ffmpeg_enable_vaapi");
-#endif
+  config->unregister_callbacks (config, NULL, NULL, this, sizeof (*this));
 
   free (this);
 }
 
-void *init_video_plugin (xine_t *xine, void *data) {
+void *init_video_plugin (xine_t *xine, const void *data) {
 
   ff_video_class_t *this;
   config_values_t  *config;
 
+  (void)data;
+
   this = calloc(1, sizeof (ff_video_class_t));
   if (!this) {
     return NULL;
@@ -2943,6 +3008,7 @@ void *init_video_plugin (xine_t *xine, void *data) {
    * moment */
   config = xine->config;
 
+#ifdef HAVE_POSTPROC
   this->pp_quality = xine->config->register_range(config, "video.processing.ffmpeg_pp_quality", 3,
     0, PP_QUALITY_MAX,
     _("MPEG-4 postprocessing quality"),
@@ -2952,6 +3018,7 @@ void *init_video_plugin (xine_t *xine, void *data) {
       "too heavy post processing can actually make the image worse by blurring it "
       "too much."),
     10, pp_quality_cb, this);
+#endif /* HAVE_POSTPROC */
 
   this->thread_count = xine->config->register_num(config, "video.processing.ffmpeg_thread_count", 1,
     _("FFmpeg video decoding thread count"),
