@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -55,6 +55,8 @@
 #include <xine/xineutils.h>
 #include <xine/demux.h>
 
+#include "flac_demuxer.h"
+
 #ifndef LEGACY_FLAC
 # define FLAC__SeekableStreamDecoder FLAC__StreamDecoder
 #endif
@@ -88,17 +90,6 @@ typedef struct demux_flac_s {
 } demux_flac_t ;
 
 
-/* FLAC Demuxer class */
-typedef struct demux_flac_class_s {
-  demux_class_t     demux_class;
-
-  xine_t           *xine;
-  config_values_t  *config;
-
-} demux_flac_class_t;
-
-void *demux_flac_init_class (xine_t *xine, void *data);
-
 /* FLAC Callbacks */
 static
 #ifdef LEGACY_FLAC
@@ -119,6 +110,8 @@ flac_read_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = this->input;
     off_t offset = *bytes;
 
+    (void)decoder;
+
     lprintf("flac_read_callback\n");
 
     /* This should only be called when flac is reading the metadata
@@ -168,6 +161,8 @@ flac_seek_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = ((demux_flac_t *)client_data)->input;
     off_t offset;
 
+    (void)decoder;
+
     lprintf("flac_seek_callback\n");
 
     offset = input->seek (input, absolute_byte_offset, SEEK_SET);
@@ -199,6 +194,8 @@ flac_tell_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = ((demux_flac_t *)client_data)->input;
     off_t offset;
 
+    (void)decoder;
+
     lprintf("flac_tell_callback\n");
 
     offset = input->get_current_pos (input);
@@ -225,6 +222,8 @@ flac_length_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = ((demux_flac_t *)client_data)->input;
     off_t offset;
 
+    (void)decoder;
+
     lprintf("flac_length_callback\n");
 
     offset = input->get_length (input);
@@ -247,6 +246,8 @@ flac_eof_callback (const FLAC__SeekableStreamDecoder *decoder,
 {
     demux_flac_t *this = (demux_flac_t *)client_data;
 
+    (void)decoder;
+
     lprintf("flac_eof_callback\n");
 
     if (this->status == DEMUX_FINISHED)
@@ -273,6 +274,11 @@ flac_write_callback (const FLAC__SeekableStreamDecoder *decoder,
      * is seeking. We do the decoding in the decoder
      */
 
+  (void)decoder;
+  (void)frame;
+  (void)buffer;
+  (void)client_data;
+
   lprintf("Error: Write callback was called!\n");
 
   return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
@@ -285,6 +291,8 @@ flac_metadata_callback (const FLAC__SeekableStreamDecoder *decoder,
 {
     demux_flac_t *this = (demux_flac_t *)client_data;
 
+    (void)decoder;
+
     lprintf("IN: Metadata callback\n");
 
     /* This should be called when we first look at a flac stream,
@@ -297,6 +305,7 @@ flac_metadata_callback (const FLAC__SeekableStreamDecoder *decoder,
        this->bits_per_sample = metadata->data.stream_info.bits_per_sample;
        this->channels        = metadata->data.stream_info.channels;
        this->sample_rate     = metadata->data.stream_info.sample_rate;
+       if (this->sample_rate)
        this->length_in_msec  = (this->total_samples * 1000) /
                                 this->sample_rate;
      }
@@ -313,6 +322,8 @@ flac_error_callback (const FLAC__SeekableStreamDecoder *decoder,
      * in the stream.
      */
 
+    (void)decoder;
+
     xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_flac: flac_error_callback\n");
 
     if (status == FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
@@ -339,7 +350,7 @@ demux_flac_send_chunk (demux_plugin_t *this_gen) {
     buf_element_t *buf = NULL;
     off_t current_file_pos, file_size = 0;
     int64_t current_pts;
-    unsigned int remaining_sample_bytes = 0;
+    /*unsigned*/ int remaining_sample_bytes;
 
     remaining_sample_bytes = 2048;
 
@@ -473,6 +484,8 @@ static int
 demux_flac_seek (demux_plugin_t *this_gen, off_t start_pos, int start_time, int playing) {
     demux_flac_t *this = (demux_flac_t *) this_gen;
 
+    (void)playing;
+
     lprintf("demux_flac_seek\n");
 
     start_pos = (off_t) ( (double) start_pos / 65535 *
@@ -542,6 +555,8 @@ static uint32_t
 demux_flac_get_capabilities (demux_plugin_t *this_gen) {
   lprintf("demux_flac_get_capabilities\n");
 
+  (void)this_gen;
+
   return DEMUX_CAP_NOCAP;
 }
 
@@ -549,6 +564,10 @@ static int
 demux_flac_get_optional_data (demux_plugin_t *this_gen, void *data, int dtype) {
   lprintf("demux_flac_get_optional_data\n");
 
+  (void)this_gen;
+  (void)data;
+  (void)dtype;
+
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -563,25 +582,13 @@ open_plugin (demux_class_t *class_gen,
     switch (stream->content_detection_method) {
         case METHOD_BY_CONTENT:
         {
-          uint8_t      buf[MAX_PREVIEW_SIZE];
-          int          len;
+          uint8_t      buf[4];
 
           /*
            * try to get a preview of the data
            */
-          len = input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW);
-          if (len == INPUT_OPTIONAL_UNSUPPORTED) {
-
-            if (input->get_capabilities (input) & INPUT_CAP_SEEKABLE) {
-
-              input->seek (input, 0, SEEK_SET);
-              if ( (len=input->read (input, buf, 1024)) <= 0)
-                return NULL;
-              input->seek (input, 0, SEEK_SET);
-
-            } else
-              return NULL;
-          }
+          if (_x_demux_read_header(input, buf, sizeof(buf)) < 4)
+            return NULL;
 
           /* FIXME: Skip id3v2 tag */
           /* Look for fLaC tag at the beginning of file */
@@ -603,6 +610,9 @@ open_plugin (demux_class_t *class_gen,
     */
 
     this         = calloc(1, sizeof (demux_flac_t));
+    if (!this)
+      return NULL;
+
     this->stream = stream;
     this->input  = input;
 
@@ -710,23 +720,22 @@ open_plugin (demux_class_t *class_gen,
 
 /* FLAC Demuxer class */
 void *
-demux_flac_init_class (xine_t *xine, void *data) {
+demux_flac_init_class (xine_t *xine, const void *data) {
 
-    demux_flac_class_t     *this;
+    static const demux_class_t demux_flac_class = {
+        .open_plugin     = open_plugin,
+        .description     = N_("FLAC demux plugin"),
+        .identifier      = "FLAC",
+        .mimetypes       = "application/x-flac: flac: FLAC Audio;"
+                           "application/flac: flac: FLAC Audio;",
+        .extensions      = "flac",
+        .dispose         = NULL,
+    };
+
+    (void)xine;
+    (void)data;
 
     lprintf("demux_flac_init_class\n");
 
-    this         = calloc(1, sizeof (demux_flac_class_t));
-    this->config = xine->config;
-    this->xine   = xine;
-
-    this->demux_class.open_plugin     = open_plugin;
-    this->demux_class.description     = N_("FLAC demux plugin");
-    this->demux_class.identifier      = "FLAC";
-    this->demux_class.mimetypes       = "application/x-flac: flac: FLAC Audio;"
-                                        "application/flac: flac: FLAC Audio;";
-    this->demux_class.extensions      = "flac";
-    this->demux_class.dispose         = default_demux_class_dispose;
-
-    return this;
+    return (void *)&demux_flac_class;
 }
