@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -308,7 +308,7 @@ static void send_ogg_packet (demux_ogg_t *this,
 
   buf_element_t *buf;
 
-  int done=0,todo=op->bytes;
+  size_t done = 0, todo = op->bytes;
   const size_t op_size = sizeof(ogg_packet);
 
   while (done<todo) {
@@ -397,7 +397,7 @@ static void ogg_handle_event (demux_ogg_t *this) {
           int c_chap = this->chapter_info->current_chapter;
           if (c_chap+1 < this->chapter_info->max_chapter) {
             int start_time = this->chapter_info->entries[c_chap+1].start_pts / 90;
-            this->demux_plugin.seek((demux_plugin_t *)this, 0, start_time, 1);
+            this->demux_plugin.seek(&this->demux_plugin, 0, start_time, 1);
           }
         }
       }
@@ -408,7 +408,7 @@ static void ogg_handle_event (demux_ogg_t *this) {
           int c_chap = this->chapter_info->current_chapter;
           if (c_chap >= 1) {
             int start_time = this->chapter_info->entries[c_chap-1].start_pts / 90;
-            this->demux_plugin.seek((demux_plugin_t *)this, 0, start_time, 1);
+            this->demux_plugin.seek(&this->demux_plugin, 0, start_time, 1);
           }
         }
       }
@@ -468,9 +468,9 @@ static void prepare_read_comments (demux_ogg_t *this)
 
 static int read_comments (demux_ogg_t *this, const char *comment)
 {
-  int i;
+  unsigned i;
 
-  for (i = 0; i < sizeof (metadata) / sizeof (struct ogg_meta); ++i) {
+  for (i = 0; i < sizeof (metadata) / sizeof (metadata[0]); ++i) {
     size_t ml = strlen (metadata[i].tag);
     if (!strncasecmp (metadata[i].tag, comment, ml) && comment[ml]) {
       if (metadata[i].append && this->meta[metadata[i].meta]) {
@@ -634,10 +634,10 @@ static void update_chapter_display (demux_ogg_t *this, int stream_num, ogg_packe
       if (this->meta[XINE_META_INFO_TITLE]) {
         data.str_len = snprintf(data.str, sizeof(data.str), "%s / %s", this->meta[XINE_META_INFO_TITLE], this->chapter_info->entries[chapter].name);
       } else {
-	strncpy(data.str, this->chapter_info->entries[chapter].name, sizeof(data.str)-1);
+	strlcpy(data.str, this->chapter_info->entries[chapter].name, sizeof(data.str));
       }
     } else {
-      strncpy(data.str, this->meta[XINE_META_INFO_TITLE], sizeof(data.str));
+      strlcpy(data.str, this->meta[XINE_META_INFO_TITLE], sizeof(data.str));
     }
     if ( data.str_len == 0 )
       data.str_len = strlen(data.str);
@@ -690,6 +690,7 @@ static void send_ogg_buf (demux_ogg_t *this,
 
     if ((this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_AUDIO_SPEEX ||
         (this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_AUDIO_FLAC ||
+        (this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_AUDIO_OPUS ||
         (this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_AUDIO_VORBIS) {
       data = op->packet;
       size = op->bytes;
@@ -717,8 +718,9 @@ static void send_ogg_buf (demux_ogg_t *this,
 		       normpos,
 		       pts / 90, this->time_length, 0);
 
+  }
 #ifdef HAVE_THEORA
-  } else if ((this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_VIDEO_THEORA) {
+  else if ((this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_VIDEO_THEORA) {
 
     int64_t pts;
     theora_info t_info;
@@ -750,9 +752,10 @@ static void send_ogg_buf (demux_ogg_t *this,
 
     theora_comment_clear (&t_comment);
     theora_info_clear (&t_info);
+  }
 #endif
 
-  } else if ((this->si[stream_num]->buf_types & 0xFF000000) == BUF_VIDEO_BASE) {
+  else if ((this->si[stream_num]->buf_types & 0xFF000000) == BUF_VIDEO_BASE) {
 
     uint8_t *data;
     int size;
@@ -933,6 +936,7 @@ static void decode_speex_header (demux_ogg_t *this, const int stream_num, ogg_pa
     this->si[stream_num]->headers += header->extra_headers;
   }
 #else
+  (void)op;
   xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "Speex stream detected, unable to play\n");
 
   this->si[stream_num]->buf_types = BUF_CONTROL_NOP;
@@ -1229,6 +1233,7 @@ static void decode_text_header (demux_ogg_t *this, const int stream_num, ogg_pac
   uint32_t *val;
   buf_element_t *buf;
 
+  (void)op;
   lprintf ("textstream detected.\n");
   this->si[stream_num]->headers = 2;
   channel = this->num_spu_streams++;
@@ -1295,6 +1300,7 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
     this->si[stream_num]->headers = 0; /* FIXME: don't know */
   }
 #else
+  (void)op;
   this->si[stream_num]->buf_types = BUF_VIDEO_THEORA;
   this->num_video_streams++;
   this->unhandled_video_streams++;
@@ -1304,7 +1310,7 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
 
 static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
   xine_flac_metadata_header header;
-  xine_flac_streaminfo_block streaminfo = {};
+  xine_flac_streaminfo_block streaminfo = {.samplerate = 0};
   buf_element_t *buf;
   xine_waveformatex wave;
 
@@ -1366,6 +1372,36 @@ static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_pac
   op->packet += 9;
 }
 
+static void decode_opus_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
+  buf_element_t *buf;
+  xine_waveformatex wave;
+
+  memset(&wave, 0, sizeof(wave));
+
+  this->si[stream_num]->headers = 0;
+
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, _X_BE_32(&op->packet[12]));
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, op->packet[9]);
+
+  this->si[stream_num]->buf_types = BUF_AUDIO_OPUS + this->num_audio_streams++;
+  this->si[stream_num]->factor = 90000;
+
+  buf = this->audio_fifo->buffer_pool_alloc(this->audio_fifo);
+
+  buf->type = this->si[stream_num]->buf_types;
+  buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAME_END;
+
+  buf->decoder_info[0] = 0;
+  buf->decoder_info[1] = _X_BE_32(&op->packet[12]); /* sample rate */
+  buf->decoder_info[2] = 0;
+  buf->decoder_info[3] = op->packet[9]; /* channel count */
+  buf->size = sizeof(xine_waveformatex);
+  wave.cbSize = 0;
+  memcpy(buf->content, &wave, sizeof(xine_waveformatex));
+
+  this->audio_fifo->put(this->audio_fifo, buf);
+}
+
 static void decode_annodex_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
   lprintf ("Annodex stream detected\n");
   this->si[stream_num]->buf_types = BUF_CONTROL_NOP;
@@ -1528,6 +1564,8 @@ static void send_header (demux_ogg_t *this) {
           decode_anxdata_header(this, stream_num, &op);
 	} else if (!memcmp (&op.packet[0], "CMML", 4)) {
 	  decode_cmml_header(this, stream_num, &op);
+        } else if (!memcmp (&op.packet[0], "OpusHead", 8)) {
+          decode_opus_header(this, stream_num, &op);
         } else {
           xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
                   "demux_ogg: unknown stream type (signature >%.8s<). hex dump of bos packet follows:\n",
@@ -1823,7 +1861,8 @@ static void demux_ogg_send_headers (demux_plugin_t *this_gen) {
   this->num_spu_streams   = 0;
   this->avg_bitrate       = 1;
 
-  this->input->seek (this->input, 0, SEEK_SET);
+  if (this->input->seek (this->input, 0, SEEK_SET) != 0)
+    this->status = DEMUX_FINISHED;
 
   if (this->status == DEMUX_OK) {
     _x_demux_control_start(this->stream);
@@ -2038,8 +2077,7 @@ static int demux_ogg_get_optional_data(demux_plugin_t *this_gen,
   }
 }
 
-static int detect_ogg_content (int detection_method, demux_class_t *class_gen,
-                               input_plugin_t *input) {
+static int detect_ogg_content (int detection_method, input_plugin_t *input) {
 
   switch (detection_method) {
 
@@ -2061,10 +2099,9 @@ static int detect_ogg_content (int detection_method, demux_class_t *class_gen,
   }
 }
 
-static int detect_anx_content (int detection_method, demux_class_t *class_gen,
-    input_plugin_t *input) {
+static int detect_anx_content (int detection_method, input_plugin_t *input) {
 
-  if (detect_ogg_content(detection_method, class_gen, input) == 0)
+  if (detect_ogg_content(detection_method, input) == 0)
     return 0;
 
   switch (detection_method) {
@@ -2101,7 +2138,7 @@ static demux_plugin_t *anx_open_plugin (demux_class_t *class_gen,
   demux_ogg_t *this;
   int i;
 
-  if (detect_anx_content(stream->content_detection_method, class_gen, input) == 0)
+  if (detect_anx_content(stream->content_detection_method, input) == 0)
     return NULL;
 
   /*
@@ -2148,7 +2185,7 @@ static demux_plugin_t *ogg_open_plugin (demux_class_t *class_gen,
   demux_ogg_t *this;
   int i;
 
-  if (detect_ogg_content(stream->content_detection_method, class_gen, input) == 0)
+  if (detect_ogg_content(stream->content_detection_method, input) == 0)
     return NULL;
 
   /*
@@ -2187,10 +2224,14 @@ static demux_plugin_t *ogg_open_plugin (demux_class_t *class_gen,
 /*
  * Annodex demuxer class
  */
-static void *anx_init_class (xine_t *xine, void *data) {
+static void *anx_init_class (xine_t *xine, const void *data) {
   demux_anx_class_t     *this;
 
+  (void)xine;
+  (void)data;
   this = calloc(1, sizeof(demux_anx_class_t));
+  if (!this)
+    return NULL;
 
   this->demux_class.open_plugin     = anx_open_plugin;
   this->demux_class.description     = N_("Annodex demux plugin");
@@ -2211,10 +2252,14 @@ static void *anx_init_class (xine_t *xine, void *data) {
 /*
  * ogg demuxer class
  */
-static void *ogg_init_class (xine_t *xine, void *data) {
+static void *ogg_init_class (xine_t *xine, const void *data) {
   demux_ogg_class_t     *this;
 
+  (void)xine;
+  (void)data;
   this = calloc(1, sizeof(demux_ogg_class_t));
+  if (!this)
+    return NULL;
 
   this->demux_class.open_plugin     = ogg_open_plugin;
   this->demux_class.description     = N_("OGG demux plugin");
@@ -2226,10 +2271,11 @@ static void *ogg_init_class (xine_t *xine, void *data) {
     "application/x-ogm-video: ogv: Ogg Video;"
     "application/x-ogg: ogx: Ogg Stream;"
     "audio/ogg: oga: Ogg Audio;"
+    "audio/opus: opus: Opus Audio;"
     "audio/x-ogg: oga: Ogg Audio;"
     "video/ogg: ogv: Ogg Video;"
     "video/x-ogg: ogv: Ogg Video;";
-  this->demux_class.extensions      = "ogx ogv oga ogg spx ogm";
+  this->demux_class.extensions      = "ogx ogv oga ogg spx ogm opus";
   this->demux_class.dispose         = default_demux_class_dispose;
 
   return this;
@@ -2239,11 +2285,11 @@ static void *ogg_init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 static const demuxer_info_t demux_info_anx = {
-  20                       /* priority */
+  .priority = 20,
 };
 
 static const demuxer_info_t demux_info_ogg = {
-  10                       /* priority */
+  .priority = 10,
 };
 
 extern const decoder_info_t dec_info_vorbis;
@@ -2263,5 +2309,5 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
 #ifdef HAVE_THEORA
   { PLUGIN_VIDEO_DECODER, 19, "theora", XINE_VERSION_CODE, &dec_info_theora, theora_init_plugin },
 #endif
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
