@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -49,10 +49,6 @@
 
 #define MAX_FRAME_SIZE 2000
 
-typedef struct {
-  audio_decoder_class_t   decoder_class;
-} speex_class_t;
-
 typedef struct speex_decoder_s {
   audio_decoder_t   audio_decoder;
 
@@ -115,7 +111,7 @@ static
 void read_metadata (speex_decoder_t *this, char * comments, int length)
 {
   char * c = comments;
-  int len, i, nb_fields;
+  unsigned len, i, nb_fields;
   char * end;
 
   _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "speex");
@@ -174,14 +170,14 @@ void read_metadata (speex_decoder_t *this, char * comments, int length)
       size_t keylen = strlen(speex_comment_keys[i].key);
 
       if ( !strncasecmp (speex_comment_keys[i].key, c,
-			 keylen) ) {
-	char meta_info[(len - keylen) + 1];
+                         keylen) ) {
+        char meta_info[(len - keylen) + 1];
 
-	lprintf ("known metadata %d %d\n",
-		 i, speex_comment_keys[i].xine_metainfo_index);
+        lprintf ("known metadata %u %d\n",
+                 i, speex_comment_keys[i].xine_metainfo_index);
 
-	strncpy(meta_info, &c[keylen], len-keylen);
-	_x_meta_info_set_utf8(this->stream, speex_comment_keys[i].xine_metainfo_index, meta_info);
+        strncpy(meta_info, &c[keylen], len-keylen);
+        _x_meta_info_set_utf8(this->stream, speex_comment_keys[i].xine_metainfo_index, meta_info);
       }
     }
 
@@ -195,7 +191,7 @@ static void speex_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
   char *const buf_content = (char*)buf->content;
 
   llprintf (LOG_BUFFERS, "decode buf=%8p content=%8p flags=%08x\n",
-	    buf, buf->content, buf->decoder_flags);
+            buf, buf->content, buf->decoder_flags);
 
   if ( (buf->decoder_flags & BUF_FLAG_HEADER) &&
        !(buf->decoder_flags & BUF_FLAG_STDHEADER) ) {
@@ -205,68 +201,68 @@ static void speex_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
       if (!this->st) {
         const SpeexMode * spx_mode;
-	SpeexHeader * spx_header;
-	unsigned int modeID;
-	int bitrate;
+        SpeexHeader * spx_header;
+        unsigned int modeID;
+        int bitrate;
 
-	speex_bits_init (&this->bits);
+        speex_bits_init (&this->bits);
 
-	spx_header = speex_packet_to_header (buf_content, buf->size);
+        spx_header = speex_packet_to_header (buf_content, buf->size);
 
-	if (!spx_header) {
-	  xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: could not read Speex header\n");
-	  return;
-	}
+        if (!spx_header) {
+          xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: could not read Speex header\n");
+          return;
+        }
 
-	modeID = (unsigned int)spx_header->mode;
-	if (modeID >= SPEEX_NB_MODES) {
-	  xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": invalid mode ID %u\n", modeID);
-	  return;
-	}
+        modeID = (unsigned int)spx_header->mode;
+        if (modeID >= SPEEX_NB_MODES) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": invalid mode ID %u\n", modeID);
+          return;
+        }
 
         spx_mode = speex_mode_list[modeID];
 
-	if (spx_mode->bitstream_version != spx_header->mode_bitstream_version) {
-	  xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: incompatible Speex mode bitstream version\n");
-	  return;
-	}
+        if (spx_mode->bitstream_version != spx_header->mode_bitstream_version) {
+          xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: incompatible Speex mode bitstream version\n");
+          return;
+        }
 
-	this->st = speex_decoder_init (spx_mode);
-	if (!this->st) {
-	  xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: decoder initialization failed\n");
-	  return;
-	}
+        this->st = speex_decoder_init (spx_mode);
+        if (!this->st) {
+          xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: decoder initialization failed\n");
+          return;
+        }
 
-	this->rate = spx_header->rate;
-	speex_decoder_ctl (this->st, SPEEX_SET_SAMPLING_RATE, &this->rate);
-	_x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE,
-	  this->rate);
+        this->rate = spx_header->rate;
+        speex_decoder_ctl (this->st, SPEEX_SET_SAMPLING_RATE, &this->rate);
+        _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE,
+          this->rate);
 
-	this->channels = spx_header->nb_channels;
-	if (this->channels == 2) {
-	  SpeexCallback callback;
+        this->channels = spx_header->nb_channels;
+        if (this->channels == 2) {
+          SpeexCallback callback;
 
-	  callback.callback_id = SPEEX_INBAND_STEREO;
-	  callback.func = speex_std_stereo_request_handler;
-	  callback.data = &this->stereo;
-	  speex_decoder_ctl (this->st, SPEEX_SET_HANDLER, &callback);
-	}
+          callback.callback_id = SPEEX_INBAND_STEREO;
+          callback.func = speex_std_stereo_request_handler;
+          callback.data = &this->stereo;
+          speex_decoder_ctl (this->st, SPEEX_SET_HANDLER, &callback);
+        }
 
-	this->nframes = spx_header->frames_per_packet;
-	if (!this->nframes) this->nframes = 1;
+        this->nframes = spx_header->frames_per_packet;
+        if (!this->nframes) this->nframes = 1;
 
-	speex_decoder_ctl (this->st, SPEEX_GET_FRAME_SIZE, &this->frame_size);
+        speex_decoder_ctl (this->st, SPEEX_GET_FRAME_SIZE, &this->frame_size);
 
-	speex_decoder_ctl (this->st, SPEEX_GET_BITRATE, &bitrate);
-	if (bitrate <= 1) bitrate = 16000; /* assume 16 kbit */
-	_x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITRATE, bitrate);
+        speex_decoder_ctl (this->st, SPEEX_GET_BITRATE, &bitrate);
+        if (bitrate <= 1) bitrate = 16000; /* assume 16 kbit */
+        _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITRATE, bitrate);
 
-	this->header_count += spx_header->extra_headers;
-	this->expect_metadata = 1;
+        this->header_count += spx_header->extra_headers;
+        this->expect_metadata = 1;
 
-	free (spx_header);
+        free (spx_header);
       } else if (this->expect_metadata) {
-	read_metadata (this, buf_content, buf->size);
+        read_metadata (this, buf_content, buf->size);
       }
 
       this->header_count--;
@@ -274,15 +270,15 @@ static void speex_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
       if (!this->header_count) {
         int mode = _x_ao_channels2mode(this->channels);
 
-	if (!this->output_open) {
-	  this->output_open =
-	    (this->stream->audio_out->open) (this->stream->audio_out,
-					  this->stream,
-					  16,
-					  this->rate,
-					  mode);
+        if (!this->output_open) {
+          this->output_open =
+            (this->stream->audio_out->open) (this->stream->audio_out,
+                                          this->stream,
+                                          16,
+                                          this->rate,
+                                          mode);
             lprintf ("this->output_open after attempt is %d\n", this->output_open);
-	}
+        }
       }
     }
 
@@ -303,18 +299,18 @@ static void speex_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
       ret = speex_decode_int (this->st, &this->bits, audio_buffer->mem);
 
       if (ret==-1)
-	break;
+        break;
       if (ret==-2) {
-	xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: Decoding error, corrupted stream?\n");
-	break;
+        xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: Decoding error, corrupted stream?\n");
+        break;
       }
       if (speex_bits_remaining(&this->bits)<0) {
-	xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: Decoding overflow, corrupted stream?\n");
-	break;
+        xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libspeex: Decoding overflow, corrupted stream?\n");
+        break;
       }
 
       if (this->channels == 2) {
-	speex_decode_stereo_int (audio_buffer->mem, this->frame_size, &this->stereo);
+        speex_decode_stereo_int (audio_buffer->mem, this->frame_size, &this->stereo);
       }
 
       speex_decoder_ctl (this->st, SPEEX_GET_BITRATE, &bitrate);
@@ -352,12 +348,16 @@ static void speex_dispose (audio_decoder_t *this_gen) {
 }
 
 static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen,
-				     xine_stream_t *stream) {
+                                     xine_stream_t *stream) {
 
   speex_decoder_t *this ;
   static SpeexStereoState init_stereo = SPEEX_STEREO_STATE_INIT;
 
+  (void)class_gen;
+
   this = (speex_decoder_t *) calloc(1, sizeof(speex_decoder_t));
+  if (!this)
+    return NULL;
 
   this->audio_decoder.decode_data         = speex_decode_data;
   this->audio_decoder.reset               = speex_reset;
@@ -382,25 +382,26 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen,
  * speex plugin class
  */
 
-void *speex_init_plugin (xine_t *xine, void *data) {
+void *speex_init_plugin (xine_t *xine, const void *data) {
 
-  speex_class_t *this;
+  static const audio_decoder_class_t decode_speex_class = {
+    .open_plugin     = open_plugin,
+    .identifier      = "speex",
+    .description     = N_("Speex audio decoder plugin"),
+    .dispose         = NULL,
+  };
 
-  this = (speex_class_t *) calloc(1, sizeof(speex_class_t));
+  (void)xine;
+  (void)data;
 
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "speex";
-  this->decoder_class.description     = N_("Speex audio decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
-
-  return this;
+  return (void*)&decode_speex_class;
 }
 
 static const uint32_t audio_types[] = {
   BUF_AUDIO_SPEEX, 0
- };
+};
 
 const decoder_info_t dec_info_speex = {
-  audio_types,         /* supported types */
-  5                    /* priority        */
+  .supported_types = audio_types,
+  .priority = 5,
 };
