@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -108,19 +108,34 @@ typedef struct {
   int                  total_time;
 } demux_film_t ;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_film_class_t;
+static int probe_film_file(input_plugin_t *input, int *film_header_size) {
+  unsigned char header[8];
 
+  if (_x_demux_read_header(input, header, 8) != 8)
+    return 0;
+
+  /* FILM signature correct? */
+  if (!_x_is_fourcc(header, "FILM"))
+    return 0;
+
+  llprintf(DEBUG_FILM_LOAD, "found 'FILM' signature\n");
+
+  /* header size = header size - 16-byte FILM signature */
+  *film_header_size = _X_BE_32(&header[4]);
+  if (*film_header_size < 16)
+    return 0;
+  *film_header_size -= 16;
+
+  return 1;
+}
 
 /* Open a FILM file
  * This function is called from the _open() function of this demuxer.
  * It returns 1 if FILM file was opened successfully. */
-static int open_film_file(demux_film_t *film) {
+static int open_film_file(demux_film_t *film, int film_header_size) {
 
   unsigned char *film_header;
-  unsigned int film_header_size;
-  unsigned char scratch[16];
+  unsigned int hsize = film_header_size < 0 ? 0 : film_header_size;
   unsigned int chunk_type;
   unsigned int chunk_size;
   unsigned int i, j;
@@ -136,25 +151,13 @@ static int open_film_file(demux_film_t *film) {
   film->audio_bits = 0;
   film->audio_channels = 0;
 
-  /* get the signature, header length and file version */
-  if (_x_demux_read_header(film->input, scratch, 16) != 16)
+  if (film->input->seek(film->input, 8, SEEK_SET) != 8)
     return 0;
 
-  /* FILM signature correct? */
-  if (!_x_is_fourcc(scratch, "FILM"))
+  /* read version */
+  if (film->input->read(film->input, film->version, 4) != 4)
     return 0;
 
-  llprintf(DEBUG_FILM_LOAD, "found 'FILM' signature\n");
-
-  /* file is qualified; skip over the header bytes in the stream */
-  film->input->seek(film->input, 16, SEEK_SET);
-
-  /* header size = header size - 16-byte FILM signature */
-  film_header_size = _X_BE_32(&scratch[4]) - 16;
-  film_header = malloc(film_header_size);
-  if (!film_header)
-    return 0;
-  memcpy(film->version, &scratch[8], 4);
   llprintf(DEBUG_FILM_LOAD, "0x%X header bytes, version %c%c%c%c\n",
     film_header_size,
     film->version[0],
@@ -162,11 +165,17 @@ static int open_film_file(demux_film_t *film) {
     film->version[2],
     film->version[3]);
 
+  /* file is qualified; skip over the header bytes in the stream */
+  if (film->input->seek(film->input, 16, SEEK_SET) != 16)
+    return 0;
+
+  film_header = malloc(film_header_size);
+  if (!film_header)
+    return 0;
+
   /* load the rest of the FILM header */
   if (film->input->read(film->input, film_header, film_header_size) !=
     film_header_size) {
-    free (film->interleave_buffer);
-    free (film->sample_table);
     free (film_header);
     return 0;
   }
@@ -174,18 +183,18 @@ static int open_film_file(demux_film_t *film) {
   /* get the starting offset */
   film->data_start = film->input->get_current_pos(film->input);
   film->data_size = film->input->get_length(film->input) - film->data_start;
+  if (film->data_size < 0)
+    film->data_size = 0;
 
   /* traverse the FILM header */
   i = 0;
-  while (i < film_header_size) {
+  while (i + 7 < hsize) {
     chunk_type = _X_BE_32(&film_header[i]);
     chunk_size = _X_BE_32(&film_header[i + 4]);
 
     /* sanity check the chunk size */
-    if (i + chunk_size > film_header_size) {
+    if (i + chunk_size > hsize) {
       xine_log(film->stream->xine, XINE_LOG_MSG, _("invalid FILM chunk size\n"));
-      free (film->interleave_buffer);
-      free (film->sample_table);
       free (film_header);
       return 0;
     }
@@ -344,8 +353,6 @@ static int open_film_file(demux_film_t *film) {
     default:
       xine_log(film->stream->xine, XINE_LOG_MSG, _("unrecognized FILM chunk\n"));
     film_abort:
-      free (film->interleave_buffer);
-      free (film->sample_table);
       free (film_header);
       return 0;
     }
@@ -386,8 +393,7 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
 
   /* check if all the samples have been sent */
   if (i >= this->sample_count) {
-    this->status = DEMUX_FINISHED;
-    return this->status;
+    goto finished;
   }
 
   /* check if we're only sending audio samples until the next keyframe */
@@ -406,17 +412,23 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
   if ((!this->sample_table[i].audio) &&
     (this->video_type == BUF_VIDEO_CINEPAK)) {
     /* do a special song and dance when loading CVID data */
-    if (this->version[0])
+    if (this->version[0]) {
+      if (this->sample_table[i].sample_size < 2)
+        return this->status;
       cvid_chunk_size = this->sample_table[i].sample_size - 2;
-    else
+    } else {
+      if (this->sample_table[i].sample_size < 6)
+        return this->status;
       cvid_chunk_size = this->sample_table[i].sample_size - 6;
+    }
 
     /* reset flag */
     fixed_cvid_header = 0;
 
     remaining_sample_bytes = cvid_chunk_size;
-    this->input->seek(this->input, this->sample_table[i].sample_offset,
-      SEEK_SET);
+    if (this->input->seek(this->input, this->sample_table[i].sample_offset, SEEK_SET) < 0) {
+      goto finished;
+    }
 
     while (remaining_sample_bytes) {
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
@@ -432,29 +444,26 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
       buf->decoder_flags |= BUF_FLAG_FRAMERATE;
       buf->decoder_info[0] = this->sample_table[i].duration;
 
-      if (remaining_sample_bytes > buf->max_size)
+      if ((int)remaining_sample_bytes > buf->max_size)
         buf->size = buf->max_size;
       else
         buf->size = remaining_sample_bytes;
       remaining_sample_bytes -= buf->size;
 
-      if (!fixed_cvid_header) {
+      if (!fixed_cvid_header && buf->size >= 10) {
         if (this->input->read(this->input, buf->content, 10) != 10) {
-          buf->free_buffer(buf);
-          this->status = DEMUX_FINISHED;
-          break;
+          goto fail_buf;
         }
 
         /* skip over the extra non-spec CVID bytes */
-        this->input->seek(this->input,
-          this->sample_table[i].sample_size - cvid_chunk_size, SEEK_CUR);
-
+        if (this->input->seek(this->input,
+            this->sample_table[i].sample_size - cvid_chunk_size, SEEK_CUR) < 0) {
+          goto fail_buf;
+        }
         /* load the rest of the chunk */
         if (this->input->read(this->input, buf->content + 10,
           buf->size - 10) != buf->size - 10) {
-          buf->free_buffer(buf);
-          this->status = DEMUX_FINISHED;
-          break;
+          goto fail_buf;
         }
 
         /* adjust the length in the CVID data chunk */
@@ -466,9 +475,7 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
       } else {
         if (this->input->read(this->input, buf->content, buf->size) !=
           buf->size) {
-          buf->free_buffer(buf);
-          this->status = DEMUX_FINISHED;
-          break;
+          goto fail_buf;
         }
       }
 
@@ -486,8 +493,9 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
 
     /* load a non-cvid video chunk */
     remaining_sample_bytes = this->sample_table[i].sample_size;
-    this->input->seek(this->input, this->sample_table[i].sample_offset,
-      SEEK_SET);
+    if (this->input->seek(this->input, this->sample_table[i].sample_offset, SEEK_SET) < 0) {
+      goto finished;
+    }
 
     while (remaining_sample_bytes) {
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
@@ -503,17 +511,14 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
       buf->decoder_flags |= BUF_FLAG_FRAMERATE;
       buf->decoder_info[0] = this->sample_table[i].duration;
 
-      if (remaining_sample_bytes > buf->max_size)
+      if ((int)remaining_sample_bytes > buf->max_size)
         buf->size = buf->max_size;
       else
         buf->size = remaining_sample_bytes;
       remaining_sample_bytes -= buf->size;
 
-      if (this->input->read(this->input, buf->content, buf->size) !=
-        buf->size) {
-        buf->free_buffer(buf);
-        this->status = DEMUX_FINISHED;
-        break;
+      if (this->input->read(this->input, buf->content, buf->size) != buf->size) {
+        goto fail_buf;
       }
 
       if (this->sample_table[i].keyframe)
@@ -529,8 +534,9 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
 
     /* load a mono audio sample and packetize it */
     remaining_sample_bytes = this->sample_table[i].sample_size;
-    this->input->seek(this->input, this->sample_table[i].sample_offset,
-      SEEK_SET);
+    if (this->input->seek(this->input, this->sample_table[i].sample_offset, SEEK_SET) < 0) {
+      goto finished;
+    }
 
     first_buf = 1;
     while (remaining_sample_bytes) {
@@ -551,30 +557,27 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
         buf->pts = 0;
       buf->extra_info->input_time = buf->pts / 90;
 
-      if (remaining_sample_bytes > buf->max_size)
+      if ((int)remaining_sample_bytes > buf->max_size)
         buf->size = buf->max_size;
       else
         buf->size = remaining_sample_bytes;
       remaining_sample_bytes -= buf->size;
 
-      if (this->input->read(this->input, buf->content, buf->size) !=
-        buf->size) {
-        buf->free_buffer(buf);
-        this->status = DEMUX_FINISHED;
-        break;
+      if (this->input->read(this->input, buf->content, buf->size) != buf->size) {
+        goto fail_buf;
       }
 
       if (this->video_type == BUF_VIDEO_SEGA) {
         /* if the file uses the SEGA video codec, assume this is
          * sign/magnitude audio */
-        for (j = 0; j < buf->size; j++)
+        for (j = 0; j < (unsigned int)(buf->size); j++)
           if (buf->content[j] < 0x80)
             buf->content[j] += 0x80;
           else
             buf->content[j] = -(buf->content[j] & 0x7F) + 0x80;
       } else if (this->audio_bits == 8) {
         /* convert 8-bit data from signed -> unsigned */
-        for (j = 0; j < buf->size; j++)
+        for (j = 0; j < (unsigned int)(buf->size); j++)
           buf->content[j] += 0x80;
       }
 
@@ -593,8 +596,7 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
     /* load the whole chunk into the buffer */
     if (this->input->read(this->input, this->interleave_buffer,
       this->sample_table[i].sample_size) != this->sample_table[i].sample_size) {
-      this->status = DEMUX_FINISHED;
-      return this->status;
+      goto finished;
     }
 
     /* proceed to de-interleave into individual buffers */
@@ -619,31 +621,31 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
         buf->pts = 0;
       buf->extra_info->input_time = buf->pts / 90;
 
-      if (remaining_sample_bytes > buf->max_size / 2)
+      if ((int)remaining_sample_bytes > buf->max_size / 2)
         buf->size = buf->max_size;
       else
         buf->size = remaining_sample_bytes * 2;
       remaining_sample_bytes -= buf->size / 2;
 
       if (this->audio_bits == 16) {
-        for (j = 0, k = interleave_index; j < buf->size; j += 4, k += 2) {
+        for (j = 0, k = interleave_index; j < (unsigned int)(buf->size); j += 4, k += 2) {
           buf->content[j] =     this->interleave_buffer[k];
           buf->content[j + 1] = this->interleave_buffer[k + 1];
         }
         for (j = 2,
              k = interleave_index + this->sample_table[i].sample_size / 2;
-             j < buf->size; j += 4, k += 2) {
+             j < (unsigned int)(buf->size); j += 4, k += 2) {
           buf->content[j] =     this->interleave_buffer[k];
           buf->content[j + 1] = this->interleave_buffer[k + 1];
         }
         interleave_index += buf->size / 2;
       } else {
-        for (j = 0, k = interleave_index; j < buf->size; j += 2, k += 1) {
+        for (j = 0, k = interleave_index; j < (unsigned int)(buf->size); j += 2, k += 1) {
           buf->content[j] = this->interleave_buffer[k] += 0x80;
         }
         for (j = 1,
              k = interleave_index + this->sample_table[i].sample_size / 2;
-             j < buf->size; j += 2, k += 1) {
+             j < (unsigned int)(buf->size); j += 2, k += 1) {
           buf->content[j] = this->interleave_buffer[k] += 0x80;
         }
         interleave_index += buf->size / 2;
@@ -659,13 +661,18 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
   }
 
   return this->status;
+
+ fail_buf:
+  buf->free_buffer(buf);
+ finished:
+  this->status = DEMUX_FINISHED;
+  return this->status;
 }
 
 static void demux_film_send_headers(demux_plugin_t *this_gen) {
   demux_film_t *this = (demux_film_t *) this_gen;
   buf_element_t *buf;
   int64_t initial_duration = 3000;
-  int i;
 
   this->video_fifo  = this->stream->video_fifo;
   this->audio_fifo  = this->stream->audio_fifo;
@@ -692,6 +699,7 @@ static void demux_film_send_headers(demux_plugin_t *this_gen) {
 
   /* send init info to decoders */
   if (this->video_type) {
+    unsigned int i;
     /* find the first video frame duration */
     for (i = 0; i < this->sample_count; i++)
       if (!this->sample_table[i].audio) {
@@ -827,8 +835,8 @@ static int demux_film_seek (demux_plugin_t *this_gen, off_t start_pos, int start
 static void demux_film_dispose (demux_plugin_t *this_gen) {
   demux_film_t *this = (demux_film_t *) this_gen;
 
-  free(this->sample_table);
-  free(this->interleave_buffer);
+  _x_freep(&this->sample_table);
+  _x_freep(&this->interleave_buffer);
   free(this);
 }
 
@@ -845,11 +853,15 @@ static int demux_film_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_film_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_film_get_optional_data(demux_plugin_t *this_gen,
 					void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -857,8 +869,23 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
                                     input_plugin_t *input) {
 
   demux_film_t    *this;
+  int film_header_size;
+
+  switch (stream->content_detection_method) {
+    case METHOD_BY_MRL:
+    case METHOD_BY_CONTENT:
+    case METHOD_EXPLICIT:
+      if (!probe_film_file(input, &film_header_size))
+        return NULL;
+      break;
+    default:
+      return NULL;
+  }
+
+  this = calloc(1, sizeof(demux_film_t));
+  if (!this)
+    return NULL;
 
-  this         = calloc(1, sizeof(demux_film_t));
   this->stream = stream;
   this->input  = input;
 
@@ -874,38 +901,27 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   this->status = DEMUX_FINISHED;
 
-  switch (stream->content_detection_method) {
-
-  case METHOD_BY_MRL:
-  case METHOD_BY_CONTENT:
-  case METHOD_EXPLICIT:
-
-    if (!open_film_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  break;
-
-  default:
-    free (this);
+  if (!open_film_file(this, film_header_size)) {
+    demux_film_dispose(&this->demux_plugin);
     return NULL;
   }
 
   return &this->demux_plugin;
 }
 
-void *demux_film_init_plugin (xine_t *xine, void *data) {
-  demux_film_class_t     *this;
+void *demux_film_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_film_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("FILM (CPK) demux plugin");
-  this->demux_class.identifier      = "FILM (CPK)";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "cpk cak film";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_film_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("FILM (CPK) demux plugin"),
+    .identifier      = "FILM (CPK)",
+    .mimetypes       = NULL,
+    .extensions      = "cpk cak film",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_film_class;
 }
