@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -43,6 +43,8 @@
 #define LOG
 */
 
+#define USE_FRAME_BUF
+
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/compat.h>
@@ -71,12 +73,282 @@ typedef struct {
   flac_seekpoint_t    *seekpoints;
   int                  seekpoint_count;
 
+#ifdef USE_FRAME_BUF
+  uint8_t              frame_head_crc_tab[256];
+  uint8_t              frame_head[16];
+  uint8_t             *frame_buf;
+  uint32_t             frame_buf_used;
+  uint32_t             frame_buf_size;
+  off_t                frame_buf_filepos;
+  struct {
+    off_t              filepos;
+    uint32_t           rate;
+    uint32_t           bits;
+    uint32_t           channels;
+    uint32_t           vbs;
+    uint32_t           hsize;
+    uint32_t           bsize;
+    uint32_t           num;
+    uint32_t           max_size;
+    uint32_t           buf_pos;
+  }                    frame1, frame2;
+  int64_t              last_pts;
+  int                  seek_flag;
+  int                  read_errs;
+#endif
+
   unsigned char        streaminfo[sizeof(xine_waveformatex) + FLAC_STREAMINFO_SIZE];
 } demux_flac_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_flac_class_t;
+#ifdef USE_FRAME_BUF
+static const uint32_t flac_sample_rates[16] = {
+      0, 88200, 176400, 192000,
+   8000, 16000,  22050,  24000,
+  32000, 44100,  48000,  96000,
+      1,     2,      3,      0
+};
+
+static const uint32_t flac_blocksizes[16] = {
+         0,      192,      576,   2 * 576,
+   4 * 576,  8 * 576,        1,         2,
+       256,  2 * 256,  4 * 256,   8 * 256,
+  16 * 256, 32 * 256, 64 * 256, 128 * 256
+};
+
+static const uint8_t flac_sample_sizes[8] = {
+   0,  8, 12, 0,
+  16, 20, 24, 0
+};
+
+static const uint8_t flac_channels[16] = {
+  1, /* mono */
+  2, /* stereo */
+  3, /* surround */
+  4, /* quadrophonic */
+  5, /* 5.0 */
+  6, /* 5.1 */
+  7, /* 6.1 */
+  8, /* 7.1 */
+  2, /* left + side */
+  2, /* right + side */
+  2, /* mid + side */
+  0,
+  0,
+  0,
+  0,
+  0
+};
+
+static void flac_init_frame_head (demux_flac_t *flac) {
+  uint32_t i, j, v;
+  for (i = 0; i < 256; i++) {
+    v = i << 24;
+    for (j = 0; j < 8; j++)
+      v = (v << 1) ^ (0x07000000 & (((int32_t)v) >> 31));
+    flac->frame_head_crc_tab[i] = v >> 24;
+  }
+}
+
+static void flac_reset_head (demux_flac_t *flac) {
+  flac->frame1.buf_pos = 0;
+  flac->frame2.buf_pos = 0;
+  flac->frame1.hsize   = 0;
+  flac->frame2.hsize   = 0;
+  flac->frame_buf_used = 0;
+}
+
+static uint32_t flac_test_frame_head (demux_flac_t *flac, uint32_t len) {
+  uint8_t *p = flac->frame_head;
+  uint32_t v = 0;
+  while (len--)
+    v = (v >> 8) ^ flac->frame_head_crc_tab[*p++ ^ (v & 0xff)];
+  return v;
+}
+
+static int flac_parse_frame_head (demux_flac_t *flac) {
+  const uint8_t *p = flac->frame_head;
+  uint32_t v;
+  /* sync word */
+  if (p[0] != 0xff)
+    return 1;
+  if ((p[1] & 0xfe) != 0xf8)
+    return 1;
+  /* reserved */
+  if (flac->frame_head[3] & 1)
+    return 2;
+  /* variable block size */
+  flac->frame2.vbs = flac->frame_head[1] & 0x01;
+  /* channels */
+  v = flac_channels[flac->frame_head[3] >> 4];
+  if (v == 0)
+    return 2;
+  flac->frame2.channels = v;
+  /* bits */
+  v = flac_sample_sizes[(flac->frame_head[3] >> 1) & 7];
+  if (v == 0)
+    return 2;
+  flac->frame2.bits = v;
+  /* frame num (fixed block size) or sample num (utf8) */
+  p += 4;
+  v = _X_BE_32 (p);
+  if ((v & 0x80000000) == 0) {
+    flac->frame2.num = v >> 24;
+    p += 1;
+  } else if ((v & 0xe0c00000) == 0xc0800000) {
+    flac->frame2.num = ((v & 0x1f000000) >> 18)
+                     | ((v & 0x003f0000) >> 16);
+    p += 2;
+  } else if ((v & 0xf0c0c000) == 0xe0808000) {
+    flac->frame2.num = ((v & 0x0f000000) >> 12)
+                     | ((v & 0x003f0000) >> 10)
+                     | ((v & 0x00003f00) >>  8);
+    p += 3;
+  } else if ((v & 0xf8c0c0c0) == 0xf0808080) {
+    flac->frame2.num = ((v & 0x07000000) >>  6)
+                     | ((v & 0x003f0000) >>  4)
+                     | ((v & 0x00003f00) >>  2)
+                     |  (v & 0x0000003f);
+    p += 4;
+  } else if ((v & 0xfcc0c0c0) == 0xf8808080) {
+    flac->frame2.num =  (v & 0x03000000)
+                     | ((v & 0x003f0000) <<  2)
+                     | ((v & 0x00003f00) <<  4)
+                     | ((v & 0x0000003f) <<  6)
+                     |  (p[4] & 0x3f);
+    p += 5;
+  } else if ((v & 0xfec0c0c0) == 0xfc808080) {
+    flac->frame2.num = ((v & 0x01000000) <<  6)
+                     | ((v & 0x003f0000) <<  8)
+                     | ((v & 0x00003f00) << 10)
+                     | ((v & 0x0000003f) << 12)
+                     | ((p[4] & 0x3f)    <<  6)
+                     |  (p[5] & 0x3f);
+    p += 6;
+  } else {
+    return 2;
+  }
+  /* block size */
+  v = flac_blocksizes[flac->frame_head[2] >> 4];
+  if (v < 4) {
+    switch (v) {
+      case 1:
+        v = *p++ + 1;
+        break;
+      case 2:
+        v = _X_BE_16 (p) + 1; p += 2;
+        break;
+      default:
+        return 2;
+    }
+  }
+  flac->frame2.bsize = v;
+  /* sample rate */
+  v = flac_sample_rates[flac->frame_head[2] & 0x0f];
+  if (v < 4) {
+    switch (v) {
+      case 1:
+        v = *p++ * 1000;
+        break;
+      case 2:
+        v = _X_BE_16 (p); p += 2;
+        break;
+      case 3:
+        v = _X_BE_16 (p) * 10; p += 2;
+        break;
+      default:
+        return 2;
+    }
+  }
+  flac->frame2.rate = v;
+  /* crc test */
+  p++;
+  flac->frame2.hsize = p - flac->frame_head;
+  if (flac_test_frame_head (flac, flac->frame2.hsize) != 0)
+    return 2;
+  /* flac shall make things smaller than the uncompressed size. */
+  /* frame head + frame crc + channel heads */
+  v = 18 + flac->frame2.channels * (((flac->frame2.bits + 7) >> 3) + 1);
+  /* uncompressed samples */
+  if (flac->frame2.channels == 2)
+    v += ((2 * flac->frame2.bits + 1) * flac->frame2.bsize + 7) >> 3;
+  else
+    v += (flac->frame2.channels * flac->frame2.bits * flac->frame2.bsize + 7) >> 3;
+  flac->frame2.max_size = v;
+  return 0;
+}
+
+static int flac_get_frame (demux_flac_t *flac) {
+  int r = -1;
+  uint32_t v;
+  uint8_t *p, *e;
+
+  flac->frame1 = flac->frame2;
+  p = flac->frame_buf + flac->frame2.buf_pos + flac->frame2.hsize;
+  e = flac->frame_buf + flac->frame_buf_used;
+  memcpy (e, "\xff\xf8\x00\x00", 4);
+  v = _X_BE_32 (p); p += 4;
+
+  while (1) {
+    uint32_t l;
+    int32_t  s;
+    while ((v & 0xfffe0001) != 0xfff80000)
+      v = (v << 8) | *p++;
+    if (p + sizeof (flac->frame_head) - 4 <= e) {
+      flac->frame2.buf_pos = p - 4 - flac->frame_buf;
+      flac->frame2.filepos = flac->frame_buf_filepos + flac->frame2.buf_pos;
+      memcpy (flac->frame_head, flac->frame_buf + flac->frame2.buf_pos, sizeof (flac->frame_head));
+      r = flac_parse_frame_head (flac);
+      if (r == 0)
+        break;
+      v = (v << 8) | *p++;
+      continue;
+    }
+    if (flac->frame1.bits == 0) {
+      p = flac->frame_buf + 4;
+      flac->frame1.buf_pos = 0;
+      flac->frame_buf_used = 0;
+    } else if (flac->frame1.buf_pos) {
+      l = flac->frame_buf_used - flac->frame1.buf_pos;
+      if (l) {
+        if (l <= flac->frame1.buf_pos)
+          memcpy (flac->frame_buf, flac->frame_buf + flac->frame1.buf_pos, l);
+        else
+          memmove (flac->frame_buf, flac->frame_buf + flac->frame1.buf_pos, l);
+      }
+      p -= flac->frame1.buf_pos;
+      flac->frame1.buf_pos = 0;
+      flac->frame_buf_used = l;
+    }
+    flac->frame2.buf_pos = p - 4 - flac->frame_buf;
+    flac->frame2.hsize = 0;
+    l = flac->frame_buf_size - flac->frame_buf_used;
+    if (!l)
+      break;
+    flac->frame_buf_filepos = flac->input->get_current_pos (flac->input) - flac->frame_buf_used;
+    s = flac->input->read (flac->input, flac->frame_buf + flac->frame_buf_used, l);
+    if (s <= 0)
+      break;
+    flac->frame_buf_used += s;
+    e = flac->frame_buf + flac->frame_buf_used;
+    memcpy (e, "\xff\xf8\x00\x00", 4);
+    v = _X_BE_32 (p - 4);
+  }
+
+  /* enlarge buf? */
+  if ((r == 0) && (flac->frame2.max_size > flac->frame_buf_size)) {
+    uint32_t need = 3 * flac->frame2.max_size / 2;
+    uint8_t *n = realloc (flac->frame_buf, need + 16);
+    if (n) {
+      flac->frame_buf = n;
+      flac->frame_buf_size = need;
+      xprintf (flac->stream->xine, XINE_VERBOSITY_DEBUG,
+        "demux_flac: frame buffer enlarged to %u bytes.\n", (unsigned int)need);
+    }
+  }
+
+  return flac->frame2.buf_pos - flac->frame1.buf_pos;
+}
+#endif
 
 /* Open a flac file
  * This function is called from the _open() function of this demuxer.
@@ -90,8 +362,6 @@ static int open_flac_file(demux_flac_t *flac) {
   unsigned char *streaminfo = flac->streaminfo + sizeof(xine_waveformatex);
   int i;
 
-  flac->seekpoints = NULL;
-
   /* fetch the file signature, 4 bytes will read both the fLaC
    * signature and the */
   if (_x_demux_read_header(flac->input, &signature, 4) != 4)
@@ -166,10 +436,13 @@ static int open_flac_file(demux_flac_t *flac) {
     /* SEEKTABLE */
     case 3:
       lprintf ("SEEKTABLE metadata, %d bytes\n", block_length);
+      if (!flac->sample_rate)
+        break;
       flac->seekpoint_count = block_length / FLAC_SEEKPOINT_SIZE;
-      flac->seekpoints = xine_xcalloc(flac->seekpoint_count,
-				      sizeof(flac_seekpoint_t));
-      if (flac->seekpoint_count && !flac->seekpoints)
+      if (!flac->seekpoint_count)
+        break;
+      flac->seekpoints = calloc(flac->seekpoint_count, sizeof(flac_seekpoint_t));
+      if (!flac->seekpoints)
         return 0;
       for (i = 0; i < flac->seekpoint_count; i++) {
         if (flac->input->read(flac->input, buffer, FLAC_SEEKPOINT_SIZE) != FLAC_SEEKPOINT_SIZE)
@@ -194,33 +467,46 @@ static int open_flac_file(demux_flac_t *flac) {
     case 4:
       lprintf ("VORBIS_COMMENT metadata\n");
       {
-        char comments[block_length + 1]; /* last byte for NUL termination */
-        char *ptr = comments;
+        char *comments;
         uint32_t length, user_comment_list_length, cn;
         char *comment;
         char c;
 
+        if (block_length < 8)
+          break;
+
+        comments = malloc(block_length + 1); /* last byte for NUL termination */
+        if (!comments)
+          break;
+
         if (flac->input->read(flac->input, comments, block_length) == block_length) {
+          char *ptr = comments;
           int tracknumber = -1;
           int tracktotal = -1;
 
           length = _X_LE_32(ptr);
           ptr += 4 + length;
-          if (length > block_length - 8)
+          if (length > block_length - 8) {
+            free(comments);
             return 0; /* bad length or too little left in the buffer */
+          }
 
           user_comment_list_length = _X_LE_32(ptr);
           ptr += 4;
 
           cn = 0;
           for (; cn < user_comment_list_length; cn++) {
-            if (ptr > comments + block_length - 4)
+            if (ptr > comments + block_length - 4) {
+              free(comments);
               return 0; /* too little left in the buffer */
+            }
 
             length = _X_LE_32(ptr);
             ptr += 4;
-            if (length >= block_length || ptr + length > comments + block_length)
+            if (length >= block_length || ptr + length > comments + block_length) {
+              free(comments);
               return 0; /* bad length */
+            }
 
             comment = (char*) ptr;
             c = comment[length];
@@ -272,6 +558,7 @@ static int open_flac_file(demux_flac_t *flac) {
             _x_meta_info_set(flac->stream, XINE_META_INFO_TRACK_NUMBER, tn);
           }
         }
+        free(comments);
       }
       break;
 
@@ -291,6 +578,11 @@ static int open_flac_file(demux_flac_t *flac) {
 
   } while ((preamble[0] & 0x80) == 0);
 
+  /* do not bail out yet, maybe decoder can handle this
+  if (!flac->sample_rate)
+    return 0;
+  */
+
   flac->data_start = flac->input->get_current_pos(flac->input);
   flac->data_size = flac->input->get_length(flac->input) - flac->data_start;
 
@@ -305,8 +597,72 @@ static int open_flac_file(demux_flac_t *flac) {
 static int demux_flac_send_chunk(demux_plugin_t *this_gen) {
   demux_flac_t *this = (demux_flac_t *) this_gen;
   buf_element_t *buf = NULL;
+#ifdef USE_FRAME_BUF
+  int l;
+#else
   int64_t input_time_guess;
+#endif
 
+#ifdef USE_FRAME_BUF
+  l = flac_get_frame (this);
+  if (l) {
+    uint32_t flags, normpos = 0;
+    uint8_t *p;
+    int64_t  pts;
+    int      itime;
+
+    if (this->data_size > 0)
+      normpos = (double)(this->frame1.filepos - this->data_start) * 65535 / this->data_size;
+
+    if (this->frame1.rate) {
+      if (this->frame1.vbs)
+        pts = (double)this->frame1.num * 90000 / this->frame1.rate;
+      else
+        pts = (double)this->frame1.num * this->frame1.bsize * 90000 / this->frame1.rate;
+      itime = pts / 90;
+    } else {
+      pts = 0;
+      itime = 0;
+    }
+
+    if (this->seek_flag) {
+      _x_demux_control_newpts (this->stream, pts, BUF_FLAG_SEEK);
+      this->seek_flag = 0;
+    } else {
+      int64_t diff = pts - this->last_pts;
+      if (diff < 0)
+        diff = -diff;
+      if (diff > 220000)
+        _x_demux_control_newpts (this->stream, pts, 0);
+    }
+    this->last_pts = pts;
+
+    this->read_errs = 3;
+    flags = BUF_FLAG_FRAME_START;
+    p = this->frame_buf + this->frame1.buf_pos;
+    while (l) {
+      buf = this->audio_fifo->buffer_pool_size_alloc (this->audio_fifo, l);
+      buf->type = BUF_AUDIO_FLAC;
+      buf->size = l > buf->max_size ? buf->max_size : l;
+      memcpy (buf->content, p, buf->size);
+      p += buf->size;
+      l -= buf->size;
+      if (!l)
+        flags |= BUF_FLAG_FRAME_END;
+      buf->decoder_flags |= flags;
+      flags &= ~BUF_FLAG_FRAME_START;
+      buf->extra_info->input_normpos = normpos;
+      buf->pts = pts;
+      buf->extra_info->input_time = itime;
+      pts = 0;
+      itime = 0;
+      this->audio_fifo->put (this->audio_fifo, buf);
+    }
+  } else if (!this->seek_flag) {
+    if (--this->read_errs < 0)
+      this->status = DEMUX_FINISHED;
+  }
+#else
   /* just send a buffer-sized chunk; let the decoder figure out the
    * boundaries and let the engine figure out the pts */
   buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
@@ -328,12 +684,14 @@ static int demux_flac_send_chunk(demux_plugin_t *this_gen) {
    */
 
   /* do this one step at a time to make sure all the numbers stay safe */
-  input_time_guess = this->total_samples;
-  input_time_guess /= this->sample_rate;
-  input_time_guess *= 1000;
-  input_time_guess *= buf->extra_info->input_normpos;
-  input_time_guess /= 65535;
-  buf->extra_info->input_time = input_time_guess;
+  if (this->sample_rate) {
+    input_time_guess = this->total_samples;
+    input_time_guess /= this->sample_rate;
+    input_time_guess *= 1000;
+    input_time_guess *= buf->extra_info->input_normpos;
+    input_time_guess /= 65535;
+    buf->extra_info->input_time = input_time_guess;
+  }
 
   if (this->input->read(this->input, buf->content, buf->size) !=
     buf->size) {
@@ -343,7 +701,7 @@ static int demux_flac_send_chunk(demux_plugin_t *this_gen) {
   }
   buf->decoder_flags |= BUF_FLAG_FRAME_END;
   this->audio_fifo->put(this->audio_fifo, buf);
-
+#endif
   return this->status;
 }
 
@@ -401,7 +759,9 @@ static int demux_flac_seek (demux_plugin_t *this_gen,
   demux_flac_t *this = (demux_flac_t *) this_gen;
   int seekpoint_index = 0;
   int64_t start_pts;
+#ifndef USE_FRAME_BUF
   unsigned char buf[4];
+#endif
 
   start_pos = (off_t) ( (double) start_pos / 65535 *
               this->data_size );
@@ -427,6 +787,10 @@ static int demux_flac_seek (demux_plugin_t *this_gen,
       this->status = DEMUX_OK;
       start_pos += this->data_start;
       this->input->seek(this->input, start_pos, SEEK_SET);
+#ifdef USE_FRAME_BUF
+      flac_reset_head (this);
+      this->seek_flag = 1;
+#else
       while(1){ /* here we try to find something that resembles a frame header */
 
 	if (this->input->read(this->input, buf, 2) != 2){
@@ -439,13 +803,17 @@ static int demux_flac_seek (demux_plugin_t *this_gen,
 		  * to decide, but this way we reduce the number of warnings */
 	start_pos +=2;
       }
-
+#endif
       _x_demux_flush_engine(this->stream);
       this->input->seek(this->input, start_pos, SEEK_SET);
       _x_demux_control_newpts(this->stream, 0, BUF_FLAG_SEEK);
       return this->status;
 
     } else {
+#ifdef USE_FRAME_BUF
+      flac_reset_head (this);
+      this->seek_flag = 1;
+#endif
       /* do a lazy, linear seek based on the assumption that there are not
        * that many seek points; time-based seek */
       start_pts = start_time;
@@ -476,6 +844,9 @@ static void demux_flac_dispose (demux_plugin_t *this_gen) {
   demux_flac_t *this = (demux_flac_t *) this_gen;
 
   free(this->seekpoints);
+#ifdef USE_FRAME_BUF
+  free (this->frame_buf);
+#endif
   free(this);
 }
 
@@ -489,6 +860,9 @@ static int demux_flac_get_stream_length (demux_plugin_t *this_gen) {
   demux_flac_t *this = (demux_flac_t *) this_gen;
   int64_t length = this->total_samples;
 
+  if (!this->sample_rate)
+    return 0;
+
   length *= 1000;
   length /= this->sample_rate;
 
@@ -496,11 +870,15 @@ static int demux_flac_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_flac_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_flac_get_optional_data(demux_plugin_t *this_gen,
                                         void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -515,7 +893,39 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     return NULL;
   }
 
-  this         = calloc(1, sizeof(demux_flac_t));
+  this = calloc(1, sizeof(demux_flac_t));
+  if (!this)
+    return NULL;
+#ifndef HAVE_ZERO_SAFE_MEM
+#  ifdef USE_FRAME_BUF
+  this->frame_buf_filepos = 0;
+  this->frame_buf_used    = 0;
+  this->frame2.filepos    = 0;
+  this->frame2.rate       = 0;
+  this->frame2.bits       = 0;
+  this->frame2.channels   = 0;
+  this->frame2.vbs        = 0;
+  this->frame2.bsize      = 0;
+  this->frame2.num        = 0;
+  this->frame2.max_size   = 0;
+  this->frame2.buf_pos    = 0;
+  this->last_pts          = 0;
+  this->seek_flag         = 0;
+#  endif
+  this->seekpoints        = NULL;
+#endif
+
+#ifdef USE_FRAME_BUF
+  this->frame_buf_size = 8 << 10;
+  this->frame_buf      = malloc (this->frame_buf_size + 16);
+  if (!this->frame_buf) {
+    free (this);
+    return NULL;
+  }
+  flac_init_frame_head (this);
+  this->read_errs = 3;
+#endif
+
   this->stream = stream;
   this->input  = input;
 
@@ -545,6 +955,9 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   break;
 
   default:
+#ifdef USE_FRAME_BUF
+    free (this->frame_buf);
+#endif
     free (this);
     return NULL;
   }
@@ -552,19 +965,23 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_flac_init_plugin (xine_t *xine, void *data) {
-  demux_flac_class_t     *this;
+void *demux_flac_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_flac_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Free Lossless Audio Codec (flac) demux plugin");
-  this->demux_class.identifier      = "FLAC";
-  this->demux_class.mimetypes       =
-    "audio/x-flac: flac: FLAC Audio;"
-    "audio/flac: flac: FLAC Audio;";
-  this->demux_class.extensions      = "flac";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_flac_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Free Lossless Audio Codec (flac) demux plugin"),
+    .identifier      = "FLAC",
+    .mimetypes       =
+      "audio/x-flac: flac: FLAC Audio;"
+      "audio/flac: flac: FLAC Audio;",
+    .extensions      = "flac",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_flac_class;
 }
+
+
