@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2017 the xine project
+ * Copyright (C) 2004-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -103,15 +103,14 @@ typedef struct {
 
   int                  audiodelay;   /* fine tune a/v sync */
 
+  uint32_t             video_time;
+
   unsigned int         zero_pts_count;
 
 #define TEMPBUFSIZE 4096
   uint8_t             *tempbuf;
 } demux_flv_t ;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_flv_class_t;
 
 /* an early FLV specification had 24bit bigendian timestamps. This
    limited clip duration to 4:39:37.215. The backwards compatible solution:
@@ -169,7 +168,10 @@ typedef enum {
   VF_SCREEN2, /* v2 */
   VF_H264,    /* MPEG4 part 10, usually with global sequence parameter set */
   VF_H263,
-  VF_MP4      /* MPEG4 part 2, usually with global sequence parameter set */
+  VF_MP4,     /* MPEG4 part 2, usually with global sequence parameter set */
+  VF_reserved10,
+  VF_reserved11,
+  VF_HEVC     /* H265 */
 } vf_t;
 
 /* redefine abs as macro to handle 64-bit diffs.
@@ -200,40 +202,6 @@ static void check_newpts (demux_flv_t *this, int64_t pts, int video) {
   this->last_pts[video] = pts;
 }
 
-/* returns 1 if the FLV file was opened successfully, 0 otherwise */
-static int open_flv_file(demux_flv_t *this) {
-  unsigned char buffer[9];
-
-  if (_x_demux_read_header(this->input, buffer, 9) != 9)
-    return 0;
-
-  if ((buffer[0] != 'F') || (buffer[1] != 'L') || (buffer[2] != 'V'))
-    return 0;
-
-  if (buffer[3] != 0x01) {
-    xprintf(this->xine, XINE_VERBOSITY_LOG,
-      _("unsupported FLV version (%d).\n"), buffer[3]);
-    return 0;
-  }
-
-  this->flags = buffer[4];
-  if ((this->flags & (FLV_FLAG_HAS_VIDEO | FLV_FLAG_HAS_AUDIO)) == 0) {
-    xprintf(this->xine, XINE_VERBOSITY_LOG,
-      _("neither video nor audio stream in this file.\n"));
-    return 0;
-  }
-
-  this->start = _X_BE_32(&buffer[5]);
-  this->size = this->input->get_length(this->input);
-
-  this->input->seek(this->input, this->start, SEEK_SET);
-
-  lprintf("  qualified FLV file, repositioned @ offset 0x%" PRIxMAX "\n",
-          (intmax_t)this->start);
-
-  return 1;
-}
-
 #define BE_F64(buf) ({\
   union { uint64_t q; double d; } _tmp;\
   _tmp.q = _X_BE_64(buf);\
@@ -268,6 +236,40 @@ typedef enum {
 #define SPC (space + 2 * (MAX_AMF_LEVELS - level))
 #define NEEDBYTES(n) if ((unsigned long int)(end - p) < n) return 0
 
+static unsigned int key_num (const char *key) {
+  static const char * const known_keys[] = {
+    "\x0b""audiocodecid",
+    "\x0a""audiodatarate",
+    "\x0d""audiodelay",
+    "\x07""audiosamplerate",
+    "\x08""audiosamplesize",
+    "\x01""duration",
+    "\x10""filepositions",
+    "\x0c""filesize",
+    "\x04""framerate",
+    "\x03""height",
+    "\x0e""onMetaData",
+    "\x09""stereo",
+    "\x0f""times",
+    "\x06""videocodecid",
+    "\x05""videodatarate",
+    "\x04""videoframerate",
+    "\x02""width"
+  };
+  unsigned int b = 0, e = sizeof (known_keys) / sizeof (known_keys[0]), m = e >> 1;
+  do {
+    int d = strcmp (key, known_keys[m] + 1);
+    if (d < 0)
+      e = m;
+    else if (d > 0)
+      b = m + 1;
+    else
+      return known_keys[m][0];
+    m = (b + e) >> 1;
+  } while (b != e);
+  return 0;
+}
+
 static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
   unsigned char *p = buf, *end = buf + size, *name, space[2 * MAX_AMF_LEVELS + 3];
   int level = 0, i, type, count[MAX_AMF_LEVELS], info = 0;
@@ -282,6 +284,7 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
   /* top level has nameless vars */
   count[0] = 10000;
   while (1) {
+    unsigned int keynum;
     if (count[level] > 0) {
       /* next strict array item */
       if (--count[level] == 0) {
@@ -293,6 +296,7 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
       if (p >= end) break;
       type = *p++;
       name = NULL;
+      keynum = 0;
       xprintf (this->xine, XINE_VERBOSITY_DEBUG, "%s", SPC);
     } else {
       /* get current name */
@@ -313,6 +317,7 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
       type = *p;
       *p++ = 0x00;
       xprintf (this->xine, XINE_VERBOSITY_DEBUG, "%s%s = ", SPC, name);
+      keynum = key_num ((char *)name);
     }
     switch (type) {
       case AMF0_NUMBER:
@@ -322,44 +327,56 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
         if (i == val) xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "%d\n", i);
         else xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "%.03lf\n", val);
         p += 8;
-        if (name && info) {
-          if (!strcmp (name, "duration")) {
-            this->length = val * (double)1000;
-          } else if (!strcmp (name, "width")) {
-            this->width = i;
-            _x_stream_info_set (this->stream, XINE_STREAM_INFO_VIDEO_WIDTH, i);
-          } else if (!strcmp (name, "height")) {
-            this->height = i;
-            _x_stream_info_set (this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT, i);
-          } else if (!strcmp (name, "framerate") || !strcmp (name, "videoframerate")) {
-            if ((i > 0) && (i < 1000)) {
-              this->duration = (double)90000 / val;
-              _x_stream_info_set (this->stream, XINE_STREAM_INFO_FRAME_DURATION,
-                this->duration);
-            }
-          } else if (!strcmp (name, "videodatarate")) {
-            _x_stream_info_set (this->stream, XINE_STREAM_INFO_VIDEO_BITRATE,
-              val * (double)1000);
-          } else if (!strcmp (name, "videocodecid")) {
-            this->videocodec = i;
-          } else if (!strcmp (name, "audiosamplerate")) {
-            this->samplerate = i;
-            _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, i);
-          } else if (!strcmp (name, "audiosamplesize")) {
-            this->samplesize = i;
-            _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_BITS, i);
-          } else if (!strcmp (name, "stereo")) {
-            this->audio_channels = i ? 2 : 1;
-            _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, i);
-          } else if (!strcmp (name, "audiodatarate")) {
-            _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_BITRATE,
-              val * (double)1000);
-          } else if (!strcmp (name, "audiocodecid")) {
-            this->audiocodec = i;
-          } else if (!strcmp (name, "filesize")) {
-            this->filesize = val;
-          } else if (!strcmp (name, "audiodelay")) {
-            this->audiodelay = val * (double)-1000;
+        if (info) {
+          switch (keynum) {
+            case 0x01: /* "duration" */
+              this->length = val * (double)1000;
+            break;
+            case 0x02: /* "width" */
+              this->width = i;
+              _x_stream_info_set (this->stream, XINE_STREAM_INFO_VIDEO_WIDTH, i);
+            break;
+            case 0x03: /* "height" */
+              this->height = i;
+              _x_stream_info_set (this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT, i);
+            break;
+            case 0x04: /* "framerate", "videoframerate" */
+              if ((i > 0) && (i < 1000)) {
+                this->duration = (double)90000 / val;
+                _x_stream_info_set (this->stream, XINE_STREAM_INFO_FRAME_DURATION, this->duration);
+              }
+            break;
+            case 0x05: /* "videodatarate" */
+              _x_stream_info_set (this->stream, XINE_STREAM_INFO_VIDEO_BITRATE, val * (double)1000);
+            break;
+            case 0x06: /* "videocodecid" */
+              this->videocodec = i;
+            break;
+            case 0x07: /* "audiosamplerate" */
+              this->samplerate = i;
+              _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, i);
+            break;
+            case 0x08: /* "audiosamplesize" */
+              this->samplesize = i;
+              _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_BITS, i);
+            break;
+            case 0x09: /* "stereo" */
+              this->audio_channels = i ? 2 : 1;
+              _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, i);
+            break;
+            case 0x0a: /* "audiodatarate" */
+              _x_stream_info_set (this->stream, XINE_STREAM_INFO_AUDIO_BITRATE, val * (double)1000);
+            break;
+            case 0x0b: /* "audiocodecid" */
+              this->audiocodec = i;
+            break;
+            case 0x0c: /* "filesize" */
+              this->filesize = val;
+            break;
+            case 0x0d: /* "audiodelay" */
+              this->audiodelay = val * (double)-1000;
+            break;
+            default: ;
           }
         }
       break;
@@ -367,8 +384,8 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
         NEEDBYTES (1);
         i = !!(*p++);
         xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "%s\n", i ? "yes" : "no");
-        if (name && info) {
-          if (!strcmp (name, "stereo"))
+        if (info) {
+          if (keynum == 0x09) /* "stereo" */
             this->audio_channels = i ? 2 : 1;
         }
       break;
@@ -380,15 +397,22 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
         c = p[u];
         p[u] = 0x00;
         xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "\"%s\"\n", p);
-        if (!level && !strcmp (p, "onMetaData")) info = this->got_info = 1;
-        if (name && info) {
-          if ((!strcmp (name, "audiocodecid")) && (!strcmp (p, "mp4a")))
-            this->audiocodec = AF_AAC;
-          else if ((!strcmp (name, "videocodecid")) && (!strcmp (p, "avc1")))
-            this->videocodec = VF_H264;
-          else if (!strcmp (name, "stereo")) {
-            if (!strcmp (p, "true") || !strcmp (p, "yes"))
-              this->audio_channels = 2;
+        if (!level && !memcmp (p, "onMetaData", 11)) info = this->got_info = 1;
+        if (info) {
+          switch (keynum) {
+            case 0x0b: /* "audiocodecid" */
+              if (!memcmp (p, "mp4a", 5))
+                this->audiocodec = AF_AAC;
+            break;
+            case 0x06: /* "videocodecid" */
+              if (!memcmp (p, "avc1", 5))
+                this->videocodec = VF_H264;
+            break;
+            case 0x09: /* "stereo" */
+              if (!memcmp (p, "true", 5) || !memcmp (p, "yes", 4))
+                this->audio_channels = 2;
+            break;
+            default: ;
           }
         }
         p[u] = c;
@@ -423,12 +447,7 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
         NEEDBYTES (4);
         u = _X_BE_32 (p);
         p += 4;
-        c = 0;
-        if (name) {
-          if (!strcmp (name, "times")) c = 1;
-          else if (!strcmp (name, "filepositions")) c = 2;
-        }
-        if (c) {
+        if ((keynum == 0x0f) || (keynum == 0x10)) { /* "times", "filepositions" */
           NEEDBYTES (u * 9);
           xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "[%d] {..}\n", u);
           if (!this->index || (this->num_indices != u)) {
@@ -437,14 +456,18 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
             if (!this->index) return 0;
             this->num_indices = u;
           }
-          if (c == 1) for (i = 0; i < (int)u; i++) {
-            if (*p++ != AMF0_NUMBER) return 0;
-            this->index[i].pts = BE_F64 (p) * (double)1000;
-            p += 8;
-          } else for (i = 0; i < (int)u; i++) {
-            if (*p++ != AMF0_NUMBER) return 0;
-            this->index[i].offset = BE_F64 (p);
-            p += 8;
+          if (keynum == 0x0f) { /* "times" */
+            for (i = 0; i < (int)u; i++) {
+              if (*p++ != AMF0_NUMBER) return 0;
+              this->index[i].pts = BE_F64 (p) * (double)1000;
+              p += 8;
+            }
+          } else { /* "filepositions" */
+            for (i = 0; i < (int)u; i++) {
+              if (*p++ != AMF0_NUMBER) return 0;
+              this->index[i].offset = BE_F64 (p);
+              p += 8;
+            }
           }
         } else {
           if (++level >= MAX_AMF_LEVELS) return 0;
@@ -520,9 +543,9 @@ static int read_flv_packet(demux_flv_t *this, int preview) {
        12 .. 14  stream id (usually 0)
        15        codec id / frame type / audio params (A/V only)
        16        right / bottom crop (VP6(F))
-                 frame subtype (AAC, H264, MPEG4)
+                 frame subtype (AAC, H264, MPEG4, HEVC)
        17 .. 19  ?? (VP6F)
-                 pts - dts (H264, MPEG4) */
+                 pts - dts (H264, MPEG4, HEVC) */
     unsigned char  buffer[20];
     /* skip rest, if any */
     if (remaining_bytes)
@@ -533,7 +556,7 @@ static int read_flv_packet(demux_flv_t *this, int preview) {
       this->status = DEMUX_FINISHED;
       return this->status;
     }
-    remaining_bytes = _X_BE_24(&buffer[5]);
+    remaining_bytes = _X_BE_32 (&buffer[4]) & 0xffffff;
     /* skip empty tags */
     if (--remaining_bytes < 0)
       continue;
@@ -642,13 +665,18 @@ static int read_flv_packet(demux_flv_t *this, int preview) {
         switch (this->videocodec) {
           case VF_FLV1:
             buf_type = BUF_VIDEO_FLV1;
+            this->video_time = pts;
           break;
           case VF_H263:
             buf_type = BUF_VIDEO_H263;
+            this->video_time = pts;
           break;
           case VF_MP4:
             buf_type = BUF_VIDEO_MPEG4;
           goto comm_mpeg4;
+          case VF_HEVC:
+            buf_type = BUF_VIDEO_HEVC;
+          goto comm_mpeg4;
           case VF_H264:
             buf_type = BUF_VIDEO_H264;
             /* AVC extra header */
@@ -658,8 +686,10 @@ static int read_flv_packet(demux_flv_t *this, int preview) {
               continue; /* skip sequence footer */
             if (!buffer[16])
               mp4header = 1;
+            else
+              this->video_time = pts;
             /* pts really is dts here, buffer[17..19] has (pts - dts) signed big endian. */
-            ptsoffs = _X_BE_24 (buffer + 17);
+            ptsoffs = _X_BE_32 (buffer + 16) & 0xffffff;
             if (ptsoffs & 0x800000)
               ptsoffs |= ~0xffffff;
             /* better: +/- 16 frames, but we cannot trust header framerate */
@@ -670,13 +700,16 @@ static int read_flv_packet(demux_flv_t *this, int preview) {
           case VF_VP6:
             buf_type = BUF_VIDEO_VP6F;
             GETBYTES (1);
+            this->video_time = pts;
           break;
           case VF_VP6A:
             buf_type = BUF_VIDEO_VP6F;
             GETBYTES (4);
+            this->video_time = pts;
           break;
           case VF_JPEG:
             buf_type = BUF_VIDEO_JPEG;
+            this->video_time = pts;
           break;
           default:
             lprintf("  unsupported video format (%d)...\n", this->videocodec);
@@ -832,6 +865,7 @@ static void seek_flv_file (demux_flv_t *this, off_t seek_pos, int seek_pts) {
     this->input->seek(this->input, this->start, SEEK_SET);
     this->cur_pts = 0;
     this->zero_pts_count = 0;
+    this->video_time = ~0u;
     return;
   }
  
@@ -841,22 +875,30 @@ static void seek_flv_file (demux_flv_t *this, off_t seek_pos, int seek_pts) {
     uint32_t a = 0, b, c = this->num_indices;
     while (a + 1 < c) {
       b = (a + c) >> 1;
-      if (this->index[b].pts <= seek_pts) a = b; else c = b;
+      if (this->index[b].pts <= (unsigned int)seek_pts) a = b; else c = b;
     }
     x = &this->index[a];
     if ((x->offset >= this->start + 4) && (x->offset + 15 < size)) {
-      this->input->seek (this->input, x->offset, SEEK_SET);
-      this->input->read (this->input, (char *)buf, 15);
-      if (!buf[8] && !buf[9] && !buf[10] && (
-        ((buf[0] == FLV_TAG_TYPE_VIDEO) && ((buf[11] >> 4) == 1)) ||
-        (buf[0] == FLV_TAG_TYPE_AUDIO)
-        )) {
-        xprintf (this->xine, XINE_VERBOSITY_DEBUG,
-          "demux_flv: seek_index (%u.%03u, %"PRId64")\n",
-          x->pts / 1000, x->pts % 1000, (int64_t)x->offset);
-        this->input->seek (this->input, x->offset - 4, SEEK_SET);
-        this->cur_pts = x->pts;
-        return;
+      if (this->input->seek (this->input, x->offset, SEEK_SET) == x->offset) {
+        if (this->input->read (this->input, (char *)buf, 15) == 15) {
+          if (!buf[8] && !buf[9] && !buf[10] && (
+             ((buf[0] == FLV_TAG_TYPE_VIDEO) && ((buf[11] >> 4) == 1)) ||
+              (buf[0] == FLV_TAG_TYPE_AUDIO)
+          )) {
+            uint32_t found_pts = gettimestamp (buf, 4);
+            if (found_pts == 0)
+              found_pts = x->pts;
+            if ((found_pts < x->pts + 1000) && (x->pts < found_pts + 1000)) {
+              xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+                "demux_flv: seek_index (%u.%03u, %"PRId64")\n",
+                found_pts / 1000, found_pts % 1000, (int64_t)x->offset);
+              this->input->seek (this->input, x->offset - 4, SEEK_SET);
+              this->cur_pts = found_pts;
+              this->video_time = buf[0] == FLV_TAG_TYPE_VIDEO ? found_pts : ~0u;
+              return;
+            }
+          }
+        }
       }
     }
     xprintf (this->xine, XINE_VERBOSITY_LOG, _("demux_flv: Not using broken seek index.\n"));
@@ -920,13 +962,13 @@ static void seek_flv_file (demux_flv_t *this, off_t seek_pos, int seek_pts) {
       if ((buf[4] == FLV_TAG_TYPE_VIDEO) && ((buf[15] >> 4) == 1)) {
         keypos = pos;
         /* Shortcut xine_keyframes users. */
-        if (now == seek_pts) break;
+        if (now == (unsigned int)seek_pts) break;
       }
       /* Non seekable file ?? */
       if (now == 0) break;
       /* Stop at reversal of direction, dont slow when tags are not sorted by pts properly. */
       /* Stop at latest tag at, or at first tag right after seek_pts to make step 3 work.   */
-      if (now > seek_pts) {
+      if (now > (unsigned int)seek_pts) {
         found = pos;
         if (i > 0) break;
         /* Traverse backwards. */
@@ -938,7 +980,7 @@ static void seek_flv_file (demux_flv_t *this, off_t seek_pos, int seek_pts) {
         if (!this->videocodec) found = pos;
         if (i < 0) break;
         /* Traverse forward. */
-        pos += _X_BE_24 (&buf[5]) + 15;
+        pos += (_X_BE_32 (&buf[4]) & 0xffffff) + 15;
         i = 1;
       }
     }
@@ -959,6 +1001,7 @@ static void seek_flv_file (demux_flv_t *this, off_t seek_pos, int seek_pts) {
       if ((buf[4] == FLV_TAG_TYPE_VIDEO) && ((buf[15] >> 4) == 1)) {
         found = pos;
         now = gettimestamp (buf, 8);
+        this->video_time = now;
         xprintf (this->xine, XINE_VERBOSITY_DEBUG,
           "demux_flv: seek_keyframe (%u.%03u, %"PRId64")\n", now / 1000, now % 1000, (int64_t)found);
         break;
@@ -969,10 +1012,12 @@ static void seek_flv_file (demux_flv_t *this, off_t seek_pos, int seek_pts) {
   }
 
   /* we are there!! */
-  this->input->seek (this->input, found + 4, SEEK_SET);
-  this->input->read (this->input, (char *)buf, 4);
-  this->cur_pts = gettimestamp (buf, 0);
-  this->input->seek (this->input, found - 4, SEEK_SET);
+  if (this->input->seek (this->input, found + 4, SEEK_SET) == found + 4) {
+    if (this->input->read (this->input, (char *)buf, 4) == 4) {
+      this->cur_pts = gettimestamp (buf, 0);
+      this->input->seek (this->input, found - 4, SEEK_SET);
+    }
+  }
 
   return;
 }
@@ -1040,7 +1085,7 @@ static int demux_flv_seek (demux_plugin_t *this_gen,
      It usually works at least due to xine input cache.
      Even if not, no problem there. */
   if ((!start_time && !start_pos) || INPUT_IS_SEEKABLE (this->input)) {
-    if (!this->length || start_time < this->length) {
+    if (!this->length || (unsigned int)start_time < this->length) {
       _x_demux_flush_engine(this->stream);
       seek_flv_file(this, start_pos, start_time);
     }
@@ -1070,11 +1115,11 @@ static int demux_flv_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_flv_get_capabilities(demux_plugin_t *this_gen) {
-  return DEMUX_CAP_AUDIOLANG;
+  demux_flv_t *this = (demux_flv_t *) this_gen;
+  return DEMUX_CAP_AUDIOLANG | (this->video_time != ~0u ? DEMUX_CAP_VIDEO_TIME : 0);
 }
 
-static int demux_flv_get_optional_data(demux_plugin_t *this_gen,
-                                       void *data, int data_type) {
+static int demux_flv_get_optional_data (demux_plugin_t *this_gen, void *data, int data_type) {
   demux_flv_t *this = (demux_flv_t *) this_gen;
 
   /* be a bit paranoid */
@@ -1082,16 +1127,24 @@ static int demux_flv_get_optional_data(demux_plugin_t *this_gen,
     return DEMUX_OPTIONAL_UNSUPPORTED;
 
   switch (data_type) {
-    case DEMUX_OPTIONAL_DATA_AUDIOLANG: {
-      char *str   = data;
-      int channel = *((int *)data);
-      if (channel != 0) {
-        strcpy (str, "none");
-      } else {
-        strcpy (str, "und");
+    case DEMUX_OPTIONAL_DATA_AUDIOLANG:
+      if (data) {
+        char *str = data;
+        int channel;
+        memcpy (&channel, data, sizeof (channel));
+        if (channel != 0) {
+          strcpy (str, "none");
+        } else {
+          strcpy (str, "und");
+          return DEMUX_OPTIONAL_SUCCESS;
+        }
+      }
+    break;
+    case DEMUX_OPTIONAL_DATA_VIDEO_TIME:
+      if ((this->video_time != ~0u) && data) {
+        memcpy (data, &this->video_time, sizeof (this->video_time));
         return DEMUX_OPTIONAL_SUCCESS;
       }
-    }
     break;
     default: ;
   }
@@ -1101,10 +1154,50 @@ static int demux_flv_get_optional_data(demux_plugin_t *this_gen,
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
                                     input_plugin_t *input) {
   demux_flv_t *this;
+  unsigned char buffer[9];
+  off_t start;
 
-  this         = calloc (1, sizeof (demux_flv_t) + 32 + TEMPBUFSIZE);
+  switch (stream->content_detection_method) {
+    case METHOD_BY_MRL:
+    case METHOD_BY_CONTENT:
+    case METHOD_EXPLICIT:
+      break;
+    default:
+      return NULL;
+  }
+
+  if (_x_demux_read_header (input, buffer, 9) != 9)
+    return NULL;
+
+  if ((buffer[0] != 'F') || (buffer[1] != 'L') || (buffer[2] != 'V'))
+    return NULL;
+  if (buffer[3] != 0x01) {
+    xprintf (stream->xine, XINE_VERBOSITY_LOG,
+      _("unsupported FLV version (%d).\n"), (int)buffer[3]);
+    return NULL;
+  }
+  if ((buffer[4] & (FLV_FLAG_HAS_VIDEO | FLV_FLAG_HAS_AUDIO)) == 0) {
+    xprintf (stream->xine, XINE_VERBOSITY_LOG,
+      _("neither video nor audio stream in this file.\n"));
+    return NULL;
+  }
+
+  this = calloc (1, sizeof (*this) + 32 + TEMPBUFSIZE);
   if (!this)
     return NULL;
+
+  start = _X_BE_32 (buffer + 5);
+  if (input->seek (input, start, SEEK_SET) != start) {
+    input->seek (input, 0, SEEK_SET);
+    free (this);
+    return NULL;
+  }
+  lprintf ("  qualified FLV file, repositioned @ offset 0x%" PRIxMAX "\n", (intmax_t)start);
+
+  this->flags = buffer[4];
+  this->start = start;
+  this->size = input->get_length (input);
+
   this->tempbuf = (uint8_t *)(((uintptr_t)this + sizeof (demux_flv_t) + 31) & ~(uintptr_t)31);
   this->xine   = stream->xine;
   this->stream = stream;
@@ -1120,40 +1213,29 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.get_optional_data = demux_flv_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
 
+  this->video_time = ~0u;
   this->status = DEMUX_FINISHED;
 
-  switch (stream->content_detection_method) {
-    case METHOD_BY_MRL:
-    case METHOD_BY_CONTENT:
-    case METHOD_EXPLICIT:
-      if (!open_flv_file(this)) {
-        free (this);
-        return NULL;
-      }
-      break;
-
-    default:
-      free (this);
-      return NULL;
-  }
-
   return &this->demux_plugin;
 }
 
-void *demux_flv_init_class (xine_t *xine, void *data) {
-  demux_flv_class_t     *this;
+void *demux_flv_init_class (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof (demux_flv_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Flash Video file demux plugin");
-  this->demux_class.identifier      = "FLV";
-  this->demux_class.mimetypes       = "video/x-flv: flv: Flash video;"
-				      "video/flv: flv: Flash video;"
-				      "application/x-flash-video: flv: Flash video;";
-  this->demux_class.extensions      = "flv";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_flv_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Flash Video file demux plugin"),
+    .identifier      = "FLV",
+    .mimetypes       =
+      "video/x-flv: flv: Flash video;"
+      "video/flv: flv: Flash video;"
+      "application/x-flash-video: flv: Flash video;",
+    .extensions      = "flv",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_flv_class;
 }
 
