@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -107,15 +107,12 @@ typedef struct {
   unsigned int         audio_type;
   unsigned int         audio_frame_count;
 
-  palette_entry_t      palette[PALETTE_COUNT];
+  palette_entry_t      palette[PALETTE_COUNT + 1];
   unsigned char       *decode_map;
   int                  decode_map_size;
   int                  new_palette;
 } demux_ipmovie_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_ipmovie_class_t;
 
 /* This function loads and processes a single chunk in an IP movie file.
  * It returns the type of chunk that was processed. */
@@ -206,12 +203,14 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
 
       case OPCODE_END_OF_STREAM:
         lprintf("end of stream\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_END_OF_CHUNK:
         lprintf("end of chunk\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_CREATE_TIMER:
@@ -264,7 +263,8 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
 
       case OPCODE_START_STOP_AUDIO:
         lprintf("start/stop audio\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_INIT_VIDEO_BUFFERS:
@@ -299,12 +299,14 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
       case OPCODE_UNKNOWN_14:
       case OPCODE_UNKNOWN_15:
         lprintf("unknown (but documented) opcode %02X\n", opcode_type);
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_SEND_BUFFER:
         lprintf("send buffer\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_AUDIO_FRAME:
@@ -317,7 +319,8 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
           opcode_size -= 6;
           this->audio_frame_count +=
             (opcode_size / this->wave.nChannels / (this->wave.wBitsPerSample / 8));
-          this->input->seek(this->input, 6, SEEK_CUR);
+          if (this->input->seek(this->input, 6, SEEK_CUR) < 0)
+            return CHUNK_BAD;
         } else
           this->audio_frame_count +=
             (opcode_size - 6) / this->wave.nChannels;
@@ -356,23 +359,27 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
             this->audio_fifo->put (this->audio_fifo, buf);
           }
         }else{
-          this->input->seek(this->input, opcode_size, SEEK_CUR);
+          if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+            return CHUNK_BAD;
         }
         break;
 
       case OPCODE_SILENCE_FRAME:
         lprintf("silence frame\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_INIT_VIDEO_MODE:
         lprintf("initialize video mode\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_CREATE_GRADIENT:
         lprintf("create gradient\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_SET_PALETTE:
@@ -411,13 +418,14 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
 
       case OPCODE_SET_PALETTE_COMPRESSED:
         lprintf("set palette compressed\n");
-        this->input->seek(this->input, opcode_size, SEEK_CUR);
+        if (this->input->seek(this->input, opcode_size, SEEK_CUR) < 0)
+          return CHUNK_BAD;
         break;
 
       case OPCODE_SET_DECODING_MAP:
         lprintf("set decoding map\n");
 
-        current_file_pos = this->input->get_current_pos(this->input);
+        //current_file_pos = this->input->get_current_pos(this->input);
         lprintf("sending decoding map along with duration %d\n",
           this->frame_pts_inc);
 
@@ -536,7 +544,8 @@ static int open_ipmovie_file(demux_ipmovie_t *this) {
     return 0;
 
   /* file is qualified; skip over the signature bytes (+ 6 unknown) in the stream */
-  this->input->seek(this->input, IPMOVIE_SIGNATURE_SIZE+6, SEEK_SET);
+  if (this->input->seek(this->input, IPMOVIE_SIGNATURE_SIZE+6, SEEK_SET) < 0)
+    return 0;
 
   /* process the first chunk which should be CHUNK_INIT_VIDEO */
   if (process_ipmovie_chunk(this) != CHUNK_INIT_VIDEO)
@@ -629,6 +638,9 @@ static int demux_ipmovie_seek (demux_plugin_t *this_gen,
 
   demux_ipmovie_t *this = (demux_ipmovie_t *) this_gen;
 
+  (void)start_pos;
+  (void)start_time;
+
   /* if thread is not running, initialize demuxer */
   if( !playing ) {
 
@@ -655,15 +667,20 @@ static int demux_ipmovie_get_status (demux_plugin_t *this_gen) {
 }
 
 static int demux_ipmovie_get_stream_length (demux_plugin_t *this_gen) {
+  (void)this_gen;
   return 0;
 }
 
 static uint32_t demux_ipmovie_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_ipmovie_get_optional_data(demux_plugin_t *this_gen,
                                            void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -672,10 +689,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_ipmovie_t    *this;
 
-  this         = calloc(1, sizeof(demux_ipmovie_t));
-  if (!this) {
+  this = calloc(1, sizeof(demux_ipmovie_t));
+  if (!this)
     return NULL;
-  }
+
   this->stream = stream;
   this->input  = input;
 
@@ -713,19 +730,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_ipmovie_init_plugin (xine_t *xine, void *data) {
-  demux_ipmovie_class_t     *this;
+void *demux_ipmovie_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_ipmovie_class_t));
-  if (!this) {
-    return NULL;
-  }
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Interplay MVE Movie demux plugin");
-  this->demux_class.identifier      = "Interplay MVE";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "mve mv8";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  (void)xine;
+  (void)data;
 
-  return this;
+  static const demux_class_t demux_ipmovie_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Interplay MVE Movie demux plugin"),
+    .identifier      = "Interplay MVE",
+    .mimetypes       = NULL,
+    .extensions      = "mve mv8",
+    .dispose         = NULL,
+  };
+
+  return (void *)&demux_ipmovie_class;
 }
