@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -54,6 +54,9 @@
 #define PTS_AUDIO 0
 #define PTS_VIDEO 1
 
+/* disabled for now.
+#define PTS_BOUNCE
+*/
 
 /* redefine abs as macro to handle 64-bit diffs.
    i guess llabs may not be available everywhere */
@@ -67,16 +70,19 @@ typedef struct demux_mpeg_pes_s {
   fifo_buffer_t        *video_fifo;
 
   input_plugin_t       *input;
+  int                   is_vdr;
 
   int                   status;
 
   int                   rate;
 
-  uint8_t              *scratch;
-
   int64_t               nav_last_end_pts;
   int64_t               nav_last_start_pts;
   int64_t               last_pts[2];
+#ifdef PTS_BOUNCE
+  int64_t               apts, bpts;
+  int32_t               bounce_left;
+#endif
   int64_t               scr;
   uint32_t              packet_len;
   uint32_t              stream_id;
@@ -99,16 +105,6 @@ typedef struct demux_mpeg_pes_s {
   off_t                 preview_size, preview_done;
 } demux_mpeg_pes_t ;
 
-typedef struct {
-
-  demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
-} demux_mpeg_pes_class_t;
-
 static int32_t parse_video_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf);
 static int32_t parse_audio_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf);
 static int32_t parse_ancillary_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf);
@@ -132,6 +128,61 @@ static int32_t parse_IEC14496_FlexMux_stream(demux_mpeg_pes_t *this, uint8_t *p,
 static int32_t parse_program_stream_directory(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf);
 static int32_t parse_program_stream_pack_header(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf);
 
+#ifdef PTS_BOUNCE
+#define ts_abs(x) (((x) < 0) ? -(x) : (x))
+/* bounce detection like in metronom. looks like double work but saves
+ * up to 80 discontinuity messages. */
+static void check_newpts (demux_mpeg_pes_t *this, int64_t pts, int video) {
+#ifdef TS_LOG
+  printf ("demux_mpeg_pes: check_newpts %lld, send_newpts %d, buf_flag_seek %d\n",
+    pts, this->send_newpts, this->buf_flag_seek);
+#endif
+/*if (pts)*/
+  {
+    int64_t diff;
+    do {
+      this->last_pts[video] = pts;
+      if (!this->apts) {
+        diff = 0;
+        this->apts = pts;
+        break;
+      }
+      diff = pts - this->apts;
+      if (ts_abs (diff) <= WRAP_THRESHOLD) {
+        this->apts = pts;
+        break;
+      }
+      if (this->bpts) {
+        diff = pts - this->bpts;
+        if (ts_abs (diff) <= WRAP_THRESHOLD) {
+          this->bpts = pts;
+          break;
+        }
+      }
+      this->bpts = this->apts;
+      this->apts = pts;
+      this->bounce_left = WRAP_THRESHOLD;
+      _x_demux_control_newpts (this->stream, pts, this->buf_flag_seek ? BUF_FLAG_SEEK : 0);
+      this->send_newpts = 0;
+      this->buf_flag_seek = 0;
+      return;
+    } while (0);
+    if (this->bounce_left) {
+      this->bounce_left -= diff;
+      if (this->bounce_left <= 0) {
+        this->bpts = 0;
+        this->bounce_left = 0;
+      }
+    }
+    if (this->send_newpts || this->buf_flag_seek) {
+      _x_demux_control_newpts (this->stream, pts, this->buf_flag_seek ? BUF_FLAG_SEEK : 0);
+      this->send_newpts = 0;
+      this->buf_flag_seek = 0;
+    }
+  }
+}
+#else /* !PTS_BOUNCE */
+
 static int detect_pts_discontinuity( demux_mpeg_pes_t *this, int64_t pts, int video )
 {
   int64_t diff;
@@ -229,6 +280,7 @@ static void check_newpts( demux_mpeg_pes_t *this, int64_t pts, int video )
   if( pts )
     this->last_pts[video] = pts;
 }
+#endif
 
 static off_t read_data(demux_mpeg_pes_t *this, uint8_t *buf, off_t nlen)
 {
@@ -342,7 +394,7 @@ static void demux_mpeg_pes_parse_pack (demux_mpeg_pes_t *this, int preview_mode)
   this->packet_len = p[4] << 8 | p[5];
   lprintf("stream_id=0x%x, packet_len=%d\n",this->stream_id, this->packet_len);
 
-  if (this->packet_len <= (buf->max_size - 6)) {
+  if ((int)(this->packet_len) <= (buf->max_size - 6)) {
     i = read_data(this, buf->mem+6, (off_t) this->packet_len);
     if (i != this->packet_len) {
       buf->free_buffer (buf);
@@ -425,11 +477,32 @@ static void demux_mpeg_pes_parse_pack (demux_mpeg_pes_t *this, int preview_mode)
   return;
 }
 
+static void demux_mpeg_pes_vdr_seek_0 (demux_mpeg_pes_t *this, int n) {
+  xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+    "demux_mpeg_pes: vdr sync point #%d.\n", n);
+  this->last_cell_time = 0;
+  this->send_newpts = 1;
+  this->buf_flag_seek = 0;
+  this->nav_last_end_pts = this->nav_last_start_pts = 0;
+  this->status   = DEMUX_OK ;
+  this->last_pts[0]   = 0;
+  this->last_pts[1]   = 0;
+#ifdef PTS_BOUNCE
+  this->apts = this->bpts = 0;
+  this->bounce_left = 0;
+#endif
+}
+
 static int32_t parse_padding_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* Just skip padding. */
   int todo = 6 + this->packet_len;
   int done = buf->size;
 
+  (void)p;
+
+  if (this->is_vdr && (buf->content[4] == 0xff))
+    demux_mpeg_pes_vdr_seek_0 (this, buf->content[5]);
+
   while (done < todo)
   {
     /* Handle Jumbo frames from VDR. */
@@ -455,6 +528,7 @@ static int32_t parse_padding_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elem
 
 static int32_t parse_program_stream_map(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x.\n", this->stream_id);
   buf->free_buffer (buf);
@@ -462,6 +536,7 @@ static int32_t parse_program_stream_map(demux_mpeg_pes_t *this, uint8_t *p, buf_
 }
 static int32_t parse_ecm_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -469,6 +544,7 @@ static int32_t parse_ecm_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_
 }
 static int32_t parse_emm_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -476,6 +552,7 @@ static int32_t parse_emm_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_
 }
 static int32_t parse_dsmcc_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -483,6 +560,7 @@ static int32_t parse_dsmcc_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elemen
 }
 static int32_t parse_iec_13522_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -490,6 +568,7 @@ static int32_t parse_iec_13522_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_el
 }
 static int32_t parse_h222_typeA_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -497,6 +576,7 @@ static int32_t parse_h222_typeA_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_e
 }
 static int32_t parse_h222_typeB_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -504,6 +584,7 @@ static int32_t parse_h222_typeB_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_e
 }
 static int32_t parse_h222_typeC_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -511,6 +592,7 @@ static int32_t parse_h222_typeC_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_e
 }
 static int32_t parse_h222_typeD_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -518,6 +600,7 @@ static int32_t parse_h222_typeD_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_e
 }
 static int32_t parse_h222_typeE_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -525,6 +608,7 @@ static int32_t parse_h222_typeE_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_e
 }
 static int32_t parse_IEC14496_SL_packetized_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -532,6 +616,7 @@ static int32_t parse_IEC14496_SL_packetized_stream(demux_mpeg_pes_t *this, uint8
 }
 static int32_t parse_IEC14496_FlexMux_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -539,6 +624,7 @@ static int32_t parse_IEC14496_FlexMux_stream(demux_mpeg_pes_t *this, uint8_t *p,
 }
 static int32_t parse_program_stream_directory(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -546,6 +632,7 @@ static int32_t parse_program_stream_directory(demux_mpeg_pes_t *this, uint8_t *p
 }
 static int32_t parse_ancillary_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* FIXME: Implement */
+  (void)p;
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
           "xine-lib:demux_mpeg_pes: Unhandled stream_id 0x%02x\n", this->stream_id);
   buf->free_buffer (buf);
@@ -637,6 +724,7 @@ static int32_t parse_program_stream_pack_header(demux_mpeg_pes_t *this, uint8_t
 static int32_t parse_program_stream_system_header(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf) {
   /* program stream system header */
   /* FIXME: Implement */
+  (void)p;
   buf->free_buffer (buf);
   return 6 + this->packet_len;
 }
@@ -1074,16 +1162,24 @@ static int32_t parse_private_stream_1(demux_mpeg_pes_t *this, uint8_t *p, buf_el
       if( !this->preview_mode )
         check_newpts( this, this->pts, PTS_AUDIO );
 
-      this->audio_fifo->put (this->audio_fifo, buf);
-      lprintf ("A52 PACK put on fifo\n");
+      if (this->audio_fifo) {
+        this->audio_fifo->put (this->audio_fifo, buf);
+        lprintf ("A52 PACK put on fifo\n");
+      } else {
+        buf->free_buffer(buf);
+        /* skip tail */
+        if (this->input->seek (this->input, this->packet_len - size, SEEK_CUR) < 0)
+          return -1;
+        return this->packet_len + result;
+      }
 
-      if (size == this->packet_len) {
+      if (size == (int)(this->packet_len)) {
         return this->packet_len + result;
       }
 
       /* Handle Jumbo A52 frames from VDR. */
       offset = size;
-      while (offset < this->packet_len) {
+      while (offset < (int)(this->packet_len)) {
         int i;
         buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
         size = this->packet_len - offset;
@@ -1100,12 +1196,8 @@ static int32_t parse_private_stream_1(demux_mpeg_pes_t *this, uint8_t *p, buf_el
         buf->type      = BUF_AUDIO_A52;
         buf->pts       = 0;
 
-        if(this->audio_fifo) {
-          this->audio_fifo->put (this->audio_fifo, buf);
-          lprintf ("A52 PACK put on fifo\n");
-        } else {
-          buf->free_buffer(buf);
-        }
+        this->audio_fifo->put (this->audio_fifo, buf);
+        lprintf ("A52 PACK put on fifo\n");
       }
 
       return this->packet_len + result;
@@ -1137,7 +1229,7 @@ static int32_t parse_video_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elemen
 
   buf->content = p;
   payload_size = buf->max_size - result;
-  if (payload_size > this->packet_len)
+  if (payload_size > (int)(this->packet_len))
     payload_size = this->packet_len;
 
   /* H.264 broadcasts via DVB-S use standard video PES packets,
@@ -1167,7 +1259,7 @@ static int32_t parse_video_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elemen
    */
   if (this->mpeg12_h264_detected < 2) {
     uint8_t *pp = p + 2, *pp_limit = p + payload_size - 1;
-    while (0 < pp && pp < pp_limit) {
+    while (pp && pp < pp_limit) {
       if (pp[0] == 0x01 && pp[-1] == 0x00 && pp[-2] == 0x00) {
         if (pp[1] >= 0x80 || !pp[1]) { /* MPEG 1/2 start code */
           this->mpeg12_h264_detected = 2;
@@ -1211,7 +1303,7 @@ static int32_t parse_video_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elemen
     }
   }
 
-  if (this->packet_len <= (buf->max_size - result)) {
+  if ((int)(this->packet_len) <= (buf->max_size - result)) {
     buf->size = this->packet_len;
     /* VDR ensures that H.264 still images end with an end of sequence NAL unit. We
        need to detect this to inform the decoder that the current frame is complete.
@@ -1234,7 +1326,7 @@ static int32_t parse_video_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elemen
   this->video_fifo->put (this->video_fifo, buf);
   while (todo_length > 0) {
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    if (todo_length < buf->max_size) {
+    if ((int)todo_length < buf->max_size) {
       chunk_length = todo_length;
     } else {
       chunk_length = buf->max_size;
@@ -1354,7 +1446,8 @@ static int demux_mpeg_pes_estimate_rate (demux_mpeg_pes_t *this) {
      they should continue to be so throughout.
    */
 
-  this->input->seek (this->input, pos, SEEK_SET);
+  if (this->input->seek (this->input, pos, SEEK_SET) != pos)
+    return 0;
 
   while ( (buf = this->input->read_block (this->input, this->video_fifo, 2048))
           && count < MAX_SAMPLES && reads++ < MAX_READS ) {
@@ -1467,14 +1560,6 @@ static int demux_mpeg_pes_estimate_rate (demux_mpeg_pes_t *this) {
 }
 #endif /*ESTIMATE_RATE_FIXED*/
 
-static void demux_mpeg_pes_dispose (demux_plugin_t *this_gen) {
-
-  demux_mpeg_pes_t *this = (demux_mpeg_pes_t *) this_gen;
-
-  xine_free_aligned (this->scratch);
-  free (this);
-}
-
 static int demux_mpeg_pes_get_status (demux_plugin_t *this_gen) {
   demux_mpeg_pes_t *this = (demux_mpeg_pes_t *) this_gen;
 
@@ -1506,7 +1591,10 @@ static void demux_mpeg_pes_send_headers (demux_plugin_t *this_gen) {
 
     int num_buffers = NUM_PREVIEW_BUFFERS;
 
-    this->input->seek (this->input, 0, SEEK_SET);
+    if (this->input->seek (this->input, 0, SEEK_SET) != 0) {
+      this->status = DEMUX_FINISHED;
+      return;
+    }
 
     this->status = DEMUX_OK ;
     while ( (num_buffers>0) && (this->status == DEMUX_OK) ) {
@@ -1582,6 +1670,10 @@ static int demux_mpeg_pes_seek (demux_plugin_t *this_gen,
     this->status   = DEMUX_OK ;
     this->last_pts[0]   = 0;
     this->last_pts[1]   = 0;
+#ifdef PTS_BOUNCE
+    this->apts = this->bpts = 0;
+    this->bounce_left = 0;
+#endif
   } else {
     this->buf_flag_seek = 1;
     this->nav_last_end_pts = this->nav_last_start_pts = 0;
@@ -1609,128 +1701,53 @@ static int demux_mpeg_pes_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_mpeg_pes_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_mpeg_pes_get_optional_data(demux_plugin_t *this_gen,
                                         void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                   input_plugin_t *input_gen) {
-
-  input_plugin_t     *input = (input_plugin_t *) input_gen;
+                                   input_plugin_t *input) {
   demux_mpeg_pes_t *this;
 
-  this         = calloc(1, sizeof(demux_mpeg_pes_t));
-  this->stream = stream;
-  this->input  = input;
-
-  this->demux_plugin.send_headers      = demux_mpeg_pes_send_headers;
-  this->demux_plugin.send_chunk        = demux_mpeg_pes_send_chunk;
-  this->demux_plugin.seek              = demux_mpeg_pes_seek;
-  this->demux_plugin.dispose           = demux_mpeg_pes_dispose;
-  this->demux_plugin.get_status        = demux_mpeg_pes_get_status;
-  this->demux_plugin.get_stream_length = demux_mpeg_pes_get_stream_length;
-  this->demux_plugin.get_capabilities  = demux_mpeg_pes_get_capabilities;
-  this->demux_plugin.get_optional_data = demux_mpeg_pes_get_optional_data;
-  this->demux_plugin.demux_class       = class_gen;
-
-  this->scratch    = xine_mallocz_aligned (4096);
-  this->status     = DEMUX_FINISHED;
-  /* Don't start demuxing stream until we see a program_stream_pack_header */
-  /* We need to system header in order to identify is the stream is mpeg1 or mpeg2. */
-  this->wait_for_program_stream_pack_header = 1;
-  /* trigger detection of MPEG 1/2 respectively H.264 content */
-  this->mpeg12_h264_detected = 0;
-
-  this->preview_size = 0;
-
   lprintf ("open_plugin:detection_method=%d\n",
            stream->content_detection_method);
 
   switch (stream->content_detection_method) {
 
   case METHOD_BY_CONTENT: {
+    uint8_t buf[6];
 
     /* use demux_mpeg_block for block devices */
     if ((input->get_capabilities(input) & INPUT_CAP_BLOCK)) {
-      xine_free_aligned (this->scratch);
-      free (this);
       return NULL;
     }
 
-    if (((input->get_capabilities(input) & INPUT_CAP_PREVIEW) != 0) ) {
-
-      int preview_size = input->get_optional_data(input, &this->preview_data, INPUT_OPTIONAL_DATA_PREVIEW);
-
-      if (preview_size >= 6) {
-        lprintf("open_plugin:get_optional_data worked\n");
-
-        if (this->preview_data[0] || this->preview_data[1]
-            || (this->preview_data[2] != 0x01) ) {
-          lprintf("open_plugin:preview_data failed\n");
-
-          xine_free_aligned (this->scratch);
-          free (this);
-          return NULL;
-        }
-        switch(this->preview_data[3]) {
-
-        case 0xe0 ... 0xef:
-        case 0xc0 ... 0xdf:
-        case 0xbd ... 0xbe:
-          break;
-        default:
-          xine_free_aligned (this->scratch);
-          free (this);
-          return NULL;
-        }
-
-        lprintf("open_plugin:Accepting detection_method XINE_DEMUX_CONTENT_STRATEGY (preview_data)\n");
-
-        break;
-      }
+    if (_x_demux_read_header(input, buf, sizeof(buf)) != 6) {
+      return NULL;
     }
 
-    if (((input->get_capabilities(input) & INPUT_CAP_SEEKABLE) != 0) ) {
-
-      input->seek(input, 0, SEEK_SET);
-      if (input->read(input, (char *)this->scratch, 6) == 6) {
-        lprintf("open_plugin:read worked\n");
-
-        if (this->scratch[0] || this->scratch[1]
-            || (this->scratch[2] != 0x01) ) {
-          lprintf("open_plugin:scratch failed\n");
-
-          xine_free_aligned (this->scratch);
-          free (this);
-          return NULL;
-        }
-        switch(this->scratch[3]) {
-
-        case 0xe0 ... 0xef:
-        case 0xc0 ... 0xdf:
-        case 0xbd ... 0xbe:
-          break;
-        default:
-          xine_free_aligned (this->scratch);
-          free (this);
-          return NULL;
-        }
-
-        input->seek(input, 0, SEEK_SET);
-
-        lprintf("open_plugin:Accepting detection_method XINE_DEMUX_CONTENT_STRATEGY \n");
-
+    if (buf[0] || buf[1] || (buf[2] != 0x01)) {
+      lprintf("open_plugin:preview_data failed\n");
+      return NULL;
+    }
+    switch (buf[3]) {
+      case 0xe0 ... 0xef:
+      case 0xc0 ... 0xdf:
+      case 0xbd ... 0xbe:
         break;
-      }
+      default:
+        return NULL;
     }
 
-    xine_free_aligned (this->scratch);
-    free (this);
-    return NULL;
+    lprintf("open_plugin:Accepting detection_method XINE_DEMUX_CONTENT_STRATEGY (preview_data)\n");
   }
   break;
 
@@ -1741,28 +1758,56 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     break;
 
   default:
-    xine_free_aligned (this->scratch);
-    free (this);
     return NULL;
   }
+
+  this = calloc(1, sizeof(demux_mpeg_pes_t));
+  if (!this)
+    return NULL;
+
+  /* trigger detection of MPEG 1/2 respectively H.264 content */
+  this->mpeg12_h264_detected = 0;
+  this->preview_size = 0;
+  this->stream = stream;
+  this->input  = input;
+  this->status = DEMUX_FINISHED;
+
+  this->is_vdr = 0;
+  if (input->input_class->identifier && !strcmp (input->input_class->identifier, "VDR"))
+    this->is_vdr = 1;
+
+  /* Don't start demuxing stream until we see a program_stream_pack_header */
+  /* We need to system header in order to identify is the stream is mpeg1 or mpeg2. */
+  this->wait_for_program_stream_pack_header = 1;
+
+  this->demux_plugin.send_headers      = demux_mpeg_pes_send_headers;
+  this->demux_plugin.send_chunk        = demux_mpeg_pes_send_chunk;
+  this->demux_plugin.seek              = demux_mpeg_pes_seek;
+  this->demux_plugin.dispose           = default_demux_plugin_dispose;
+  this->demux_plugin.get_status        = demux_mpeg_pes_get_status;
+  this->demux_plugin.get_stream_length = demux_mpeg_pes_get_stream_length;
+  this->demux_plugin.get_capabilities  = demux_mpeg_pes_get_capabilities;
+  this->demux_plugin.get_optional_data = demux_mpeg_pes_get_optional_data;
+  this->demux_plugin.demux_class       = class_gen;
+
   return &this->demux_plugin;
 }
 
-void *demux_pes_init_class (xine_t *xine, void *data) {
+void *demux_pes_init_class (xine_t *xine, const void *data) {
 
-  demux_mpeg_pes_class_t     *this;
-  this         = calloc(1, sizeof(demux_mpeg_pes_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("mpeg pes demux plugin");
-  this->demux_class.identifier      = "MPEG_PES";
-  this->demux_class.mimetypes       = "video/mp2p: m2p: MPEG2 program stream;";
-  this->demux_class.extensions      = "pes vdr:/ netvdr:/";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_mpeg_pes_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("mpeg pes demux plugin"),
+    .identifier      = "MPEG_PES",
+    .mimetypes       = "video/mp2p: m2p: MPEG2 program stream;",
+    .extensions      = "pes vdr:/ netvdr:/",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_mpeg_pes_class;
 }
 
 
