@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -148,7 +148,7 @@ typedef struct {
   xing_header_t       *xing_header;
   vbri_header_t       *vbri_header;
 
-  int                  found_next_frame:1;
+  unsigned int         found_next_frame : 1;
   int                  free_bitrate_count;
   off_t                free_bitrate_size; /* use this size if 3 free bitrate frames are encountered */
   uint8_t              next_header[4];
@@ -444,6 +444,8 @@ static vbri_header_t *XINE_MALLOC parse_vbri_header(mpg_audio_frame_t *frame,
   uint8_t *ptr = buf;
   vbri_header_t *vbri;
 
+  (void)frame;
+
   ptr += (32 + 4);
 
   if ((ptr + 4) >= (buf + bufsize)) return 0;
@@ -542,7 +544,7 @@ static int parse_frame_payload(demux_mpgaudio_t *this,
 
   buf = this->audio_fifo->buffer_pool_alloc(this->audio_fifo);
 
-  if (this->cur_frame.size > buf->max_size) {
+  if ((int)(this->cur_frame.size) > buf->max_size) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
             LOG_MODULE ": frame size is greater than fifo buffer size\n");
     buf->free_buffer(buf);
@@ -685,7 +687,7 @@ static int sniff_buffer_looks_like_mp3 (uint8_t *buf, int buflen, int *version,
         /* Since one frame is available, is there another frame
          * just to be sure this is more likely to be a real MP3
          * buffer? */
-        if (offset + size + 4 >= buflen) {
+        if (offset + (int)size + 4 >= buflen) {
           return 0;
         }
 
@@ -830,30 +832,6 @@ static int demux_mpgaudio_get_status (demux_plugin_t *this_gen) {
   return this->status;
 }
 
-static int demux_mpgaudio_read_head(input_plugin_t *input, uint8_t *buf) {
-
-  int       bs = 0;
-
-  if (INPUT_IS_SEEKABLE(input)) {
-    input->seek(input, 0, SEEK_SET);
-
-    bs = input->read(input, buf, MAX_PREVIEW_SIZE);
-
-    lprintf("stream is seekable\n");
-
-  } else if ((input->get_capabilities(input) & INPUT_CAP_PREVIEW) != 0) {
-
-    lprintf("input plugin provides preview\n");
-
-    bs = input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW);
-
-  } else {
-    lprintf("not seekable, no preview\n");
-    return 0;
-  }
-  return bs;
-}
-
 /*
  * mp3 stream detection
  * return 1 if detected, 0 otherwise
@@ -865,7 +843,7 @@ static int detect_mpgaudio_file(input_plugin_t *input,
   uint32_t head;
 
   *version = *layer = 0;
-  preview_len = demux_mpgaudio_read_head(input, buf);
+  preview_len = _x_demux_read_header(input, buf, sizeof(buf));
   if (preview_len < 4)
     return 0;
 
@@ -884,7 +862,8 @@ static int detect_mpgaudio_file(input_plugin_t *input,
      * try to skip by seeking.
      */
     if (INPUT_IS_SEEKABLE (input)) {
-      input->seek (input, 10 + tag_size, SEEK_SET);
+      if (input->seek (input, 10 + tag_size, SEEK_SET) < 0)
+        return 0;
       preview_len = input->read (input, buf, MAX_PREVIEW_SIZE);
       if (!sniff_buffer_looks_like_mp3 (buf, preview_len, version, layer)) {
         lprintf ("sniff_buffer_looks_like_mp3 failed\n");
@@ -942,7 +921,10 @@ static void demux_mpgaudio_send_headers (demux_plugin_t *this_gen) {
     }
 
     /* seek back to the beginning */
-    this->input->seek (this->input, 0, SEEK_SET);
+    if (this->input->seek (this->input, 0, SEEK_SET) != 0) {
+      this->status = DEMUX_FINISHED;
+      return;
+    }
 
     /*
      * send preview buffers
@@ -1116,7 +1098,7 @@ static int demux_mpgaudio_seek (demux_plugin_t *this_gen,
 
     if (start_time < 0)
       start_time = 0;
-    if (start_time > this->stream_length)
+    if ((unsigned int)start_time > this->stream_length)
       start_time = this->stream_length;
 
     if (this->stream_length > 0) {
@@ -1160,11 +1142,15 @@ static int demux_mpgaudio_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_mpgaudio_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_mpgaudio_get_optional_data(demux_plugin_t *this_gen,
 					void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -1232,19 +1218,16 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 /*
  * demux mpegaudio class
  */
-void *demux_mpgaudio_init_class (xine_t *xine, void *data) {
+void *demux_mpgaudio_init_class (xine_t *xine, const void *data) {
 
-  demux_class_t     *this;
+  (void)xine;
+  (void)data;
 
-  this         = calloc(1, sizeof(demux_class_t));
-  if (!this)
-    return NULL;
-
-  this->open_plugin     = open_plugin;
-  this->description     = N_("MPEG audio demux plugin");
-  this->identifier      = "MPEGAUDIO";
-  if( _x_decoder_available(xine, BUF_AUDIO_MPEG) ) {
-    this->mimetypes =
+  static const demux_class_t demux_mpgaudio_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("MPEG audio demux plugin"),
+    .identifier      = "MPEGAUDIO",
+    .mimetypes =
       "audio/mpeg2: mp2: MPEG audio;"
       "audio/x-mpeg2: mp2: MPEG audio;"
       "audio/mpeg3: mp3: MPEG audio;"
@@ -1254,13 +1237,10 @@ void *demux_mpgaudio_init_class (xine_t *xine, void *data) {
       "audio/x-mpegurl: mp3: MPEG audio;"
       "audio/mpegurl: mp3: MPEG audio;"
       "audio/mp3: mp3: MPEG audio;"
-      "audio/x-mp3: mp3: MPEG audio;";
-    this->extensions    = "mp3 mp2 mpa mpega";
-  } else {
-    this->mimetypes     = NULL;
-    this->extensions    = NULL;
-  }
-  this->dispose         = default_demux_class_dispose;
+      "audio/x-mp3: mp3: MPEG audio;",
+    .extensions    = "mp3 mp2 mpa mpega",
+    .dispose       = NULL,
+  };
 
-  return this;
+  return (void *)&demux_mpgaudio_class;
 }
