@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2017 the xine project
+ * Copyright (C) 2007-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -70,10 +70,6 @@ typedef struct {
   int                status;
 } demux_playlist_t;
 
-typedef struct {
-  demux_class_t      demux_class;
-} demux_playlist_class_t;
-
 
 static playlist_t detect_by_extension (input_plugin_t *input) {
   char *ext;
@@ -203,6 +199,8 @@ static void parse_ref (demux_playlist_t *this, char *data, int length) {
   char *end;
   int   alt = 0;
 
+  (void)length; /* FIXME: test */
+
   while (src && *src) {
     end = strchr (src, '\n');
     if (end)
@@ -228,6 +226,8 @@ static void parse_m3u (demux_playlist_t *this, char *data, int length) {
   char *end;
   char *title = NULL;
 
+  (void)length; /* FIXME: test */
+
   while (src && *src) {
     end = strchr (src, '\n');
     if (end)
@@ -256,6 +256,8 @@ static void parse_ram (demux_playlist_t *this, char *data, int length) {
   char *src = data;
   char *end;
 
+  (void)length; /* FIXME: test */
+
   while (src && *src) {
     end = strchr (src, '\n');
     if (end)
@@ -296,6 +298,8 @@ static void parse_pls (demux_playlist_t *this, char *data, int length) {
   char *src = data;
   char *end;
 
+  (void)length; /* FIXME: test */
+
   while (src && *src) {
     end = strchr (src, '\n');
     if (end)
@@ -562,7 +566,8 @@ static void demux_playlist_send_headers (demux_plugin_t *this_gen) {
 
   _x_demux_control_start (this->stream);
 
-  this->input->seek (this->input, 0, SEEK_SET);
+  if (this->input->seek (this->input, 0, SEEK_SET) != 0)
+    this->status = DEMUX_FINISHED;
 }
 
 
@@ -573,9 +578,13 @@ static int demux_playlist_send_chunk (demux_plugin_t *this_gen) {
 
   length = this->input->get_length (this->input);
   if (length > 0) {
-    data = calloc(1, length+1);
-    if (data)
-      this->input->read (this->input, data, length);
+    data = malloc (length + 1);
+    if (data) {
+      length = this->input->read (this->input, data, length);
+      if (length < 0)
+        length = 0;
+      data[length] = '\0';
+    }
   }
   else {
     char buf[1024];
@@ -583,10 +592,10 @@ static int demux_playlist_send_chunk (demux_plugin_t *this_gen) {
 
     length = 0;
     while ((len = this->input->read (this->input, buf, sizeof(buf))) > 0) {
-      data = realloc (data, length+len+1);
-      if (!data)
+      void *tmp = realloc (data, length+len+1);
+      if (!tmp)
         break;
-
+      data = tmp;
       memcpy (data+length, buf, len);
       length += len;
       data[length] = '\0';
@@ -639,6 +648,10 @@ static int demux_playlist_send_chunk (demux_plugin_t *this_gen) {
 
 static int demux_playlist_seek (demux_plugin_t *this_gen,
                                 off_t start_pos, int start_time, int playing) {
+  (void)this_gen;
+  (void)start_pos;
+  (void)start_time;
+  (void)playing;
   return DEMUX_OK;
 }
 
@@ -649,15 +662,20 @@ static int demux_playlist_get_status (demux_plugin_t *this_gen) {
 }
 
 static int demux_playlist_get_stream_length (demux_plugin_t *this_gen) {
+  (void)this_gen;
   return 0;
 }
 
 static uint32_t demux_playlist_get_capabilities (demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_playlist_get_optional_data (demux_plugin_t *this_gen,
                                              void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -665,15 +683,35 @@ static int demux_playlist_get_optional_data (demux_plugin_t *this_gen,
 static demux_plugin_t *open_plugin (demux_class_t *class_gen,
                                     xine_stream_t *stream, input_plugin_t *input) {
   demux_playlist_t *this;
+  playlist_t playlist;
+
+  switch (stream->content_detection_method) {
+    case METHOD_BY_MRL:
+      lprintf ("detect by extension\n");
+      playlist = detect_by_extension (input);
+      break;
+
+    case METHOD_BY_CONTENT:
+    case METHOD_EXPLICIT:
+      lprintf ("detect by content\n");
+      playlist = detect_by_content (input);
+      break;
+
+    default:
+      return NULL;
+  }
+
+  if (playlist == XINE_PLT_NONE)
+    return NULL;
 
   this = calloc (1, sizeof (demux_playlist_t));
-  if (!this) {
+  if (!this)
     return NULL;
-  }
 
   this->xine   = stream->xine;
   this->stream = stream;
   this->input  = input;
+  this->playlist = playlist;
 
   this->demux_plugin.send_headers      = demux_playlist_send_headers;
   this->demux_plugin.send_chunk        = demux_playlist_send_chunk;
@@ -685,73 +723,47 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   this->demux_plugin.get_optional_data = demux_playlist_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
 
-  switch (stream->content_detection_method) {
-    case METHOD_BY_MRL:
-      lprintf ("detect by extension\n");
-      this->playlist = detect_by_extension (input);
-      if (!this->playlist) {
-        free (this);
-        return NULL;
-      }
-      break;
-
-    case METHOD_BY_CONTENT:
-    case METHOD_EXPLICIT:
-      lprintf ("detect by content\n");
-      this->playlist = detect_by_content (input);
-      if (!this->playlist) {
-        free (this);
-        return NULL;
-      }
-      break;
-
-    default:
-      free (this);
-      return NULL;
-  }
-
   lprintf ("playlist:0x%08x (%s)\n", this->playlist, (char*)&this->playlist);
 
   return &this->demux_plugin;
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
-  demux_playlist_class_t     *this;
+static void *init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_playlist_class_t));
-  if (!this) {
-    return NULL;
-  }
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Playlist demux plugin");
-  this->demux_class.identifier      = "playlist";
-  this->demux_class.mimetypes       =
-    "audio/mpegurl: m3u: M3U playlist;"
-    "audio/x-mpegurl: m3u: M3U playlist;"
-    //"audio/x-pn-realaudio: ram: RAM playlist;"
-    //"audio/vnd.rn-realaudio: ram: RAM playlist;"
-    "audio/x-scpls: pls: Winamp playlist;"
-    "audio/x-ms-wax: wax, asx: WAX playlist;"
-    "audio/x-ms-wvx: wvx, asx: WVX playlist;"
-    "application/smil: smi, smil: SMIL playlist;"
-    "application/x-quicktimeplayer: qtl: Quicktime playlist;"
-    "application/xspf+xml: xspf: XSPF playlist;";
-  this->demux_class.extensions      = "m3u ram pls asx wax wvx smi smil qtl xspf rss";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_playlist_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Playlist demux plugin"),
+    .identifier      = "playlist",
+    .mimetypes       =
+      "audio/mpegurl: m3u: M3U playlist;"
+      "audio/x-mpegurl: m3u: M3U playlist;"
+      //"audio/x-pn-realaudio: ram: RAM playlist;"
+      //"audio/vnd.rn-realaudio: ram: RAM playlist;"
+      "audio/x-scpls: pls: Winamp playlist;"
+      "audio/x-ms-wax: wax, asx: WAX playlist;"
+      "audio/x-ms-wvx: wvx, asx: WVX playlist;"
+      "application/smil: smi, smil: SMIL playlist;"
+      "application/x-quicktimeplayer: qtl: Quicktime playlist;"
+      "application/xspf+xml: xspf: XSPF playlist;",
+    .extensions      = "m3u ram pls asx wax wvx smi smil qtl xspf rss",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_playlist_class;
 }
 
 /*
  * exported plugin catalog entry
  */
-static const demuxer_info_t demux_info_flv = {
-  10                       /* priority */
+static const demuxer_info_t demux_info_playlist = {
+  .priority = 10,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "playlist", XINE_VERSION_CODE, &demux_info_flv, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_DEMUX, 27, "playlist", XINE_VERSION_CODE, &demux_info_playlist, init_plugin },
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
