@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 the xine project
+ * Copyright (C) 2005-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -46,7 +46,6 @@ typedef struct {
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-  fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
   input_plugin_t      *input;
   int                  status;
@@ -54,15 +53,11 @@ typedef struct {
   int                  seek_flag;  /* this is set when a seek just occurred */
 } demux_shn_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_shn_class_t;
 
-
-static int open_shn_file(demux_shn_t *this) {
+static int probe_shn_file(input_plugin_t *input) {
   uint8_t peak[4];
 
-  if (_x_demux_read_header(this->input, peak, 4) != 4)
+  if (_x_demux_read_header(input, peak, 4) != 4)
       return 0;
 
   if ((peak[0] != 'a') || (peak[1] != 'j') ||
@@ -107,7 +102,6 @@ static void demux_shn_send_headers(demux_plugin_t *this_gen) {
   demux_shn_t *this = (demux_shn_t *) this_gen;
   buf_element_t *buf;
 
-  this->video_fifo  = this->stream->video_fifo;
   this->audio_fifo  = this->stream->audio_fifo;
 
   this->status = DEMUX_OK;
@@ -138,6 +132,9 @@ static int demux_shn_seek (demux_plugin_t *this_gen,
                            off_t start_pos, int start_time, int playing) {
   demux_shn_t *this = (demux_shn_t *) this_gen;
 
+  (void)start_pos;
+  (void)start_time;
+
   /* if thread is not running, initialize demuxer */
   if( !playing ) {
 
@@ -159,15 +156,20 @@ static int demux_shn_get_status (demux_plugin_t *this_gen) {
 static int demux_shn_get_stream_length (demux_plugin_t *this_gen) {
 //  demux_shn_t *this = (demux_shn_t *) this_gen;
 
+  (void)this_gen;
   return 0;
 }
 
 static uint32_t demux_shn_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_shn_get_optional_data(demux_plugin_t *this_gen,
                                        void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -176,7 +178,22 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_shn_t    *this;
 
-  this         = calloc(1, sizeof(demux_shn_t));
+  switch (stream->content_detection_method) {
+    case METHOD_BY_MRL:
+    case METHOD_BY_CONTENT:
+    case METHOD_EXPLICIT:
+      if (!probe_shn_file(input)) {
+        return NULL;
+      }
+      break;
+    default:
+      return NULL;
+  }
+
+  this = calloc(1, sizeof(demux_shn_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -191,36 +208,23 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.demux_class       = class_gen;
 
   this->status = DEMUX_FINISHED;
-  switch (stream->content_detection_method) {
-
-  case METHOD_BY_MRL:
-  case METHOD_BY_CONTENT:
-  case METHOD_EXPLICIT:
-    if (!open_shn_file(this)) {
-      free (this);
-      return NULL;
-    }
-    break;
-
-  default:
-    free (this);
-    return NULL;
-  }
 
   return &this->demux_plugin;
 }
 
-void *demux_shn_init_plugin (xine_t *xine, void *data) {
-  demux_shn_class_t     *this;
+void *demux_shn_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_shn_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Shorten demux plugin");
-  this->demux_class.identifier      = "Shorten";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "shn";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_shn_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Shorten demux plugin"),
+    .identifier      = "Shorten",
+    .mimetypes       = NULL,
+    .extensions      = "shn",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_shn_class;
 }
