@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -66,6 +66,7 @@
 #include <xine/video_out.h>
 #include "dxr3.h"
 #include "video_out_dxr3.h"
+#include "group_dxr3.h"
 
 #ifdef HAVE_FFMPEG_AVUTIL_H
 #  include <mem.h>
@@ -73,44 +74,11 @@
 #  include <libavutil/mem.h>
 #endif
 
-#include "compat.c"
-
 /* the amount of extra time we give the card for decoding */
 #define DECODE_PIPE_PREBUFFER 10000
 
-
-/* plugin class initialization functions */
-static void                *dxr3_x11_init_plugin(xine_t *xine, void *visual_gen);
-static void                *dxr3_aa_init_plugin(xine_t *xine, void *visual_gen);
-static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen);
-
-
-/* plugin catalog information */
-#ifdef HAVE_X11
-static const vo_info_t   vo_info_dxr3_x11 = {
-  10,                  /* priority        */
-  XINE_VISUAL_TYPE_X11 /* visual type     */
-};
-#endif
-
-static const vo_info_t   vo_info_dxr3_aa = {
-  10,                  /* priority        */
-  XINE_VISUAL_TYPE_AA  /* visual type     */
-};
-
-const plugin_info_t      xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-#ifdef HAVE_X11
-  { PLUGIN_VIDEO_OUT, 22, "dxr3", XINE_VERSION_CODE, &vo_info_dxr3_x11, &dxr3_x11_init_plugin },
-#endif
-  { PLUGIN_VIDEO_OUT, 22, "aadxr3", XINE_VERSION_CODE, &vo_info_dxr3_aa, &dxr3_aa_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
-
 /* plugin class functions */
 static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const void *visual);
-static void         dxr3_vo_class_dispose(video_driver_class_t *class_gen);
 
 /* plugin instance functions */
 static uint32_t    dxr3_get_capabilities(vo_driver_t *this_gen);
@@ -152,30 +120,21 @@ static void        dxr3_update_swap_fields(void *data, xine_cfg_entry_t *entry);
 static void        dxr3_update_enhanced_mode(void *this_gen, xine_cfg_entry_t *entry);
 
 
-#ifdef HAVE_X11
-static void *dxr3_x11_init_plugin(xine_t *xine, void *visual_gen)
+int dxr3_present(xine_stream_t *stream)
 {
-  dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
+  int present = 0;
 
-  if (!this) return NULL;
-  this->visual_type = XINE_VISUAL_TYPE_X11;
-  return &this->video_driver_class;
-}
-#endif
-
-static void *dxr3_aa_init_plugin(xine_t *xine, void *visual_gen)
-{
-  dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
-
-  if (!this) return NULL;
-  this->visual_type = XINE_VISUAL_TYPE_AA;
-  return &this->video_driver_class;
+  if (stream->video_driver)
+    present = stream->video_driver->dispose == dxr3_dispose;
+  llprintf(LOG_VID, "dxr3 %s\n", present ? "present" : "not present");
+  return present;
 }
 
-static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen)
+static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, const void *visual_gen)
 {
   dxr3_driver_class_t *this;
 
+  (void)visual_gen;
   this = calloc(1, sizeof(dxr3_driver_class_t));
   if (!this) return NULL;
 
@@ -185,26 +144,34 @@ static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen)
   this->video_driver_class.open_plugin     = dxr3_vo_open_plugin;
   this->video_driver_class.identifier      = DXR3_VO_ID;
   this->video_driver_class.description     = N_("video output plugin displaying images through your DXR3 decoder card");
-  this->video_driver_class.dispose         = dxr3_vo_class_dispose;
+  this->video_driver_class.dispose         = default_video_driver_class_dispose;
 
   this->xine                               = xine;
 
   this->instance                           = 0;
 
-  this->scr                                = dxr3_scr_init(xine);
-
   return this;
 }
 
-static void dxr3_vo_class_dispose(video_driver_class_t *class_gen)
+#ifdef HAVE_X11
+void *dxr3_x11_init_plugin(xine_t *xine, const void *visual_gen)
 {
-  dxr3_driver_class_t *class = (dxr3_driver_class_t *)class_gen;
+  dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
 
-  if(class->scr)
-    class->scr->scr_plugin.exit(&class->scr->scr_plugin);
-  free(class_gen);
+  if (!this) return NULL;
+  this->visual_type = XINE_VISUAL_TYPE_X11;
+  return &this->video_driver_class;
 }
+#endif
 
+void *dxr3_aa_init_plugin(xine_t *xine, const void *visual_gen)
+{
+  dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
+
+  if (!this) return NULL;
+  this->visual_type = XINE_VISUAL_TYPE_AA;
+  return &this->video_driver_class;
+}
 
 static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const void *visual_gen)
 {
@@ -261,6 +228,8 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
   this->vo_driver.gui_data_exchange    = dxr3_gui_data_exchange;
   this->vo_driver.dispose              = dxr3_dispose;
 
+  this->scr                            = dxr3_scr_init(class->xine);
+
   pthread_mutex_init(&this->video_device_lock, NULL);
   pthread_mutex_init(&this->spu_device_lock, NULL);
 
@@ -524,6 +493,7 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
 
 static uint32_t dxr3_get_capabilities(vo_driver_t *this_gen)
 {
+  (void)this_gen;
   return VO_CAP_YV12 | VO_CAP_YUY2;
 }
 
@@ -577,6 +547,8 @@ static void dxr3_frame_proc_slice(vo_frame_t *frame_gen, uint8_t **src)
 static void dxr3_frame_field(vo_frame_t *vo_img, int which_field)
 {
   /* dummy function */
+  (void)vo_img;
+  (void)which_field;
 }
 
 static void dxr3_frame_dispose(vo_frame_t *frame_gen)
@@ -602,7 +574,7 @@ static void dxr3_update_frame_format(vo_driver_t *this_gen, vo_frame_t *frame_ge
     if (this->fd_video >= 0) {
       metronom_clock_t *clock = this->class->xine->clock;
 
-      clock->unregister_scr(clock, &this->class->scr->scr_plugin);
+      clock->unregister_scr(clock, &this->scr->scr_plugin);
       close(this->fd_video);
       this->fd_video = -1;
       /* inform the encoder on next frame's arrival */
@@ -655,8 +627,8 @@ static void dxr3_update_frame_format(vo_driver_t *this_gen, vo_frame_t *frame_ge
 
     /* start the scr plugin */
     time = clock->get_current_time(clock);
-    this->class->scr->scr_plugin.start(&this->class->scr->scr_plugin, time);
-    clock->register_scr(clock, &this->class->scr->scr_plugin);
+    this->scr->scr_plugin.start(&this->scr->scr_plugin, time);
+    clock->register_scr(clock, &this->scr->scr_plugin);
 
     this->scale.force_redraw = 1;
   }
@@ -708,7 +680,7 @@ static void dxr3_update_frame_format(vo_driver_t *this_gen, vo_frame_t *frame_ge
   }
 
   /* if dimensions changed, we need to re-allocate frame memory */
-  if ((frame->vo_frame.width != width) || (frame->vo_frame.height != height) ||
+  if ((frame->vo_frame.width != (int)width) || (frame->vo_frame.height != (int)height) ||
       (frame->oheight != oheight) || (frame->vo_frame.format != format)) {
     av_freep(&frame->mem);
 
@@ -943,7 +915,7 @@ static void dxr3_display_frame(vo_driver_t *this_gen, vo_frame_t *frame_gen)
   if (this->overlay_enabled) {
     if (this->scale.force_redraw                         ||
 	this->scale.delivered_width  != frame_gen->width ||
-	this->scale.delivered_height != frame->oheight   ||
+	this->scale.delivered_height != (int)frame->oheight ||
 	this->scale.delivered_ratio  != frame_gen->ratio ||
 	this->scale.user_ratio       != (this->widescreen_enabled ? frame->aspect : XINE_VO_ASPECT_4_3)) {
 
@@ -980,9 +952,9 @@ static void dxr3_display_frame(vo_driver_t *this_gen, vo_frame_t *frame_gen)
 	    reg.reg = 0;
 	    reg.val = MVCOMMAND_SYNC;
 	    ioctl(this->fd_control, EM8300_IOCTL_WRITEREG, &reg);
-	    pthread_mutex_lock(&this->class->scr->mutex);
-	    this->class->scr->sync = 1;
-	    pthread_mutex_unlock(&this->class->scr->mutex);
+            pthread_mutex_lock(&this->scr->mutex);
+            this->scr->sync = 1;
+            pthread_mutex_unlock(&this->scr->mutex);
 	  }
 	}
 	this->need_update = 0;
@@ -1151,6 +1123,7 @@ static int dxr3_set_property(vo_driver_t *this_gen, int property, int value)
 static void dxr3_get_property_min_max(vo_driver_t *this_gen, int property,
   int *min, int *max)
 {
+  (void)this_gen;
   switch (property) {
   case VO_PROP_SATURATION:
   case VO_PROP_CONTRAST:
@@ -1258,6 +1231,9 @@ static void dxr3_dispose(vo_driver_t *this_gen)
 
   _x_alphablend_free(&this->alphablend_extra_data);
 
+  if (this->scr)
+    this->scr->scr_plugin.exit(&this->scr->scr_plugin);
+
   free(this);
 }
 
