@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 the xine project
+ * Copyright (C) 2017-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -20,14 +20,13 @@
  * simple network input plugins
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <xine/xine_internal.h>
-#include <xine/input_plugin.h>
 
-void *input_net_init_class  (xine_t *xine, void *data);
-void *input_http_init_class (xine_t *xine, void *data);
-void *input_rtsp_init_class (xine_t *xine, void *data);
-void *input_pnm_init_class  (xine_t *xine, void *data);
+void *input_net_init_class  (xine_t *xine, const void *data);
+void *input_tls_init_class  (xine_t *xine, const void *data);
+void *input_http_init_class (xine_t *xine, const void *data);
+void *input_rtsp_init_class (xine_t *xine, const void *data);
+void *input_pnm_init_class  (xine_t *xine, const void *data);
+void *input_ftp_init_class  (xine_t *xine, const void *data);
+void *input_ftpes_init_class(xine_t *xine, const void *data);
+void *input_hls_init_class  (xine_t *xine, const void *data);
