@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -49,6 +49,8 @@
 #include <xine/compat.h>
 #include <xine/input_plugin.h>
 
+#include "input_helper.h"
+
 #define MAXFILES      65535
 
 #ifndef WIN32
@@ -62,10 +64,8 @@ typedef struct {
   input_class_t     input_class;
 
   xine_t           *xine;
-  config_values_t  *config;
 
-  char             *origin_path;
-  int               show_hidden_files;
+  const char       *origin_path;
 
   int               mrls_allocated_entries;
   xine_mrl_t      **mrls;
@@ -99,11 +99,11 @@ static uint32_t file_input_get_capabilities (input_plugin_t *this_gen) {
 
 #ifdef _MSC_VER
     /*return INPUT_CAP_SEEKABLE | INPUT_CAP_GET_DIR;*/
-	return INPUT_CAP_SEEKABLE;
+	return INPUT_CAP_CLONE | INPUT_CAP_SEEKABLE;
 #else
   if (fstat (this->fh, &buf) == 0) {
     if (S_ISREG(buf.st_mode))
-      return INPUT_CAP_SEEKABLE;
+      return INPUT_CAP_CLONE | INPUT_CAP_SEEKABLE;
     else
       return 0;
   } else
@@ -166,22 +166,21 @@ static off_t file_input_read (input_plugin_t *this_gen, void *buf, off_t len) {
 
 static buf_element_t *file_input_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
 
-  file_input_plugin_t  *this = (file_input_plugin_t *) this_gen;
-  buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
-
-  if (todo > buf->max_size)
-    todo = buf->max_size;
-  if (todo < 0) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->type = BUF_DEMUX_BLOCK;
-
 #ifdef HAVE_MMAP
+  file_input_plugin_t  *this = (file_input_plugin_t *) this_gen;
   if ( file_input_check_mmap(this) ) {
+    buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
     off_t len = todo;
 
+    if (todo > buf->max_size)
+      todo = buf->max_size;
+    if (todo < 0) {
+      buf->free_buffer (buf);
+      return NULL;
+    }
+
+    buf->type = BUF_DEMUX_BLOCK;
+
     if ( (this->mmap_curr + len) > (this->mmap_base + this->mmap_len) )
       len = (this->mmap_base + this->mmap_len) - this->mmap_curr;
 
@@ -196,34 +195,12 @@ static buf_element_t *file_input_read_block (input_plugin_t *this_gen, fifo_buff
     /* free(buf->mem); buf->mem = NULL; */
 
     this->mmap_curr += len;
-  } else
-#endif
-  {
-    off_t num_bytes, total_bytes = 0;
 
-    buf->content = buf->mem;
-
-    while (total_bytes < todo) {
-      num_bytes = read (this->fh, buf->mem + total_bytes, todo-total_bytes);
-      if (num_bytes <= 0) {
-	if (num_bytes < 0) {
-	  xine_log (this->stream->xine, XINE_LOG_MSG,
-		    _("input_file: read error (%s)\n"), strerror(errno));
-	  _x_message(this->stream, XINE_MSG_READ_ERROR,
-                     this->mrl, NULL);
-	}
-	buf->free_buffer (buf);
-	buf = NULL;
-	break;
-      }
-      total_bytes += num_bytes;
-    }
-
-    if( buf != NULL )
-      buf->size = total_bytes;
+    return buf;
   }
+#endif
 
-  return buf;
+  return _x_input_default_read_block(this_gen, fifo, todo);
 }
 
 static off_t file_input_seek (input_plugin_t *this_gen, off_t offset, int origin) {
@@ -287,10 +264,6 @@ static off_t file_input_get_length (input_plugin_t *this_gen) {
   return 0;
 }
 
-static uint32_t file_input_get_blocksize (input_plugin_t *this_gen) {
-  return 0;
-}
-
 /*
  * Return 1 if filepathname is a directory, otherwise 0
  */
@@ -309,11 +282,6 @@ static const char* file_input_get_mrl (input_plugin_t *this_gen) {
   return this->mrl;
 }
 
-static int file_input_get_optional_data (input_plugin_t *this_gen, void *data, int data_type) {
-
-  return INPUT_OPTIONAL_UNSUPPORTED;
-}
-
 static void file_input_dispose (input_plugin_t *this_gen ) {
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
@@ -417,23 +385,26 @@ static int file_input_open (input_plugin_t *this_gen ) {
   return 1;
 }
 
+static int file_input_get_optional_data (input_plugin_t *this_gen, void *data, int data_type);
+
 static input_plugin_t *file_input_get_instance (input_class_t *cls_gen, xine_stream_t *stream,
-				    const char *data) {
+                                                const char *mrl) {
 
   /* file_input_class_t  *cls = (file_input_class_t *) cls_gen; */
   file_input_plugin_t *this;
-  char                *mrl = strdup(data);
 
   lprintf("file_input_get_instance\n");
 
   if ((strncasecmp (mrl, "file:", 5)) && strstr (mrl, ":/") && (strstr (mrl, ":/") < strchr(mrl, '/'))) {
-    free (mrl);
     return NULL;
   }
 
   this = (file_input_plugin_t *) calloc(1, sizeof (file_input_plugin_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
-  this->mrl    = mrl;
+  this->mrl    = strdup(mrl);
   this->fh     = -1;
 
   this->input_plugin.open               = file_input_open;
@@ -443,7 +414,7 @@ static input_plugin_t *file_input_get_instance (input_class_t *cls_gen, xine_str
   this->input_plugin.seek               = file_input_seek;
   this->input_plugin.get_current_pos    = file_input_get_current_pos;
   this->input_plugin.get_length         = file_input_get_length;
-  this->input_plugin.get_blocksize      = file_input_get_blocksize;
+  this->input_plugin.get_blocksize      = _x_input_default_get_blocksize;
   this->input_plugin.get_mrl            = file_input_get_mrl;
   this->input_plugin.get_optional_data  = file_input_get_optional_data;
   this->input_plugin.dispose            = file_input_dispose;
@@ -452,6 +423,23 @@ static input_plugin_t *file_input_get_instance (input_class_t *cls_gen, xine_str
   return &this->input_plugin;
 }
 
+static int file_input_get_optional_data (input_plugin_t *this_gen, void *data, int data_type) {
+  if ((data_type == INPUT_OPTIONAL_DATA_CLONE) && data) {
+    file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+    input_plugin_t *new = file_input_get_instance (this->input_plugin.input_class, this->stream, this->mrl);
+    if (new) {
+      if (new->open (new) < 0) {
+        new->dispose (new);
+      } else {
+        input_plugin_t **q = data;
+        *q = new;
+        return INPUT_OPTIONAL_SUCCESS;
+      }
+    }
+  }
+  return INPUT_OPTIONAL_UNSUPPORTED;
+}
+
 
 /*
  * plugin class functions
@@ -482,11 +470,7 @@ static input_plugin_t *file_input_get_instance (input_class_t *cls_gen, xine_str
 /*
  * Callback for config changes.
  */
-static void file_input_hidden_bool_cb (void *data, xine_cfg_entry_t *cfg) {
-  file_input_class_t *this = (file_input_class_t *) data;
 
-  this->show_hidden_files = cfg->num_value;
-}
 static void file_input_origin_change_cb (void *data, xine_cfg_entry_t *cfg) {
   file_input_class_t *this = (file_input_class_t *) data;
 
@@ -575,8 +559,9 @@ static uint32_t file_input_get_file_type (char *filepathname, char *origin, xine
   uint32_t     file_type = 0;
   char         buf[XINE_PATH_MAX + XINE_NAME_MAX + 1];
 
+  (void)xine;
   if((lstat(filepathname, &pstat)) < 0) {
-    snprintf(buf, sizeof(buf), "%s/%s", origin, filepathname);
+    snprintf_buf(buf, "%s/%s", origin, filepathname);
     if((lstat(buf, &pstat)) < 0) {
       lprintf ("lstat failed for %s{%s}\n", filepathname, origin);
       file_type |= mrl_unknown;
@@ -619,10 +604,10 @@ static uint32_t file_input_get_file_type (char *filepathname, char *origin, xine
  */
 static off_t file_input_get_file_size (const char *filepathname, const char *origin) {
   struct stat  pstat;
-  char         buf[XINE_PATH_MAX + XINE_NAME_MAX + 1];
+  char         buf[XINE_PATH_MAX * 2 + XINE_NAME_MAX + 3];
 
   if((lstat(filepathname, &pstat)) < 0) {
-    snprintf(buf, sizeof(buf), "%s/%s", origin, filepathname);
+    snprintf_buf(buf, "%s/%s", origin, filepathname);
     if((lstat(buf, &pstat)) < 0)
       return (off_t) 0;
   }
@@ -639,26 +624,29 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
   DIR                  *pdir;
   xine_mrl_t           *hide_files, *dir_files, *norm_files;
   char                  current_dir[XINE_PATH_MAX + 1];
-  char                  current_dir_slashed[XINE_PATH_MAX + 1];
-  char                  fullfilename[XINE_PATH_MAX + XINE_NAME_MAX + 1];
+  char                  current_dir_slashed[XINE_PATH_MAX + 2];
+  char                  fullfilename[XINE_PATH_MAX + XINE_NAME_MAX + 2];
   int                   num_hide_files  = 0;
   int                   num_dir_files   = 0;
   int                   num_norm_files  = 0;
   int                   num_files       = -1;
   int                 (*func) ()        = file_input_sortfiles_default;
   int                   already_tried   = 0;
+  int                   show_hidden_files;
 
   *nFiles = 0;
   memset(current_dir, 0, sizeof(current_dir));
 
+  show_hidden_files = _x_input_get_show_hidden_files(this->xine->config);
+
   /*
    * No origin location, so got the content of the current directory
    */
   if(!filename) {
-    snprintf(current_dir, XINE_PATH_MAX, "%s", this->origin_path);
+    snprintf_buf(current_dir, "%s", this->origin_path);
   }
   else {
-    snprintf(current_dir, XINE_PATH_MAX, "%s", filename);
+    snprintf_buf(current_dir, "%s", filename);
 
     /* Remove exceed '/' */
     while((current_dir[strlen(current_dir) - 1] == '/') && strlen(current_dir) > 1)
@@ -668,10 +656,10 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
   /* Store new origin path */
  try_again_from_home:
 
-  this->config->update_string(this->config, "media.files.origin_path", current_dir);
+  this->xine->config->update_string(this->xine->config, "media.files.origin_path", current_dir);
 
   if(strcasecmp(current_dir, "/"))
-    snprintf(current_dir_slashed, sizeof(current_dir_slashed), "%s/", current_dir);
+    snprintf_buf(current_dir_slashed, "%s/", current_dir);
   else
     strcpy(current_dir_slashed, "/");
 
@@ -682,7 +670,7 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
 
     if(!already_tried) {
       /* Try one more time with user homedir */
-      snprintf(current_dir, XINE_PATH_MAX, "%s", xine_get_homedir());
+      snprintf_buf(current_dir, "%s", xine_get_homedir());
       already_tried++;
       goto try_again_from_home;
     }
@@ -697,12 +685,12 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
   while((pdirent = readdir(pdir)) != NULL) {
 
     memset(fullfilename, 0, sizeof(fullfilename));
-    snprintf(fullfilename, sizeof(fullfilename), "%s/%s", current_dir, pdirent->d_name);
+    snprintf_buf(fullfilename, "%s/%s", current_dir, pdirent->d_name);
 
     if(file_input_is_dir(fullfilename)) {
 
       /* if user don't want to see hidden files, ignore them */
-      if(this->show_hidden_files == 0 &&
+      if (show_hidden_files == 0 &&
 	 ((strlen(pdirent->d_name) > 1)
 	  && (pdirent->d_name[0] == '.' &&  pdirent->d_name[1] != '.'))) {
 	;
@@ -742,7 +730,7 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
 	    && (pdirent->d_name[0] == '.' &&  pdirent->d_name[1] != '.')) {
 
       /* if user don't want to see hidden files, ignore them */
-      if(this->show_hidden_files) {
+      if (show_hidden_files) {
 
 	hide_files[num_hide_files].origin = strdup(current_dir);
 	hide_files[num_hide_files].mrl    = _x_asprintf("%s%s", current_dir_slashed, pdirent->d_name);
@@ -842,7 +830,7 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
       else
 	MRL_ZERO(this->mrls[num_files]);
 
-      MRL_DUPLICATE(&dir_files[i], this->mrls[num_files]);
+      *(this->mrls[num_files]) = dir_files[i];
 
       num_files++;
     }
@@ -860,7 +848,7 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
       else
 	MRL_ZERO(this->mrls[num_files]);
 
-      MRL_DUPLICATE(&hide_files[i], this->mrls[num_files]);
+      *(this->mrls[num_files]) = hide_files[i];
 
       num_files++;
     }
@@ -878,24 +866,15 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
       else
 	MRL_ZERO(this->mrls[num_files]);
 
-      MRL_DUPLICATE(&norm_files[i], this->mrls[num_files]);
+      *(this->mrls[num_files]) = norm_files[i];
 
       num_files++;
     }
 
     /* Some cleanups before leaving */
-    for(i = num_dir_files; i == 0; i--)
-      MRL_ZERO(&dir_files[i]);
     free(dir_files);
-
-    for(i = num_hide_files; i == 0; i--)
-      MRL_ZERO(&hide_files[i]);
     free(hide_files);
-
-    for(i = num_norm_files; i == 0; i--)
-      MRL_ZERO(&norm_files[i]);
     free(norm_files);
-
   }
   else {
     free(hide_files);
@@ -913,8 +892,9 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
    * Freeing exceeded mrls if exists.
    */
   while(this->mrls_allocated_entries > num_files) {
-    MRL_ZERO(this->mrls[this->mrls_allocated_entries - 1]);
-    _x_freep(&this->mrls[this->mrls_allocated_entries--]);
+    this->mrls_allocated_entries--;
+    MRL_ZERO(this->mrls[this->mrls_allocated_entries]);
+    _x_freep(&this->mrls[this->mrls_allocated_entries]);
   }
 
   /*
@@ -942,27 +922,29 @@ static void file_input_class_dispose (input_class_t *this_gen) {
   file_input_class_t  *this = (file_input_class_t *) this_gen;
   config_values_t     *config = this->xine->config;
 
-  config->unregister_callback(config, "media.files.origin_path");
-  config->unregister_callback(config, "media.files.show_hidden_files");
+  config->unregister_callbacks (config, NULL, NULL, this, sizeof (*this));
 
   while(this->mrls_allocated_entries) {
-    MRL_ZERO(this->mrls[this->mrls_allocated_entries - 1]);
-    _x_freep(&this->mrls[this->mrls_allocated_entries--]);
+    this->mrls_allocated_entries--;
+    MRL_ZERO(this->mrls[this->mrls_allocated_entries]);
+    _x_freep(&this->mrls[this->mrls_allocated_entries]);
   }
   _x_freep (&this->mrls);
 
   free (this);
 }
 
-static void *file_input_init_plugin (xine_t *xine, void *data) {
+static void *file_input_init_plugin (xine_t *xine, const void *data) {
 
   file_input_class_t  *this;
   config_values_t     *config;
 
+  (void)data;
   this = (file_input_class_t *) calloc(1, sizeof (file_input_class_t));
+  if (!this)
+    return NULL;
 
   this->xine   = xine;
-  this->config = xine->config;
   config       = xine->config;
 
   this->input_class.get_instance       = file_input_get_instance;
@@ -990,12 +972,7 @@ static void *file_input_init_plugin (xine_t *xine, void *data) {
 						0, file_input_origin_change_cb, (void *) this);
   }
 
-  this->show_hidden_files = config->register_bool(config,
-						  "media.files.show_hidden_files",
-						  0, _("list hidden files"),
-						  _("If enabled, the browser to select the file to "
-						    "play will also show hidden files."),
-						  10, file_input_hidden_bool_cb, (void *) this);
+  _x_input_register_show_hidden_files(config);
 
   return this;
 }
@@ -1010,7 +987,7 @@ static void *file_input_init_plugin (xine_t *xine, void *data) {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   INPUT_FILE_CATALOG,
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 #endif
 
