@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2017 the xine project
+ * Copyright (C) 2002-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -54,6 +54,7 @@
 #include "mms.h"
 #include "mmsh.h"
 #include "net_buf_ctrl.h"
+#include "input_helper.h"
 
 #define LOG_MODULE "input_mms"
 
@@ -64,6 +65,7 @@
 /* network bandwidth */
 static const uint32_t mms_bandwidths[]={14400,19200,28800,33600,34430,57600,
 					115200,262200,393216,524300,1544000,10485800};
+#define NUM_BANDWIDTHS (sizeof (mms_bandwidths) / sizeof (mms_bandwidths[0]))
 
 static const char *const mms_bandwidth_strs[]={"14.4 Kbps (Modem)", "19.2 Kbps (Modem)",
 					       "28.8 Kbps (Modem)", "33.6 Kbps (Modem)",
@@ -88,7 +90,6 @@ typedef struct {
 
   char             scratch[1025];
 
-  int              bandwidth;
   int              protocol;       /* mmst or mmsh */
 
 } mms_input_plugin_t;
@@ -97,8 +98,8 @@ typedef struct {
 
   input_class_t       input_class;
 
-  mms_input_plugin_t *ip;
   int                 protocol;    /* autoprobe, mmst or mmsh */
+  int                 bandwidth;
 
   xine_t             *xine;
 } mms_input_class_t;
@@ -123,39 +124,8 @@ static off_t mms_plugin_read (input_plugin_t *this_gen,
   return n;
 }
 
-static buf_element_t *mms_plugin_read_block (input_plugin_t *this_gen,
-                                             fifo_buffer_t *fifo, off_t todo) {
-  /*mms_input_plugin_t   *this = (mms_input_plugin_t *) this_gen; */
-  buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
-  int                   total_bytes;
-
-  lprintf ("mms_plugin_read_block: %"PRId64" bytes...\n", todo);
-
-  if (todo > buf->max_size)
-    todo = buf->max_size;
-  if (todo < 0) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->content = buf->mem;
-  buf->type = BUF_DEMUX_BLOCK;
-
-  total_bytes = mms_plugin_read (this_gen, (char*)buf->content, todo);
-
-  if (total_bytes != todo) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->size = total_bytes;
-
-  return buf;
-}
-
 static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   mms_input_plugin_t   *this = (mms_input_plugin_t *) this_gen;
-  off_t                 dest = 0;
   off_t                 curpos = 0;
 
   lprintf ("mms_plugin_seek: %"PRId64" offset, %d origin...\n", offset, origin);
@@ -170,49 +140,8 @@ static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
       break;
   }
 
-  switch (origin) {
-    case SEEK_SET:
-      dest = offset;
-      break;
-    case SEEK_CUR:
-      dest = curpos + offset;
-      break;
-    default:
-      printf ("input_mms: unknown origin in seek!\n");
-      return curpos;
-  }
-
-  if (curpos > dest) {
-    printf ("input_mms: cannot seek back!\n");
-    return curpos;
-  }
-
-  while (curpos < dest) {
-    int n = 0;
-    int diff;
-
-    diff = dest - curpos;
-
-    if (diff > 1024)
-      diff = 1024;
-
-    switch (this->protocol) {
-      case PROTOCOL_MMST:
-        n = mms_read (this->mms, this->scratch, diff);
-        break;
-      case PROTOCOL_MMSH:
-        n = mmsh_read (this->mmsh, this->scratch, diff);
-        break;
-    }
-
-    curpos += n;
-
-    if (n < diff)
-      return curpos;
-
-  }
-
-  return curpos;
+  return _x_input_seek_preview(this_gen, offset, origin,
+                               &curpos, -1, -1);
 }
 
 static off_t mms_plugin_seek_time (input_plugin_t *this_gen, int time_offset, int origin) {
@@ -259,14 +188,6 @@ static off_t mms_plugin_get_length (input_plugin_t *this_gen) {
 
 }
 
-static uint32_t mms_plugin_get_capabilities (input_plugin_t *this_gen) {
-  return INPUT_CAP_PREVIEW;
-}
-
-static uint32_t mms_plugin_get_blocksize (input_plugin_t *this_gen) {
-  return 0;
-}
-
 static off_t mms_plugin_get_current_pos (input_plugin_t *this_gen){
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
   off_t curpos = 0;
@@ -338,17 +259,9 @@ static int mms_plugin_get_optional_data (input_plugin_t *this_gen,
 
 static void bandwidth_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
   mms_input_class_t *class = (mms_input_class_t*) this_gen;
-
   lprintf ("bandwidth_changed_cb %d\n", entry->num_value);
-
-  if(!class)
-   return;
-
-  if(class->ip && ((entry->num_value >= 0) && (entry->num_value <= 11))) {
-    mms_input_plugin_t *this = class->ip;
-
-    this->bandwidth = mms_bandwidths[entry->num_value];
-  }
+  if (class && (entry->num_value >= 0) && (entry->num_value < (int)NUM_BANDWIDTHS))
+    class->bandwidth = mms_bandwidths[entry->num_value];
 }
 
 static void protocol_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
@@ -362,24 +275,25 @@ static void protocol_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
 
 static int mms_plugin_open (input_plugin_t *this_gen) {
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
+  mms_input_class_t  *class = (mms_input_class_t *) this->input_plugin.input_class;
   mms_t              *mms  = NULL;
   mmsh_t             *mmsh = NULL;
 
   switch (this->protocol) {
     case PROTOCOL_UNDEFINED:
-      mms = mms_connect (this->stream, this->mrl, this->bandwidth);
+      mms = mms_connect (this->stream, this->mrl, class->bandwidth);
       if (mms) {
         this->protocol = PROTOCOL_MMST;
       } else {
-        mmsh = mmsh_connect (this->stream, this->mrl, this->bandwidth);
+        mmsh = mmsh_connect (this->stream, this->mrl, class->bandwidth);
         this->protocol = PROTOCOL_MMSH;
       }
       break;
     case PROTOCOL_MMST:
-      mms = mms_connect (this->stream, this->mrl, this->bandwidth);
+      mms = mms_connect (this->stream, this->mrl, class->bandwidth);
       break;
     case PROTOCOL_MMSH:
-      mmsh = mmsh_connect (this->stream, this->mrl, this->bandwidth);
+      mmsh = mmsh_connect (this->stream, this->mrl, class->bandwidth);
       break;
   }
 
@@ -416,7 +330,6 @@ static input_plugin_t *mms_class_get_instance (input_class_t *cls_gen, xine_stre
   }
 
   this = calloc(1, sizeof (mms_input_plugin_t));
-  cls->ip = this;
   this->stream   = stream;
   this->mms      = NULL;
   this->mmsh     = NULL;
@@ -430,14 +343,14 @@ static input_plugin_t *mms_class_get_instance (input_class_t *cls_gen, xine_stre
   }
 
   this->input_plugin.open              = mms_plugin_open;
-  this->input_plugin.get_capabilities  = mms_plugin_get_capabilities;
+  this->input_plugin.get_capabilities  = _x_input_get_capabilities_preview;
   this->input_plugin.read              = mms_plugin_read;
-  this->input_plugin.read_block        = mms_plugin_read_block;
+  this->input_plugin.read_block        = _x_input_default_read_block;
   this->input_plugin.seek              = mms_plugin_seek;
   this->input_plugin.seek_time         = mms_plugin_seek_time;
   this->input_plugin.get_current_pos   = mms_plugin_get_current_pos;
   this->input_plugin.get_length        = mms_plugin_get_length;
-  this->input_plugin.get_blocksize     = mms_plugin_get_blocksize;
+  this->input_plugin.get_blocksize     = _x_input_default_get_blocksize;
   this->input_plugin.get_mrl           = mms_plugin_get_mrl;
   this->input_plugin.dispose           = mms_plugin_dispose;
   this->input_plugin.get_optional_data = mms_plugin_get_optional_data;
@@ -454,21 +367,21 @@ static input_plugin_t *mms_class_get_instance (input_class_t *cls_gen, xine_stre
 static void mms_class_dispose (input_class_t *this_gen) {
   mms_input_class_t  *this = (mms_input_class_t *) this_gen;
 
-  this->xine->config->unregister_callback(this->xine->config,
-					  "media.network.bandwidth");
-  this->xine->config->unregister_callback(this->xine->config,
-					  "media.network.mms_protocol");
+  this->xine->config->unregister_callbacks (this->xine->config, NULL, NULL, this, sizeof (*this));
   free (this);
 }
 
-static void *init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, const void *data) {
 
   mms_input_class_t  *this;
+  int i;
 
+  (void)data;
   this = calloc(1, sizeof (mms_input_class_t));
+  if (!this)
+    return NULL;
 
   this->xine   = xine;
-  this->ip                             = NULL;
 
   this->input_class.get_instance       = mms_class_get_instance;
   this->input_class.identifier         = "mms";
@@ -478,16 +391,17 @@ static void *init_class (xine_t *xine, void *data) {
   this->input_class.dispose            = mms_class_dispose;
   this->input_class.eject_media        = NULL;
 
-  xine->config->register_enum(xine->config, "media.network.bandwidth", 10,
-			      (char **)mms_bandwidth_strs,
-			      _("network bandwidth"),
-			      _("Specify the bandwidth of your internet connection here. "
-			        "This will be used when streaming servers offer different versions "
-				"with different bandwidth requirements of the same stream."),
-			      0, bandwidth_changed_cb, (void*) this);
+  i = xine->config->register_enum (xine->config, "media.network.bandwidth",
+    10,
+    (char **)mms_bandwidth_strs,
+    _("network bandwidth"),
+    _("Specify the bandwidth of your internet connection here. "
+      "This will be used when streaming servers offer different versions "
+      "with different bandwidth requirements of the same stream."),
+    0, bandwidth_changed_cb, (void*) this);
+  this->bandwidth = mms_bandwidths[((i >= 0) && (i < (int)NUM_BANDWIDTHS)) ? i : 10];
 
-  this->protocol = xine->config->register_enum(xine->config,
-    "media.network.mms_protocol",
+  this->protocol = xine->config->register_enum (xine->config, "media.network.mms_protocol",
     0,
     (char **)mms_protocol_strs,
     _("MMS protocol"),
@@ -505,5 +419,5 @@ static void *init_class (xine_t *xine, void *data) {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "mms", XINE_VERSION_CODE, NULL, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
