@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -41,33 +41,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <fcntl.h>
 #include <string.h>
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-
-#ifdef ENABLE_IPV6
-#include <sys/types.h>
-#endif
-
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-
-#ifdef WIN32
-#include <winsock.h>
-#endif
-
-#include <errno.h>
-#include <sys/time.h>
 
 #define LOG_MODULE "input_net"
 #define LOG_VERBOSE
@@ -78,186 +52,35 @@
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/input_plugin.h>
+#include "tls/xine_tls.h"
 #include "net_buf_ctrl.h"
 #include "group_network.h"
+#include "input_helper.h"
 
 #define NET_BS_LEN 2324
-#define BUFSIZE                 1024
 
 typedef struct {
   input_plugin_t   input_plugin;
 
   xine_stream_t   *stream;
 
-  int              fh;
+  xine_tls_t      *tls;
   char            *mrl;
   char            *host_port;
 
-  char             preview[MAX_PREVIEW_SIZE];
-  off_t            preview_size;
-
   off_t            curpos;
 
   nbc_t           *nbc;
 
-  /* scratch buffer for forward seeking */
-  char             seek_buf[BUFSIZE];
-
+  off_t            preview_size;
+  char             preview[MAX_PREVIEW_SIZE];
 
 } net_input_plugin_t;
 
-typedef struct {
-
-  input_class_t     input_class;
-
-  xine_t           *xine;
-  config_values_t  *config;
-
-} net_input_class_t;
-
 /* **************************************************************** */
 /*                       Private functions                          */
 /* **************************************************************** */
 
-#ifndef ENABLE_IPV6
-static int host_connect_attempt_ipv4(struct in_addr ia, int port, xine_t *xine) {
-
-  int                s;
-  union {
-    struct sockaddr_in in;
-    struct sockaddr    sa;
-  } sa;
-
-  s = xine_socket_cloexec(PF_INET, SOCK_STREAM, IPPROTO_TCP);
-  if (s==-1) {
-    xine_log(xine, XINE_LOG_MSG,
-             _("input_net: socket(): %s\n"), strerror(errno));
-    return -1;
-  }
-
-  sa.in.sin_family = AF_INET;
-  sa.in.sin_addr   = ia;
-  sa.in.sin_port   = htons(port);
-
-#ifndef WIN32
-  if (connect(s, &sa.sa, sizeof(sa.in))==-1 && errno != EINPROGRESS)
-#else
-  if (connect(s, &sa.sa, sizeof(sa.in))==-1 && WSAGetLastError() != WSAEINPROGRESS)
-#endif
-  {
-    xine_log(xine, XINE_LOG_MSG,
-             _("input_net: connect(): %s\n"), strerror(errno));
-    close(s);
-    return -1;
-  }
-
-  return s;
-}
-#else
-static int host_connect_attempt(int family, struct sockaddr* sin, int addrlen, xine_t *xine) {
-
-  int s = xine_socket_cloexec(family, SOCK_STREAM, IPPROTO_TCP);
-
-  if (s == -1) {
-    xine_log(xine, XINE_LOG_MSG,
-             _("input_net: socket(): %s\n"), strerror(errno));
-    return -1;
-  }
-
-#ifndef WIN32
-  if (connect(s, sin, addrlen)==-1 && errno != EINPROGRESS)
-#else
-  if (connect(s, sin, addrlen)==-1 && WSAGetLastError() != WSAEINPROGRESS)
-#endif
-  {
-    xine_log(xine, XINE_LOG_MSG,
-             _("input_net: connect(): %s\n"), strerror(errno));
-    close(s);
-    return -1;
-  }
-
-  return s;
-}
-#endif
-
-#ifndef ENABLE_IPV6
-static int host_connect_ipv4(const char *host, int port, xine_t *xine) {
-  struct hostent *h;
-  int             i;
-  int             s;
-
-  h = gethostbyname(host);
-  if (h==NULL) {
-    xine_log(xine, XINE_LOG_MSG,
-             _("input_net: unable to resolve '%s'.\n"), host);
-    return -1;
-  }
-
-  for (i=0; h->h_addr_list[i]; i++) {
-    struct in_addr ia;
-    memcpy (&ia, h->h_addr_list[i],4);
-    s = host_connect_attempt_ipv4 (ia, port, xine);
-    if (s != -1)
-      return s;
-  }
-
-  xine_log(xine, XINE_LOG_MSG,
-           _("input_net: unable to connect to '%s'.\n"), host);
-  return -1;
-}
-#endif
-
-static int host_connect(const char *host, int port, xine_t *xine) {
-
-#ifndef ENABLE_IPV6
-  return host_connect_ipv4(host, port, xine);
-#else
-
-  struct addrinfo hints, *res, *tmpaddr;
-  int error;
-  char strport[16];
-  int             s;
-
-  memset(&hints, 0, sizeof(hints));
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_family = PF_UNSPEC;
-
-  snprintf(strport, sizeof(strport), "%d", port);
-
-  lprintf("Resolving host '%s' at port '%s'\n", host, strport);
-
-  error = getaddrinfo(host, strport, &hints, &res);
-
-  if (error) {
-
-    xine_log(xine, XINE_LOG_MSG,
-             _("input_net: unable to resolve '%s'.\n"), host);
-    return -1;
-  }
-
-  /* We loop over all addresses and try to connect */
-  tmpaddr = res;
-  while (tmpaddr) {
-
-      s = host_connect_attempt (tmpaddr->ai_family,
-				tmpaddr->ai_addr, tmpaddr->ai_addrlen, xine);
-      if (s != -1)
-	  return s;
-
-      tmpaddr = tmpaddr->ai_next;
-  }
-
-  xine_log(xine, XINE_LOG_MSG,
-           _("input_net: unable to connect to '%s'.\n"), host);
-  return -1;
-
-#endif
-
-}
-
-#define LOW_WATER_MARK  50
-#define HIGH_WATER_MARK 100
-
 static off_t net_plugin_read (input_plugin_t *this_gen,
 			      void *buf_gen, off_t len) {
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
@@ -283,7 +106,7 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
   }
 
   if( (len-total) > 0 ) {
-    n = _x_read_abort (this->stream, this->fh, &buf[total], len-total);
+    n = _x_tls_read (this->tls, &buf[total], len-total);
 
     xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_net: got %" PRIdMAX " bytes (%" PRIdMAX "/%" PRIdMAX " bytes read)\n", (intmax_t)n, (intmax_t)total, (intmax_t)len);
 
@@ -298,46 +121,9 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
   return total;
 }
 
-static buf_element_t *net_plugin_read_block (input_plugin_t *this_gen,
-					     fifo_buffer_t *fifo, off_t todo) {
-  /* net_input_plugin_t   *this = (net_input_plugin_t *) this_gen; */
-  buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
-  off_t                 total_bytes;
-
-  if (todo > buf->max_size)
-    todo = buf->max_size;
-  if (todo < 0) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->content = buf->mem;
-  buf->type = BUF_DEMUX_BLOCK;
-
-  total_bytes = net_plugin_read (this_gen, (char*)buf->content, todo);
-
-  if (total_bytes != todo) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->size = total_bytes;
-
-  return buf;
-}
-
-static off_t net_plugin_get_length (input_plugin_t *this_gen) {
-
-  return 0;
-}
-
-static uint32_t net_plugin_get_capabilities (input_plugin_t *this_gen) {
-
-  return INPUT_CAP_PREVIEW;
-}
-
 static uint32_t net_plugin_get_blocksize (input_plugin_t *this_gen) {
 
+  (void)this_gen;
   return NET_BS_LEN;
 
 }
@@ -351,40 +137,8 @@ static off_t net_plugin_get_current_pos (input_plugin_t *this_gen){
 static off_t net_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
 
-  if ((origin == SEEK_CUR) && (offset >= 0)) {
-
-    for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
-      if( this_gen->read (this_gen, this->seek_buf, BUFSIZE) <= 0 )
-        return this->curpos;
-    }
-
-    this_gen->read (this_gen, this->seek_buf, offset);
-  }
-
-  if (origin == SEEK_SET) {
-
-    if (offset < this->curpos) {
-
-      if( this->curpos <= this->preview_size )
-        this->curpos = offset;
-      else
-        xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-                "input_net: cannot seek back! (%" PRIdMAX " > %" PRIdMAX ")\n",
-                (intmax_t)this->curpos, (intmax_t)offset);
-
-    } else {
-      offset -= this->curpos;
-
-      for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
-        if( this_gen->read (this_gen, this->seek_buf, BUFSIZE) <= 0 )
-          return this->curpos;
-      }
-
-      this_gen->read (this_gen, this->seek_buf, offset);
-    }
-  }
-
-  return this->curpos;
+  return _x_input_seek_preview(this_gen, offset, origin,
+                               &this->curpos, -1, this->preview_size);
 }
 
 
@@ -413,13 +167,10 @@ static int net_plugin_get_optional_data (input_plugin_t *this_gen,
 static void net_plugin_dispose (input_plugin_t *this_gen ) {
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
 
-  if (this->fh != -1) {
-    close(this->fh);
-    this->fh = -1;
-  }
+  _x_tls_close (&this->tls);
 
-  free (this->mrl);
-  free (this->host_port);
+  _x_freep (&this->mrl);
+  _x_freep (&this->host_port);
 
   if (this->nbc) {
     nbc_close (this->nbc);
@@ -444,22 +195,26 @@ static int net_plugin_open (input_plugin_t *this_gen ) {
     sscanf(pptr,"%d", &port);
   }
 
-  this->fh     = host_connect(filename, port, this->stream->xine);
   this->curpos = 0;
 
-  if (this->fh == -1) {
+  this->tls = _x_tls_connect(this->stream->xine, this->stream, filename, port);
+  if (!this->tls) {
     return 0;
   }
 
+  if (!strncasecmp(this->mrl, "tls", 3)) {
+    if (_x_tls_handshake(this->tls, filename, -1) < 0)
+      return 0;
+  }
+
   /*
    * fill preview buffer
    */
   while ((toread > 0) && (trycount < 10)) {
-#ifndef WIN32
-    this->preview_size += read (this->fh, this->preview + this->preview_size, toread);
-#else
-    this->preview_size += recv (this->fh, this->preview + this->preview_size, toread, 0);
-#endif
+    int got = _x_tls_read (this->tls, this->preview + this->preview_size, toread);
+    if (got < 0)
+      break;
+    this->preview_size += got;
     trycount++;
     toread = MAX_PREVIEW_SIZE - this->preview_size;
   }
@@ -475,7 +230,9 @@ static input_plugin_t *net_class_get_instance (input_class_t *cls_gen, xine_stre
   nbc_t *nbc = NULL;
   const char *filename;
 
-  if (!strncasecmp (mrl, "tcp://", 6)) {
+  if (!strncasecmp (mrl, "tcp://", 6) ||
+      !strncasecmp (mrl, "tls://", 6)) {
+
     filename = &mrl[6];
 
     if((!filename) || (strlen(filename) == 0)) {
@@ -507,18 +264,18 @@ static input_plugin_t *net_class_get_instance (input_class_t *cls_gen, xine_stre
   this->mrl           = strdup(mrl);
   this->host_port     = strdup(filename);
   this->stream        = stream;
-  this->fh            = -1;
+  this->tls           = NULL;
   this->curpos        = 0;
   this->nbc           = nbc;
   this->preview_size  = 0;
 
   this->input_plugin.open              = net_plugin_open;
-  this->input_plugin.get_capabilities  = net_plugin_get_capabilities;
+  this->input_plugin.get_capabilities  = _x_input_get_capabilities_preview;
   this->input_plugin.read              = net_plugin_read;
-  this->input_plugin.read_block        = net_plugin_read_block;
+  this->input_plugin.read_block        = _x_input_default_read_block;
   this->input_plugin.seek              = net_plugin_seek;
   this->input_plugin.get_current_pos   = net_plugin_get_current_pos;
-  this->input_plugin.get_length        = net_plugin_get_length;
+  this->input_plugin.get_length        = _x_input_default_get_length;
   this->input_plugin.get_blocksize     = net_plugin_get_blocksize;
   this->input_plugin.get_mrl           = net_plugin_get_mrl;
   this->input_plugin.get_optional_data = net_plugin_get_optional_data;
@@ -533,22 +290,37 @@ static input_plugin_t *net_class_get_instance (input_class_t *cls_gen, xine_stre
  *  net plugin class
  */
 
-void *input_net_init_class (xine_t *xine, void *data) {
+void *input_net_init_class (xine_t *xine, const void *data) {
 
-  net_input_class_t  *this;
+  static const input_class_t input_net_class = {
+    .get_instance      = net_class_get_instance,
+    .description       = N_("net input plugin as shipped with xine"),
+    .identifier        = "TCP",
+    .get_dir           = NULL,
+    .get_autoplay_list = NULL,
+    .dispose           = NULL,
+    .eject_media       = NULL,
+  };
 
-  this         = calloc(1, sizeof(net_input_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
-
-  this->input_class.get_instance      = net_class_get_instance;
-  this->input_class.description       = N_("net input plugin as shipped with xine");
-  this->input_class.identifier        = "TCP";
-  this->input_class.get_dir           = NULL;
-  this->input_class.get_autoplay_list = NULL;
-  this->input_class.dispose           = default_input_class_dispose;
-  this->input_class.eject_media       = NULL;
-
-  return this;
+  (void)xine;
+  (void)data;
+  return (void *)&input_net_class;
 }
 
+void *input_tls_init_class (xine_t *xine, const void *data) {
+
+  static const input_class_t input_tls_class = {
+    .get_instance      = net_class_get_instance,
+    .description       = N_("tls input plugin"),
+    .identifier        = "TLS",
+    .get_dir           = NULL,
+    .get_autoplay_list = NULL,
+    .dispose           = NULL,
+    .eject_media       = NULL,
+  };
+
+  (void)xine;
+  (void)data;
+
+  return (void *)&input_tls_class;
+}
