@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2017 the xine project
+ * Copyright (C) 2002-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -53,6 +53,7 @@
 
 #include "librtsp/rtsp_session.h"
 #include "net_buf_ctrl.h"
+#include "input_helper.h"
 #include "group_network.h"
 
 #define BUFSIZE 1025
@@ -90,36 +91,6 @@ static off_t rtsp_plugin_read (input_plugin_t *this_gen,
   return n;
 }
 
-static buf_element_t *rtsp_plugin_read_block (input_plugin_t *this_gen,
-                                             fifo_buffer_t *fifo, off_t todo) {
-  /*rtsp_input_plugin_t   *this = (rtsp_input_plugin_t *) this_gen; */
-  buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
-  int                   total_bytes;
-
-  lprintf ("rtsp_plugin_read_block: %"PRId64" bytes...\n", todo);
-
-  if (todo > buf->max_size)
-    todo = buf->max_size;
-  if (todo < 0) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->content = buf->mem;
-  buf->type = BUF_DEMUX_BLOCK;
-
-  total_bytes = rtsp_plugin_read (this_gen, (char*)buf->content, todo);
-
-  if (total_bytes != todo) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->size = total_bytes;
-
-  return buf;
-}
-
 static off_t rtsp_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
 
   rtsp_input_plugin_t *this = (rtsp_input_plugin_t *) this_gen;
@@ -127,23 +98,7 @@ static off_t rtsp_plugin_seek (input_plugin_t *this_gen, off_t offset, int origi
   lprintf ("seek %"PRId64" bytes, origin %d\n", offset, origin);
 
   /* only realtive forward-seeking is implemented */
-
-  if ((origin == SEEK_CUR) && (offset >= 0)) {
-
-    for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
-      off_t n = rtsp_plugin_read (this_gen, this->scratch, BUFSIZE);
-      if (n <= 0)
-	return this->curpos;
-      this->curpos += n;
-    }
-
-    off_t n = rtsp_plugin_read (this_gen, this->scratch, offset);
-    if (n <= 0)
-      return this->curpos;
-    this->curpos += n;
-  }
-
-  return this->curpos;
+  return _x_input_seek_preview(this_gen, offset, origin, &this->curpos, -1, -1);
 }
 
 static off_t rtsp_plugin_seek_time (input_plugin_t *this_gen, int time_offset, int origin) {
@@ -165,17 +120,15 @@ static off_t rtsp_plugin_get_length (input_plugin_t *this_gen) {
   off_t                 length;
   */
 
+  (void)this_gen;
   return -1;
 }
 
 static uint32_t rtsp_plugin_get_capabilities (input_plugin_t *this_gen) {
+  (void)this_gen;
   return INPUT_CAP_PREVIEW | INPUT_CAP_RIP_FORBIDDEN;
 }
 
-static uint32_t rtsp_plugin_get_blocksize (input_plugin_t *this_gen) {
-  return 0;
-}
-
 static off_t rtsp_plugin_get_current_pos (input_plugin_t *this_gen){
   rtsp_input_plugin_t *this = (rtsp_input_plugin_t *) this_gen;
 
@@ -273,12 +226,12 @@ static input_plugin_t *rtsp_class_get_instance (input_class_t *cls_gen, xine_str
   this->input_plugin.open              = rtsp_plugin_open;
   this->input_plugin.get_capabilities  = rtsp_plugin_get_capabilities;
   this->input_plugin.read              = rtsp_plugin_read;
-  this->input_plugin.read_block        = rtsp_plugin_read_block;
+  this->input_plugin.read_block        = _x_input_default_read_block;
   this->input_plugin.seek              = rtsp_plugin_seek;
   this->input_plugin.seek_time         = rtsp_plugin_seek_time;
   this->input_plugin.get_current_pos   = rtsp_plugin_get_current_pos;
   this->input_plugin.get_length        = rtsp_plugin_get_length;
-  this->input_plugin.get_blocksize     = rtsp_plugin_get_blocksize;
+  this->input_plugin.get_blocksize     = _x_input_default_get_blocksize;
   this->input_plugin.get_mrl           = rtsp_plugin_get_mrl;
   this->input_plugin.dispose           = rtsp_plugin_dispose;
   this->input_plugin.get_optional_data = rtsp_plugin_get_optional_data;
@@ -290,20 +243,19 @@ static input_plugin_t *rtsp_class_get_instance (input_class_t *cls_gen, xine_str
 /*
  * rtsp input plugin class stuff
  */
-void *input_rtsp_init_class (xine_t *xine, void *data) {
+void *input_rtsp_init_class (xine_t *xine, const void *data) {
 
-  input_class_t  *this;
-
-  this = calloc(1, sizeof (input_class_t));
-
-  this->get_instance       = rtsp_class_get_instance;
-  this->identifier         = "rtsp";
-  this->description        = N_("rtsp streaming input plugin");
-  this->get_dir            = NULL;
-  this->get_autoplay_list  = NULL;
-  this->dispose            = default_input_class_dispose;
-  this->eject_media        = NULL;
-
-  return this;
+  static const input_class_t input_rtsp_class = {
+    .get_instance       = rtsp_class_get_instance,
+    .identifier         = "rtsp",
+    .description        = N_("rtsp streaming input plugin"),
+    .get_dir            = NULL,
+    .get_autoplay_list  = NULL,
+    .dispose            = NULL,
+    .eject_media        = NULL,
+  };
+  (void)xine;
+  (void)data;
+  return (void *)&input_rtsp_class;
 }
 
