@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,6 +40,7 @@
 #include <xine/xineutils.h>
 #include <xine/input_plugin.h>
 #include "net_buf_ctrl.h"
+#include "input_helper.h"
 
 #define BUFSIZE                 1024
 #if defined(WIN32) || defined(__CYGWIN__)
@@ -59,15 +60,20 @@ typedef struct {
   int              fh;
   off_t            curpos;
 
-  int              num_reads, num_waits;
+  int              num_reads, num_fd_reads, num_fd_waits;
   long int         old_mode, mode;
   int              nonblock;
   int              timeout, requery_timeout;
 
+#define RING_LD 15
+#define RING_SIZE (1 << RING_LD)
+#define RING_MASK (RING_SIZE - 1)
+  int              ring_write;
+  int              ring_read;
+  uint8_t         *ring_buf;
+
   off_t            preview_size;
   char             preview[MAX_PREVIEW_SIZE];
-
-  char             seek_buf[BUFSIZE];
 } stdin_input_plugin_t;
 
 static off_t stdin_plugin_get_current_pos (input_plugin_t *this_gen);
@@ -76,11 +82,10 @@ static off_t stdin_plugin_get_current_pos (input_plugin_t *this_gen);
 static int stdin_plugin_wait (stdin_input_plugin_t *this) {
   int ret;
   if (this->requery_timeout <= 0) {
-    xine_cfg_entry_t cfgentry;
     this->requery_timeout = 1 << 20;
-    if (xine_config_lookup_entry (this->xine, "media.network.timeout", &cfgentry))
-      this->timeout = cfgentry.num_value * 1000;
+    this->timeout = _x_query_network_timeout (this->xine) * 1000;
   }
+  this->num_fd_waits += 1;
   ret = _x_io_select (this->stream, this->fh, XIO_READ_READY, this->timeout);
   if (ret != XIO_READY) {
     if (ret == XIO_ABORTED) {
@@ -101,7 +106,7 @@ static off_t stdin_plugin_read (input_plugin_t *this_gen, void *buf_gen, off_t l
   stdin_input_plugin_t  *this = (stdin_input_plugin_t *) this_gen;
   char *buf = (char *)buf_gen;
   long int n, rest = len, done;
-  int waited, e;
+  int e;
 
   lprintf ("reading %"PRId64" bytes...\n", len);
   if (rest <= 0)
@@ -126,31 +131,107 @@ static off_t stdin_plugin_read (input_plugin_t *this_gen, void *buf_gen, off_t l
     rest -= n;
   }
 
-  /* Let input_cache handle the demux_len <= done <= cache_fill_len case without wait */
-  waited = 0;
   if (this->nonblock) {
-    while (1) {
-      n = read (this->fh, buf + done, rest);
-      if (n >= 0) {
-        this->curpos += n;
-        done += n;
-        lprintf ("got %ld bytes (%ld/%"PRId64" bytes read)\n", n, done, len);
-        if (this->requery_timeout > 0)
-          this->requery_timeout -= n;
-        this->num_waits += waited;
-        return done;
+    if (this->ring_buf) {
+      while (1) {
+        e = 0;
+        n = 1;
+        /* Get from buf. */
+        {
+          int part = this->ring_write - this->ring_read;
+          if (part < 0) {
+            part = RING_SIZE - this->ring_read;
+            if (part <= rest) {
+              rest -= part;
+              xine_fast_memcpy (buf + done, this->ring_buf + this->ring_read, part);
+              this->ring_read = 0;
+              this->curpos += part;
+              done += part;
+              part = this->ring_write;
+            }
+          }
+          if (part > rest)
+            part = rest;
+          if (part > 0) {
+            xine_fast_memcpy (buf + done, this->ring_buf + this->ring_read, part);
+            this->ring_read += part;
+            this->curpos += part;
+            done += part;
+            rest -= part;
+          }
+        }
+
+        /* Are we done already? */
+        if (rest <= 0) {
+          if (this->requery_timeout > 0)
+            this->requery_timeout -= n;
+          return done;
+        }
+
+        /* Always try an immediate buf refill. Dont fill up all as that would look like buf empty later. */
+        do {
+          int part = this->ring_read - this->ring_write;
+          if (part <= 0) {
+            int room = RING_SIZE + part - 32;
+            if (room <= 0)
+              break;
+            part = RING_SIZE - this->ring_write;
+            if (part > room)
+              part = room;
+            this->num_fd_reads += 1;
+            n = read (this->fh, this->ring_buf + this->ring_write, part);
+            if (n <= 0)
+              break;
+            this->ring_write = (this->ring_write + n) & RING_MASK;
+            if (this->ring_write > 0)
+              break;
+            part = this->ring_read;
+          }
+          part -= 32;
+          if (part <= 0)
+            break;
+          this->num_fd_reads += 1;
+          n = read (this->fh, this->ring_buf + this->ring_write, part);
+          if (n >= 0)
+            this->ring_write += n;
+        } while (0);
+        if (n < 0)
+          e = errno;
+
+        /* Continue / Wait / Bail out. */
+        if (n == 0)
+          return done;
+        if (n < 0) {
+          if (e != EAGAIN)
+            break;
+          if (stdin_plugin_wait (this) != XIO_READY)
+            return done;
+        }
+      }
+    } else {
+      /* Let input_cache handle the demux_len <= done <= cache_fill_len case without wait */
+      while (1) {
+        this->num_fd_reads += 1;
+        n = read (this->fh, buf + done, rest);
+        if (n >= 0) {
+          this->curpos += n;
+          done += n;
+          lprintf ("got %ld bytes (%ld/%"PRId64" bytes read)\n", n, done, len);
+          if (this->requery_timeout > 0)
+            this->requery_timeout -= n;
+          return done;
+        }
+        e = errno;
+        if (e != EAGAIN)
+          break;
+        if (stdin_plugin_wait (this) != XIO_READY)
+          return done;
       }
-      e = errno;
-      if (e != EAGAIN)
-        break;
-      waited = 1;
-      if (stdin_plugin_wait (this) != XIO_READY)
-        return done;
     }
   } else {
-    waited = 1;
     if (stdin_plugin_wait (this) != XIO_READY)
       return done;
+    this->num_fd_reads += 1;
     n = read (this->fh, buf + done, rest);
     if (n >= 0) {
       this->curpos += n;
@@ -158,7 +239,6 @@ static off_t stdin_plugin_read (input_plugin_t *this_gen, void *buf_gen, off_t l
       lprintf ("got %ld bytes (%ld/%"PRId64" bytes read)\n", n, done, len);
       if (this->requery_timeout > 0)
         this->requery_timeout -= n;
-      this->num_waits += waited;
       return done;
     }
     e = errno;
@@ -177,35 +257,6 @@ static off_t stdin_plugin_read (input_plugin_t *this_gen, void *buf_gen, off_t l
   return done;
 }
 
-static buf_element_t *stdin_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo,
-					       off_t todo) {
-
-  off_t                 total_bytes;
-  /* stdin_input_plugin_t  *this = (stdin_input_plugin_t *) this_gen; */
-  buf_element_t         *buf = fifo->buffer_pool_alloc (fifo);
-
-  if (todo > buf->max_size)
-    todo = buf->max_size;
-  if (todo < 0) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->content = buf->mem;
-  buf->type = BUF_DEMUX_BLOCK;
-
-  total_bytes = stdin_plugin_read (this_gen, (char*)buf->content, todo);
-
-  if (total_bytes != todo) {
-    buf->free_buffer (buf);
-    return NULL;
-  }
-
-  buf->size = total_bytes;
-
-  return buf;
-}
-
 /* forward reference */
 static off_t stdin_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
 
@@ -213,54 +264,13 @@ static off_t stdin_plugin_seek (input_plugin_t *this_gen, off_t offset, int orig
 
   lprintf ("seek %"PRId64" offset, %d origin...\n", offset, origin);
 
-  if ((origin == SEEK_CUR) && (offset >= 0)) {
-
-    for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
-      if( this_gen->read (this_gen, this->seek_buf, BUFSIZE) <= 0 )
-        return this->curpos;
-    }
-
-    this_gen->read (this_gen, this->seek_buf, offset);
-  }
-
-  if (origin == SEEK_SET) {
-
-    if (offset < this->curpos) {
-
-      if( this->curpos <= this->preview_size )
-        this->curpos = offset;
-      else
-        xprintf (this->xine, XINE_VERBOSITY_LOG,
-                 _("stdin: cannot seek back! (%" PRIdMAX " > %" PRIdMAX ")\n"),
-                 (intmax_t)this->curpos, (intmax_t)offset);
-
-    } else {
-      offset -= this->curpos;
-
-      for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
-        if( this_gen->read (this_gen, this->seek_buf, BUFSIZE) <= 0 )
-          return this->curpos;
-      }
-
-      this_gen->read (this_gen, this->seek_buf, offset);
-    }
-  }
-
-  return this->curpos;
+  return _x_input_seek_preview (this_gen, offset, origin,
+                                &this->curpos, -1, this->preview_size);
 }
 
 static off_t stdin_plugin_get_length(input_plugin_t *this_gen) {
 
-  return 0;
-}
-
-static uint32_t stdin_plugin_get_capabilities(input_plugin_t *this_gen) {
-
-  return INPUT_CAP_PREVIEW;
-}
-
-static uint32_t stdin_plugin_get_blocksize(input_plugin_t *this_gen) {
-
+  (void)this_gen;
   return 0;
 }
 
@@ -279,8 +289,11 @@ static const char* stdin_plugin_get_mrl (input_plugin_t *this_gen) {
 static void stdin_plugin_dispose (input_plugin_t *this_gen ) {
   stdin_input_plugin_t *this = (stdin_input_plugin_t *) this_gen;
 
-  xprintf (this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": waited %d of %d reads.\n",
-    this->num_waits, this->num_reads);
+  xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+    LOG_MODULE ": %d reads, %d fd reads, %d fd waits.\n",
+    this->num_reads, this->num_fd_reads, this->num_fd_waits);
+
+  free (this->ring_buf);
 
   if (this->nbc)
     nbc_close (this->nbc);
@@ -300,17 +313,34 @@ static void stdin_plugin_dispose (input_plugin_t *this_gen ) {
   free (this);
 }
 
+static uint32_t stdin_plugin_get_capabilities (input_plugin_t *this_gen) {
+  stdin_input_plugin_t *this = (stdin_input_plugin_t *) this_gen;
+
+  return INPUT_CAP_PREVIEW | INPUT_CAP_SIZED_PREVIEW | ((this && this->ring_buf) ? INPUT_CAP_NO_CACHE : 0);
+}
+
 static int stdin_plugin_get_optional_data (input_plugin_t *this_gen,
 					   void *data, int data_type) {
   stdin_input_plugin_t *this = (stdin_input_plugin_t *) this_gen;
 
   switch (data_type) {
-  case INPUT_OPTIONAL_DATA_PREVIEW:
-
-    memcpy (data, this->preview, this->preview_size);
-    return this->preview_size;
-
-    break;
+    case INPUT_OPTIONAL_DATA_PREVIEW:
+      if (!data || (this->preview_size <= 0))
+        break;
+      memcpy (data, this->preview, this->preview_size);
+      return this->preview_size;
+    case INPUT_OPTIONAL_DATA_SIZED_PREVIEW:
+      if (!data || (this->preview_size <= 0))
+        break;
+      {
+        int want;
+        memcpy (&want, data, sizeof (want));
+        want = want < 0 ? 0
+             : want > this->preview_size ? this->preview_size
+             : want;
+        memcpy (data, this->preview, want);
+        return want;
+      }
   }
 
   return INPUT_OPTIONAL_UNSUPPORTED;
@@ -334,6 +364,15 @@ static int stdin_plugin_open (input_plugin_t *this_gen ) {
       return 0;
     }
   }
+
+  this->num_reads = 0;
+  this->num_fd_reads = 0;
+  this->num_fd_waits = 0;
+
+  this->ring_write = 0;
+  this->ring_read = 0;
+  _x_freep (&this->ring_buf);
+
 #ifdef WIN32
   else setmode(this->fh, FILE_FLAGS);
 #else
@@ -343,6 +382,8 @@ static int stdin_plugin_open (input_plugin_t *this_gen ) {
     this->mode = fcntl (this->fh, F_GETFL);
     this->nonblock = !!(this->mode & O_NONBLOCK);
   }
+  if (this->nonblock)
+    this->ring_buf = malloc (RING_SIZE);
 #endif
 
   /* mrl accepted and opened successfully at this point */
@@ -396,24 +437,31 @@ static input_plugin_t *stdin_class_get_instance (input_class_t *class_gen,
     return NULL;
   }
 
+#ifndef HAVE_ZERO_SAFE_MEM
+  this->num_reads       = 0;
+  this->num_fd_reads    = 0;
+  this->num_fd_waits    = 0;
+  this->ring_write      = 0;
+  this->ring_read       = 0;
+  this->ring_buf        = NULL;
+  this->curpos          = 0;
+  this->requery_timeout = 0;
+#endif
+
   this->stream          = stream;
   this->mrl             = strdup (data);
   this->fh              = fh;
   this->xine            = stream->xine;
-  this->curpos          = 0;
-  this->num_reads       = 0;
-  this->num_waits       = 0;
   this->timeout         = 30000;
-  this->requery_timeout = 0;
 
   this->input_plugin.open              = stdin_plugin_open;
   this->input_plugin.get_capabilities  = stdin_plugin_get_capabilities;
   this->input_plugin.read              = stdin_plugin_read;
-  this->input_plugin.read_block        = stdin_plugin_read_block;
+  this->input_plugin.read_block        = _x_input_default_read_block;
   this->input_plugin.seek              = stdin_plugin_seek;
   this->input_plugin.get_current_pos   = stdin_plugin_get_current_pos;
   this->input_plugin.get_length        = stdin_plugin_get_length;
-  this->input_plugin.get_blocksize     = stdin_plugin_get_blocksize;
+  this->input_plugin.get_blocksize     = _x_input_default_get_blocksize;
   this->input_plugin.get_mrl           = stdin_plugin_get_mrl;
   this->input_plugin.dispose           = stdin_plugin_dispose;
   this->input_plugin.get_optional_data = stdin_plugin_get_optional_data;
@@ -425,21 +473,20 @@ static input_plugin_t *stdin_class_get_instance (input_class_t *class_gen,
 /*
  * stdin input plugin class stuff
  */
-static void *stdin_plugin_init_class (xine_t *xine, void *data) {
+static void *stdin_plugin_init_class (xine_t *xine, const void *data) {
 
-  input_class_t  *this;
-
-  this = calloc(1, sizeof (input_class_t));
-
-  this->get_instance       = stdin_class_get_instance;
-  this->identifier         = "stdin_fifo";
-  this->description        = N_("stdin streaming input plugin");
-  this->get_dir            = NULL;
-  this->get_autoplay_list  = NULL;
-  this->dispose            = default_input_class_dispose;
-  this->eject_media        = NULL;
-
-  return this;
+  (void)xine;
+  (void)data;
+  static const input_class_t input_stdin_class = {
+    .get_instance       = stdin_class_get_instance,
+    .identifier         = "stdin_fifo",
+    .description        = N_("stdin streaming input plugin"),
+    .get_dir            = NULL,
+    .get_autoplay_list  = NULL,
+    .dispose            = NULL,
+    .eject_media        = NULL,
+  };
+  return (void *)&input_stdin_class;
 }
 
 /*
@@ -452,7 +499,6 @@ static void *stdin_plugin_init_class (xine_t *xine, void *data) {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   INPUT_STDIN_CATALOG,
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 #endif
-
