@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 the xine project
+ * Copyright (C) 2003-2018 the xine project
  * Copyright (C) 2003 J.Asselman <j.asselman@itsec-ps.nl>
  *
  * This file is part of xine, a free video player.
@@ -256,6 +256,7 @@ struct pvrscr_s {
 
 static int pvrscr_get_priority(scr_plugin_t *scr)
 {
+  (void)scr;
   return 10; /* high priority */
 }
 
@@ -518,12 +519,12 @@ static void store_vid_frame (buf_element_t *frame)
   pthread_mutex_unlock (&this->vid_frames_lock);
 }
 
-static int extract_mrl(v4l_input_plugin_t *this, char *mrl)
+static int extract_mrl(v4l_input_plugin_t *this, const char *mrl)
 {
   char   *tuner_name = NULL;
   int     frequency  = 0;
-  char   *locator    = NULL;
-  char   *begin      = NULL;
+  const char   *locator    = NULL;
+  const char   *begin      = NULL;
 
   if (mrl == NULL) {
     lprintf("Someone passed an empty mrl\n");
@@ -1243,6 +1244,9 @@ static int v4l_adjust_realtime_speed(v4l_input_plugin_t *this, fifo_buffer_t *fi
  */
 static off_t v4l_plugin_read (input_plugin_t *this_gen, void *buf, off_t len) {
   lprintf("Read not supported\n");
+  (void)this_gen;
+  (void)buf;
+  (void)len;
   return 0;
 }
 
@@ -1271,6 +1275,9 @@ static buf_element_t *v4l_plugin_read_block (input_plugin_t *this_gen, fifo_buff
   static char           video = 0;
   int                   speed = _x_get_speed(this->stream);
 
+#ifndef LOG
+  (void)todo;
+#endif
   v4l_event_handler(this);
 
 #ifdef HAVE_ALSA
@@ -1448,6 +1455,11 @@ static buf_element_t *v4l_plugin_read_block (input_plugin_t *this_gen, fifo_buff
 static off_t v4l_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   v4l_input_plugin_t *this = (v4l_input_plugin_t *) this_gen;
 
+  (void)this_gen;
+#ifndef LOG
+  (void)offset;
+  (void)origin;
+#endif
   lprintf("seek %lld bytes, origin %d\n", offset, origin);
   return this->curpos;
 }
@@ -1462,6 +1474,7 @@ static off_t v4l_plugin_get_length (input_plugin_t *this_gen) {
     off_t                 length;
   */
 
+  (void)this_gen;
   return -1;
 }
 
@@ -1485,6 +1498,7 @@ static uint32_t v4l_plugin_get_capabilities (input_plugin_t *this_gen)
  */
 static uint32_t v4l_plugin_get_blocksize (input_plugin_t *this_gen)
 {
+  (void)this_gen;
   return 0;
 }
 
@@ -1653,6 +1667,9 @@ static int v4l_plugin_get_optional_data (input_plugin_t *this_gen,
                                          void *data, int data_type) {
   /* v4l_input_plugin_t *this = (v4l_input_plugin_t *) this_gen; */
 
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
@@ -1721,20 +1738,18 @@ static input_plugin_t *v4l_class_get_instance (input_class_t *cls_gen,
 #ifdef HAVE_ALSA
   cfg_entry_t        *entry;
 #endif
-  char               *mrl     = strdup(data);
 
   /* Example mrl:  v4l:/Television/62500 */
-  if(!mrl || strncasecmp(mrl, "v4l:/", 5)) {
-    free(mrl);
+  if(!data || strncasecmp(data, "v4l:/", 5)) {
     return NULL;
   }
 
   this = calloc(1, sizeof (v4l_input_plugin_t));
-
-  extract_mrl(this, mrl);
+  if (!this)
+    return NULL;
 
   this->stream        = stream;
-  this->mrl           = mrl;
+  this->mrl           = strdup(data);
   this->video_buf     = NULL;
   this->video_fd      = -1;
   this->radio_fd      = -1;
@@ -1744,6 +1759,8 @@ static input_plugin_t *v4l_class_get_instance (input_class_t *cls_gen,
   this->pcm_data      = NULL;
   this->pcm_hwparams  = NULL;
 
+  extract_mrl(this, this->mrl);
+
   /* Audio */
   this->pcm_stream    = SND_PCM_STREAM_CAPTURE;
   entry = this->stream->xine->config->lookup_entry(this->stream->xine->config,
@@ -1890,21 +1907,20 @@ static input_plugin_t *v4l_class_get_radio_instance (input_class_t *cls_gen,
 /*
  * v4l input plugin class stuff
  */
-static void *init_video_class (xine_t *xine, void *data)
+static void *init_video_class (xine_t *xine, const void *data)
 {
-  input_class_t      *this;
+  static const input_class_t v4l_video_input_class = {
+    .get_instance       = v4l_class_get_video_instance,
+    .identifier         = "v4l",
+    .description        = N_("v4l tv input plugin"),
+    .get_dir            = NULL,
+    .get_autoplay_list  = NULL,
+    .dispose            = NULL,
+    .eject_media        = NULL,
+  };
+
   config_values_t    *config = xine->config;
 
-  this = calloc(1, sizeof (input_class_t));
-
-  this->get_instance       = v4l_class_get_video_instance;
-  this->identifier         = "v4l";
-  this->description        = N_("v4l tv input plugin");
-  this->get_dir            = NULL;
-  this->get_autoplay_list  = NULL;
-  this->dispose            = default_input_class_dispose;
-  this->eject_media        = NULL;
-
   config->register_filename (config, "media.video4linux.video_device",
 			   VIDEO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 			   _("v4l video device"),
@@ -1923,31 +1939,32 @@ static void *init_video_class (xine_t *xine, void *data)
                         _("Selects the TV standard of the input signals. "
                         "Either: AUTO, PAL, NTSC or SECAM. "), 20, NULL, NULL);
 
-  return this;
+  (void)data;
+  return (void *)&v4l_video_input_class;
 }
 
-static void *init_radio_class (xine_t *xine, void *data)
+static void *init_radio_class (xine_t *xine, const void *data)
 {
-  input_class_t *this;
+  static const input_class_t v4l_radio_input_class = {
+    .get_instance       = v4l_class_get_radio_instance,
+    .identifier         = "v4l",
+    .description        = N_("v4l radio input plugin"),
+    .get_dir            = NULL,
+    .get_autoplay_list  = NULL,
+    .dispose            = NULL,
+    .eject_media        = NULL,
+  };
+
   config_values_t *config = xine->config;
 
-  this = calloc (1, sizeof (input_class_t));
-
-  this->get_instance       = v4l_class_get_radio_instance;
-  this->identifier         = "v4l";
-  this->description        = N_("v4l radio input plugin");
-  this->get_dir            = NULL;
-  this->get_autoplay_list  = NULL;
-  this->dispose            = default_input_class_dispose;
-  this->eject_media        = NULL;
-
   config->register_filename (config, "media.video4linux.radio_device",
 			   RADIO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 			   _("v4l radio device"),
 			   _("The path to your Video4Linux radio device."),
 			   10, NULL, NULL);
 
-  return this;
+  (void)data;
+  return (void *)&v4l_radio_input_class;
 }
 
 /*
@@ -1958,7 +1975,7 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "v4l_radio", XINE_VERSION_CODE, NULL, init_radio_class },
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "v4l_tv", XINE_VERSION_CODE, NULL, init_video_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
 /*
