@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -797,11 +797,13 @@ static off_t vcd_plugin_get_current_pos (input_plugin_t *this_gen){
 
 static uint32_t vcd_plugin_get_capabilities (input_plugin_t *this_gen) {
 
+  (void)this_gen;
   return INPUT_CAP_SEEKABLE | INPUT_CAP_BLOCK ;
 }
 
 static uint32_t vcd_plugin_get_blocksize (input_plugin_t *this_gen) {
 
+  (void)this_gen;
   return VCDSECTORSIZE;
 }
 
@@ -825,6 +827,9 @@ static const char* vcd_plugin_get_mrl (input_plugin_t *this_gen) {
 static int vcd_plugin_get_optional_data (input_plugin_t *this_gen,
 					 void *data, int data_type) {
 
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
@@ -888,21 +893,21 @@ static int vcd_plugin_open (input_plugin_t *this_gen) {
 }
 
 static input_plugin_t *vcd_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream,
-				    const char *data) {
+                                               const char *mrl) {
 
   vcd_input_class_t  *cls = (vcd_input_class_t *) cls_gen;
   vcd_input_plugin_t *this;
-  char               *mrl = strdup(data);
 
   if (strncasecmp (mrl, "vcdo:/",6)) {
-    free (mrl);
     return 0;
   }
 
   this = calloc(1, sizeof(vcd_input_plugin_t));
+  if (!this)
+    return NULL;
 
   this->stream = stream;
-  this->mrl    = mrl;
+  this->mrl    = strdup(mrl);
   this->fd     = -1;
 
   this->input_plugin.open              = vcd_plugin_open;
@@ -945,7 +950,14 @@ static void vcd_class_dispose (input_class_t *this_gen) {
   config->unregister_callback(config, "media.vcd.device");
 
   vcd_filelist_dispose(this);
-  free (this->mrls);
+
+  while (this->mrls_allocated_entries) {
+    this->mrls_allocated_entries--;
+    MRL_ZERO(this->mrls[this->mrls_allocated_entries]);
+    _x_freep(&this->mrls[this->mrls_allocated_entries]);
+  }
+  _x_freep(&this->mrls);
+
   free (this);
 }
 
@@ -999,7 +1011,7 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
       this->mrls[(i-1)] = calloc(1, sizeof(xine_mrl_t));
     }
     else {
-      memset(this->mrls[(i-1)], 0, sizeof(xine_mrl_t));
+      MRL_ZERO(this->mrls[(i-1)]);
     }
 
     this->mrls[i-1]->mrl  = _x_asprintf("vcdo:/%d", i);
@@ -1014,8 +1026,9 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
    * Freeing exceeded mrls if exists.
    */
   while(this->mrls_allocated_entries > *num_files) {
-    MRL_ZERO(this->mrls[this->mrls_allocated_entries - 1]);
-    free(this->mrls[this->mrls_allocated_entries--]);
+    this->mrls_allocated_entries--;
+    MRL_ZERO(this->mrls[this->mrls_allocated_entries]);
+    _x_freep(&this->mrls[this->mrls_allocated_entries]);
   }
 
   this->mrls[*num_files] = NULL;
@@ -1063,12 +1076,15 @@ static const char * const * vcd_class_get_autoplay_list (input_class_t *this_gen
   return (const char * const *)this->filelist;
 }
 
-static void *init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, const void *data) {
 
   vcd_input_class_t  *this;
   config_values_t    *config = xine->config;
 
+  (void)data;
   this = calloc(1, sizeof (vcd_input_class_t));
+  if (!this)
+    return NULL;
 
   this->xine   = xine;
 
@@ -1099,5 +1115,5 @@ static void *init_class (xine_t *xine, void *data) {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "VCDO", XINE_VERSION_CODE, NULL, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
