@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2017 the xine project
+ * Copyright (C) 2002-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -30,30 +30,20 @@
 #include "config.h"
 #endif
 
-#include <unistd.h>
 #include <stdio.h>
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
 #include <stdlib.h>
-#include <time.h>
 
 #if defined(HAVE_ICONV) && defined(HAVE_NL_LANGINFO)
 #define USE_ICONV
 #include <iconv.h>
 #include <locale.h>
 #include <langinfo.h>
+#else
+# ifndef ICONV_CONST
+#   define ICONV_CONST const
+# endif
 #endif
 
 /********** logging **********/
@@ -117,13 +107,7 @@ struct mms_s {
   int           s;
 
   /* url parsing */
-  char         *url;
-  char         *proto;
-  char         *host;
-  int           port;
-  char         *user;
-  char         *password;
-  char         *uri;
+  xine_url_t    url;
 
   /* command to send */
   char          scmd[CMD_HEADER_LEN + CMD_BODY_LEN];
@@ -542,7 +526,7 @@ static int interp_asf_header (mms_t *this) {
 
 static const char mmst_proto_s[][8] = { "mms", "mmst", "" };
 
-static int mmst_valid_proto (char *proto) {
+static int mmst_valid_proto (const char *proto) {
   int i = 0;
 
   lprintf("mmst_valid_proto\n");
@@ -581,16 +565,17 @@ static void report_progress (xine_stream_t *stream, int p) {
 static int mms_tcp_connect(mms_t *this) {
   int progress, res;
 
-  if (!this->port) this->port = MMST_PORT;
+  if (!this->url.port)
+    this->url.port = MMST_PORT;
 
   /*
    * try to connect
    */
-  lprintf("try to connect to %s on port %d \n", this->host, this->port);
-  this->s = _x_io_tcp_connect (this->stream, this->host, this->port);
+  lprintf("try to connect to %s on port %d \n", this->url.host, this->url.port);
+  this->s = _x_io_tcp_connect (this->stream, this->url.host, this->url.port);
   if (this->s < 0) {
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-             "failed to connect '%s'\n", this->host);
+             "failed to connect '%s'\n", this->url.host);
     return 1;
   }
 
@@ -689,7 +674,6 @@ mms_t *mms_connect (xine_stream_t *stream, const char *url, int bandwidth) {
   this = calloc(1, sizeof (mms_t));
 
   this->stream          = stream;
-  this->url             = strdup (url);
   this->s               = -1;
   this->seq_num         = 0;
   this->scmd_body       = this->scmd + CMD_HEADER_LEN + CMD_PREFIX_LEN;
@@ -703,13 +687,12 @@ mms_t *mms_connect (xine_stream_t *stream, const char *url, int bandwidth) {
 
   report_progress (stream, 0);
 
-  if (!_x_parse_url (this->url, &this->proto, &this->host, &this->port,
-                     &this->user, &this->password, &this->uri, NULL)) {
+  if (!_x_url_parse2 (url, &this->url)) {
     lprintf ("invalid url\n");
     goto fail;
   }
 
-  if (!mmst_valid_proto(this->proto)) {
+  if (!mmst_valid_proto(this->url.proto)) {
     lprintf ("unsupported protocol\n");
     goto fail;
   }
@@ -730,7 +713,7 @@ mms_t *mms_connect (xine_stream_t *stream, const char *url, int bandwidth) {
   lprintf("send command 0x01\n");
   mms_gen_guid(this->guid);
   snprintf (str, sizeof(str), "\x1c\x03NSPlayer/7.0.0.1956; {%s}; Host: %s",
-    this->guid, this->host);
+    this->guid, this->url.host);
   string_utf16 (url_conv, this->scmd_body, str, strlen(str) + 2);
 
   if (!send_command (this, 1, 0, 0x0004000b, strlen(str) * 2 + 8)) {
@@ -780,7 +763,7 @@ mms_t *mms_connect (xine_stream_t *stream, const char *url, int bandwidth) {
     char *path, *unescaped;
     size_t pathlen;
 
-    unescaped = strdup (this->uri);
+    unescaped = strdup (this->url.uri);
     if (!unescaped)
       goto fail;
     _x_mrl_unescape (unescaped);
@@ -913,19 +896,9 @@ fail:
 #endif
 
   if (this->s != -1)
-    close (this->s);
-  if (this->url)
-    free(this->url);
-  if (this->proto)
-    free(this->proto);
-  if (this->host)
-    free(this->host);
-  if (this->user)
-    free(this->user);
-  if (this->password)
-    free(this->password);
-  if (this->uri)
-    free(this->uri);
+    _x_io_tcp_close (this->stream, this->s);
+
+  _x_url_cleanup(&this->url);
 
   free (this);
   return NULL;
@@ -1153,19 +1126,10 @@ int mms_read (mms_t *this, char *data, int len) {
 void mms_close (mms_t *this) {
 
   if (this->s != -1)
-    close (this->s);
-  if (this->url)
-    free(this->url);
-  if (this->proto)
-    free(this->proto);
-  if (this->host)
-    free(this->host);
-  if (this->user)
-    free(this->user);
-  if (this->password)
-    free(this->password);
-  if (this->uri)
-    free(this->uri);
+    _x_io_tcp_close (this->stream, this->s);
+
+  _x_url_cleanup(&this->url);
+
   if (this->asf_header)
     asf_header_delete(this->asf_header);
   free (this);
