@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2017 the xine project
+ * Copyright (C) 2002-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -32,24 +32,10 @@
 #include "config.h"
 #endif
 
-#include <unistd.h>
 #include <stdio.h>
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
 #include <stdlib.h>
-#include <time.h>
 
 #define LOG_MODULE "mmsh"
 #define LOG_VERBOSE
@@ -155,13 +141,7 @@ struct mmsh_s {
   int           s;
 
   /* url parsing */
-  char         *url;
-  char         *proto;
-  char         *host;
-  int           port;
-  char         *user;
-  char         *password;
-  char         *uri;
+  xine_url_t    url;
 
   char          str[SCRATCH_SIZE]; /* scratch buffer to built strings */
 
@@ -198,7 +178,7 @@ static int send_command (mmsh_t *this, char *cmd)  {
   lprintf ("send_command:\n%s\n", cmd);
 
   const size_t length = strlen(cmd);
-  if (_x_io_tcp_write(this->stream, this->s, cmd, length) != length) {
+  if ((size_t)_x_io_tcp_write (this->stream, this->s, cmd, length) != length) {
     xprintf (this->stream->xine, XINE_LOG_MSG, _("libmmsh: send error\n"));
     return 0;
   }
@@ -452,7 +432,7 @@ static int interp_header (mmsh_t *this) {
 
 static const char mmsh_proto_s[][8] = { "mms", "mmsh", "" };
 
-static int mmsh_valid_proto (char *proto) {
+static int mmsh_valid_proto (const char *proto) {
   int i = 0;
 
   lprintf("mmsh_valid_proto\n");
@@ -490,18 +470,19 @@ static void report_progress (xine_stream_t *stream, int p) {
 static int mmsh_tcp_connect(mmsh_t *this) {
   int progress, res;
 
-  if (!this->port) this->port = MMSH_PORT;
+  if (!this->url.port)
+    this->url.port = MMSH_PORT;
 
   /*
    * try to connect
    */
-  lprintf("try to connect to %s on port %d \n", this->host, this->port);
+  lprintf("try to connect to %s on port %d \n", this->url.host, this->url.port);
 
-  this->s = _x_io_tcp_connect (this->stream, this->host, this->port);
+  this->s = _x_io_tcp_connect (this->stream, this->url.host, this->url.port);
 
   if (this->s < 0) {
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-             "libmmsh: failed to connect '%s'\n", this->host);
+             "libmmsh: failed to connect '%s'\n", this->url.host);
     return 1;
   }
 
@@ -531,8 +512,8 @@ static int mmsh_connect_int(mmsh_t *this, int bandwidth) {
   /* first request */
   lprintf("first http request\n");
 
-  snprintf (this->str, SCRATCH_SIZE, mmsh_FirstRequest, this->uri,
-            this->host, this->port, 1);
+  snprintf (this->str, SCRATCH_SIZE, mmsh_FirstRequest, this->url.uri,
+            this->url.host, this->url.port, 1);
 
   if (!send_command (this, this->str))
     return 0;
@@ -545,7 +526,8 @@ static int mmsh_connect_int(mmsh_t *this, int bandwidth) {
   if (!interp_header (this))
     return 0;
 
-  close (this->s);
+  _x_io_tcp_close (this->stream, this->s);
+  this->s = -1;
   report_progress (this->stream, 20);
 
   asf_header_choose_streams (this->asf_header, bandwidth,
@@ -574,6 +556,7 @@ static int mmsh_connect_int2(mmsh_t *this, int bandwidth) {
   /* second request */
   lprintf("second http request\n");
 
+  (void)bandwidth;
   /* stream selection string */
   /* The same selection is done with mmst */
   /* 0 means selected */
@@ -598,13 +581,13 @@ static int mmsh_connect_int2(mmsh_t *this, int bandwidth) {
 
   switch (this->stream_type) {
     case MMSH_SEEKABLE:
-      snprintf (this->str, SCRATCH_SIZE, mmsh_SeekableRequest, this->uri,
-                this->host, this->port, this->start_time, 0, 0, 2, 0,
+      snprintf (this->str, SCRATCH_SIZE, mmsh_SeekableRequest, this->url.uri,
+                this->url.host, this->url.port, this->start_time, 0, 0, 2, 0,
                 this->asf_header->stream_count, stream_selection);
       break;
     case MMSH_LIVE:
-      snprintf (this->str, SCRATCH_SIZE, mmsh_LiveRequest, this->uri,
-                this->host, this->port, 2,
+      snprintf (this->str, SCRATCH_SIZE, mmsh_LiveRequest, this->url.uri,
+                this->url.host, this->url.port, 2,
                 this->asf_header->stream_count, stream_selection);
       break;
   }
@@ -642,7 +625,7 @@ mmsh_t *mmsh_connect (xine_stream_t *stream, const char *url, int bandwidth) {
   this = calloc(1, sizeof (mmsh_t));
 
   this->stream          = stream;
-  this->url             = strdup(url);
+  //this->url             = strdup(url);
   this->s               = -1;
   this->asf_header_len  = 0;
   this->asf_header_read = 0;
@@ -653,13 +636,12 @@ mmsh_t *mmsh_connect (xine_stream_t *stream, const char *url, int bandwidth) {
 
   report_progress (stream, 0);
 
-  if (!_x_parse_url (this->url, &this->proto, &this->host, &this->port,
-                     &this->user, &this->password, &this->uri, NULL)) {
+  if (!_x_url_parse2 (url, &this->url)) {
     xine_log (this->stream->xine, XINE_LOG_MSG, _("invalid url\n"));
     goto fail;
   }
 
-  if (!mmsh_valid_proto(this->proto)) {
+  if (!mmsh_valid_proto(this->url.proto)) {
     xine_log (this->stream->xine, XINE_LOG_MSG, _("unsupported protocol\n"));
     goto fail;
   }
@@ -681,14 +663,9 @@ mmsh_t *mmsh_connect (xine_stream_t *stream, const char *url, int bandwidth) {
 fail:
   lprintf("mmsh_connect: failed\n" );
   if (this->s != -1)
-    close(this->s);
+    _x_io_tcp_close(this->stream, this->s);
 
-  free(this->url);
-  free(this->proto);
-  free(this->host);
-  free(this->user);
-  free(this->password);
-  free(this->uri);
+  _x_url_cleanup(&this->url);
 
   free(this);
 
@@ -718,7 +695,8 @@ static int get_media_packet (mmsh_t *this) {
         if (this->chunk_seq_number == 0)
           return 0;
 
-        close(this->s);
+        _x_io_tcp_close(this->stream, this->s);
+        this->s = -1;
 
         if (mmsh_tcp_connect(this))
           return 0;
@@ -865,17 +843,12 @@ void mmsh_close (mmsh_t *this) {
   lprintf("mmsh_close\n");
 
   if (this->s != -1)
-    close(this->s);
+    _x_io_tcp_close(this->stream, this->s);
 
   if (this->asf_header)
     asf_header_delete(this->asf_header);
 
-  free(this->url);
-  free(this->proto);
-  free(this->host);
-  free(this->user);
-  free(this->password);
-  free(this->uri);
+  _x_url_cleanup(&this->url);
 
   free(this);
 }
