@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -23,12 +23,18 @@
 #ifndef HAVE_NET_BUF_CTRL_H
 #define HAVE_NET_BUF_CTRL_H
 
-#include <xine/xine_internal.h>
-
-typedef struct nbc_s nbc_t;
-
-nbc_t *nbc_init (xine_stream_t *xine) XINE_MALLOC;
-
-void nbc_close (nbc_t *this);
-
+#ifdef __cplusplus
+extern "C" {
 #endif
+
+#include <xine/input_plugin.h>
+
+#define nbc_t xine_nbc_t
+#define nbc_init(s) xine_nbc_init (s)
+#define nbc_close(nbc) xine_nbc_close (nbc)
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* HAVE_NET_BUF_CTRL_H */
