@@ -704,7 +704,7 @@ static off_t
 vcd_plugin_read (input_plugin_t *this_gen, void *vbuf, const off_t nlen)
 {
   vcd_input_plugin_t *this = (vcd_input_plugin_t *)this_gen;
-  char *buf = vbuf;
+  uint8_t *buf = vbuf;
   dbg_print (this->class, (INPUT_DBG_CALL|INPUT_DBG_EXT),
             "Called with nlen %u\n", (unsigned int) nlen);
 
@@ -1120,22 +1120,31 @@ static bool vcd_handle_events (vcd_input_plugin_t *this) {
      */
     case XINE_EVENT_INPUT_NUMBER_9:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_8:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_7:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_6:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_5:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_4:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_3:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_2:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_1:
       digit_entered++;
+      /* fall through */
     case XINE_EVENT_INPUT_NUMBER_0:
       {
         number_addend *= 10;
@@ -1880,13 +1889,15 @@ VCD_ENUM_CALLBACK (vcd_default_autoplay_cb, vcdinfo_item_enum_t, default_autopla
 VCD_NUM_CALLBACK (vcd_debug_cb, vcdplayer_debug)
 
 static void *
-vcd_init (xine_t *xine, void *data)
+vcd_init (xine_t *xine, const void *data)
 {
   vcd_input_class_t  *class;
   config_values_t    *config;
 
   xprintf (xine, XINE_VERBOSITY_DEBUG, "input_vcd: init class\n");
 
+  (void)data;
+
   class = calloc (1, sizeof (vcd_input_class_t));
   if (!class)
     return NULL;
@@ -2074,7 +2085,7 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, SHORT_PLUGIN_NAME,
     XINE_VERSION_CODE, NULL, vcd_init },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
 /*
