@@ -1,3 +1,7 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -91,6 +95,7 @@ static void iflow_execute(FastInstructionFlow *_this, GoomSL *gsl);
 void iflow_free(InstructionFlow *_this)
 { /* {{{ */
   goom_hash_free(_this->labels);
+  free(_this->instr);
   free(_this); /*TODO: finir cette fonction */
 } /* }}} */
 
@@ -1423,9 +1428,21 @@ int gsl_is_compiled(GoomSL *gss)
 
 void gsl_free(GoomSL *gss)
 { /* {{{ */
-  iflow_free(gss->iflow);
-  free(gss->vars);
-  free(gss->functions);
+  if (gss->iflow)
+    iflow_free(gss->iflow);
+  if (gss->vars)
+    goom_hash_free(gss->vars);
+  if (gss->functions)
+    goom_hash_free(gss->functions);
+
+  free(gss->gsl_struct);
+  free(gss->ptrArray);
+
+  if (gss->data_heap)
+    goom_heap_delete(gss->data_heap);
+  if (gss->structIDS)
+    goom_hash_free(gss->structIDS);
+
   free(gss);
 } /* }}} */
 
@@ -1480,7 +1497,7 @@ void gsl_append_file_to_buffer(const char *fname, char **buffer)
     }
     
     /* add fname to the already imported files. */
-    strcpy(gsl_already_imported[gsl_nb_import++], fname);
+    strlcpy(gsl_already_imported[gsl_nb_import], fname, sizeof(gsl_already_imported[0]));
 
     /* load the file */
     fbuffer = gsl_read_file(fname);
