@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -39,17 +39,6 @@
 
 /* FIXME: This plugin needs to handle overlays as well. */
 
-/* plugin class initialization function */
-static void *mosaico_init_plugin(xine_t *xine, void *);
-
-/* plugin catalog information */
-static const post_info_t mosaico_special_info = { XINE_POST_TYPE_VIDEO_COMPOSE };
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_POST, 10, "mosaico", XINE_VERSION_CODE, &mosaico_special_info, &mosaico_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
 
 typedef struct mosaico_parameters_s {
   unsigned int  pip_num;
@@ -69,14 +58,8 @@ PARAM_ITEM(POST_PARAM_TYPE_INT, h, NULL, 0, INT_MAX, 150,
   "height of the pasted picture")
 END_PARAM_DESCR(mosaico_param_descr)
 
-typedef struct post_class_mosaico_s post_class_mosaico_t;
 typedef struct post_mosaico_s post_mosaico_t;
 
-struct post_class_mosaico_s {
-  post_class_t    class;
-  xine_t         *xine;
-};
-
 /* plugin structures */
 typedef struct mosaico_pip_s mosaico_pip_t;
 struct mosaico_pip_s {
@@ -87,7 +70,6 @@ struct mosaico_pip_s {
 
 struct post_mosaico_s {
   post_plugin_t    post;
-  xine_post_in_t   parameter_input;
 
   mosaico_pip_t   *pip;
   int64_t          vpts_limit;
@@ -98,137 +80,17 @@ struct post_mosaico_s {
   unsigned int     pip_count;
 };
 
-/* plugin class functions */
-static post_plugin_t *mosaico_open_plugin(post_class_t *class_gen, int inputs,
-					 xine_audio_port_t **audio_target,
-					 xine_video_port_t **video_target);
-
-/* plugin instance functions */
-static void           mosaico_dispose(post_plugin_t *this_gen);
-
 /* parameter functions */
-static xine_post_api_descr_t *mosaico_get_param_descr(void);
-static int            mosaico_set_parameters(xine_post_t *this_gen, void *param_gen);
-static int            mosaico_get_parameters(xine_post_t *this_gen, void *param_gen);
-static char          *mosaico_get_help(void);
-
-/* replaced video port functions */
-static void           mosaico_close(xine_video_port_t *port_gen, xine_stream_t *stream);
-
-/* frame intercept check */
-static int            mosaico_intercept_frame(post_video_port_t *port, vo_frame_t *frame);
-
-/* replaced vo_frame functions */
-static int            mosaico_draw_background(vo_frame_t *frame, xine_stream_t *stream);
-static int            mosaico_draw(vo_frame_t *frame, xine_stream_t *stream);
-
-
-static void *mosaico_init_plugin(xine_t *xine, void *data)
-{
-  post_class_mosaico_t *this = calloc(1, sizeof(post_class_mosaico_t));
-
-  if (!this)
-    return NULL;
-
-  this->class.open_plugin     = mosaico_open_plugin;
-  this->class.identifier      = "mosaico";
-  this->class.description     = N_("Mosaico is a picture in picture (pip) post plugin");
-  this->class.dispose         = default_post_class_dispose;
-  this->xine                  = xine;
-
-  return &this->class;
-}
-
-static post_plugin_t *mosaico_open_plugin(post_class_t *class_gen, int inputs,
-					 xine_audio_port_t **audio_target,
-					 xine_video_port_t **video_target)
-{
-  post_mosaico_t       *this = calloc(1, sizeof(post_mosaico_t));
-  post_in_t            *input;
-  xine_post_in_t       *input_api;
-  post_out_t           *output;
-  post_video_port_t    *port;
-  static xine_post_api_t post_api =
-    { mosaico_set_parameters, mosaico_get_parameters, mosaico_get_param_descr, mosaico_get_help };
-  int i;
-
-  lprintf("mosaico open\n");
-
-  if (inputs < 2 || !this || !video_target || !video_target[0]) {
-    free(this);
-    return NULL;
-  }
-
-  _x_post_init(&this->post, 0, inputs);
-
-  this->pip       = (mosaico_pip_t *)calloc((inputs - 1), sizeof(mosaico_pip_t));
-  this->pip_count = inputs - 1;
-
-  pthread_cond_init(&this->vpts_limit_changed, NULL);
-  pthread_mutex_init(&this->mutex, NULL);
-
-  /* the port for the background video */
-  port = _x_post_intercept_video_port(&this->post, video_target[0], &input, &output);
-  port->intercept_frame = mosaico_intercept_frame;
-  port->new_frame->draw = mosaico_draw_background;
-  port->port_lock       = &this->mutex;
-  port->frame_lock      = &this->mutex;
-  input->xine_in.name   = "video in 0";
-  this->post.xine_post.video_input[0] = &port->new_port;
-
-  for (i = 0; i < inputs - 1; i++) {
-    this->pip[i].x = 50;
-    this->pip[i].y = 50;
-    this->pip[i].w = 150;
-    this->pip[i].h = 150;
-    this->pip[i].input_name = _x_asprintf("video in %d", i+1);
-
-    port = _x_post_intercept_video_port(&this->post, video_target[0], &input, NULL);
-    port->new_port.close  = mosaico_close;
-    port->intercept_frame = mosaico_intercept_frame;
-    port->new_frame->draw = mosaico_draw;
-    port->port_lock       = &this->mutex;
-    port->frame_lock      = &this->mutex;
-    input->xine_in.name   = this->pip[i].input_name;
-    this->post.xine_post.video_input[i+1] = &port->new_port;
-  }
-
-  input_api       = &this->parameter_input;
-  input_api->name = "parameters";
-  input_api->type = XINE_POST_DATA_PARAMETERS;
-  input_api->data = &post_api;
-  xine_list_push_back(this->post.input, input_api);
-
-  this->post.dispose = mosaico_dispose;
-
-  return &this->post;
-}
-
-static void mosaico_dispose(post_plugin_t *this_gen)
-{
-  post_mosaico_t *this = (post_mosaico_t *)this_gen;
-
-  if (_x_post_dispose(this_gen)) {
-    int i;
-    for (i = 0; i < this->pip_count; i++)
-      free(this->pip[i].input_name);
-    free(this->pip);
-    pthread_cond_destroy(&this->vpts_limit_changed);
-    pthread_mutex_destroy(&this->mutex);
-    free(this);
-  }
-}
-
 
 static xine_post_api_descr_t *mosaico_get_param_descr(void)
 {
   return &mosaico_param_descr;
 }
 
-static int mosaico_set_parameters(xine_post_t *this_gen, void *param_gen)
+static int mosaico_set_parameters(xine_post_t *this_gen, const void *param_gen)
 {
   post_mosaico_t *this = (post_mosaico_t *)this_gen;
-  mosaico_parameters_t *param = (mosaico_parameters_t *)param_gen;
+  const mosaico_parameters_t *param = (const mosaico_parameters_t *)param_gen;
 
   if (param->pip_num > this->pip_count) return 0;
   this->pip[param->pip_num - 1].x = param->x;
@@ -255,22 +117,25 @@ static int mosaico_get_parameters(xine_post_t *this_gen, void *param_gen)
 static char *mosaico_get_help(void)
 {
   return _("Mosaico does simple picture in picture effects.\n"
-	   "\n"
-	   "Parameters\n"
-	   "  pip_num: the number of the picture slot the following settings apply to\n"
-	   "  x: the x coordinate of the left upper corner of the picture\n"
-	   "  y: the y coordinate of the left upper corner of the picture\n"
-	   "  w: the width of the picture\n"
-	   "  h: the height of the picture\n");
+           "\n"
+           "Parameters\n"
+           "  pip_num: the number of the picture slot the following settings apply to\n"
+           "  x: the x coordinate of the left upper corner of the picture\n"
+           "  y: the y coordinate of the left upper corner of the picture\n"
+           "  w: the width of the picture\n"
+           "  h: the height of the picture\n");
 }
 
+/* replaced video port functions */
 
 static void mosaico_close(xine_video_port_t *port_gen, xine_stream_t *stream)
 {
   post_video_port_t *port = (post_video_port_t *)port_gen;
   post_mosaico_t *this = (post_mosaico_t *)port->post;
   vo_frame_t *free_frame;
-  int pip_num;
+  unsigned int pip_num;
+
+  (void)stream;
 
   for (pip_num = 0; pip_num < this->pip_count; pip_num++)
     if (this->post.xine_post.video_input[pip_num+1] == port_gen) break;
@@ -287,13 +152,17 @@ static void mosaico_close(xine_video_port_t *port_gen, xine_stream_t *stream)
   _x_post_dec_usage(port);
 }
 
+/* frame intercept check */
 
 static int mosaico_intercept_frame(post_video_port_t *port, vo_frame_t *frame)
 {
+  (void)port;
+
   /* TODO: only YV12 supported */
   return (frame->format == XINE_IMGFMT_YV12);
 }
 
+/* replaced vo_frame functions */
 
 static void frame_copy_content(vo_frame_t *to, vo_frame_t *from)
 {
@@ -354,8 +223,8 @@ static void frame_paste(post_mosaico_t *this, vo_frame_t *background, int pip_nu
     target_offset = 0;
     for (j = 0; j < target_height; j++, target_offset += (background_width - target_width))
       for (i = 0; i < target_width; i++, target_offset++) {
-	source_offset = ((i * scale_x) >> shift_x) + (((j * scale_y) >> shift_y) * source_width);
-	background->base[0][pos + target_offset] = this->pip[pip_num].frame->base[0][source_offset];
+        source_offset = ((i * scale_x) >> shift_x) + (((j * scale_y) >> shift_y) * source_width);
+        background->base[0][pos + target_offset] = this->pip[pip_num].frame->base[0][source_offset];
       }
 
     background_width = (background_width + 1) / 2;
@@ -370,16 +239,16 @@ static void frame_paste(post_mosaico_t *this, vo_frame_t *background, int pip_nu
     target_offset = 0;
     for (j = 0; j < target_height; j++, target_offset += (background_width - target_width))
       for (i = 0; i < target_width; i++, target_offset++) {
-	source_offset = ((i * scale_x) >> shift_x) + (((j * scale_y) >> shift_y) * source_width);
-	background->base[1][pos + target_offset] = this->pip[pip_num].frame->base[1][source_offset];
+        source_offset = ((i * scale_x) >> shift_x) + (((j * scale_y) >> shift_y) * source_width);
+        background->base[1][pos + target_offset] = this->pip[pip_num].frame->base[1][source_offset];
       }
 
     /* V */
     target_offset = 0;
     for (j = 0; j < target_height; j++, target_offset += (background_width - target_width))
       for (i = 0; i < target_width; i++, target_offset++) {
-	source_offset = ((i * scale_x) >> shift_x) + (((j * scale_y) >> shift_y) * source_width);
-	background->base[2][pos + target_offset] = this->pip[pip_num].frame->base[2][source_offset];
+        source_offset = ((i * scale_x) >> shift_x) + (((j * scale_y) >> shift_y) * source_width);
+        background->base[2][pos + target_offset] = this->pip[pip_num].frame->base[2][source_offset];
       }
 
     break;
@@ -391,7 +260,8 @@ static int mosaico_draw_background(vo_frame_t *frame, xine_stream_t *stream)
   post_video_port_t *port = (post_video_port_t *)frame->port;
   post_mosaico_t *this = (post_mosaico_t *)port->post;
   vo_frame_t *background;
-  int pip_num, skip;
+  unsigned int pip_num;
+  int skip;
 
   pthread_mutex_lock(&this->mutex);
 
@@ -443,7 +313,8 @@ static int mosaico_draw(vo_frame_t *frame, xine_stream_t *stream)
   post_video_port_t *port = (post_video_port_t *)frame->port;
   post_mosaico_t *this = (post_mosaico_t *)port->post;
   vo_frame_t *free_frame;
-  int pip_num, skip;
+  unsigned int pip_num;
+  int skip;
 
   for (pip_num = 0; pip_num < this->pip_count; pip_num++)
     if (this->post.xine_post.video_input[pip_num+1] == frame->port) break;
@@ -478,3 +349,119 @@ static int mosaico_draw(vo_frame_t *frame, xine_stream_t *stream)
 
   return skip;
 }
+
+/* plugin instance functions */
+
+static void mosaico_dispose(post_plugin_t *this_gen)
+{
+  post_mosaico_t *this = (post_mosaico_t *)this_gen;
+
+  if (_x_post_dispose(this_gen)) {
+    unsigned int i;
+    for (i = 0; i < this->pip_count; i++)
+      free(this->pip[i].input_name);
+    free(this->pip);
+    pthread_cond_destroy(&this->vpts_limit_changed);
+    pthread_mutex_destroy(&this->mutex);
+    free(this);
+  }
+}
+
+static post_plugin_t *mosaico_open_plugin(post_class_t *class_gen, int inputs,
+                                          xine_audio_port_t **audio_target,
+                                          xine_video_port_t **video_target)
+{
+  post_mosaico_t       *this = calloc(1, sizeof(post_mosaico_t));
+  post_in_t            *input;
+  post_out_t           *output;
+  post_video_port_t    *port;
+  int i;
+
+  static const xine_post_api_t post_api = {
+    .set_parameters  = mosaico_set_parameters,
+    .get_parameters  = mosaico_get_parameters,
+    .get_param_descr = mosaico_get_param_descr,
+    .get_help        = mosaico_get_help,
+  };
+  static const xine_post_in_t params_input = {
+    .name = "parameters",
+    .type = XINE_POST_DATA_PARAMETERS,
+    .data = (void *)&post_api,
+  };
+
+  lprintf("mosaico open\n");
+
+  (void)class_gen;
+  (void)audio_target;
+
+  if (inputs < 2 || !this || !video_target || !video_target[0]) {
+    free(this);
+    return NULL;
+  }
+
+  _x_post_init(&this->post, 0, inputs);
+
+  this->pip       = (mosaico_pip_t *)calloc((inputs - 1), sizeof(mosaico_pip_t));
+  this->pip_count = inputs - 1;
+
+  pthread_cond_init(&this->vpts_limit_changed, NULL);
+  pthread_mutex_init(&this->mutex, NULL);
+
+  /* the port for the background video */
+  port = _x_post_intercept_video_port(&this->post, video_target[0], &input, &output);
+  port->intercept_frame = mosaico_intercept_frame;
+  port->new_frame->draw = mosaico_draw_background;
+  port->port_lock       = &this->mutex;
+  port->frame_lock      = &this->mutex;
+  input->xine_in.name   = "video in 0";
+  this->post.xine_post.video_input[0] = &port->new_port;
+
+  for (i = 0; i < inputs - 1; i++) {
+    this->pip[i].x = 50;
+    this->pip[i].y = 50;
+    this->pip[i].w = 150;
+    this->pip[i].h = 150;
+    this->pip[i].input_name = _x_asprintf("video in %d", i+1);
+
+    port = _x_post_intercept_video_port(&this->post, video_target[0], &input, NULL);
+    port->new_port.close  = mosaico_close;
+    port->intercept_frame = mosaico_intercept_frame;
+    port->new_frame->draw = mosaico_draw;
+    port->port_lock       = &this->mutex;
+    port->frame_lock      = &this->mutex;
+    input->xine_in.name   = this->pip[i].input_name;
+    this->post.xine_post.video_input[i+1] = &port->new_port;
+  }
+
+  xine_list_push_back(this->post.input, (void *)&params_input);
+
+  this->post.dispose = mosaico_dispose;
+
+  return &this->post;
+}
+
+static void *mosaico_init_plugin(xine_t *xine, const void *data)
+{
+  static const post_class_t post_mosaico_class = {
+    .open_plugin     = mosaico_open_plugin,
+    .identifier      = "mosaico",
+    .description     = N_("Mosaico is a picture in picture (pip) post plugin"),
+    .dispose         = NULL,
+  };
+
+  (void)xine;
+  (void)data;
+
+  return (void *)&post_mosaico_class;
+}
+
+/* plugin catalog information */
+static const post_info_t mosaico_special_info = {
+  .type = XINE_POST_TYPE_VIDEO_COMPOSE,
+};
+
+const plugin_info_t xine_plugin_info[] EXPORTED = {
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_POST, 10, "mosaico", XINE_VERSION_CODE, &mosaico_special_info, &mosaico_init_plugin },
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
+};
