@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -22,72 +22,16 @@
  * simple video inverter plugin
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "planar.h"
+
 #include <xine/xine_internal.h>
 #include <xine/post.h>
 
 
-/* plugin class initialization function */
-void *invert_init_plugin(xine_t *xine, void *);
-
-/* plugin class functions */
-static post_plugin_t *invert_open_plugin(post_class_t *class_gen, int inputs,
-					 xine_audio_port_t **audio_target,
-					 xine_video_port_t **video_target);
-
-/* plugin instance functions */
-static void           invert_dispose(post_plugin_t *this_gen);
-
-/* frame intercept check */
-static int            invert_intercept_frame(post_video_port_t *port, vo_frame_t *frame);
-
-/* replaced vo_frame functions */
-static int            invert_draw(vo_frame_t *frame, xine_stream_t *stream);
-
-
-void *invert_init_plugin(xine_t *xine, void *data)
-{
-  post_class_t *class = calloc(1, sizeof(post_class_t));
-
-  if (!class)
-    return NULL;
-
-  class->open_plugin     = invert_open_plugin;
-  class->identifier      = "invert";
-  class->description     = N_("inverts the colours of every video frame");
-  class->dispose         = default_post_class_dispose;
-
-  return class;
-}
-
-
-static post_plugin_t *invert_open_plugin(post_class_t *class_gen, int inputs,
-					 xine_audio_port_t **audio_target,
-					 xine_video_port_t **video_target)
-{
-  post_plugin_t *this   = calloc(1, sizeof(post_plugin_t));
-  post_in_t     *input;
-  post_out_t    *output;
-  post_video_port_t *port;
-
-  if (!this || !video_target || !video_target[0]) {
-    free(this);
-    return NULL;
-  }
-
-  _x_post_init(this, 0, 1);
-
-  port = _x_post_intercept_video_port(this, video_target[0], &input, &output);
-  port->intercept_frame = invert_intercept_frame;
-  port->new_frame->draw = invert_draw;
-  input->xine_in.name   = "video";
-  output->xine_out.name = "inverted video";
-  this->xine_post.video_input[0] = &port->new_port;
-
-  this->dispose = invert_dispose;
-
-  return this;
-}
-
 static void invert_dispose(post_plugin_t *this)
 {
   if (_x_post_dispose(this))
@@ -97,6 +41,7 @@ static void invert_dispose(post_plugin_t *this)
 
 static int invert_intercept_frame(post_video_port_t *port, vo_frame_t *frame)
 {
+  (void)port;
   return (frame->format == XINE_IMGFMT_YV12 || frame->format == XINE_IMGFMT_YUY2);
 }
 
@@ -145,3 +90,50 @@ static int invert_draw(vo_frame_t *frame, xine_stream_t *stream)
 
   return skip;
 }
+
+static post_plugin_t *invert_open_plugin(post_class_t *class_gen, int inputs,
+                                         xine_audio_port_t **audio_target,
+                                         xine_video_port_t **video_target)
+{
+  post_plugin_t *this   = calloc(1, sizeof(post_plugin_t));
+  post_in_t     *input;
+  post_out_t    *output;
+  post_video_port_t *port;
+
+  if (!this || !video_target || !video_target[0]) {
+    free(this);
+    return NULL;
+  }
+
+  (void)class_gen;
+  (void)inputs;
+  (void)audio_target;
+
+  _x_post_init(this, 0, 1);
+
+  port = _x_post_intercept_video_port(this, video_target[0], &input, &output);
+  port->intercept_frame = invert_intercept_frame;
+  port->new_frame->draw = invert_draw;
+  input->xine_in.name   = "video";
+  output->xine_out.name = "inverted video";
+  this->xine_post.video_input[0] = &port->new_port;
+
+  this->dispose = invert_dispose;
+
+  return this;
+}
+
+void *invert_init_plugin(xine_t *xine, const void *data)
+{
+  static const post_class_t post_invert_class = {
+    .open_plugin     = invert_open_plugin,
+    .identifier      = "invert",
+    .description     = N_("inverts the colours of every video frame"),
+    .dispose         = NULL,
+  };
+
+  (void)xine;
+  (void)data;
+
+  return (void *)&post_invert_class;
+}
