@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -24,6 +24,8 @@
 #include "config.h"
 #endif
 
+#include "planar.h"
+
 #include <config.h>
 
 #include <xine/xine_internal.h>
@@ -44,9 +46,6 @@
 
 #define PP_STRING_SIZE 256 /* size of pp mode string (including all options) */
 
-/* plugin class initialization function */
-void *pp_init_plugin(xine_t *xine, void *);
-
 typedef struct post_plugin_pp_s post_plugin_pp_t;
 
 /*
@@ -79,7 +78,6 @@ struct post_plugin_pp_s {
   int                frame_height;
 
   pp_parameters_t    params;
-  xine_post_in_t     params_input;
 
   /* libpostproc specific stuff */
   int                pp_flags;
@@ -90,16 +88,19 @@ struct post_plugin_pp_s {
 };
 
 
-static int set_parameters (xine_post_t *this_gen, void *param_gen) {
+static int set_parameters (xine_post_t *this_gen, const void *param_gen) {
 #ifdef PP_PARAMETERS_T
   post_plugin_pp_t *this = (post_plugin_pp_t *)this_gen;
-  pp_parameters_t *param = (pp_parameters_t *)param_gen;
+  const pp_parameters_t *param = (const pp_parameters_t *)param_gen;
 
   pthread_mutex_lock (&this->lock);
 
   memcpy( &this->params, param, sizeof(pp_parameters_t) );
 
   pthread_mutex_unlock (&this->lock);
+#else
+  (void)this_gen;
+  (void)param_gen;
 #endif
   return 1;
 }
@@ -111,6 +112,9 @@ static int get_parameters (xine_post_t *this_gen, void *param_gen) {
 
 
   memcpy( param, &this->params, sizeof(pp_parameters_t) );
+#else
+  (void)this_gen;
+  (void)param_gen;
 #endif
   return 1;
 }
@@ -137,105 +141,6 @@ static char * get_help (void) {
   return help;
 }
 
-static xine_post_api_t post_api = {
-  set_parameters,
-  get_parameters,
-  get_param_descr,
-  get_help,
-};
-
-
-/* plugin class functions */
-static post_plugin_t *pp_open_plugin(post_class_t *class_gen, int inputs,
-					 xine_audio_port_t **audio_target,
-					 xine_video_port_t **video_target);
-
-/* plugin instance functions */
-static void           pp_dispose(post_plugin_t *this_gen);
-
-/* frame intercept check */
-static int            pp_intercept_frame(post_video_port_t *port, vo_frame_t *frame);
-
-/* replaced vo_frame functions */
-static int            pp_draw(vo_frame_t *frame, xine_stream_t *stream);
-
-
-void *pp_init_plugin(xine_t *xine, void *data)
-{
-  post_class_t *class = calloc(1, sizeof(post_class_t));
-
-  if (!class)
-    return NULL;
-
-  class->open_plugin     = pp_open_plugin;
-  class->identifier      = "pp";
-  class->description     = N_("plugin for ffmpeg libpostprocess");
-  class->dispose         = default_post_class_dispose;
-
-  return class;
-}
-
-
-static post_plugin_t *pp_open_plugin(post_class_t *class_gen, int inputs,
-					 xine_audio_port_t **audio_target,
-					 xine_video_port_t **video_target)
-{
-  post_plugin_pp_t  *this = calloc(1, sizeof(post_plugin_pp_t));
-  post_in_t         *input;
-  xine_post_in_t    *input_api;
-  post_out_t        *output;
-  post_video_port_t *port;
-#if defined(ARCH_X86)
-  uint32_t           cpu_caps;
-#endif
-
-  if (!this || !video_target || !video_target[0]) {
-    free(this);
-    return NULL;
-  }
-
-  _x_post_init(&this->post, 0, 1);
-
-  this->params.quality = 3;
-  strcpy(this->params.mode, "de");
-
-  this->pp_flags = PP_FORMAT_420;
-#if defined(ARCH_X86)
-  /* Detect what cpu accel we have */
-  cpu_caps = xine_mm_accel();
-  if(cpu_caps & MM_ACCEL_X86_MMX)
-    this->pp_flags |= PP_CPU_CAPS_MMX;
-  if(cpu_caps & MM_ACCEL_X86_MMXEXT)
-    this->pp_flags |= PP_CPU_CAPS_MMX2;
-  if(cpu_caps & MM_ACCEL_X86_3DNOW)
-    this->pp_flags |= PP_CPU_CAPS_3DNOW;
-#endif
-
-  this->our_mode = NULL;
-  this->our_context = NULL;
-
-  pthread_mutex_init (&this->lock, NULL);
-
-  port = _x_post_intercept_video_port(&this->post, video_target[0], &input, &output);
-  port->intercept_frame = pp_intercept_frame;
-  port->new_frame->draw = pp_draw;
-
-  input_api       = &this->params_input;
-  input_api->name = "parameters";
-  input_api->type = XINE_POST_DATA_PARAMETERS;
-  input_api->data = &post_api;
-  xine_list_push_back(this->post.input, input_api);
-
-  input->xine_in.name     = "video";
-  output->xine_out.name   = "pped video";
-
-  this->post.xine_post.video_input[0] = &port->new_port;
-
-  this->post.dispose = pp_dispose;
-
-  return &this->post;
-}
-
 static void pp_dispose(post_plugin_t *this_gen)
 {
   post_plugin_pp_t *this = (post_plugin_pp_t *)this_gen;
@@ -256,6 +161,7 @@ static void pp_dispose(post_plugin_t *this_gen)
 
 static int pp_intercept_frame(post_video_port_t *port, vo_frame_t *frame)
 {
+  (void)port;
   return (frame->format == XINE_IMGFMT_YV12 || frame->format == XINE_IMGFMT_YUY2);
 }
 
@@ -350,3 +256,89 @@ static int pp_draw(vo_frame_t *frame, xine_stream_t *stream)
 
   return skip;
 }
+
+static post_plugin_t *pp_open_plugin(post_class_t *class_gen, int inputs,
+                                     xine_audio_port_t **audio_target,
+                                     xine_video_port_t **video_target)
+{
+  post_plugin_pp_t  *this = calloc(1, sizeof(post_plugin_pp_t));
+  post_in_t         *input;
+  post_out_t        *output;
+  post_video_port_t *port;
+#if defined(ARCH_X86)
+  uint32_t           cpu_caps;
+#endif
+
+  static const xine_post_api_t post_api = {
+    .set_parameters  = set_parameters,
+    .get_parameters  = get_parameters,
+    .get_param_descr = get_param_descr,
+    .get_help        = get_help,
+  };
+  static const xine_post_in_t params_input = {
+    .name = "parameters",
+    .type = XINE_POST_DATA_PARAMETERS,
+    .data = (void *)&post_api,
+  };
+
+  if (!this || !video_target || !video_target[0]) {
+    free(this);
+    return NULL;
+  }
+
+  (void)class_gen;
+  (void)inputs;
+  (void)audio_target;
+
+  _x_post_init(&this->post, 0, 1);
+
+  this->params.quality = 3;
+  strcpy(this->params.mode, "de");
+
+  this->pp_flags = PP_FORMAT_420;
+#if defined(ARCH_X86)
+  /* Detect what cpu accel we have */
+  cpu_caps = xine_mm_accel();
+  if(cpu_caps & MM_ACCEL_X86_MMX)
+    this->pp_flags |= PP_CPU_CAPS_MMX;
+  if(cpu_caps & MM_ACCEL_X86_MMXEXT)
+    this->pp_flags |= PP_CPU_CAPS_MMX2;
+  if(cpu_caps & MM_ACCEL_X86_3DNOW)
+    this->pp_flags |= PP_CPU_CAPS_3DNOW;
+#endif
+
+  this->our_mode = NULL;
+  this->our_context = NULL;
+
+  pthread_mutex_init (&this->lock, NULL);
+
+  port = _x_post_intercept_video_port(&this->post, video_target[0], &input, &output);
+  port->intercept_frame = pp_intercept_frame;
+  port->new_frame->draw = pp_draw;
+
+  xine_list_push_back(this->post.input, (void *)&params_input);
+
+  input->xine_in.name     = "video";
+  output->xine_out.name   = "pped video";
+
+  this->post.xine_post.video_input[0] = &port->new_port;
+
+  this->post.dispose = pp_dispose;
+
+  return &this->post;
+}
+
+void *pp_init_plugin(xine_t *xine, const void *data)
+{
+  static const post_class_t post_pp_class = {
+    .open_plugin     = pp_open_plugin,
+    .identifier      = "pp",
+    .description     = N_("plugin for ffmpeg libpostprocess"),
+    .dispose         = NULL,
+  };
+
+  (void)xine;
+  (void)data;
+
+  return (void *)&post_pp_class;
+}
