@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -49,14 +49,6 @@
 
 typedef struct post_plugin_fftscope_s post_plugin_fftscope_t;
 
-typedef struct post_class_fftscope_s post_class_fftscope_t;
-
-struct post_class_fftscope_s {
-  post_class_t post_class;
-
-  xine_t      *xine;
-};
-
 struct post_plugin_fftscope_s {
   post_plugin_t post;
 
@@ -171,7 +163,7 @@ static void draw_fftscope(post_plugin_fftscope_t *this, vo_frame_t *frame) {
 
       map_ptr = ((FFT_HEIGHT * (c+1) / this->channels -1 ) * FFT_WIDTH + i * 2) / 2;
       map_ptr_bkp = map_ptr;
-      amp_float = fft_amp(i, this->wave[c], FFT_BITS);
+      amp_float = fft_amp2 (this->fft, i, this->wave[c]);
       if (amp_float == 0)
         amp_int = 0;
       else
@@ -275,6 +267,9 @@ static int fftscope_port_open(xine_audio_port_t *port_gen, xine_stream_t *stream
   post_plugin_fftscope_t *this = (post_plugin_fftscope_t *)port->post;
   int c, i;
 
+  if (!this->metronom)
+    this->metronom = _x_metronom_init(1, 0, stream->xine);
+
   _x_post_rewire(&this->post);
   _x_post_inc_usage(port);
 
@@ -424,13 +419,14 @@ static void fftscope_dispose(post_plugin_t *this_gen)
 {
   post_plugin_fftscope_t *this = (post_plugin_fftscope_t *)this_gen;
 
-  if (_x_post_dispose(this_gen)) {
+  if (!_x_post_dispose(this_gen))
+    return;
 
+  if (this->metronom)
     this->metronom->exit(this->metronom);
 
-    _x_freep(&this->buf.mem);
-    free(this);
-  }
+  _x_freep(&this->buf.mem);
+  free(this);
 }
 
 /* plugin class functions */
@@ -439,7 +435,6 @@ static post_plugin_t *fftscope_open_plugin(post_class_t *class_gen, int inputs,
 					 xine_video_port_t **video_target)
 {
   post_plugin_fftscope_t *this  = calloc(1, sizeof(post_plugin_fftscope_t));
-  post_class_fftscope_t  *class = (post_class_fftscope_t *)class_gen;
   post_in_t              *input;
   post_out_t             *output;
   post_out_t             *outputv;
@@ -450,9 +445,10 @@ static post_plugin_t *fftscope_open_plugin(post_class_t *class_gen, int inputs,
     return NULL;
   }
 
-  _x_post_init(&this->post, 1, 0);
+  (void)class_gen;
+  (void)inputs;
 
-  this->metronom = _x_metronom_init(1, 0, class->xine);
+  _x_post_init(&this->post, 1, 0);
 
   this->vo_port = video_target[0];
 
@@ -477,19 +473,17 @@ static post_plugin_t *fftscope_open_plugin(post_class_t *class_gen, int inputs,
 }
 
 /* plugin class initialization function */
-void *fftscope_init_plugin(xine_t *xine, void *data)
+void *fftscope_init_plugin(xine_t *xine, const void *data)
 {
-  post_class_fftscope_t *class = calloc(1, sizeof(post_class_fftscope_t));
+  static const post_class_t post_fftscope_class = {
+    .open_plugin     = fftscope_open_plugin,
+    .identifier      = "FFT Scope",
+    .description     = N_("FFT Scope"),
+    .dispose         = NULL,
+  };
 
-  if (!class)
-    return NULL;
+  (void)xine;
+  (void)data;
 
-  class->post_class.open_plugin     = fftscope_open_plugin;
-  class->post_class.identifier      = "FFT Scope";
-  class->post_class.description     = N_("FFT Scope");
-  class->post_class.dispose         = default_post_class_dispose;
-
-  class->xine                       = xine;
-
-  return &class->post_class;
+  return (void *)&post_fftscope_class;
 }
