@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2017 the xine project
+ * Copyright (C) 2016-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -32,11 +32,6 @@
 #include <xine/post.h>
 #include "visualizations.h"
 
-typedef struct {
-  post_class_t  post_class;
-  xine_t       *xine;
-} post_class_tdaan_t;
-
 typedef struct {
   int x, y, width, height;
   /* these are in dB (0..-64) */
@@ -803,6 +798,9 @@ static int tdaan_port_open (
   post_audio_port_t *port = (post_audio_port_t *)port_gen;
   post_plugin_tdaan_t *this = (post_plugin_tdaan_t *)port->post;
 
+  if (!this->metronom)
+    this->metronom = _x_metronom_init (1, 0, stream->xine);
+
   _x_post_rewire (&this->post);
   _x_post_inc_usage (port);
 
@@ -1002,17 +1000,18 @@ static void tdaan_port_put_buffer (
 static void tdaan_dispose (post_plugin_t *this_gen) {
   post_plugin_tdaan_t *this = (post_plugin_tdaan_t *)this_gen;
 
-  if (_x_post_dispose (this_gen)) {
+  if (!_x_post_dispose (this_gen))
+    return;
+
+  if (this->metronom)
     this->metronom->exit (this->metronom);
-    free (this);
-  }
+  free (this);
 }
 
 /* plugin class functions */
 static post_plugin_t *tdaan_open_plugin (
   post_class_t *class_gen, int inputs, xine_audio_port_t **audio_target, xine_video_port_t **video_target) {
 
-  post_class_tdaan_t   *class = (post_class_tdaan_t *)class_gen;
   post_plugin_tdaan_t  *this  = calloc (1, sizeof (post_plugin_tdaan_t));
   post_in_t          *input;
   post_out_t         *output;
@@ -1024,9 +1023,10 @@ static post_plugin_t *tdaan_open_plugin (
     return NULL;
   }
 
-  _x_post_init (&this->post, 1, 0);
+  (void)class_gen;
+  (void)inputs;
 
-  this->metronom = _x_metronom_init (1, 0, class->xine);
+  _x_post_init (&this->post, 1, 0);
 
   this->vo_port = video_target[0];
 
@@ -1051,21 +1051,19 @@ static post_plugin_t *tdaan_open_plugin (
 }
 
 /* plugin class initialization function */
-void *tdaan_init_plugin (xine_t *xine, void *data) {
+void *tdaan_init_plugin (xine_t *xine, const void *data) {
 
-  post_class_tdaan_t *class = calloc (1, sizeof (post_class_tdaan_t));
+  static const post_class_t post_tdaan_class = {
+    .open_plugin     = tdaan_open_plugin,
+    .identifier      = "tdaudioanalyzer",
+    .description     = N_("Time Domain Audio Analyzer Visualisation"),
+    .dispose         = NULL,
+  };
 
-  if (!class)
-    return NULL;
+  (void)xine;
+  (void)data;
 
-  class->post_class.open_plugin     = tdaan_open_plugin;
-  class->post_class.identifier      = "tdaudioanalyzer";
-  class->post_class.description     = N_("Time Domain Audio Analyzer Visualisation");
-  class->post_class.dispose         = default_post_class_dispose;
-
-  class->xine                       = xine;
-
-  return class;
+  return (void *)&post_tdaan_class;
 }
 
 
