@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2017 the xine project
+ * Copyright (C) 2002-2018 the xine project
  *
  * Copyright (C) James Courtier-Dutton James@superbug.demon.co.uk - July 2001
  *
@@ -286,7 +286,7 @@ void spudec_update_nav(spudec_decoder_t *this)
 {
   metronom_clock_t *clock = this->stream->xine->clock;
 
-  if (this->pci_cur.next && this->pci_cur.next->vpts <= clock->get_current_time(clock)) {
+  if (this->pci_cur.next && (int64_t)this->pci_cur.next->vpts <= clock->get_current_time(clock)) {
     pci_node_t *node = this->pci_cur.next;
     xine_fast_memcpy(&this->pci_cur, this->pci_cur.next, sizeof(pci_node_t));
     spudec_process_nav(this);
@@ -883,6 +883,8 @@ static void spudec_discover_clut(xine_t *xine, spudec_state_t *state, vo_overlay
   CLUT_Y_CR_CB_INIT(0xff, 0x90, 0x00)
   };
 
+  (void)xine;
+
   rle = ovl->rle;
 
   /* this seems to be a problem somewhere else,
