@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2017 the xine project
+ * Copyright (C) 2010-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -241,17 +241,9 @@ typedef struct {
   sparse_array_t	object_pos;
 } dvbsub_func_t;
 
-typedef struct		dvb_spu_class_s {
-  spu_decoder_class_t	class;
-  xine_t	       *xine;
-
-  int                   ignore_pts;
-} dvb_spu_class_t;
-
 typedef struct dvb_spu_decoder_s {
   spu_decoder_t	spu_decoder;
 
-  dvb_spu_class_t      *class;
   xine_stream_t        *stream;
 
   spu_dvb_descriptor_t *spu_descriptor;
@@ -440,6 +432,9 @@ static void decode_2bit_pixel_code_string (dvb_spu_decoder_t * this, int r, int
   int j;
   const uint8_t *lut = lookup_lut (dvbsub, r);
 
+  (void)object_id;
+  (void)ofs;
+
   if (dvbsub->in_scanline == 0)
     dvbsub->in_scanline = 1;
 
@@ -506,6 +501,9 @@ static void decode_4bit_pixel_code_string (dvb_spu_decoder_t * this, int r, int
   int j;
   const uint8_t *lut = lookup_lut (dvbsub, r);
 
+  (void)object_id;
+  (void)ofs;
+
   if (dvbsub->in_scanline == 0)
     dvbsub->in_scanline = 1;
 
@@ -576,6 +574,9 @@ static void decode_8bit_pixel_code_string (dvb_spu_decoder_t * this, int r, int
   dvbsub_func_t *dvbsub = this->dvbsub;
   int j;
 
+  (void)object_id;
+  (void)ofs;
+
   if (dvbsub->in_scanline == 0)
     dvbsub->in_scanline = 1;
 
@@ -938,6 +939,7 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
 static void process_display_definition_segment(dvb_spu_decoder_t *this)
 {
   /* FIXME: not implemented. */
+  (void)this;
 }
 
 static void unlock_mutex_cancellation_func(void *mutex_gen)
@@ -1040,7 +1042,7 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 	  continue;
         /* clear osd */
         this->stream->osd_renderer->clear( this->dvbsub->regions[r].osd );
-        if ( this->dvbsub->regions[r].width>dest_width && !(this->stream->video_driver->get_capabilities(this->stream->video_driver) & VO_CAP_CUSTOM_EXTENT_OVERLAY)) {
+        if ( this->dvbsub->regions[r].width>dest_width && !(this->stream->video_out->get_capabilities(this->stream->video_out) & VO_CAP_CUSTOM_EXTENT_OVERLAY)) {
 	  downscale_region_image(&this->dvbsub->regions[r], tmp, dest_width);
 	  reg = tmp;
 	  reg_width = dest_width;
@@ -1133,7 +1135,7 @@ static void spudec_decode_data (spu_decoder_t * this_gen, buf_element_t * buf)
   /* don't ask metronom for a vpts but rather do the calculation
    * because buf->pts could be too far in future and metronom won't accept
    * further backwards pts (see metronom_got_spu_packet) */
-  if (!this->class->ignore_pts && buf->pts > 0) {
+  if (buf->pts > 0) {
     metronom_t *metronom = this->stream->metronom;
     int64_t vpts_offset = metronom->get_option( metronom, METRONOM_VPTS_OFFSET );
     int64_t spu_offset = metronom->get_option( metronom, METRONOM_SPU_OFFSET );
@@ -1229,6 +1231,7 @@ static void spudec_reset (spu_decoder_t * this_gen)
 static void spudec_discontinuity (spu_decoder_t * this_gen)
 {
   /* do nothing */
+  (void)this_gen;
 }
 
 static void spudec_dispose_internal (dvb_spu_decoder_t * this, int thread_running)
@@ -1268,9 +1271,9 @@ static void spudec_dispose (spu_decoder_t * this_gen)
 
 static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen, xine_stream_t * stream)
 {
-  dvb_spu_class_t *class = (dvb_spu_class_t *)class_gen;
   dvb_spu_decoder_t *this;
 
+  (void)class_gen;
   this = calloc(1, sizeof (dvb_spu_decoder_t));
   if (!this)
     return NULL;
@@ -1310,7 +1313,6 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
   this->spu_decoder.get_interact_info = NULL;
   this->spu_decoder.set_button = NULL;
 
-  this->class = class;
   this->stream = stream;
 
   pthread_mutex_init(&this->dvbsub_osd_mutex, NULL);
@@ -1350,7 +1352,7 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
   this->dvbsub_hide_timeout.tv_sec = time(NULL);
 
   if (pthread_create(&this->dvbsub_timer_thread, NULL, dvbsub_timer_func, this)) {
-    xprintf (this->class->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": pthread_create() failed\n");
+    xprintf (stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": pthread_create() failed\n");
     spudec_dispose_internal(this, 0);
     return NULL;
   }
@@ -1358,31 +1360,17 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
   return (spu_decoder_t *) this;
 }
 
-static void dvb_spu_decoder_class_dispose (spu_decoder_class_t * this_gen)
+static void *init_spu_decoder_plugin (xine_t * xine, const void *data)
 {
-  dvb_spu_class_t *this = (dvb_spu_class_t *) this_gen;
-
-  this->xine->config->unregister_callback(this->xine->config, "subtitles.dvb.ignore_pts");
-
-  free (this);
-}
-
-static void *init_spu_decoder_plugin (xine_t * xine, void *data)
-{
-
-  dvb_spu_class_t *this;
-  this = calloc(1, sizeof (dvb_spu_class_t));
-  if (!this)
-    return NULL;
-
-  this->class.open_plugin = dvb_spu_class_open_plugin;
-  this->class.identifier  = "spudvb";
-  this->class.description = N_("DVB subtitle decoder plugin");
-  this->class.dispose = dvb_spu_decoder_class_dispose;
-
-  this->xine = xine;
-
-  return &this->class;
+  (void)xine;
+  (void)data;
+  static const spu_decoder_class_t decode_dvb_spu_class = {
+    .open_plugin = dvb_spu_class_open_plugin,
+    .identifier  = "spudvb",
+    .description = N_("DVB subtitle decoder plugin"),
+    .dispose = NULL,
+  };
+  return (void *)&decode_dvb_spu_class;
 }
 
 
@@ -1390,13 +1378,13 @@ static void *init_spu_decoder_plugin (xine_t * xine, void *data)
 static const uint32_t supported_types[] = { BUF_SPU_DVB, 0 };
 
 static const decoder_info_t spudec_info = {
-  supported_types,		/* supported types */
-  1				/* priority        */
+  .supported_types = supported_types,
+  .priority        = 1,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
 /* type, API, "name", version, special_info, init_function */
   {PLUGIN_SPU_DECODER, 17, "spudvb", XINE_VERSION_CODE, &spudec_info,
    &init_spu_decoder_plugin},
-  {PLUGIN_NONE, 0, "", 0, NULL, NULL}
+  {PLUGIN_NONE, 0, NULL, 0, NULL, NULL}
 };
