@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * Copyright (C) 2009 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
@@ -31,16 +31,18 @@
 #include <unistd.h>
 #include <string.h>
 
+#define LOG_MODULE "spu_hdmv"
+/*
+#define LOG
+*/
+
 #include <xine/xine_internal.h>
 #include <xine/buffer.h>
 #include <xine/xineutils.h>
 #include <xine/video_out.h>
 #include <xine/video_overlay.h>
 
-#define XINE_HDMV_TRACE(x...) printf(x)
-/*#define XINE_HDMV_TRACE(x...) */
-#define XINE_HDMV_ERROR(x...) fprintf(stderr, "spuhdmv: " x)
-/*#define XINE_HDMV_ERROR(x...) lprintf(x) */
+#define XINE_HDMV_TRACE lprintf
 
 #ifndef MAX
 #  define MAX(a,b) (a>b)?(a):(b)
@@ -258,7 +260,7 @@ static void segbuf_parse_segment_header(segment_buffer_t *buf)
     if ( buf->segment_type < 0x14 ||
          ( buf->segment_type > 0x18 &&
            buf->segment_type != 0x80)) {
-      XINE_HDMV_ERROR("unknown segment type 0x%02x, resetting\n", buf->segment_type);
+      XINE_HDMV_TRACE("unknown segment type 0x%02x, resetting\n", buf->segment_type);
       segbuf_reset(buf);
     }
   } else {
@@ -318,7 +320,7 @@ static uint8_t segbuf_get_u8(segment_buffer_t *buf)
 {
   if (!(buf->error = ++buf->segment_data > buf->segment_end))
     return buf->segment_data[-1];
-  XINE_HDMV_ERROR("segbuf_get_u8: read failed (end of segment reached) !\n");
+  XINE_HDMV_TRACE("segbuf_get_u8: read failed (end of segment reached) !\n");
   return 0;
 }
 
@@ -350,10 +352,12 @@ static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
     return NULL;
 
   if (len % 5) {
-    XINE_HDMV_ERROR("  decode_palette: segment size error (%zd ; expected %zd for %zd entries)\n",
+    XINE_HDMV_TRACE("  decode_palette: segment size error (%zd ; expected %zd for %zd entries)\n",
                     len, (5 * entries), entries);
     return NULL;
   }
+
+  (void)palette_version_number;
   XINE_HDMV_TRACE("decode_palette: %zd items (id %d, version %d)\n",
                   entries, palette_id, palette_version_number);
 
@@ -455,6 +459,7 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
   uint8_t  version   = segbuf_get_u8 (buf);
   uint8_t  seq_desc  = segbuf_get_u8 (buf);
 
+  (void)version;
   XINE_HDMV_TRACE("  decode_object: object_id %d, version %d, seq 0x%x\n",
                   object_id, version, seq_desc);
 
@@ -521,7 +526,7 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
 
   /* store partial RLE data in HDMV format */
   if (objects->raw_data_size < objects->raw_data_len + segbuf_data_length(buf)) {
-    XINE_HDMV_ERROR("object larger than object size !\n");
+    XINE_HDMV_TRACE("object larger than object size !\n");
     return NULL;
   }
   memcpy(objects->raw_data + objects->raw_data_len, buf->segment_data, segbuf_data_length(buf));
@@ -555,6 +560,7 @@ static window_def_t *segbuf_decode_window_definition(segment_buffer_t *buf)
 {
   window_def_t *wnd;
   uint8_t a;
+
   wnd = calloc(1, sizeof(window_def_t));
   if (!wnd)
     return NULL;
@@ -566,6 +572,7 @@ static window_def_t *segbuf_decode_window_definition(segment_buffer_t *buf)
   wnd->width  = segbuf_get_u16 (buf);
   wnd->height = segbuf_get_u16 (buf);
 
+  (void)a;
   XINE_HDMV_TRACE("  window: [%02x %d]  %d,%d %dx%d\n", a,
                   wnd->id, wnd->xpos, wnd->ypos, wnd->width, wnd->height);
 
@@ -583,6 +590,9 @@ static int segbuf_decode_video_descriptor(segment_buffer_t *buf)
   uint16_t height     = segbuf_get_u16(buf);
   uint8_t  frame_rate = segbuf_get_u8 (buf);
 
+  (void)width;
+  (void)height;
+  (void)frame_rate;
   XINE_HDMV_TRACE("  video_descriptor: %dx%d fps %d\n", width, height, frame_rate);
 
   return buf->error;
@@ -672,6 +682,7 @@ static presentation_segment_t *segbuf_decode_presentation_segment(segment_buffer
 static rle_elem_t *copy_crop_rle(subtitle_object_t *obj, composition_object_t *cobj)
 {
   /* TODO: cropping (w,h sized image from pos x,y) */
+  (void)cobj;
 
   rle_elem_t *rle = calloc (obj->num_rle, sizeof(rle_elem_t));
   if (rle) {
@@ -774,8 +785,8 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
 {
   video_overlay_manager_t *ovl_manager = this->stream->video_out->get_overlay_manager(this->stream->video_out);
   metronom_t              *metronom    = this->stream->metronom;
-  video_overlay_event_t    event       = {0};
-  vo_overlay_t             overlay     = {0};
+  video_overlay_event_t    event       = {.vpts = 0};
+  vo_overlay_t             overlay     = {.x = 0};
 
   /* find palette */
   subtitle_clut_t *clut = this->cluts;
@@ -871,7 +882,7 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
 
 static void hide_overlays(spuhdmv_decoder_t *this, int64_t pts)
 {
-  video_overlay_event_t event = {0};
+  video_overlay_event_t event = {.vpts = 0};
   int i = 0;
 
   while (this->overlay_handles[i] >= 0) {
@@ -911,7 +922,8 @@ static void update_overlays(spuhdmv_decoder_t *this)
 
       for (i = 0; i < pseg->object_number; i++) {
         if (!cobj) {
-          XINE_HDMV_ERROR("show_overlays: composition object %d missing !\n", i);
+          xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
+                   "Error displaying overlay %d: missing composition object\n", i);
         } else {
           show_overlay(this, cobj, pseg->palette_id_ref, i, pseg->pts, !pseg->shown);
           cobj = cobj->next;
@@ -927,25 +939,27 @@ static void update_overlays(spuhdmv_decoder_t *this)
 
 static void decode_segment(spuhdmv_decoder_t *this)
 {
+  int err = 0;
+
   XINE_HDMV_TRACE("*** new segment, pts %010"PRId64": 0x%02x (%8d bytes)\n",
                   this->pts, this->buf->segment_type, this->buf->segment_len);
 
   switch (segbuf_segment_type(this->buf)) {
   case SEGTYPE_PALETTE:
     XINE_HDMV_TRACE("  segment: PALETTE\n");
-    decode_palette(this);
+    err = decode_palette(this);
     break;
   case SEGTYPE_OBJECT:
     XINE_HDMV_TRACE("  segment: OBJECT\n");
-    decode_object(this);
+    err = decode_object(this);
     break;
   case SEGTYPE_PRESENTATION_SEGMENT:
     XINE_HDMV_TRACE("  segment: PRESENTATION SEGMENT\n");
-    decode_presentation_segment(this);
+    err = decode_presentation_segment(this);
     break;
   case SEGTYPE_WINDOW_DEFINITION:
     XINE_HDMV_TRACE("  segment: WINDOW DEFINITION\n");
-    decode_window_definition(this);
+    err = decode_window_definition(this);
     break;
   case SEGTYPE_INTERACTIVE:
     XINE_HDMV_TRACE("  segment: INTERACTIVE\n");
@@ -958,11 +972,13 @@ static void decode_segment(spuhdmv_decoder_t *this)
 #endif
     break;
   default:
-    XINE_HDMV_ERROR("  segment type 0x%x unknown, skipping\n", segbuf_segment_type(this->buf));
+    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
+             "Unknown segment type 0x%x, skipping\n", segbuf_segment_type(this->buf));
     break;
   }
-  if (this->buf->error) {
-    XINE_HDMV_ERROR("*** DECODE ERROR ***\n");
+  if (err || this->buf->error) {
+    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
+             "Error decoding segment 0x%x\n", segbuf_segment_type(this->buf));
   }
 
   update_overlays (this);
@@ -1046,6 +1062,8 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
 {
   spuhdmv_decoder_t *this;
 
+  (void)class_gen;
+
   this = (spuhdmv_decoder_t *) calloc(1, sizeof (spuhdmv_decoder_t));
   if (!this)
     return NULL;
@@ -1069,32 +1087,30 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
   return &this->spu_decoder;
 }
 
-static void *init_plugin (xine_t *xine, void *data)
+static void *init_plugin (xine_t *xine, const void *data)
 {
-  spu_decoder_class_t *this;
+  (void)xine;
+  (void)data;
 
-  this = calloc(1, sizeof (spu_decoder_class_t));
-  if (!this)
-    return NULL;
-
-  this->open_plugin = open_plugin;
-  this->identifier  = "spuhdmv";
-  this->description = "HDMV/BluRay bitmap SPU decoder plugin";
-  this->dispose     = default_spu_decoder_class_dispose;
-
-  return this;
+  static const spu_decoder_class_t decode_spu_hdmv_class = {
+    .open_plugin = open_plugin,
+    .identifier  = "spuhdmv",
+    .description = "HDMV/BluRay bitmap SPU decoder plugin",
+    .dispose     = NULL,
+  };
+  return (void *)&decode_spu_hdmv_class;
 }
 
 /* plugin catalog information */
 static const uint32_t supported_types[] = { BUF_SPU_HDMV, 0 };
 
 static const decoder_info_t dec_info_data = {
-  supported_types,     /* supported types */
-  5                    /* priority        */
+  .supported_types = supported_types,
+  .priority        = 5,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_SPU_DECODER, 17, "spuhdmv", XINE_VERSION_CODE, &dec_info_data, &init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
