@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,6 +40,8 @@
 #include <xine/xineutils.h>
 #include <xine/osd.h>
 
+#include "sputext_demuxer.h"
+
 #define SUB_MAX_TEXT  5      /* lines */
 #define SUB_BUFSIZE   256    /* chars per line */
 
@@ -543,7 +545,7 @@ static int is_cjk_encoding(const char *enc) {
     "SHIFT_JISX0213",
   };
 
-  int pstr;
+  unsigned int pstr;
 
   /* return 1 if encoding string is one of the CJK(Chinese,Jananese,Korean)
    * character set strings. */
@@ -1011,7 +1013,7 @@ static void spudec_reset (spu_decoder_t *this_gen) {
 
 static void spudec_discontinuity (spu_decoder_t *this_gen) {
   /* sputext_decoder_t *this = (sputext_decoder_t *) this_gen; */
-
+  (void)this_gen;
 }
 
 static void spudec_dispose (spu_decoder_t *this_gen) {
@@ -1120,7 +1122,7 @@ static void update_src_encoding(void *class_gen, xine_cfg_entry_t *entry)
   xprintf(class->xine, XINE_VERBOSITY_DEBUG, "libsputext: spu_src_encoding = %s\n", class->src_encoding );
 }
 
-static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
+static void *init_spu_decoder_plugin (xine_t *xine, const void *data) {
 
   static const char *const subtitle_size_strings[] = {
     "tiny", "small", "normal", "large", "very large", "huge", NULL
@@ -1129,6 +1131,8 @@ static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
 
   lprintf("init class\n");
 
+  (void)data;
+
   this = calloc(1, sizeof(sputext_class_t));
   if (!this)
     return NULL;
@@ -1207,15 +1211,13 @@ static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
 static const uint32_t supported_types[] = { BUF_SPU_TEXT, BUF_SPU_OGM, 0 };
 
 static const decoder_info_t spudec_info = {
-  supported_types,     /* supported types */
-  1                    /* priority        */
+  .supported_types = supported_types,
+  .priority        = 1,
 };
 
-extern void *init_sputext_demux_class (xine_t *xine, void *data);
-
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_SPU_DECODER | PLUGIN_MUST_PRELOAD, 17, "sputext", XINE_VERSION_CODE, &spudec_info, &init_spu_decoder_plugin },
   { PLUGIN_DEMUX, 27, "sputext", XINE_VERSION_CODE, NULL, &init_sputext_demux_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
