@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -49,6 +49,8 @@
 #include <xine/xineutils.h>
 #include <xine/demux.h>
 
+#include "sputext_demuxer.h"
+
 #define ERR           (void *)-1
 #define SUB_MAX_TEXT  5
 #define SUB_BUFSIZE   1024
@@ -105,8 +107,6 @@ typedef struct demux_sputext_class_s {
 
 } demux_sputext_class_t;
 
-void *init_sputext_demux_class (xine_t *xine, void *data);
-
 /*
  * Demuxer code start
  */
@@ -882,6 +882,7 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 			    - strlen(line2));
 		    break;
 		}
+                /* fall through */
 	    default:
 		if (!comment) {
 		    *q = *p;
@@ -1341,6 +1342,10 @@ static int demux_sputext_seek (demux_plugin_t *this_gen,
 
   lprintf("seek() called\n");
 
+  (void)start_pos;
+  (void)start_time;
+  (void)playing;
+
   /* simple seeking approach: just go back to start.
    * decoder will discard subtitles until the desired position.
    */
@@ -1374,6 +1379,7 @@ static void demux_sputext_send_headers(demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_sputext_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
@@ -1381,12 +1387,15 @@ static int demux_sputext_get_optional_data(demux_plugin_t *this_gen,
 					   void *data, int data_type) {
   int channel = *((int *)data);
 
+  (void)this_gen;
+
   switch (data_type) {
   case DEMUX_OPTIONAL_DATA_SPULANG:
     if (channel == -1 || channel == 0) {
       strcpy(data, "sub");
       return DEMUX_OPTIONAL_SUCCESS;
     }
+    /* fall through */
   default:
     return DEMUX_OPTIONAL_UNSUPPORTED;
   }
@@ -1437,7 +1446,7 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
         return NULL;
       }
     }
-    /* falling through is intended */
+    /* fall through */
 
   case METHOD_EXPLICIT:
   /* case METHOD_BY_CONTENT: */
@@ -1484,12 +1493,15 @@ static void sputext_demux_class_dispose(demux_class_t *this_gen)
   free(this);
 }
 
-void *init_sputext_demux_class (xine_t *xine, void *data) {
+void *init_sputext_demux_class (xine_t *xine, const void *data) {
 
   demux_sputext_class_t *this ;
 
   lprintf("initializing\n");
 
+  (void)xine;
+  (void)data;
+
   this = calloc(1, sizeof (demux_sputext_class_t));
   if (!this)
     return NULL;
