@@ -301,7 +301,8 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     case 0x00:	/* picture_start_code */
 	if (mpeg2_header_picture (picture, buffer)) {
 	    fprintf (stderr, "bad picture header\n");
-	    abort();
+            /* abort(); */
+            return 0;
 	}
 
 	mpeg2dec->is_frame_needed=0;
@@ -453,7 +454,8 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     case 0xb5:	/* extension_start_code */
 	if (mpeg2_header_extension (picture, buffer)) {
 	    fprintf (stderr, "bad extension\n");
-	    abort();
+            /* abort(); */
+            return 0;
 	}
 	break;
 
@@ -465,8 +467,11 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     case 0xb8:	/* group of pictures start code */
 	if (mpeg2_header_group_of_pictures (picture, buffer)) {
 	  printf ("libmpeg2: bad group of pictures\n");
-	  abort();
+          /* abort(); */
+          return 0;
 	}
+        break;
+
     default:
         if ((code >= 0xb9) && (code != 0xe4)) {
 	  printf("Not multiplexed? 0x%x\n",code);
@@ -507,30 +512,19 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 		     picture->current_frame != picture->forward_reference_frame ) {
 			picture->current_frame->free (picture->current_frame);
 		}
-		if (picture->picture_coding_type == B_TYPE) {
-                    ratio = get_aspect_ratio(mpeg2dec);
-		    picture->current_frame =
-		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
-						     picture->coded_picture_width,
-						     picture->coded_picture_height,
-						     ratio,
-						     mpeg2dec->frame_format,
-						     flags);
-		    libmpeg2_accel_new_frame( &mpeg2dec->accel, mpeg2dec->frame_format,
-					      picture, ratio, flags);
-		} else {
-                    ratio = get_aspect_ratio(mpeg2dec);
-		    picture->current_frame =
-		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
-						     picture->coded_picture_width,
-						     picture->coded_picture_height,
-						     ratio,
-						     mpeg2dec->frame_format,
-						     flags);
 
-		    libmpeg2_accel_new_frame( &mpeg2dec->accel, mpeg2dec->frame_format,
-					      picture, ratio, flags);
+                ratio = get_aspect_ratio(mpeg2dec);
+                picture->current_frame =
+                  mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
+                                                          picture->coded_picture_width,
+                                                          picture->coded_picture_height,
+                                                          ratio,
+                                                          mpeg2dec->frame_format,
+                                                          flags);
+                libmpeg2_accel_new_frame( &mpeg2dec->accel, mpeg2dec->frame_format,
+                                          picture, ratio, flags);
 
+                if (picture->picture_coding_type != B_TYPE) {
 		    if (picture->forward_reference_frame &&
 		        picture->forward_reference_frame != picture->backward_reference_frame)
 		      picture->forward_reference_frame->free (picture->forward_reference_frame);
