@@ -1,5 +1,5 @@
 /* kate: space-indent on; indent-width 2; mixedindent off; indent-mode cstyle; remove-trailing-space on;
- * Copyright (C) 2008-2017 the xine project
+ * Copyright (C) 2008-2019 the xine project
  * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -22,6 +22,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "alterh264_decode.h"
 #include "group_vdpau.h"
 
@@ -61,6 +65,61 @@ static int nframes;
 static FILE *outfile;
 #endif
 
+static const uint8_t zigzag_4x4[16] = {
+  0, 1, 4, 8,
+  5, 2, 3, 6,
+  9, 12, 13, 10,
+  7, 11, 14, 15
+};
+
+static const uint8_t zigzag_8x8[64] = {
+  0, 1, 8, 16, 9, 2, 3, 10,
+  17, 24, 32, 25, 18, 11, 4, 5,
+  12, 19, 26, 33, 40, 48, 41, 34,
+  27, 20, 13, 6, 7, 14, 21, 28,
+  35, 42, 49, 56, 57, 50, 43, 36,
+  29, 22, 15, 23, 30, 37, 44, 51,
+  58, 59, 52, 45, 38, 31, 39, 46,
+  53, 60, 61, 54, 47, 55, 62, 63
+};
+
+static const uint8_t default_4x4_intra[16] = {
+  6, 13, 13, 20,
+  20, 20, 28, 28,
+  28, 28, 32, 32,
+  32, 37, 37, 42
+};
+
+static const uint8_t default_4x4_inter[16] = {
+  10, 14, 14, 20,
+  20, 20, 24, 24,
+  24, 24, 27, 27,
+  27, 30, 30, 34
+};
+
+static const uint8_t default_8x8_intra[64] = {
+  6, 10, 10, 13, 11, 13, 16, 16,
+  16, 16, 18, 18, 18, 18, 18, 23,
+  23, 23, 23, 23, 23, 25, 25, 25,
+  25, 25, 25, 25, 27, 27, 27, 27,
+  27, 27, 27, 27, 29, 29, 29, 29,
+  29, 29, 29, 31, 31, 31, 31, 31,
+  31, 33, 33, 33, 33, 33, 36, 36,
+  36, 36, 38, 38, 38, 40, 40, 42
+};
+
+static const uint8_t default_8x8_inter[64] = {
+  9, 13, 13, 15, 13, 15, 17, 17,
+  17, 17, 19, 19, 19, 19, 19, 21,
+  21, 21, 21, 21, 21, 22, 22, 22,
+  22, 22, 22, 22, 24, 24, 24, 24,
+  24, 24, 24, 24, 25, 25, 25, 25,
+  25, 25, 25, 27, 27, 27, 27, 27,
+  27, 28, 28, 28, 28, 28, 30, 30,
+  30, 30, 32, 32, 32, 33, 33, 35
+};
+
+
 
 /*-------- DPB -------------------------------------------*/
 static void
@@ -79,7 +138,7 @@ dpb_print (sequence_t * sequence)
     vdpau_accel_t *accel;
     if (vo)
       accel = (vdpau_accel_t *) vo->accel_data;
-    sf = (vo) ? accel->surface : -1;
+    sf = (vo) ? accel->surface : (uint32_t)-1;
     fprintf (stderr,
 	     "{ i:%d u:%d c:%d pn:%d-%d ir:%d-%d tpoc:%d bpoc:%d sf:%u }\n",
 	     i, frame->used, frame->completed, frame->PicNum[0],
@@ -467,6 +526,7 @@ parse_scaling_list (bits_reader_t * br, uint8_t * scaling_list, int len,
   int32_t delta_scale;
   uint8_t use_default_scaling_matrix_flag = 0;
   int i;
+  uint32_t u;
 
   const uint8_t *zigzag = (len == 64) ? zigzag_8x8 : zigzag_4x4;
 
@@ -494,28 +554,28 @@ parse_scaling_list (bits_reader_t * br, uint8_t * scaling_list, int len,
     case 1:
     case 2:
       {
-	for (i = 0; i < sizeof (default_4x4_intra); i++)
-	  scaling_list[zigzag_4x4[i]] = default_4x4_intra[i];
+	for (u = 0; u < sizeof (default_4x4_intra); u++)
+	  scaling_list[zigzag_4x4[u]] = default_4x4_intra[u];
 	break;
       }
     case 3:
     case 4:
     case 5:
       {
-	for (i = 0; i < sizeof (default_4x4_inter); i++)
-	  scaling_list[zigzag_4x4[i]] = default_4x4_inter[i];
+	for (u = 0; u < sizeof (default_4x4_inter); u++)
+	  scaling_list[zigzag_4x4[u]] = default_4x4_inter[u];
 	break;
       }
     case 6:
       {
-	for (i = 0; i < sizeof (default_8x8_intra); i++)
-	  scaling_list[zigzag_8x8[i]] = default_8x8_intra[i];
+	for (u = 0; u < sizeof (default_8x8_intra); u++)
+	  scaling_list[zigzag_8x8[u]] = default_8x8_intra[u];
 	break;
       }
     case 7:
       {
-	for (i = 0; i < sizeof (default_8x8_inter); i++)
-	  scaling_list[zigzag_8x8[i]] = default_8x8_inter[i];
+	for (u = 0; u < sizeof (default_8x8_inter); u++)
+	  scaling_list[zigzag_8x8[u]] = default_8x8_inter[u];
 	break;
       }
     }
@@ -528,43 +588,34 @@ static void
 scaling_list_fallback_A (uint8_t * scaling_lists_4x4,
 			 uint8_t * scaling_lists_8x8, int i)
 {
-  int j;
+  uint32_t j;
   switch (i)
   {
   case 0:
-    {
       for (j = 0; j < sizeof (default_4x4_intra); j++)
 	scaling_lists_4x4[(i * 16) + zigzag_4x4[j]] = default_4x4_intra[j];
       break;
-    }
   case 3:
-    {
       for (j = 0; j < sizeof (default_4x4_inter); j++)
 	scaling_lists_4x4[(i * 16) + zigzag_4x4[j]] = default_4x4_inter[j];
       break;
-    }
   case 1:
   case 2:
   case 4:
   case 5:
     memcpy (&scaling_lists_4x4[i * 16], &scaling_lists_4x4[(i - 1) * 16],
-	    6 * 16);
+	    16);
     break;
   case 6:
-    {
       for (j = 0; j < sizeof (default_8x8_intra); j++)
 	scaling_lists_8x8[(i - 6) * 64 + zigzag_8x8[j]] =
 	  default_8x8_intra[j];
       break;
-    }
   case 7:
-    {
       for (j = 0; j < sizeof (default_8x8_inter); j++)
 	scaling_lists_8x8[(i - 6) * 64 + zigzag_8x8[j]] =
 	  default_8x8_inter[j];
       break;
-    }
-
   }
 }
 
@@ -1388,11 +1439,11 @@ decode_poc (vdpau_h264_alter_decoder_t * this_gen)
 
     if ((sl->pic_order_cnt_lsb < prevPicOrderCntLsb)
 	&& ((prevPicOrderCntLsb - sl->pic_order_cnt_lsb) >=
-	    (MaxPicOrderCntLsb / 2)))
+	    (int32_t)(MaxPicOrderCntLsb / 2)))
       seq->cur_pic.PicOrderCntMsb = prevPicOrderCntMsb + MaxPicOrderCntLsb;
     else if ((sl->pic_order_cnt_lsb > prevPicOrderCntLsb)
 	     && ((sl->pic_order_cnt_lsb - prevPicOrderCntLsb) >
-		 (MaxPicOrderCntLsb / 2)))
+		 (int32_t)(MaxPicOrderCntLsb / 2)))
       seq->cur_pic.PicOrderCntMsb = prevPicOrderCntMsb - MaxPicOrderCntLsb;
     else
       seq->cur_pic.PicOrderCntMsb = prevPicOrderCntMsb;
@@ -1690,6 +1741,8 @@ decode_render (vdpau_h264_alter_decoder_t * vd, int bad_frame)
       || vd->decoder_width != seq->coded_width
       || vd->decoder_height != seq->coded_height)
   {
+    if (accel->lock)
+      accel->lock (accel->vo_frame);
     if (vd->decoder != VDP_INVALID_HANDLE)
     {
       accel->vdp_decoder_destroy (vd->decoder);
@@ -1699,6 +1752,8 @@ decode_render (vdpau_h264_alter_decoder_t * vd, int bad_frame)
       accel->vdp_decoder_create (accel->vdp_device, seq->profile,
 				 seq->coded_width, seq->coded_height,
 				 sp->num_ref_frames, &vd->decoder);
+    if (accel->unlock)
+      accel->unlock (accel->vo_frame);
     if (st != VDP_STATUS_OK)
       fprintf (stderr, "vdpau_h264_alter : failed to create decoder !! %s\n",
 	       accel->vdp_get_error_string (st));
@@ -1801,10 +1856,14 @@ decode_render (vdpau_h264_alter_decoder_t * vd, int bad_frame)
     vbits[(i * 2) + 1].bitstream = seq->buf + seq->slices[i].buf_offset;
     vbits[(i * 2) + 1].bitstream_bytes = seq->slices[i].len;
   }
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   st =
     accel->vdp_decoder_render (vd->decoder, accel->surface,
 			       CAST_VdpPictureInfo_PTR &info,
 			       seq->slices_count * 2, vbits);
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
   if (st != VDP_STATUS_OK)
     lprintf ("**********************DECODING failed! - surface = %d - %s\n",
 	     accel->surface, accel->vdp_get_error_string (st));
@@ -2048,7 +2107,7 @@ parse_codec_private (vdpau_h264_alter_decoder_t * this_gen, const uint8_t * buf,
 static void
 flush_buffer (sequence_t * seq)
 {
-  if ((seq->bufpos - seq->bufseek) >= seq->bufseek)
+  if (((int)seq->bufpos - seq->bufseek) >= seq->bufseek)
   {
     seq->bufsize = (seq->bufpos - seq->bufseek) + MIN_BUFFER_SIZE;
     lprintf ("buffer too short, have to allocate a new one.\n");
@@ -2129,7 +2188,7 @@ vdpau_h264_alter_decode_data (video_decoder_t * this_gen, buf_element_t * buf)
     return;
 
   int size = seq->bufpos + buf->size;
-  if (seq->bufsize < size)
+  if ((int)seq->bufsize < size)
   {
     if (seq->bufsize > MAX_BUFFER_SIZE)
     {
@@ -2159,7 +2218,7 @@ vdpau_h264_alter_decode_data (video_decoder_t * this_gen, buf_element_t * buf)
       if (buf->pts)
 	seq->pic_pts = buf->pts;
       lprintf ("frame_end && seq->mode_frame\n");
-      int fhs;
+      uint32_t fhs;
       uint8_t tb;
       uint32_t j = 0;
       while (j < seq->bufpos)
@@ -2186,7 +2245,7 @@ vdpau_h264_alter_decode_data (video_decoder_t * this_gen, buf_element_t * buf)
     return;
   }
 
-  while (seq->bufseek <= seq->bufpos - 4)
+  while (seq->bufseek <= (int)seq->bufpos - 4)
   {
     uint8_t *buffer = seq->buf + seq->bufseek;
     if (buffer[0] == 0 && buffer[1] == 0 && buffer[2] == 1)
@@ -2289,8 +2348,12 @@ vdpau_h264_alter_dispose (video_decoder_t * this_gen)
 
   if ((this->decoder != VDP_INVALID_HANDLE) && this->sequence.accel_vdpau)
   {
+    if (this->sequence.accel_vdpau->lock)
+      this->sequence.accel_vdpau->lock (this->sequence.accel_vdpau->vo_frame);
     this->sequence.accel_vdpau->vdp_decoder_destroy (this->decoder);
     this->decoder = VDP_INVALID_HANDLE;
+    if (this->sequence.accel_vdpau->unlock)
+      this->sequence.accel_vdpau->unlock (this->sequence.accel_vdpau->vo_frame);
   }
   reset_sequence (&this->sequence);
 
@@ -2321,31 +2384,44 @@ open_plugin (video_decoder_class_t * class_gen, xine_stream_t * stream)
 
   vdpau_h264_alter_decoder_t *this;
 
+  (void)class_gen;
+
   /* the videoout must be vdpau-capable to support this decoder */
   if (!
-      (stream->video_driver->
-       get_capabilities (stream->video_driver) & VO_CAP_VDPAU_H264))
+      (stream->video_out->
+       get_capabilities (stream->video_out) & VO_CAP_VDPAU_H264))
     return NULL;
 
   /* now check if vdpau has free decoder resource */
   vo_frame_t *img =
-    stream->video_out->get_frame (stream->video_out, 1920, 1080, 1,
-				  XINE_IMGFMT_VDPAU, VO_BOTH_FIELDS);
+    stream->video_out->get_frame (stream->video_out, 1920, 1080, 1, XINE_IMGFMT_VDPAU,
+                                  VO_BOTH_FIELDS | VO_GET_FRAME_MAY_FAIL);
+  if (!img) {
+    return NULL;
+  }
   vdpau_accel_t *accel = (vdpau_accel_t *) img->accel_data;
   int runtime_nr = accel->vdp_runtime_nr;
   img->free (img);
   VdpDecoder decoder;
-  VdpStatus st =
-    accel->vdp_decoder_create (accel->vdp_device,
+  VdpStatus st;
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
+  st =accel->vdp_decoder_create (accel->vdp_device,
 			       VDP_DECODER_PROFILE_H264_MAIN, 1920, 1080, 16,
 			       &decoder);
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
   if (st != VDP_STATUS_OK)
   {
     fprintf (stderr, "can't create vdpau decoder!\n");
     return NULL;
   }
 
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   accel->vdp_decoder_destroy (decoder);
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
 
   this =
     (vdpau_h264_alter_decoder_t *) calloc (1,
@@ -2359,7 +2435,6 @@ open_plugin (video_decoder_class_t * class_gen, xine_stream_t * stream)
   this->video_decoder.dispose = vdpau_h264_alter_dispose;
 
   this->stream = stream;
-  this->class = (vdpau_h264_alter_class_t *) class_gen;
 
   int i;
   for (i = 0; i < 16; i++)
@@ -2401,23 +2476,21 @@ open_plugin (video_decoder_class_t * class_gen, xine_stream_t * stream)
  * the class's member functions.
  */
 void *
-h264_alter_init_plugin (xine_t * xine, void *data)
+h264_alter_init_plugin (xine_t * xine, const void *data)
 {
+  (void)xine;
+  (void)data;
 
-  vdpau_h264_alter_class_t *this;
+  static const video_decoder_class_t decode_video_vdpau_h264_alter_class = {
+    .open_plugin = open_plugin,
+    .identifier  = "vdpau_h264_alter",
+    .description =
+      N_
+      ("vdpau_h264_alter: H264 decoder plugin using VDPAU hardware decoding.\n"
+       "Must be used along with video_out_vdpau."),
+    .dispose = NULL,
+  };
 
-  this =
-    (vdpau_h264_alter_class_t *) calloc (1,
-					 sizeof (vdpau_h264_alter_class_t));
-
-  this->decoder_class.open_plugin = open_plugin;
-  this->decoder_class.identifier = "vdpau_h264_alter";
-  this->decoder_class.description =
-    N_
-    ("vdpau_h264_alter: H264 decoder plugin using VDPAU hardware decoding.\n"
-     "Must be used along with video_out_vdpau.");
-  this->decoder_class.dispose = default_video_decoder_class_dispose;
-
-  return this;
+  return (void *)&decode_video_vdpau_h264_alter_class;
 }
 
