@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2017 the xine project
+ * Copyright (C) 2008-2019 the xine project
  * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -151,17 +151,9 @@ typedef struct {
 } sequence_t;
 
 
-
-typedef struct {
-  video_decoder_class_t   decoder_class;
-} vdpau_vc1_class_t;
-
-
-
 typedef struct vdpau_vc1_decoder_s {
   video_decoder_t         video_decoder;  /* parent video decoder structure */
 
-  vdpau_vc1_class_t    *class;
   xine_stream_t           *stream;
 
   sequence_t              sequence;
@@ -641,11 +633,15 @@ static void decode_render( vdpau_vc1_decoder_t *vd, vdpau_accel_t *accel, uint8_
 
   VdpStatus st;
   if ( vd->decoder==VDP_INVALID_HANDLE || vd->decoder_profile!=seq->profile || vd->decoder_width!=seq->coded_width || vd->decoder_height!=seq->coded_height ) {
+    if (accel->lock)
+      accel->lock (accel->vo_frame);
     if ( vd->decoder!=VDP_INVALID_HANDLE ) {
       accel->vdp_decoder_destroy( vd->decoder );
       vd->decoder = VDP_INVALID_HANDLE;
     }
     st = accel->vdp_decoder_create( accel->vdp_device, seq->profile, seq->coded_width, seq->coded_height, 2, &vd->decoder);
+    if (accel->unlock)
+      accel->unlock (accel->vo_frame);
     if ( st!=VDP_STATUS_OK )
       fprintf(stderr, "vdpau_vc1: failed to create decoder !! %s\n", accel->vdp_get_error_string( st ) );
     else {
@@ -663,7 +659,11 @@ static void decode_render( vdpau_vc1_decoder_t *vd, vdpau_accel_t *accel, uint8_
   vbit.bitstream_bytes = len;
   if ( pic->field )
     vbit.bitstream_bytes = pic->field;
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   st = accel->vdp_decoder_render( vd->decoder, accel->surface, CAST_VdpPictureInfo_PTR &pic->vdp_infos, 1, &vbit );
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
   if ( st!=VDP_STATUS_OK )
     fprintf(stderr, "vdpau_vc1: decoder failed : %d!! %s\n", st, accel->vdp_get_error_string( st ) );
   else {
@@ -705,7 +705,11 @@ static void decode_render( vdpau_vc1_decoder_t *vd, vdpau_accel_t *accel, uint8_
     }
     vbit.bitstream = buf+pic->field+4;
     vbit.bitstream_bytes = len-pic->field-4;
+    if (accel->lock)
+      accel->lock (accel->vo_frame);
     st = accel->vdp_decoder_render( vd->decoder, accel->surface, CAST_VdpPictureInfo_PTR &pic->vdp_infos, 1, &vbit );
+    if (accel->unlock)
+      accel->unlock (accel->vo_frame);
     if ( st!=VDP_STATUS_OK )
       fprintf(stderr, "vdpau_vc1: decoder failed : %d!! %s\n", st, accel->vdp_get_error_string( st ) );
     else {
@@ -730,6 +734,7 @@ static int search_field( vdpau_vc1_decoder_t *vd, uint8_t *buf, int len )
 {
   int i;
   lprintf("search_fields, len=%d\n", len);
+  (void)vd;
   for ( i=0; i<len-4; ++i ) {
     if ( buf[i]==0 && buf[i+1]==0 && buf[i+2]==1 && buf[i+3]==field_start_code ) {
       lprintf("found field_start_code at %d\n", i);
@@ -957,7 +962,7 @@ static void vdpau_vc1_decode_data (video_decoder_t *this_gen, buf_element_t *buf
   }
 
   int size = seq->bufpos+buf->size;
-  if ( seq->bufsize < size ) {
+  if ( seq->bufsize < (unsigned int)size ) {
     seq->bufsize = size+10000;
     seq->buf = realloc( seq->buf, seq->bufsize );
     lprintf("sequence buffer realloced = %d\n", seq->bufsize );
@@ -986,7 +991,7 @@ static void vdpau_vc1_decode_data (video_decoder_t *this_gen, buf_element_t *buf
   }
 
   int res;
-  while ( seq->bufseek <= seq->bufpos-4 ) {
+  while ( seq->bufseek <= (int)(seq->bufpos)-4 ) {
     uint8_t *buffer = seq->buf+seq->bufseek;
     if ( buffer[0]==0 && buffer[1]==0 && buffer[2]==1 ) {
       seq->current_code = buffer[3];
@@ -1028,6 +1033,7 @@ static void vdpau_vc1_decode_data (video_decoder_t *this_gen, buf_element_t *buf
 static void vdpau_vc1_flush (video_decoder_t *this_gen) {
 
   lprintf( "vdpau_vc1_flush\n" );
+  (void)this_gen;
 }
 
 /*
@@ -1046,6 +1052,7 @@ static void vdpau_vc1_reset (video_decoder_t *this_gen) {
 static void vdpau_vc1_discontinuity (video_decoder_t *this_gen) {
 
   lprintf( "vdpau_vc1_discontinuity\n" );
+  (void)this_gen;
 }
 
 /*
@@ -1058,8 +1065,12 @@ static void vdpau_vc1_dispose (video_decoder_t *this_gen) {
   lprintf( "vdpau_vc1_dispose\n" );
 
   if ( this->decoder!=VDP_INVALID_HANDLE && this->sequence.accel_vdpau ) {
+      if (this->sequence.accel_vdpau->lock)
+        this->sequence.accel_vdpau->lock (this->sequence.accel_vdpau->vo_frame);
       this->sequence.accel_vdpau->vdp_decoder_destroy( this->decoder );
       this->decoder = VDP_INVALID_HANDLE;
+      if (this->sequence.accel_vdpau->unlock)
+        this->sequence.accel_vdpau->unlock (this->sequence.accel_vdpau->vo_frame);
     }
 
   reset_sequence( &this->sequence );
@@ -1080,23 +1091,37 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
 
   lprintf( "open_plugin\n" );
 
+  (void)class_gen;
+
   /* the videoout must be vdpau-capable to support this decoder */
-  if ( !(stream->video_driver->get_capabilities(stream->video_driver) & VO_CAP_VDPAU_VC1) )
+  if ( !(stream->video_out->get_capabilities(stream->video_out) & VO_CAP_VDPAU_VC1) )
     return NULL;
 
   /* now check if vdpau has free decoder resource */
-  vo_frame_t *img = stream->video_out->get_frame( stream->video_out, 1920, 1080, 1, XINE_IMGFMT_VDPAU, VO_BOTH_FIELDS );
+  vo_frame_t *img = stream->video_out->get_frame( stream->video_out, 1920, 1080, 1, XINE_IMGFMT_VDPAU,
+                                                  VO_BOTH_FIELDS | VO_GET_FRAME_MAY_FAIL );
+  if (!img) {
+    return NULL;
+  }
   vdpau_accel_t *accel = (vdpau_accel_t*)img->accel_data;
   int runtime_nr = accel->vdp_runtime_nr;
   img->free(img);
   VdpDecoder decoder;
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   VdpStatus st = accel->vdp_decoder_create( accel->vdp_device, VDP_DECODER_PROFILE_VC1_MAIN, 1920, 1080, 2, &decoder );
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
   if ( st!=VDP_STATUS_OK ) {
     lprintf( "can't create vdpau decoder.\n" );
     return NULL;
   }
 
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   accel->vdp_decoder_destroy( decoder );
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
 
   this = (vdpau_vc1_decoder_t *) calloc(1, sizeof(vdpau_vc1_decoder_t));
 
@@ -1107,7 +1132,6 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   this->video_decoder.dispose             = vdpau_vc1_dispose;
 
   this->stream                            = stream;
-  this->class                             = (vdpau_vc1_class_t *) class_gen;
 
   this->sequence.bufsize = 10000;
   this->sequence.buf = (uint8_t*)malloc(this->sequence.bufsize);
@@ -1136,19 +1160,21 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
  * This function allocates a private video decoder class and initializes
  * the class's member functions.
  */
-void *vc1_init_plugin (xine_t *xine, void *data) {
+void *vc1_init_plugin (xine_t *xine, const void *data) {
 
-  vdpau_vc1_class_t *this;
+  (void)xine;
+  (void)data;
 
-  this = (vdpau_vc1_class_t *) calloc(1, sizeof(vdpau_vc1_class_t));
+  static const video_decoder_class_t decode_video_vdpau_vc1_class = {
+    .open_plugin     = open_plugin,
+    .identifier      = "vdpau_vc1",
+    .description     =
+        N_("vdpau_vc1: vc1 decoder plugin using VDPAU hardware decoding.\n"
+           "Must be used along with video_out_vdpau."),
+    .dispose         = NULL,
+  };
 
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "vdpau_vc1";
-  this->decoder_class.description     =
-	N_("vdpau_vc1: vc1 decoder plugin using VDPAU hardware decoding.\n"
-	   "Must be used along with video_out_vdpau.");
-  this->decoder_class.dispose         = default_video_decoder_class_dispose;
-
-  return this;
+  return (void *)&decode_video_vdpau_vc1_class;
 }
 
+
