@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 the xine project
+ * Copyright (C) 2014-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -35,8 +35,6 @@
 
 #define LOG_MODULE "mmal_video_decoder"
 
-#define XINE_ENGINE_INTERNAL  /* access to stream->video_decoder_streamtype */
-
 #include <xine/xine_internal.h>
 #include <xine/video_out.h>
 #include <xine/buffer.h>
@@ -135,6 +133,12 @@ static int send_output_buffer(mmal_decoder_t *this)
             "xine frame not suitable for direct rendering\n");
 
     buffer->data = malloc(buffer->alloc_size);
+    if (!buffer->data) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": "
+              "failed to allocate output buffer\n");
+      free_output_buffer(buffer);
+      return -1;
+    }
   }
 
   status = mmal_port_send_buffer(this->decoder->output[0], buffer);
@@ -506,7 +510,7 @@ static MMAL_BUFFER_HEADER_T *get_input_buffer(mmal_decoder_t *this)
     this->input_buffer = mmal_queue_timedwait(this->input_pool->queue, 200);
     while (!this->input_buffer) {
       handle_output(this);
-      if (--retries < 1 || this->stream->emergency_brake) {
+      if (--retries < 1 || _x_action_pending(this->stream)) {
         xprintf(this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": "
                 "failed to retrieve buffer header for input data\n");
         this->discontinuity = 1;
@@ -585,17 +589,19 @@ static void mmal_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 {
   mmal_decoder_t *this = (mmal_decoder_t *) this_gen;
 
-  if (buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_SPECIAL)) {
-    return;
-  }
+  if (buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_SPECIAL | BUF_FLAG_COLOR_MATRIX | BUF_FLAG_STDHEADER)) {
+    if (buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_SPECIAL)) {
+      return;
+    }
 
-  if (buf->decoder_flags & BUF_FLAG_COLOR_MATRIX) {
-    VO_SET_FLAGS_CM (buf->decoder_info[4], this->frame_flags);
-  }
+    if (buf->decoder_flags & BUF_FLAG_COLOR_MATRIX) {
+      VO_SET_FLAGS_CM (buf->decoder_info[4], this->frame_flags);
+    }
 
-  if (buf->decoder_flags & BUF_FLAG_STDHEADER) {
-    handle_header(this, buf);
-    return;
+    if (buf->decoder_flags & BUF_FLAG_STDHEADER) {
+      handle_header(this, buf);
+      return;
+    }
   }
 
   if (!this->decoder_ok) {
@@ -727,6 +733,8 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   bcm_host_init();
 
   this = (mmal_decoder_t *) calloc(1, sizeof(mmal_decoder_t));
+  if (!this)
+    return NULL;
 
   pthread_mutex_init (&this->mutex, NULL);
 
@@ -766,7 +774,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
 
   /* test if decoder supports requested codec */
 
-  uint32_t video_type = BUF_VIDEO_BASE | (stream->video_decoder_streamtype << 16);
+  uint32_t video_type = BUF_VIDEO_BASE | (_x_get_video_streamtype(stream) << 16);
   MMAL_PORT_T *input = this->decoder->input[0];
 
   switch (video_type) {
@@ -789,7 +797,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
     default:
       xprintf(this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": "
               "unsupported video codec: 0x%x\n",
-              stream->video_decoder_streamtype);
+	      video_type);
       mmal_dispose(&this->video_decoder);
       return (video_decoder_t *)1;
   }
@@ -812,7 +820,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
             "failed to commit input format: %s (%d)\n",
             mmal_status_to_string(status), status);
     mmal_dispose(&this->video_decoder);
-    return NULL;
+    return (video_decoder_t *)1;
   }
 
   input->buffer_size = input->buffer_size_recommended;
@@ -824,24 +832,22 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
             "failed to enable input port: %s (%d)\n",
             mmal_status_to_string(status), status);
     mmal_dispose(&this->video_decoder);
-    return NULL;
+    return (video_decoder_t *)1;
   }
 
   return &this->video_decoder;
 }
 
-static void *init_plugin (xine_t *xine, void *data)
+static void *init_plugin (xine_t *xine, const void *data)
 {
-  video_decoder_class_t *this;
+  static const video_decoder_class_t decode_video_mmal_class = {
+    .open_plugin     = open_plugin,
+    .identifier      = "libmmal",
+    .description     = N_("mmal-based HW video decoder plugin"),
+    .dispose         = NULL,
+  };
 
-  this = (video_decoder_class_t *) calloc(1, sizeof(video_decoder_class_t));
-
-  this->open_plugin     = open_plugin;
-  this->identifier      = "libmmal";
-  this->description     = N_("mmal-based HW video decoder plugin");
-  this->dispose         = default_video_decoder_class_dispose;
-
-  return this;
+  return (void *)&decode_video_mmal_class;
 }
 
 /*
@@ -857,12 +863,12 @@ static const uint32_t video_types[] = {
 };
 
 static const decoder_info_t dec_info = {
-  video_types,     /* supported types */
-  10               /* priority        */
+  .supported_types = video_types,
+  .priority        = 10,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_DECODER, 19, "libmmal", XINE_VERSION_CODE, &dec_info, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
