@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project and Claudio Ciccani
+ * Copyright (C) 2000-2019 the xine project and Claudio Ciccani
  *
  * This file is part of xine, a free video player.
  *
@@ -750,6 +750,7 @@ static void directfb_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen
   frame->locked = 1;
 }
 
+#ifndef DIRECTFB_X11
 /* directfb_display_frame2(): output to underlay */
 static void directfb_display_frame2 (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
   directfb_driver_t       *this   = (directfb_driver_t *) this_gen;
@@ -875,6 +876,7 @@ static void directfb_display_frame2 (vo_driver_t *this_gen, vo_frame_t *frame_ge
                         (int  *)&frame->vo_frame.pitches[0]);
   frame->locked = 1;
 }
+#endif
 
 static int directfb_get_property (vo_driver_t *this_gen, int property) {
   directfb_driver_t *this = (directfb_driver_t *) this_gen;
@@ -1191,6 +1193,8 @@ static int directfb_gui_data_exchange (vo_driver_t *this_gen,
 static void directfb_dispose (vo_driver_t *this_gen) {
   directfb_driver_t *this = (directfb_driver_t *) this_gen;
 
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
+
   if (this->cur_frame)
     this->cur_frame->vo_frame.dispose (&this->cur_frame->vo_frame);
 
@@ -1426,6 +1430,7 @@ static DFBEnumerationResult find_underlay (DFBDisplayLayerID id,
   return DFENUM_OK;
 }
 
+#if 0
 static DFBEnumerationResult find_subpicture (DFBDisplayLayerID id,
                                              DFBDisplayLayerDescription dsc, void *ctx) {
   DFBDisplayLayerID *ret_id = (DFBDisplayLayerID *) ctx;
@@ -1440,7 +1445,9 @@ static DFBEnumerationResult find_subpicture (DFBDisplayLayerID id,
 
   return DFENUM_OK;
 }
+#endif
 
+#ifndef DIRECTFB_X11
 static void init_subpicture (directfb_driver_t *this) {
   DFBResult ret;
 
@@ -1528,6 +1535,7 @@ static void init_subpicture (directfb_driver_t *this) {
            _("video_out_directfb: using hardware subpicture acceleration.\n"));
   }
 }
+#endif
 
 static DFBResult init_device (directfb_driver_t *this) {
   IDirectFBSurface           *surface;
@@ -1736,6 +1744,7 @@ static DFBResult init_device (directfb_driver_t *this) {
   return DFB_OK;
 }
 
+#ifndef DIRECTFB_X11
 static void directfb_frame_output_cb (void *user_data, int video_width, int video_height,
                                       double video_pixel_aspect, int *dest_x, int *dest_y,
                                       int *dest_width, int *dest_height,
@@ -1751,6 +1760,7 @@ static void directfb_frame_output_cb (void *user_data, int video_width, int vide
   *win_x             = 0;
   *win_y             = 0;
 }
+#endif
 
 /*** DirectFB plugin functions ***/
 
@@ -1768,13 +1778,14 @@ static inline int convert_caps (DFBDisplayLayerCapabilities caps)
   return vo;
 }
 
+#ifndef DIRECTFB_X11
 static vo_driver_t *open_plugin_fb (video_driver_class_t *class_gen, const void *visual_gen) {
   directfb_class_t  *class  = (directfb_class_t *) class_gen;
   directfb_driver_t *this;
-  char              *args[] = { "xine", "--dfb:" DIRECTFB_OPTIONS };
+  const char        *args[] = { "xine", "--dfb:" DIRECTFB_OPTIONS };
   int                argn   = 2;
   char             **argp   = (char **) args;
-  fb_visual_t       *visual = (fb_visual_t *) visual_gen;
+  const fb_visual_t *visual = (const fb_visual_t *) visual_gen;
   config_values_t   *config = class->xine->config;
   DFBDisplayLayerID  id;
   DFBResult          ret;
@@ -1905,9 +1916,10 @@ static vo_driver_t *open_plugin_fb (video_driver_class_t *class_gen, const void
 
   return &this->vo_driver;
 }
+#endif
 
 #ifndef DIRECTFB_X11
-static void *init_class_fb (xine_t *xine, void *visual_gen) {
+static void *init_class_fb (xine_t *xine, const void *visual_gen) {
   directfb_class_t *this;
   const char       *error;
 
@@ -1922,6 +1934,9 @@ static void *init_class_fb (xine_t *xine, void *visual_gen) {
   }
 
   this = (directfb_class_t *) calloc(1, sizeof(directfb_class_t));
+  if (!this)
+    return NULL;
+
   this->driver_class.open_plugin     = open_plugin_fb;
   this->driver_class.identifier      = "DirectFB";
   this->driver_class.description     = N_("xine video output plugin using DirectFB.");
@@ -1933,8 +1948,8 @@ static void *init_class_fb (xine_t *xine, void *visual_gen) {
 }
 
 static const vo_info_t vo_info_directfb_fb = {
-  8,                   /* priority    */
-  XINE_VISUAL_TYPE_FB  /* visual type */
+  .priority    = 8,
+  .visual_type = XINE_VISUAL_TYPE_FB,
 };
 
 /*** XDirectFB plugin functions ****/
@@ -1943,10 +1958,10 @@ static const vo_info_t vo_info_directfb_fb = {
 static vo_driver_t *open_plugin_x11 (video_driver_class_t *class_gen, const void *visual_gen) {
   directfb_class_t  *class  = (directfb_class_t *) class_gen;
   directfb_driver_t *this;
-  char              *args[] = { "xine", "--dfb:" XDIRECTFB_OPTIONS };
+  const char        *args[] = { "xine", "--dfb:" XDIRECTFB_OPTIONS };
   int                argn   = 2;
   char             **argp   = (char **) args;
-  x11_visual_t      *visual = (x11_visual_t *) visual_gen;
+  const x11_visual_t *visual = (const x11_visual_t *) visual_gen;
   XWindowAttributes  attrs;
   IDirectFBScreen   *screen;
   DFBDisplayLayerID  id     = DLID_PRIMARY;
@@ -2101,9 +2116,9 @@ static vo_driver_t *open_plugin_x11 (video_driver_class_t *class_gen, const void
   return &this->vo_driver;
 }
 
-static void *init_class_x11 (xine_t *xine, void *visual_gen) {
+static void *init_class_x11 (xine_t *xine, const void *visual_gen) {
   directfb_class_t *this;
-  x11_visual_t     *visual = (x11_visual_t *) visual_gen;
+  const x11_visual_t *visual = (const x11_visual_t *) visual_gen;
   const char       *error;
 
   /* check DirectFB version */
@@ -2127,6 +2142,8 @@ static void *init_class_x11 (xine_t *xine, void *visual_gen) {
     return NULL;
 
   this = (directfb_class_t *) calloc(1, sizeof(directfb_class_t));
+  if (!this)
+    return NULL;
   this->driver_class.open_plugin     = open_plugin_x11;
   this->driver_class.identifier      = "XDirectFB";
   this->driver_class.description     = N_("xine video output plugin using DirectFB under XDirectFB.");
@@ -2138,7 +2155,7 @@ static void *init_class_x11 (xine_t *xine, void *visual_gen) {
   return this;
 }
 
-static void *init_class_x11_2 (xine_t *xine, void *visual_gen) {
+static void *init_class_x11_2 (xine_t *xine, const void *visual_gen) {
   directfb_class_t *this;
 
   this = init_class_x11( xine, visual_gen );
@@ -2149,13 +2166,13 @@ static void *init_class_x11_2 (xine_t *xine, void *visual_gen) {
 }
 
 static const vo_info_t vo_info_directfb_x11 = {
-  8,                    /* priority    */
-  XINE_VISUAL_TYPE_X11  /* visual type */
+  .priority    = 8,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
 static const vo_info_t vo_info_directfb_x11_2 = {
-  8,                      /* priority    */
-  XINE_VISUAL_TYPE_X11_2  /* visual type */
+  .priority    = 8,
+  .visual_type = XINE_VISUAL_TYPE_X11_2,
 };
 #endif /* HAVE_X11 */
 
@@ -2172,6 +2189,6 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   { PLUGIN_VIDEO_OUT, VIDEO_OUT_DRIVER_IFACE_VERSION, "XDirectFB",
     XINE_VERSION_CODE, &vo_info_directfb_x11_2, init_class_x11_2 },
 #endif
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
