@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project and Fredrik Noring
+ * Copyright (C) 2000-2018 the xine project and Fredrik Noring
  *
  * This file is part of xine, a free video player.
  *
@@ -153,6 +153,7 @@ typedef struct
 
 static uint32_t fb_get_capabilities(vo_driver_t *this_gen)
 {
+  (void)this_gen;
   return VO_CAP_YV12 | VO_CAP_YUY2 | VO_CAP_BRIGHTNESS | VO_CAP_CONTRAST | VO_CAP_SATURATION;
 }
 
@@ -265,11 +266,13 @@ static vo_frame_t *fb_alloc_frame(vo_driver_t *this_gen)
 
 static void fb_compute_ideal_size(fb_driver_t *this, fb_frame_t *frame)
 {
+  (void)this;
   _x_vo_scale_compute_ideal_size(&frame->sc);
 }
 
 static void fb_compute_rgb_size(fb_driver_t *this, fb_frame_t *frame)
 {
+  (void)this;
   _x_vo_scale_compute_output_size(&frame->sc);
 
   /* avoid problems in yuv2rgb */
@@ -374,11 +377,11 @@ static void fb_update_frame_format(vo_driver_t *this_gen,
   flags &= VO_BOTH_FIELDS;
 
   /* Find out if we need to adapt this frame. */
-  if (width != frame->sc.delivered_width    ||
-      height != frame->sc.delivered_height  ||
-      ratio != frame->sc.delivered_ratio    ||
-      flags != frame->flags                 ||
-      format != frame->format               ||
+  if ((int)width  != frame->sc.delivered_width  ||
+      (int)height != frame->sc.delivered_height ||
+      ratio       != frame->sc.delivered_ratio  ||
+      flags       != frame->flags               ||
+      format      != frame->format              ||
       this->sc.user_ratio != frame->sc.user_ratio)
   {
     lprintf("frame format (from decoder) has changed => adapt\n");
@@ -413,6 +416,7 @@ static void fb_overlay_clut_yuv2rgb(fb_driver_t *this,
   int i;
   uint32_t *rgb;
 
+  (void)this;
   if (!overlay->rgb_clut) {
     rgb = overlay->color;
     for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
@@ -484,6 +488,7 @@ static void fb_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen,
 
 static int fb_redraw_needed(vo_driver_t *this_gen)
 {
+  (void)this_gen;
   return 0;
 }
 
@@ -658,6 +663,7 @@ static void fb_get_property_min_max(vo_driver_t *this_gen,
 {
   /* fb_driver_t *this = (fb_driver_t *) this_gen;  */
 
+  (void)this_gen;
   switch (property) {
   case VO_PROP_BRIGHTNESS:
     *min = -128;
@@ -678,6 +684,9 @@ static void fb_get_property_min_max(vo_driver_t *this_gen,
 static int fb_gui_data_exchange(vo_driver_t *this_gen,
 				int data_type, void *data)
 {
+  (void)this_gen;
+  (void)data_type;
+  (void)data;
   return 0;
 }
 
@@ -685,13 +694,14 @@ static void fb_dispose(vo_driver_t *this_gen)
 {
   fb_driver_t *this = (fb_driver_t *)this_gen;
 
-  if (this->video_mem_base != (void *)-1)
+  if (this->video_mem_base != MAP_FAILED)
     munmap(this->video_mem_base, this->mem_size);
 
   if (this->fd >= 0)
     close(this->fd);
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   if (this->yuv2rgb_factory)
     this->yuv2rgb_factory->dispose (this->yuv2rgb_factory);
@@ -847,6 +857,7 @@ static int mode_visual(fb_driver_t *this, config_values_t *config,
 		       struct fb_var_screeninfo *var,
 		       struct fb_fix_screeninfo *fix)
 {
+  (void)config;
   switch(fix->visual)
   {
     case FB_VISUAL_TRUECOLOR:
@@ -987,10 +998,12 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
   if(!this)
     return NULL;
 
-  this->video_mem_base = (void*)-1;
+  this->video_mem_base = MAP_FAILED;
 
   _x_alphablend_init(&this->alphablend_extra_data, class->xine);
 
+  _x_vo_scale_init(&this->sc, 0, 0, config);
+
   register_callbacks(this);
 
   this->fd = open_fb_device(config, class->xine);
@@ -1021,7 +1034,6 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
       (this->fb_var.xres_virtual *
        this->fb_var.bits_per_pixel)/8;
 
-  _x_vo_scale_init(&this->sc, 0, 0, config);
   this->sc.gui_width  = this->fb_var.xres;
   this->sc.gui_height = this->fb_var.yres;
   this->sc.user_ratio = XINE_VO_ASPECT_AUTO;
@@ -1054,7 +1066,7 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
   this->mem_size = this->fb_fix.smem_len;
   this->video_mem_base = mmap(0, this->mem_size, PROT_READ | PROT_WRITE,
                               MAP_SHARED, this->fd, 0);
-  if (this->video_mem_base == (void*)-1)
+  if (this->video_mem_base == MAP_FAILED)
     goto error;
 
   return &this->vo_driver;
@@ -1066,9 +1078,15 @@ error:
   return 0;
 }
 
-static void *fb_init_class(xine_t *xine, void *visual_gen)
+static void *fb_init_class(xine_t *xine, const void *visual_gen)
 {
-  fb_class_t *this = calloc(1, sizeof(fb_class_t));
+  fb_class_t *this;
+
+  (void)visual_gen;
+
+  this = calloc(1, sizeof(*this));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = fb_open_plugin;
   this->driver_class.identifier      = "fb";
@@ -1080,16 +1098,15 @@ static void *fb_init_class(xine_t *xine, void *visual_gen)
   return this;
 }
 
-static const vo_info_t vo_info_fb =
-{
-  1,                    /* priority    */
-  XINE_VISUAL_TYPE_FB   /* visual type */
+static const vo_info_t vo_info_fb = {
+  .priority    = 1,
+  .visual_type = XINE_VISUAL_TYPE_FB,
 };
 
 /* exported plugin catalog entry */
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "fb", XINE_VERSION_CODE, &vo_info_fb, fb_init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
