@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2018 the xine project
  * Copyright (C) 2014 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
@@ -485,10 +485,15 @@ static mmal_overlay_t *overlay_new(mmal_driver_t *this,
                                    int x, int y, int width, int height, int layer,
 				   uint32_t *argb) {
 
-  mmal_overlay_t      *ovl = calloc(1, sizeof(mmal_overlay_t));
+  mmal_overlay_t      *ovl;
   uint32_t             image_handle;
   static VC_DISPMANX_ALPHA_T  alpha;
   VC_RECT_T            src_rect;
+
+  ovl = calloc(1, sizeof(*ovl));
+  if (!ovl)
+    return NULL;
+
   //src_width &= 31;
   //dst_width &= 15;
   ovl->src_pitch = src_pitch;
@@ -893,10 +898,12 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 /**
  * Class Functions
  */
-static void *init_class (xine_t *xine, void *visual_gen) {
+static void *init_class (xine_t *xine, const void *visual_gen) {
   mmal_class_t      *this;
 
   this = (mmal_class_t*) calloc(1, sizeof(mmal_class_t));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin      = open_plugin;
   this->driver_class.identifier       = "MMAL";
@@ -909,12 +916,12 @@ static void *init_class (xine_t *xine, void *visual_gen) {
 }
 
 static const vo_info_t vo_info_mmal = {
-  10,                  /* priority */
-  XINE_VISUAL_TYPE_FB, /* visual type supported by this plugin */
+  .priority    = 10,
+  .visual_type = XINE_VISUAL_TYPE_FB,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "mmal", XINE_VERSION_CODE, &vo_info_mmal, init_class },
-  { PLUGIN_NONE, 0, "" , 0 , NULL, NULL}
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
