@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,23 +40,20 @@
 
 typedef struct {
   vo_frame_t           vo_frame;
-  int                  width;
-  int                  height;
+  unsigned int         width;
+  unsigned int         height;
   double               ratio;
   int                  format;
-  xine_t               *xine;
 } vo_none_frame_t;
 
 typedef struct {
   vo_driver_t          vo_driver;
-  config_values_t     *config;
   int                  ratio;
   xine_t               *xine;
 } vo_none_driver_t;
 
 typedef struct {
   video_driver_class_t  driver_class;
-  config_values_t      *config;
   xine_t               *xine;
 } vo_none_class_t;
 
@@ -79,11 +76,14 @@ static void vo_none_frame_dispose(vo_frame_t *vo_frame) {
 
 static void vo_none_frame_field(vo_frame_t *vo_frame, int which_field) {
   /* do nothing */
+  (void)vo_frame;
+  (void)which_field;
 }
 
 static uint32_t vo_none_get_capabilities(vo_driver_t *vo_driver) {
   /* No, we dont crop. Neither do we interpret color matrix or range. */
   /* But we also dont ask decoders to convert data just for the trash ;-) */
+  (void)vo_driver;
   return VO_CAP_YV12 | VO_CAP_YUY2 | VO_CAP_CROP | VO_CAP_COLOR_MATRIX | VO_CAP_FULLRANGE;
 }
 
@@ -116,6 +116,7 @@ static void vo_none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_f
   vo_none_driver_t *this = (vo_none_driver_t *) vo_driver;
   vo_none_frame_t  *frame = (vo_none_frame_t *) vo_frame;
 
+  (void)flags;
   if((frame->width != width) || (frame->height != height) || (frame->format != format)) {
 
     vo_none_free_framedata(frame);
@@ -167,8 +168,9 @@ static void vo_none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_f
 
     default:
       xprintf (this->xine, XINE_VERBOSITY_DEBUG, "video_out_none: unknown frame format %04x)\n", format);
+      frame->width = 0;
+      frame->vo_frame.width = 0;
       break;
-
     }
   }
 
@@ -177,9 +179,9 @@ static void vo_none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_f
 
 static void vo_none_display_frame(vo_driver_t *vo_driver, vo_frame_t *vo_frame) {
   /* vo_none_driver_t  *driver = (vo_none_driver_t *)vo_driver; */
-  vo_none_frame_t   *frame = (vo_none_frame_t *)vo_frame;
 
-  frame->vo_frame.free(&frame->vo_frame);
+  (void)vo_driver;
+  vo_frame->free(vo_frame);
 }
 
 static int vo_none_get_property(vo_driver_t *vo_driver, int property) {
@@ -218,6 +220,8 @@ static int vo_none_set_property(vo_driver_t *vo_driver, int property, int value)
 
 static void vo_none_get_property_min_max(vo_driver_t *vo_driver,
 				      int property, int *min, int *max) {
+  (void)vo_driver;
+  (void)property;
   *min = 0;
   *max = 0;
 }
@@ -225,6 +229,8 @@ static void vo_none_get_property_min_max(vo_driver_t *vo_driver,
 static int vo_none_gui_data_exchange(vo_driver_t *vo_driver, int data_type, void *data) {
 /*   vo_none_driver_t     *this = (vo_none_driver_t *) vo_driver; */
 
+  (void)vo_driver;
+  (void)data;
   switch (data_type) {
   case XINE_GUI_SEND_COMPLETION_EVENT:
   case XINE_GUI_SEND_DRAWABLE_CHANGED:
@@ -244,6 +250,7 @@ static void vo_none_dispose(vo_driver_t *vo_driver) {
 }
 
 static int vo_none_redraw_needed(vo_driver_t *vo_driver) {
+  (void)vo_driver;
   return 0;
 }
 
@@ -251,9 +258,11 @@ static vo_driver_t *vo_none_open_plugin(video_driver_class_t *driver_class, cons
   vo_none_class_t    *class = (vo_none_class_t *) driver_class;
   vo_none_driver_t   *driver;
 
+  (void)visual;
   driver = calloc(1, sizeof(vo_none_driver_t));
+  if (!driver)
+    return NULL;
 
-  driver->config = class->config;
   driver->xine   = class->xine;
   driver->ratio  = XINE_VO_ASPECT_AUTO;
 
@@ -277,25 +286,27 @@ static vo_driver_t *vo_none_open_plugin(video_driver_class_t *driver_class, cons
 /*
  * Class related functions.
  */
-static void *vo_none_init_class (xine_t *xine, void *visual) {
+static void *vo_none_init_class (xine_t *xine, const void *visual) {
   vo_none_class_t        *this;
 
+  (void)visual;
   this = calloc(1, sizeof(vo_none_class_t));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = vo_none_open_plugin;
   this->driver_class.identifier      = "none";
   this->driver_class.description     = N_("xine video output plugin which displays nothing");
   this->driver_class.dispose         = default_video_driver_class_dispose;
 
-  this->config                       = xine->config;
   this->xine                         = xine;
 
   return this;
 }
 
 static const vo_info_t vo_info_none = {
-  5,                        /* Priority    */
-  XINE_VISUAL_TYPE_NONE     /* Visual type */
+  .priority    = 5,
+  .visual_type = XINE_VISUAL_TYPE_NONE,
 };
 
 #define VO_NONE_CATALOG { PLUGIN_VIDEO_OUT, 22, "none", XINE_VERSION_CODE, &vo_info_none, vo_none_init_class }
@@ -304,7 +315,8 @@ static const vo_info_t vo_info_none = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   VO_NONE_CATALOG,
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 #endif
 
+
