@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -132,8 +132,7 @@
 #  define YUV_SWAP_MODE      0
 #endif
 
-#define MY_PI                3.1415926
-#define MY_2PI               6.2831853
+#define MY_2PI               (M_PI * 2)
 
 
 typedef struct {
@@ -551,6 +550,8 @@ static int render_help_image_tiledtex (opengl_driver_t *this,
 }
 
 static int render_image_nop (opengl_driver_t *this, opengl_frame_t *frame) {
+  (void)this;
+  (void)frame;
   return 1;
 }
 
@@ -632,7 +633,7 @@ static int render_image_fp_yuv (opengl_driver_t *this, opengl_frame_t *frame) {
   if (! ret)
     return 0;
   if (ret == 1) {
-    char *tmp = calloc (this->tex_width * this->tex_height, 1);
+    uint8_t *tmp = calloc (this->tex_width * this->tex_height, 1);
     for (i = 0; i < frame->width+3; i++) {
       tmp[this->tex_width*(frame->height+1)+i] = 128;
       tmp[this->tex_width*(frame->height+h2+2)+i] = 128;
@@ -678,8 +679,8 @@ static int render_image_envtex (opengl_driver_t *this, opengl_frame_t *frame) {
       ! this->tex_width || ! this->tex_height) {
     ret = render_image_tex (this, frame);
     /* Texture matrix has to skale/shift tex origin + swap y coords */
-    mTex[0]  =   1.0 * frame->width  / this->tex_width;
-    mTex[5]  =  -1.0 * frame->height / this->tex_height;
+    mTex[0]  =  this->tex_width  ? ( 1.0 * frame->width  / this->tex_width)  :  1.0;
+    mTex[5]  =  this->tex_height ? (-1.0 * frame->height / this->tex_height) : -1.0;
     mTex[12] = (-2.0 * mTex[0]) / mTex[0];
     mTex[13] =  -mTex[5];
     glMatrixMode  (GL_TEXTURE);
@@ -810,6 +811,7 @@ static void render_help_check_exts (opengl_driver_t *this) {
 }
 
 static int render_help_setup_tex (opengl_driver_t *this) {
+  (void)this;
   CHECKERR ("pre-tex_setup");
   glEnable        (GL_TEXTURE_2D);
   glTexEnvi       (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,   GL_REPLACE);
@@ -1301,6 +1303,7 @@ static uint32_t opengl_get_capabilities (vo_driver_t *this_gen) {
 /*   if (this->xoverlay) */
 /*     capabilities |= VO_CAP_UNSCALED_OVERLAY; */
 
+  (void)this_gen;
   return capabilities;
 }
 
@@ -1451,7 +1454,7 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
   }
 
   /* Check frame size and format and reallocate if necessary */
-  if ((frame->width != width) || (frame->height != height) || (frame->format != format)) {
+  if ((frame->width != (int)width) || (frame->height != (int)height) || (frame->format != format)) {
 /*     lprintf ("updating frame to %d x %d (ratio=%g, format=%08x)\n", width, height, ratio, format); */
 
     /* make sure we dont pull the rug from under the renderer */
@@ -1535,6 +1538,7 @@ static void opengl_overlay_clut_yuv2rgb(opengl_driver_t  *this, vo_overlay_t *ov
   int i;
   uint32_t *rgb;
 
+  (void)this;
   if (!overlay->rgb_clut) {
     rgb = overlay->color;
     for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
@@ -1573,6 +1577,7 @@ static void opengl_overlay_begin (vo_driver_t *this_gen,
 static void opengl_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img) {
   opengl_driver_t  *this  = (opengl_driver_t *) this_gen;
 
+  (void)vo_img;
   if (this->ovl_changed && this->xoverlay) {
     XLockDisplay (this->display);
     x11osd_expose(this->xoverlay);
@@ -1793,6 +1798,7 @@ static void opengl_get_property_min_max (vo_driver_t *this_gen,
 				     int property, int *min, int *max) {
   /* opengl_driver_t *this = (opengl_driver_t *) this_gen;  */
 
+  (void)this_gen;
   switch (property) {
   case VO_PROP_BRIGHTNESS:
     *min = -128;    *max = 127;     break;
@@ -1941,6 +1947,7 @@ static void opengl_dispose_internal (opengl_driver_t *this, int thread_running)
   }
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   free (this);
 }
@@ -1972,7 +1979,7 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
   const x11_visual_t   *visual  = (const x11_visual_t *) visual_gen;
   opengl_driver_t      *this;
   const char          **render_fun_names;
-  int                   i;
+  unsigned int          i;
 
   this = (opengl_driver_t *) calloc(1, sizeof(opengl_driver_t));
 
@@ -2164,7 +2171,7 @@ static int opengl_verify_direct (x11_visual_t *vis) {
   return ret;
 }
 
-static void *opengl_init_class (xine_t *xine, void *visual_gen) {
+static void *opengl_init_class (xine_t *xine, const void *visual_gen) {
 
   opengl_class_t *this;
 
@@ -2176,7 +2183,9 @@ static void *opengl_init_class (xine_t *xine, void *visual_gen) {
       return NULL;
   }
 
-  this = (opengl_class_t *) calloc (1, sizeof(opengl_class_t));
+  this = calloc (1, sizeof(*this));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = opengl_open_plugin;
   this->driver_class.identifier      = "opengl";
@@ -2189,8 +2198,8 @@ static void *opengl_init_class (xine_t *xine, void *visual_gen) {
 
 
 static const vo_info_t vo_info_opengl = {
-  7,                    /* priority    */
-  XINE_VISUAL_TYPE_X11  /* visual type */
+  .priority    = 7,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
 
@@ -2201,5 +2210,5 @@ static const vo_info_t vo_info_opengl = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "opengl", XINE_VERSION_CODE, &vo_info_opengl, opengl_init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
