@@ -1,6 +1,6 @@
 /*
  * kate: space-indent on; indent-width 2; mixedindent off; indent-mode cstyle; remove-trailing-space on;
- * Copyright (C) 2012-2017 the xine project
+ * Copyright (C) 2012-2019 the xine project
  * Copyright (C) 2012 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -52,11 +52,8 @@
 #define GL_GLEXT_PROTOTYPES
 #include <GL/gl.h>
 #include <GL/glext.h>
-#include <GL/glx.h>
 
-
-
-typedef int (*GLXSWAPINTERVALSGI) ( int );
+#include "opengl/xine_gl.h"
 
 typedef struct {
   vo_frame_t         vo_frame;
@@ -65,7 +62,6 @@ typedef struct {
 } opengl2_frame_t;
 
 
-
 typedef struct {
   int       ovl_w, ovl_h;
   int       ovl_x, ovl_y;
@@ -104,10 +100,7 @@ typedef struct {
   vo_driver_t        vo_driver;
   vo_scale_t         sc;
 
-  Display           *display;
-  int                screen;
-  Drawable           drawable;
-  GLXContext         context;
+  xine_gl_t         *gl;
 
   int                texture_float;
   opengl2_program_t  yuv420_program;
@@ -117,7 +110,8 @@ typedef struct {
   GLuint             overlayPBO;
   GLuint             fbo;
   GLuint             videoTex, videoTex2;
-  GLXSWAPINTERVALSGI mglXSwapInterval;
+  int                last_gui_width;
+  int                last_gui_height;
 
   int                ovl_changed;
   int                ovl_vid_scale;
@@ -132,6 +126,7 @@ typedef struct {
   int                contrast;
   int                brightness;
   int                hue;
+  int                update_sharpness;
   int                sharpness;
 
   opengl2_program_t  bicubic_pass1_program;
@@ -149,7 +144,6 @@ typedef struct {
 
   config_values_t   *config;
 
-  uint32_t          capabilities;
   xine_t            *xine;
 
   int		            zoom_x;
@@ -158,6 +152,11 @@ typedef struct {
   int                cm_state;
   uint8_t            cm_lut[32];
 
+  int                max_video_width;
+  int                max_video_height;
+  int                max_display_width;
+  int                max_display_height;
+
   int                exit_indx;
   int                exiting;
 } opengl2_driver_t;
@@ -233,8 +232,8 @@ static void opengl2_exit_register (opengl2_driver_t *this) {
 
 typedef struct {
   video_driver_class_t driver_class;
-  GLXContext           ctx;
   xine_t              *xine;
+  unsigned             visual_type;
 } opengl2_class_t;
 
 
@@ -273,7 +272,6 @@ static const char *bicubic_pass2_frag=
 
 
 
-#define PI 3.14159265359
 #define LUTWIDTH 1000
 #define N_SPLINES 2
 #define CATMULLROM_SPLINE   0
@@ -283,7 +281,7 @@ static float compute_cos_spline( float x )
 {
     if ( x < 0.0 )
         x = -x;
-    return 0.5 * cos( PI * x / 2.0 ) + 0.5;
+    return 0.5 * cos( M_PI * x / 2.0 ) + 0.5;
 }
 
 static float compute_catmullrom_spline( float x )
@@ -640,13 +638,13 @@ static void opengl2_upload_overlay(opengl2_driver_t *this, opengl2_overlay_t *o,
 static void opengl2_overlay_begin (vo_driver_t *this_gen, vo_frame_t *frame_gen, int changed)
 {
   //fprintf(stderr, "opengl2_overlay_begin\n");
+  (void)frame_gen;
   opengl2_driver_t  *this = (opengl2_driver_t *) this_gen;
 
   if ( changed ) {
     this->ovl_changed = 1;
 
-    if ( !glXMakeCurrent( this->display, this->drawable, this->context ) ) {
-      xprintf( this->xine, XINE_VERBOSITY_LOG, "video_out_opengl2: display unavailable for rendering\n" );
+    if (!this->gl->make_current(this->gl)) {
       return;
     }
   }
@@ -657,6 +655,7 @@ static void opengl2_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen,
 {
   opengl2_driver_t  *this = (opengl2_driver_t *) this_gen;
 
+  (void)frame_gen;
   if ( !this->ovl_changed || this->ovl_changed>XINE_VORAW_MAX_OVL )
     return;
 
@@ -696,6 +695,7 @@ static void opengl2_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img)
   opengl2_driver_t  *this = (opengl2_driver_t *) this_gen;
   unsigned i;
 
+  (void)vo_img;
   if ( !this->ovl_changed )
     return;
 
@@ -709,7 +709,7 @@ static void opengl2_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img)
     this->overlays[i].tex = 0;
   }
 
-  glXMakeCurrent( this->display, None, NULL );
+  this->gl->release_current(this->gl);
 }
 
 
@@ -717,6 +717,7 @@ static void opengl2_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img)
 
 static void opengl2_frame_proc_slice( vo_frame_t *vo_img, uint8_t **src )
 {
+  (void)src;
   vo_img->proc_called = 1;
 }
 
@@ -724,6 +725,8 @@ static void opengl2_frame_proc_slice( vo_frame_t *vo_img, uint8_t **src )
 
 static void opengl2_frame_field( vo_frame_t *vo_img, int which_field )
 {
+  (void)vo_img;
+  (void)which_field;
 }
 
 
@@ -772,8 +775,9 @@ static void opengl2_update_frame_format( vo_driver_t *this_gen, vo_frame_t *fram
 {
   opengl2_frame_t *frame = (opengl2_frame_t *) frame_gen;
 
+  (void)this_gen;
   /* Check frame size and format and reallocate if necessary */
-  if ( (frame->width != width) || (frame->height != height) || (frame->format != format) ) {
+  if ( (frame->width != (int)width) || (frame->height != (int)height) || (frame->format != format) ) {
 
     /* (re-) allocate render space */
     xine_freep_aligned (&frame->vo_frame.base[0]);
@@ -834,7 +838,7 @@ static int opengl2_redraw_needed( vo_driver_t *this_gen )
     _x_vo_scale_compute_output_size( &this->sc );
     return 1;
   }
-  return this->update_csc;
+  return this->update_csc | this->update_sharpness;
 }
 
 
@@ -845,7 +849,7 @@ static void opengl2_update_csc_matrix (opengl2_driver_t *that, opengl2_frame_t *
   color_standard = cm_from_frame (&frame->vo_frame);
 
   if ( that->update_csc || that->color_standard != color_standard ) {
-    float hue = (float)that->hue * 3.14159265359 / 128.0;
+    float hue = (float)that->hue * M_PI / 128.0;
     float saturation = (float)that->saturation / 128.0;
     float contrast = (float)that->contrast / 128.0;
     float brightness = that->brightness;
@@ -1261,6 +1265,8 @@ static int opengl2_draw_video_cubic_y( opengl2_driver_t *that, int guiw, int gui
 static int opengl2_draw_video_simple( opengl2_driver_t *that, int guiw, int guih, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1,
     GLfloat x, GLfloat y, GLfloat x1, GLfloat y1, GLuint video_texture )
 {
+  (void)that;
+
   glViewport( 0, 0, guiw, guih );
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
@@ -1290,6 +1296,8 @@ static int opengl2_draw_video_simple( opengl2_driver_t *that, int guiw, int guih
 static void opengl2_draw_video_bilinear( opengl2_driver_t *that, int guiw, int guih, GLfloat u, GLfloat v, GLfloat u1, GLfloat v1,
     GLfloat x, GLfloat y, GLfloat x1, GLfloat y1, GLuint video_texture )
 {
+  (void)that;
+
   glViewport( 0, 0, guiw, guih );
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
@@ -1316,17 +1324,17 @@ static void opengl2_draw_video_bilinear( opengl2_driver_t *that, int guiw, int g
 
 static void opengl2_draw( opengl2_driver_t *that, opengl2_frame_t *frame )
 {
-  if ( !glXMakeCurrent( that->display, that->drawable, that->context ) ) {
-    xprintf( that->xine, XINE_VERBOSITY_LOG, "video_out_opengl2: display unavailable for rendering\n" );
+  if (!that->gl->make_current(that->gl)) {
     return;
   }
 
   if ( !opengl2_check_textures_size( that, frame->width, frame->height ) ) {
-    glXMakeCurrent( that->display, None, NULL );
+    that->gl->release_current(that->gl);
     return;
   }
 
   opengl2_update_csc_matrix( that, frame );
+  that->update_sharpness = 0;
 
   glBindFramebuffer( GL_FRAMEBUFFER, that->fbo );
 
@@ -1442,9 +1450,8 @@ static void opengl2_draw( opengl2_driver_t *that, opengl2_frame_t *frame )
   //if ( that->mglXSwapInterval )
     //that->mglXSwapInterval( 1 );
 
-  glXSwapBuffers( that->display, that->drawable );
-
-  glXMakeCurrent( that->display, None, NULL );
+  that->gl->swap_buffers(that->gl);
+  that->gl->release_current(that->gl);
 }
 
 
@@ -1474,12 +1481,18 @@ static void opengl2_display_frame( vo_driver_t *this_gen, vo_frame_t *frame_gen
 
   opengl2_redraw_needed( this_gen );
 
+  if (this->gl->resize) {
+    if (this->last_gui_width != this->sc.gui_width || this->last_gui_height != this->sc.gui_height) {
+      this->last_gui_width = this->sc.gui_width;
+      this->last_gui_height = this->sc.gui_height;
+      this->gl->resize(this->gl, this->last_gui_width, this->last_gui_height);
+    }
+  }
+
   if( !this->exiting ) {
-    XLockDisplay (this->display);
     pthread_mutex_lock(&this->drawable_lock); /* protect drawable from being changed */
     opengl2_draw( this, frame );
     pthread_mutex_unlock(&this->drawable_lock); /* allow changing drawable again */
-    XUnlockDisplay (this->display);
   }
 
   if( !this->exit_indx )
@@ -1525,6 +1538,10 @@ static int opengl2_get_property( vo_driver_t *this_gen, int property )
       return this->zoom_y;
     case VO_PROP_ASPECT_RATIO:
       return this->sc.user_ratio;
+    case VO_PROP_MAX_VIDEO_WIDTH:
+      return this->max_video_width;
+    case VO_PROP_MAX_VIDEO_HEIGHT:
+      return this->max_video_height;
   }
 
   return -1;
@@ -1565,7 +1582,7 @@ static int opengl2_set_property( vo_driver_t *this_gen, int property, int value
     case VO_PROP_SATURATION: this->saturation = value; this->update_csc = 1; break;
     case VO_PROP_CONTRAST: this->contrast = value; this->update_csc = 1; break;
     case VO_PROP_BRIGHTNESS: this->brightness = value; this->update_csc = 1; break;
-    case VO_PROP_SHARPNESS: this->sharpness = value; break;
+    case VO_PROP_SHARPNESS: this->sharpness = value; this->update_sharpness = 1; break;
   }
 
   return value;
@@ -1575,6 +1592,7 @@ static int opengl2_set_property( vo_driver_t *this_gen, int property, int value
 
 static void opengl2_get_property_min_max( vo_driver_t *this_gen, int property, int *min, int *max ) 
 {
+  (void)this_gen;
   switch ( property ) {
     case VO_PROP_HUE:
       *max = 127; *min = -128; break;
@@ -1610,7 +1628,7 @@ static int opengl2_gui_data_exchange( vo_driver_t *this_gen, int data_type, void
 
     case XINE_GUI_SEND_DRAWABLE_CHANGED: {
       pthread_mutex_lock(&this->drawable_lock); /* wait for other thread which is currently displaying */
-      this->drawable = (Drawable)data;
+      this->gl->set_native_window(this->gl, data);
       pthread_mutex_unlock(&this->drawable_lock);
       this->sc.force_redraw = 1;
       break;
@@ -1640,9 +1658,22 @@ static int opengl2_gui_data_exchange( vo_driver_t *this_gen, int data_type, void
 
 static uint32_t opengl2_get_capabilities( vo_driver_t *this_gen )
 {
-  opengl2_driver_t *this = (opengl2_driver_t *) this_gen;
+  (void)this_gen;
 
-  return this->capabilities;
+  return VO_CAP_YV12 |
+         VO_CAP_YUY2 |
+         VO_CAP_CROP |
+         VO_CAP_UNSCALED_OVERLAY |
+         VO_CAP_CUSTOM_EXTENT_OVERLAY |
+         VO_CAP_ARGB_LAYER_OVERLAY |
+      /* VO_CAP_VIDEO_WINDOW_OVERLAY | */
+         VO_CAP_COLOR_MATRIX |
+         VO_CAP_FULLRANGE |
+         VO_CAP_HUE |
+         VO_CAP_SATURATION |
+         VO_CAP_CONTRAST |
+         VO_CAP_BRIGHTNESS |
+         VO_CAP_SHARPNESS;
 }
 
 
@@ -1663,12 +1694,16 @@ static void opengl2_dispose( vo_driver_t *this_gen )
 
   opengl2_exit_unregister( this );
 
+  /* cm_close already does this.
+  this->xine->config->unregister_callbacks (this->xine->config, "video.output.opengl2_bicubic_scaling", NULL, this, sizeof (*this));
+  */
   cm_close (this);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   pthread_mutex_destroy(&this->drawable_lock);
 
-  glXMakeCurrent( this->display, this->drawable, this->context );
-  
+  this->gl->make_current(this->gl);
+
   opengl2_delete_program( &this->yuv420_program );
   opengl2_delete_program( &this->yuv422_program );
 
@@ -1709,10 +1744,9 @@ static void opengl2_dispose( vo_driver_t *this_gen )
   for ( i=0; i<XINE_VORAW_MAX_OVL; ++i ) {
     glDeleteTextures( 1, &this->overlays[i].tex );
   }
-  
-  glXMakeCurrent( this->display, None, NULL );
 
-  glXDestroyContext( this->display, this->context );
+  this->gl->release_current(this->gl);
+  this->gl->dispose(&this->gl);
 
   free (this);
 }
@@ -1722,25 +1756,78 @@ static void opengl2_dispose( vo_driver_t *this_gen )
 static vo_driver_t *opengl2_open_plugin( video_driver_class_t *class_gen, const void *visual_gen )
 {
   opengl2_class_t     *class   = (opengl2_class_t *) class_gen;
-  const x11_visual_t  *visual  = (const x11_visual_t *) visual_gen;
   opengl2_driver_t    *this;
   config_values_t     *config  = class->xine->config;
 
-  this = (opengl2_driver_t *) calloc(1, sizeof(opengl2_driver_t));
-
+  this = calloc (1, sizeof (*this));
   if (!this)
     return NULL;
+#ifndef HAVE_ZERO_SAFE_MEM
+  this->hue                            = 0;
+  this->brightness                     = 0;
+  this->update_sharpness               = 0;
+  this->sharpness                      = 0;
+  this->sharpness_program.compiled     = 0;
+  this->bicubic_pass1_program.compiled = 0;
+  this->bicubic_pass2_program.compiled = 0;
+  this->bicubic_lut_texture            = 0;
+  this->bicubic_pass1_texture          = 0;
+  this->bicubic_pass1_texture_width    = 0;
+  this->bicubic_pass1_texture_height   = 0;
+  this->bicubic_fbo                    = 0;
+  this->ovl_changed                    = 0;
+  this->num_ovls                       = 0;
+  this->yuvtex.y                       = 0;
+  this->yuvtex.u                       = 0;
+  this->yuvtex.v                       = 0;
+  this->yuvtex.yuv                     = 0;
+  this->yuvtex.width                   = 0;
+  this->yuvtex.height                  = 0;
+  this->fbo                            = 0;
+  this->videoPBO                       = 0;
+  this->videoTex                       = 0;
+  this->videoTex2                      = 0;
+  {
+    int i;
+    for (i = 0; i < XINE_VORAW_MAX_OVL; ++i) {
+      this->overlays[i].ovl_w = this->overlays[i].ovl_h = 0;
+      this->overlays[i].ovl_x = this->overlays[i].ovl_y = 0;
+      this->overlays[i].unscaled = 0;
+      this->overlays[i].tex = 0;
+      this->overlays[i].tex_w = this->overlays[i].tex_h = 0;
+    }
+  }
+#endif
 
-  this->display       = visual->display;
-  this->screen        = visual->screen;
-  this->drawable      = visual->d;
-  this->context        = class->ctx;
-  pthread_mutex_init(&this->drawable_lock, 0);
+  this->gl = _x_load_gl(class->xine, class->visual_type, visual_gen, XINE_GL_API_OPENGL);
+  if (!this->gl) {
+    goto fail_gl_init;
+  }
+
+  {
+    /* TJ. If X server link gets lost, our next render attempt will fire the
+     * Xlib fatal error handler -> exit () -> opengl2_exit () with drawable_lock held.
+     * opengl2_display_frame () does quite a lot anyway so the "recursive mutex"
+     * performance drop should not matter. */
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init (&attr);
+    pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init (&this->drawable_lock, &attr);
+    pthread_mutexattr_destroy (&attr);
+  }
 
   _x_vo_scale_init(&this->sc, 1, 0, config);
-  this->sc.frame_output_cb  = visual->frame_output_cb;
-  this->sc.dest_size_cb     = visual->dest_size_cb;
-  this->sc.user_data        = visual->user_data;
+
+  if (class->visual_type == XINE_VISUAL_TYPE_X11) {
+    const x11_visual_t  *visual  = (const x11_visual_t *) visual_gen;
+    this->sc.frame_output_cb  = visual->frame_output_cb;
+    this->sc.dest_size_cb     = visual->dest_size_cb;
+    this->sc.user_data        = visual->user_data;
+  } else /*if (class->visual_type == XINE_VISUAL_TYPE_WAYLAND)*/ {
+    const xine_wayland_visual_t  *visual  = (const xine_wayland_visual_t *) visual_gen;
+    this->sc.frame_output_cb  = visual->frame_output_cb;
+    this->sc.user_data        = visual->user_data;
+  }
   this->sc.user_ratio       = XINE_VO_ASPECT_AUTO;
 
   this->zoom_x = 100;
@@ -1763,10 +1850,31 @@ static vo_driver_t *opengl2_open_plugin( video_driver_class_t *class_gen, const
   this->vo_driver.dispose              = opengl2_dispose;
   this->vo_driver.redraw_needed        = opengl2_redraw_needed;
 
-  if ( !glXMakeCurrent( this->display, this->drawable, this->context ) ) {
+  if (!this->gl->make_current(this->gl)) {
     xprintf( this->xine, XINE_VERBOSITY_LOG, "video_out_opengl2: display unavailable for initialization\n" );
-    free( this );
-    return NULL;
+    goto fail_make_current;
+  }
+
+  {
+    GLint v[1] = {0};
+    glGetIntegerv (GL_MAX_TEXTURE_SIZE, v);
+    if (v[0] > 0) {
+      this->max_video_width  =
+      this->max_video_height = v[0];
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "video_out_opengl2: max video size %dx%d.\n",
+        this->max_video_width, this->max_video_height);
+    }
+  }
+
+  {
+    GLint v[2] = {0, 0};
+    glGetIntegerv (GL_MAX_VIEWPORT_DIMS, v);
+    if (v[0] > 0) {
+      this->max_display_width  = v[0];
+      this->max_display_height = v[1] > 0 ? v[1] : v[0];
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "video_out_opengl2: max output size %dx%d.\n",
+        this->max_display_width, this->max_display_height);
+    }
   }
 
   glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
@@ -1788,69 +1896,26 @@ static vo_driver_t *opengl2_open_plugin( video_driver_class_t *class_gen, const
 #define INITWIDTH  720
 #define INITHEIGHT 576
 
-  this->yuvtex.y = this->yuvtex.u = this->yuvtex.v = this->yuvtex.yuv = 0;
-  this->yuvtex.width = this->yuvtex.height = 0;
-  this->fbo = this->videoPBO = this->videoTex = this->videoTex2 = 0;
   if ( !opengl2_check_textures_size( this, INITWIDTH, INITHEIGHT ) ) {
-    glXMakeCurrent( this->display, None, NULL );
-    free( this );
-    return NULL;
+    goto fail;
   }
 
   if ( !opengl2_build_program( this, &this->yuv420_program, &yuv420_frag, "yuv420_frag" ) ) {
-    glXMakeCurrent( this->display, None, NULL );
-    free( this );
-    return NULL;
+    goto fail;
   }
   if ( !opengl2_build_program( this, &this->yuv422_program, &yuv422_frag, "yuv422_frag" ) ) {
-    glXMakeCurrent( this->display, None, NULL );
-    free( this );
-    return NULL;
+    goto fail;
   }
 
-  this->mglXSwapInterval = (GLXSWAPINTERVALSGI)glXGetProcAddressARB( (const GLubyte*)"glXSwapIntervalSGI" );
-
-  glXMakeCurrent( this->display, None, NULL );
-
-  this->capabilities = VO_CAP_YV12 | VO_CAP_YUY2 | VO_CAP_CROP | VO_CAP_UNSCALED_OVERLAY | VO_CAP_CUSTOM_EXTENT_OVERLAY | VO_CAP_ARGB_LAYER_OVERLAY;// | VO_CAP_VIDEO_WINDOW_OVERLAY;
-
-  this->capabilities |= VO_CAP_COLOR_MATRIX | VO_CAP_FULLRANGE;
-
-  this->capabilities |= VO_CAP_HUE;
-  this->capabilities |= VO_CAP_SATURATION;
-  this->capabilities |= VO_CAP_CONTRAST;
-  this->capabilities |= VO_CAP_BRIGHTNESS;
+  this->gl->release_current(this->gl);
 
   this->update_csc = 1;
   this->color_standard = 10;
-  this->hue = 0;
   this->saturation = 128;
   this->contrast = 128;
-  this->brightness = 0;
-  this->sharpness = 0;
-  this->sharpness_program.compiled = 0;
 
   cm_init (this);
 
-  this->bicubic_pass1_program.compiled = 0;
-  this->bicubic_pass2_program.compiled = 0;
-  this->bicubic_lut_texture = 0;
-  this->bicubic_pass1_texture = 0;
-  this->bicubic_pass1_texture_width = 0;
-  this->bicubic_pass1_texture_height = 0;
-  this->bicubic_fbo = 0;
-
-  int i;
-  for ( i=0; i<XINE_VORAW_MAX_OVL; ++i ) {
-    this->overlays[i].ovl_w = this->overlays[i].ovl_h = 0;
-    this->overlays[i].ovl_x = this->overlays[i].ovl_y = 0;
-    this->overlays[i].unscaled = 0;
-    this->overlays[i].tex = 0;
-    this->overlays[i].tex_w = this->overlays[i].tex_h = 0;
-  }
-  this->ovl_changed = 0;
-  this->num_ovls = 0;
-
   if ( this->texture_float ) {
     this->scale_bicubic = config->register_bool( config, "video.output.opengl2_bicubic_scaling", 0,
       _("opengl2: use a bicubic algo to scale the video"),
@@ -1863,72 +1928,68 @@ static vo_driver_t *opengl2_open_plugin( video_driver_class_t *class_gen, const
   xprintf( this->xine, XINE_VERBOSITY_DEBUG, "video_out_opengl2: initialized.\n");
 
   return &this->vo_driver;
+
+ fail:
+  this->gl->release_current(this->gl);
+ fail_make_current:
+  this->gl->dispose(&this->gl);
+ fail_gl_init:
+  free(this);
+  return NULL;
 }
 
-
-
-static int opengl2_check_platform( opengl2_class_t *this_gen, const x11_visual_t *vis )
+static int opengl2_check_platform( xine_t *xine, unsigned visual_type, const void *visual )
 {
-  int attribs[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_RED_SIZE, 8,
-    GLX_GREEN_SIZE, 8, GLX_BLUE_SIZE, 8, GLX_DEPTH_SIZE, 16, None };
+  const char *extensions;
+  xine_gl_t  *gl;
+  int result = 0;
 
-  Window        root;
-  XVisualInfo  *visinfo;
-  GLXContext    ctx;
-  int           ret = 1;
-
-  if ( !vis || !vis->display || !( root = RootWindow( vis->display, vis->screen ) ) )
+  gl = _x_load_gl(xine, visual_type, visual, XINE_GL_API_OPENGL);
+  if (!gl)
     return 0;
-  
-  if ( !( visinfo = glXChooseVisual( vis->display, vis->screen, attribs ) ) )
-    return 0;
-  
-  if ( !( ctx = glXCreateContext( vis->display, visinfo, NULL, GL_TRUE  ) ) ) {
-    XFree( visinfo );
-    return 0;
-  }
-  
-  if ( glXMakeCurrent( vis->display, root, ctx ) ) {
-    if ( !glXIsDirect( vis->display, ctx ) )
-      ret = 0;
-    const char *extensions = glGetString( GL_EXTENSIONS );
-    if ( !strstr( extensions, "ARB_texture_rectangle" ) )
-      ret = 0;
-    if ( !strstr( extensions, "ARB_texture_non_power_of_two" ) )
-      ret = 0;
-    if ( !strstr( extensions, "ARB_pixel_buffer_object" ) )
-      ret = 0;
-    if ( !strstr( extensions, "ARB_framebuffer_object" ) )
-      ret = 0;
-    if ( !strstr( extensions, "ARB_fragment_shader" ) )
-      ret = 0;
-    if ( !strstr( extensions, "ARB_vertex_shader" ) )
-      ret = 0;
-    glXMakeCurrent( vis->display, None, NULL );
-  }
-  else
-    ret = 0;
 
-  if ( !ret )
-    glXDestroyContext( vis->display, ctx );
-  else
-    this_gen->ctx = ctx;    
+  if (!gl->make_current(gl))
+    goto fail_released;
 
-  XFree( visinfo );
+  extensions = glGetString(GL_EXTENSIONS);
+  if (!extensions)
+    goto fail;
 
-  return ret;
+  if (!strstr( extensions, "ARB_texture_rectangle"))
+    goto fail;
+  if (!strstr( extensions, "ARB_texture_non_power_of_two"))
+    goto fail;
+  if (!strstr( extensions, "ARB_pixel_buffer_object"))
+    goto fail;
+  if (!strstr( extensions, "ARB_framebuffer_object"))
+    goto fail;
+  if (!strstr( extensions, "ARB_fragment_shader"))
+    goto fail;
+  if (!strstr( extensions, "ARB_vertex_shader"))
+    goto fail;
+  result = 1;
+
+ fail:
+  gl->release_current(gl);
+ fail_released:
+  gl->dispose(&gl);
+  return result;
 }
 
 /*
  * class functions
  */
 
-static void *opengl2_init_class( xine_t *xine, void *visual_gen )
+static void *opengl2_init_class( xine_t *xine, unsigned visual_type, const void *visual_gen )
 {
-  opengl2_class_t *this = (opengl2_class_t *) calloc(1, sizeof(opengl2_class_t));
+  opengl2_class_t *this;
 
-  if ( !opengl2_check_platform( this, (const x11_visual_t *)visual_gen ) ) {
-    free(this);
+  if (!opengl2_check_platform( xine, visual_type, visual_gen)) {
+    return NULL;
+  }
+
+  this = calloc(1, sizeof(*this));
+  if (!this) {
     return NULL;
   }
 
@@ -1937,17 +1998,31 @@ static void *opengl2_init_class( xine_t *xine, void *visual_gen )
   this->driver_class.description     = N_("xine video output plugin using opengl 2.0");
   this->driver_class.dispose         = default_video_driver_class_dispose;
   this->xine                         = xine;
+  this->visual_type                  = visual_type;
 
   return this;
 }
 
+static void *opengl2_init_class_x11( xine_t *xine, const void *visual_gen )
+{
+  return opengl2_init_class(xine, XINE_VISUAL_TYPE_X11, visual_gen);
+}
+
+static void *opengl2_init_class_wl( xine_t *xine, const void *visual_gen )
+{
+  return opengl2_init_class(xine, XINE_VISUAL_TYPE_WAYLAND, visual_gen);
+}
 
 
 static const vo_info_t vo_info_opengl2 = {
-  8,                    /* priority    */
-  XINE_VISUAL_TYPE_X11  /* visual type */
+  .priority    = 8,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
+static const vo_info_t vo_info_opengl2_wl = {
+  .priority    = 8,
+  .visual_type = XINE_VISUAL_TYPE_WAYLAND,
+};
 
 /*
  * exported plugin catalog entry
@@ -1955,6 +2030,7 @@ static const vo_info_t vo_info_opengl2 = {
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_OUT, 22, "opengl2", XINE_VERSION_CODE, &vo_info_opengl2, opengl2_init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_VIDEO_OUT, 22, "opengl2", XINE_VERSION_CODE, &vo_info_opengl2,    opengl2_init_class_x11 },
+  { PLUGIN_VIDEO_OUT, 22, "opengl2", XINE_VERSION_CODE, &vo_info_opengl2_wl, opengl2_init_class_wl },
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
