@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -57,6 +57,8 @@
 
 #include <xcb/xv.h>
 
+#define DEBUG_EMU
+
 #include "xine.h"
 #include <xine/video_out.h>
 #include <xine/xine_internal.h>
@@ -84,11 +86,13 @@ typedef struct {
 } xv_property_t;
 
 typedef struct {
+  /* public part */
   vo_frame_t         vo_frame;
-
-  int                width, height, format;
+  /* change detection */
   double             ratio;
-
+  int                req_width, req_height, format;
+  /* xv image */
+  int                width, height, xvformat;
   uint8_t           *image;
   xcb_shm_seg_t      shmseg;
   unsigned int       xv_format;
@@ -97,8 +101,9 @@ typedef struct {
   unsigned int       xv_height;
   unsigned int       xv_pitches[3];
   unsigned int       xv_offsets[3];
-
-  int                req_width, req_height;
+  /* yv12 backend for yuy2 emulation */
+  uint8_t           *base[3];
+  int                pitches[3];
 } xv_frame_t;
 
 
@@ -106,8 +111,6 @@ struct xv_driver_s {
 
   vo_driver_t        vo_driver;
 
-  config_values_t   *config;
-
   /* xcb / xv related stuff */
   xcb_connection_t  *connection;
   xcb_screen_t      *screen;
@@ -136,6 +139,8 @@ struct xv_driver_s {
 
   pthread_mutex_t    main_mutex;
 
+  int                emu_yuy2;
+
   /* color matrix switching */
   uint8_t            cm_lut[32];
   int                cm_active, cm_state;
@@ -145,7 +150,6 @@ struct xv_driver_s {
 typedef struct {
   video_driver_class_t driver_class;
 
-  config_values_t     *config;
   xine_t              *xine;
 } xv_class_t;
 
@@ -165,12 +169,18 @@ static uint32_t xv_get_capabilities (vo_driver_t *this_gen) {
 
 static void xv_frame_field (vo_frame_t *vo_img, int which_field) {
   /* not needed for Xv */
+  (void)vo_img;
+  (void)which_field;
 }
 
+static void xv_rem_yuy2_emu (xv_frame_t *f);
+
 static void xv_frame_dispose (vo_frame_t *vo_img) {
   xv_frame_t  *frame = (xv_frame_t *) vo_img ;
   xv_driver_t *this  = (xv_driver_t *) vo_img->driver;
 
+  xv_rem_yuy2_emu (frame);
+
   if (frame->shmseg) {
     pthread_mutex_lock(&this->main_mutex);
     xcb_shm_detach(this->connection, frame->shmseg);
@@ -193,18 +203,27 @@ static vo_frame_t *xv_alloc_frame (vo_driver_t *this_gen) {
   if (!frame)
     return NULL;
 
+  frame->req_width  = 0;
+  frame->req_height = 0;
+  frame->format     = 0;
+  frame->width      = 0;
+  frame->height     = 0;
+  frame->xvformat   = 0;
+  frame->image      = NULL;
+  frame->base[0]    = NULL;
+  frame->vo_frame.proc_slice = NULL;
+  frame->vo_frame.proc_frame = NULL;
+
   pthread_mutex_init (&frame->vo_frame.mutex, NULL);
 
   /*
    * supply required functions
    */
-  frame->vo_frame.proc_slice = NULL;
-  frame->vo_frame.proc_frame = NULL;
   frame->vo_frame.field      = xv_frame_field;
   frame->vo_frame.dispose    = xv_frame_dispose;
   frame->vo_frame.driver     = this_gen;
 
-  return (vo_frame_t *) frame;
+  return &frame->vo_frame;
 }
 
 static void create_ximage(xv_driver_t *this, xv_frame_t *frame, int width, int height, int format)
@@ -227,7 +246,7 @@ static void create_ximage(xv_driver_t *this, xv_frame_t *frame, int width, int h
     break;
   default:
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unknown format %08x\n",format);
-    _x_abort();
+    return;
   }
 
   if (frame->xv_format == 0) {
@@ -340,7 +359,7 @@ static void create_ximage(xv_driver_t *this, xv_frame_t *frame, int width, int h
     break;
   default:
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unknown format %08x\n",format);
-    _x_abort();
+    return;
   }
 }
 
@@ -355,74 +374,158 @@ static void dispose_ximage(xv_driver_t *this, xv_frame_t *frame)
   frame->image = NULL;
 }
 
+static void xv_slice_yuy2_emu (vo_frame_t *vo_img, uint8_t **src) {
+  xv_frame_t  *frame = (xv_frame_t *)vo_img;
+  int y = (src[0] - frame->vo_frame.base[0]) / frame->vo_frame.pitches[0];
+  int h;
+
+  if ((y < 0) || (y >= frame->height))
+    return;
+
+  if (!vo_img->proc_called)
+    vo_img->proc_called = 1;
+
+  h = frame->height - y;
+  h = h > 16 ? 16 : h;
+  yuy2_to_yv12 (src[0], frame->vo_frame.pitches[0],
+    frame->base[0] + y * frame->pitches[0], frame->pitches[0],
+    frame->base[1] + (y >> 1) * frame->pitches[1], frame->pitches[1],
+    frame->base[2] + (y >> 1) * frame->pitches[2], frame->pitches[2],
+    frame->width, h);
+}
+
+static int xv_add_yuy2_emu (xv_frame_t *f) {
+  uint8_t *base;
+  int pitch;
+  if (f->base[0])
+    return 0;
+  pitch = f->xv_pitches[0] * 2;
+  base = xine_malloc_aligned (pitch * f->xv_height);
+  if (!base)
+    return 1;
+  /* save backend */
+  f->base[0] = f->vo_frame.base[0];
+  f->base[1] = f->vo_frame.base[1];
+  f->base[2] = f->vo_frame.base[2];
+  f->pitches[0] = f->vo_frame.pitches[0];
+  f->pitches[1] = f->vo_frame.pitches[1];
+  f->pitches[2] = f->vo_frame.pitches[2];
+  /* new frontend */
+  f->vo_frame.base[0] = base;
+  f->vo_frame.base[1] =
+  f->vo_frame.base[2] = NULL;
+  f->vo_frame.pitches[0] = pitch;
+  f->vo_frame.pitches[1] =
+  f->vo_frame.pitches[2] = 0;
+  f->format = XINE_IMGFMT_YUY2;
+  f->vo_frame.proc_slice = xv_slice_yuy2_emu;
+  {
+    const union {uint8_t bytes[4]; uint32_t word;} black = {{0, 128, 0, 128}};
+    uint32_t *q = (uint32_t *)base;
+    int i;
+    for (i = pitch * f->xv_height / 4; i > 0; i--)
+      *q++ = black.word;
+  }
+  return 0;
+}
+
+static void xv_rem_yuy2_emu (xv_frame_t *f) {
+  if (!f->base[0])
+    return;
+  xine_free_aligned (f->vo_frame.base[0]);
+  f->vo_frame.base[0] = f->base[0];
+  f->vo_frame.base[1] = f->base[1];
+  f->vo_frame.base[2] = f->base[2];
+  f->vo_frame.pitches[0] = f->pitches[0];
+  f->vo_frame.pitches[1] = f->pitches[1];
+  f->vo_frame.pitches[2] = f->pitches[2];
+  f->base[0] = NULL;
+  f->format = XINE_IMGFMT_YV12;
+  f->vo_frame.proc_slice = NULL;
+}
+
 static void xv_update_frame_format (vo_driver_t *this_gen,
 				    vo_frame_t *frame_gen,
 				    uint32_t width, uint32_t height,
 				    double ratio, int format, int flags) {
   xv_driver_t  *this  = (xv_driver_t *) this_gen;
   xv_frame_t   *frame = (xv_frame_t *) frame_gen;
+  int resize;
 
+  (void)flags;
   if (this->use_pitch_alignment) {
     width = (width + 7) & ~0x7;
   }
+  resize = (frame->req_width != (int)width) || (frame->req_height != (int)height);
 
-  if ((frame->req_width != width)
-      || (frame->req_height != height)
-      || (frame->format != format)) {
+  if (resize || (frame->format != format)) {
+    int fmt = format;
+    if ((fmt == XINE_IMGFMT_YUY2) && (this->emu_yuy2 || !this->xv_format_yuy2))
+      fmt = XINE_IMGFMT_YV12;
 
-    /* printf (LOG_MODULE ": updating frame to %d x %d (ratio=%d, format=%08x)\n",width,height,ratio_code,format); */
-
-    pthread_mutex_lock(&this->main_mutex);
-
-    /*
-     * (re-) allocate xvimage
-     */
-
-    if (frame->image)
-      dispose_ximage(this, frame);
-
-    create_ximage(this, frame, width, height, format);
-    if (!frame->image) {
-      pthread_mutex_unlock(&this->main_mutex);
-      frame->vo_frame.base[0] = NULL;
-      frame->vo_frame.base[1] = NULL;
-      frame->vo_frame.base[2] = NULL;
-      frame->req_width = 0;
-      frame->vo_frame.width = 0;
-      return;
-    }
-
-    if(format == XINE_IMGFMT_YUY2) {
-      frame->vo_frame.pitches[0] = frame->xv_pitches[0];
-      frame->vo_frame.base[0] = frame->image + frame->xv_offsets[0];
-      {
-        const union {uint8_t bytes[4]; uint32_t word;} black = {{0, 128, 0, 128}};
-        uint32_t *q = (uint32_t *)frame->vo_frame.base[0];
-        int i;
-        for (i = frame->vo_frame.pitches[0] * frame->xv_height / 4; i > 0; i--)
-          *q++ = black.word;
+    if (resize || (frame->xvformat != fmt)) {
+      /* get emulation out of the way first if any */
+      xv_rem_yuy2_emu (frame);
+      /* (re-) allocate xvimage */
+      pthread_mutex_lock (&this->main_mutex);
+      if (frame->image)
+        dispose_ximage (this, frame);
+      create_ximage (this, frame, width, height, fmt);
+      pthread_mutex_unlock (&this->main_mutex);
+      if (!frame->image) {
+        frame->vo_frame.base[0] = NULL;
+        frame->vo_frame.base[1] = NULL;
+        frame->vo_frame.base[2] = NULL;
+        frame->req_width = 0;
+        frame->vo_frame.width = 0;
+        return;
+      }
+      frame->xvformat = fmt;
+      /* get params, blackfill */
+      if (fmt == XINE_IMGFMT_YUY2) {
+        frame->vo_frame.pitches[0] = frame->xv_pitches[0];
+        frame->vo_frame.base[0]    = frame->image + frame->xv_offsets[0];
+        {
+          const union {uint8_t bytes[4]; uint32_t word;} black = {{0, 128, 0, 128}};
+          uint32_t *q = (uint32_t *)frame->vo_frame.base[0];
+          int i;
+          for (i = frame->vo_frame.pitches[0] * frame->xv_height / 4; i > 0; i--)
+            *q++ = black.word;
+        }
+      } else { /* XINE_IMGFMT_YV12 */
+        frame->vo_frame.pitches[0] = frame->xv_pitches[0];
+        frame->vo_frame.pitches[1] = frame->xv_pitches[2];
+        frame->vo_frame.pitches[2] = frame->xv_pitches[1];
+        frame->vo_frame.base[0] = frame->image + frame->xv_offsets[0];
+        frame->vo_frame.base[1] = frame->image + frame->xv_offsets[2];
+        frame->vo_frame.base[2] = frame->image + frame->xv_offsets[1];
+        memset (frame->vo_frame.base[0],   0, frame->vo_frame.pitches[0] *  frame->xv_height);
+        memset (frame->vo_frame.base[1], 128, frame->vo_frame.pitches[1] * (frame->xv_height >> 1));
+        memset (frame->vo_frame.base[2], 128, frame->vo_frame.pitches[2] * (frame->xv_height >> 1));
       }
     }
-    else {
-      frame->vo_frame.pitches[0] = frame->xv_pitches[0];
-      frame->vo_frame.pitches[1] = frame->xv_pitches[2];
-      frame->vo_frame.pitches[2] = frame->xv_pitches[1];
-      frame->vo_frame.base[0] = frame->image + frame->xv_offsets[0];
-      frame->vo_frame.base[1] = frame->image + frame->xv_offsets[2];
-      frame->vo_frame.base[2] = frame->image + frame->xv_offsets[1];
-      memset (frame->vo_frame.base[0], 0, frame->vo_frame.pitches[0] * frame->xv_height);
-      memset (frame->vo_frame.base[1], 128, frame->vo_frame.pitches[1] * frame->xv_height / 2);
-      memset (frame->vo_frame.base[2], 128, frame->vo_frame.pitches[2] * frame->xv_height / 2);
-    }
-
+    /* update emulation */
+    frame->format = format;
+    if (fmt != format) {
+      if (xv_add_yuy2_emu (frame)) {
+        pthread_mutex_lock (&this->main_mutex);
+        dispose_ximage (this, frame);
+        pthread_mutex_unlock (&this->main_mutex);
+        frame->image = NULL;
+        frame->vo_frame.base[0] = NULL;
+        frame->vo_frame.base[1] = NULL;
+        frame->vo_frame.base[2] = NULL;
+        frame->req_width = 0;
+        frame->vo_frame.width = 0;
+        return;
+      }
+    } else
+      xv_rem_yuy2_emu (frame);
     /* allocated frame size may not match requested size */
     frame->req_width  = width;
     frame->req_height = height;
     frame->width  = frame->xv_width;
     frame->height = frame->xv_height;
-    frame->format = format;
-
-    pthread_mutex_unlock(&this->main_mutex);
   }
 
   if (frame->vo_frame.width > frame->width)
@@ -510,6 +613,7 @@ static void xv_overlay_begin (vo_driver_t *this_gen,
 static void xv_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img) {
   xv_driver_t  *this = (xv_driver_t *) this_gen;
 
+  (void)vo_img;
   if( this->ovl_changed && this->xoverlay ) {
     pthread_mutex_lock(&this->main_mutex);
     xcbosd_expose(this->xoverlay);
@@ -1042,6 +1146,7 @@ static void xv_dispose (vo_driver_t *this_gen) {
   }
   pthread_mutex_unlock(&this->main_mutex);
 
+  /* should do nothing as video_out has sent VO_PROP_DISCARD_FRAMES = -1 before */
   for( i=0; i < VO_NUM_RECENT_FRAMES; i++ ) {
     if( this->recent_frames[i] )
       this->recent_frames[i]->vo_frame.dispose
@@ -1058,9 +1163,13 @@ static void xv_dispose (vo_driver_t *this_gen) {
   pthread_mutex_destroy(&this->main_mutex);
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   cm_close (this);
 
+  /* cm_close already does this.
+  this->xine->config->unregister_callbacks (this->xine->config, NULL, NULL, this, sizeof (*this));
+  */
   free (this);
 }
 
@@ -1100,37 +1209,39 @@ static void xv_prop_init (xv_driver_t *this, const xv_prop_list_t *l, const xcb_
     LOG_MODULE ": port attribute %s (%d) value is %d\n", l->name, l->index, prop->value);
 }
 
-static void xv_prop_update (void *prop_gen, xine_cfg_entry_t *entry) {
-  xv_property_t *prop = (xv_property_t *)prop_gen;
+static void xv_prop_update_int (xv_property_t *prop, int value) {
   xv_driver_t   *this = prop->this;
 
-  pthread_mutex_lock(&this->main_mutex);
-  xcb_xv_set_port_attribute (this->connection, this->xv_port, prop->atom, entry->num_value);
-  pthread_mutex_unlock(&this->main_mutex);
-  prop->value = entry->num_value;
+  pthread_mutex_lock (&this->main_mutex);
+  xcb_xv_set_port_attribute (this->connection, this->xv_port, prop->atom, value);
+  pthread_mutex_unlock (&this->main_mutex);
+  prop->value = value;
 
-  xprintf (this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": %s = %d\n", prop->name, entry->num_value);
+  xprintf (this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": %s = %d\n", prop->name, value);
+}
+
+static void xv_prop_update (void *prop_gen, xine_cfg_entry_t *entry) {
+  xv_prop_update_int ((xv_property_t *)prop_gen, entry->num_value);
 }
 
 static void xv_prop_conf (xv_driver_t *this, int property,
   const char *config_name, const char *config_desc, const char *config_help) {
+  config_values_t *config = this->xine->config;
   xv_property_t *prop = &this->props[property];
   cfg_entry_t *entry;
 
   /* is this a boolean property ? */
   if ((prop->min == 0) && (prop->max == 1))
-    this->config->register_bool (this->config, config_name, prop->value,
+    config->register_bool (config, config_name, prop->value,
       config_desc, config_help, 20, xv_prop_update, prop);
   else
-    this->config->register_range (this->config, config_name, prop->value,
+    config->register_range (config, config_name, prop->value,
       prop->min, prop->max, config_desc, config_help, 20, xv_prop_update, prop);
 
-  entry = this->config->lookup_entry (this->config, config_name);
+  entry = config->lookup_entry (config, config_name);
 
-  if ((entry->num_value < prop->min) || (entry->num_value > prop->max)) {
-    this->config->update_num (this->config, config_name, ((prop->min + prop->max) >> 1));
-    entry = this->config->lookup_entry (this->config, config_name);
-  }
+  if ((entry->num_value < prop->min) || (entry->num_value > prop->max))
+    xv_prop_update_int (prop, (prop->min + prop->max) >> 1);
 
   prop->entry = entry;
 
@@ -1153,6 +1264,13 @@ static void xv_fullrange_cb_config (void *this_gen, xine_cfg_entry_t *entry) {
   this->cm_active = 0;
 }
 
+#ifdef DEBUG_EMU
+static void xv_debug_emu_cb_config (void *this_gen, xine_cfg_entry_t *entry) {
+  xv_driver_t *this = (xv_driver_t *)this_gen;
+  this->emu_yuy2 = entry->num_value;
+}
+#endif
+
 static xcb_xv_port_t xv_open_port (xv_driver_t *this, xcb_xv_port_t port) {
   {
     xcb_xv_list_image_formats_cookie_t list_formats_cookie;
@@ -1192,18 +1310,14 @@ static xcb_xv_port_t xv_open_port (xv_driver_t *this, xcb_xv_port_t port) {
     free (list_formats_reply);
   }
 
-  /* XXX
-   * Bail out if both YV12 and YUY2 are not supported.
-   * Proper fix would be implementing format conversions.
-   * - decoder should check video output CAP_ flags
-   * - video out could insert automatic conversion filter
-   * - ...
-   */
-  if (!this->xv_format_yuy2 || !this->xv_format_yv12) {
+  /* No yv12: bail out. */
+  if (!this->xv_format_yv12) {
     xprintf (this->xine, XINE_VERBOSITY_LOG,
-      LOG_MODULE ": this adaptor does not support YV12 and YUY2 formats.\n");
+      LOG_MODULE ": this adaptor does not support YV12 format.\n");
     return 0;
   }
+  /* No yuy2: just report. Who can deliver yv12 with little effort shall do that.
+   * Otherwise, accept anyway and emulate. */
 
   {
     xcb_xv_grab_port_cookie_t grab_port_cookie;
@@ -1226,8 +1340,8 @@ xv_find_adaptor_by_port (int port, xcb_xv_adaptor_info_iterator_t *adaptor_it)
 {
   for (; adaptor_it->rem; xcb_xv_adaptor_info_next(adaptor_it))
     if (adaptor_it->data->type & XCB_XV_TYPE_IMAGE_MASK)
-      if (port >= adaptor_it->data->base_id &&
-	  port < adaptor_it->data->base_id + adaptor_it->data->num_ports)
+      if (port >= (int)adaptor_it->data->base_id &&
+	  port < (int)(adaptor_it->data->base_id + adaptor_it->data->num_ports))
 	return adaptor_it;
   return NULL; /* shouldn't happen */
 }
@@ -1264,7 +1378,7 @@ static xcb_xv_port_t xv_autodetect_port(xv_driver_t *this,
 
 static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *visual_gen) {
   xv_class_t           *class = (xv_class_t *) class_gen;
-  config_values_t      *config = class->config;
+  config_values_t      *config = class->xine->config;
   xv_driver_t          *this;
   int                   i;
   const xcb_visual_t   *visual = (const xcb_visual_t *) visual_gen;
@@ -1278,15 +1392,31 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   if (!this)
     return NULL;
 
+#ifndef HAVE_ZERO_SAFE_MEM
+  /* 0/NULL inits, for optimizing away. */
+  this->xoverlay        = NULL;
+  this->ovl_changed     = 0;
+  this->xv_format_yv12  = 0;
+  this->xv_format_yuy2  = 0;
+  this->emu_yuy2        = 0;
+  this->fullrange_mode  = 0;
+  for (i = 0; i < XV_NUM_PROPERTIES; i++) {
+    this->props[i].initial_value = 0;
+    this->props[i].value = 0;
+    this->props[i].min   = 0;
+    this->props[i].max   = 0;
+    this->props[i].defer = 0;
+    this->props[i].name  = NULL;
+    this->props[i].entry = NULL;
+  }
+#endif
+
   this->capabilities = VO_CAP_CROP | VO_CAP_ZOOM_X | VO_CAP_ZOOM_Y;
   this->use_shm      = 1;
-  this->xoverlay     = NULL;
-  this->ovl_changed  = 0;
   this->xine         = class->xine;
   this->connection   = visual->connection;
   this->screen       = visual->screen;
   this->window       = visual->window;
-  this->config       = config;
 
   /*
    * check for Xvideo support
@@ -1407,14 +1537,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
    */
 
   for (i = 0; i < XV_NUM_PROPERTIES; i++) {
-    this->props[i].initial_value = 0;
-    this->props[i].value = 0;
-    this->props[i].min   = 0;
-    this->props[i].max   = 0;
     this->props[i].atom  = XCB_NONE;
-    this->props[i].defer = 0;
-    this->props[i].name  = NULL;
-    this->props[i].entry = NULL;
     this->props[i].this  = this;
   }
 
@@ -1458,52 +1581,55 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
           }
           xv_prop_init (this, l, attribute_it.data);
 
-          if (l->index == XV_PROP_ITURBT_709) { /* nvidia */
-            this->cm_active = this->props[XV_PROP_ITURBT_709].value ? 2 : 10;
-
-          } else if (l->index == XV_PROP_COLORSPACE) { /* radeonhd */
-            this->cm_active = this->props[XV_PROP_COLORSPACE].value == 2 ? 2 : (this->props[XV_PROP_COLORSPACE].value == 1 ? 10 : 0);
-
-          } else if (l->index == VO_PROP_COLORKEY) {
-            xv_prop_conf (this, VO_PROP_COLORKEY, "video.device.xv_colorkey", VIDEO_DEVICE_XV_COLORKEY_HELP);
-
-          } else if (l->index == VO_PROP_AUTOPAINT_COLORKEY) {
-            /* disable autopaint colorkey by default, might be overridden using config entry */
-            this->props[VO_PROP_AUTOPAINT_COLORKEY].value = 0;
-            xv_prop_conf (this, VO_PROP_AUTOPAINT_COLORKEY,
-              "video.device.xv_autopaint_colorkey", VIDEO_DEVICE_XV_AUTOPAINT_COLORKEY_HELP);
-
-          } else if (l->index == XV_PROP_FILTER) {
-            int xv_filter;
-            /* This setting is specific to Permedia 2/3 cards. */
-            xv_filter = config->register_range (config, "video.device.xv_filter", 0,
-              attribute_it.data->min, attribute_it.data->max, VIDEO_DEVICE_XV_FILTER_HELP,
-              20, xv_prop_update, &this->props[XV_PROP_FILTER]);
-            config->update_num (config,"video.device.xv_filter",xv_filter);
-
-          } else if (l->index == XV_PROP_DOUBLE_BUFFER) {
-            int xv_double_buffer;
-            xv_double_buffer = config->register_bool (config, "video.device.xv_double_buffer", 1,
-              VIDEO_DEVICE_XV_DOUBLE_BUFFER_HELP, 20, xv_prop_update, &this->props[XV_PROP_DOUBLE_BUFFER]);
-            config->update_num (config,"video.device.xv_double_buffer",xv_double_buffer);
-
-          } else if (l->index == XV_PROP_SYNC_TO_VBLANK) {
-            int xv_sync_to_vblank;
-            xv_sync_to_vblank = config->register_bool (config, "video.device.xv_sync_to_vblank", 1,
-              _("enable vblank sync"),
-              _("This option will synchronize the update of the video image to the "
-                "repainting of the entire screen (\"vertical retrace\"). This eliminates "
-                "flickering and tearing artifacts. On nvidia cards one may also "
-                "need to run \"nvidia-settings\" and choose which display device to "
-                "sync to under the XVideo Settings tab"),
-              20, xv_prop_update, &this->props[XV_PROP_SYNC_TO_VBLANK]);
-            config->update_num(config,"video.device.xv_sync_to_vblank",xv_sync_to_vblank);
-
-          } else if (l->index == XV_PROP_BICUBIC) {
-            int xv_bicubic;
-            xv_bicubic = config->register_enum (config, "video.device.xv_bicubic", 2,
-              (char **)bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP, 20, xv_prop_update, &this->props[XV_PROP_BICUBIC]);
-              config->update_num(config,"video.device.xv_bicubic",xv_bicubic);
+          switch (l->index) {
+            case XV_PROP_ITURBT_709: /* nvidia */
+              this->cm_active = this->props[XV_PROP_ITURBT_709].value ? 2 : 10;
+              break;
+            case XV_PROP_COLORSPACE: /* radeonhd */
+              this->cm_active = this->props[XV_PROP_COLORSPACE].value == 2 ? 2
+                              : (this->props[XV_PROP_COLORSPACE].value == 1 ? 10 : 0);
+              break;
+            case VO_PROP_COLORKEY:
+              xv_prop_conf (this, VO_PROP_COLORKEY, "video.device.xv_colorkey", VIDEO_DEVICE_XV_COLORKEY_HELP);
+              break;
+            case VO_PROP_AUTOPAINT_COLORKEY:
+              /* disable autopaint colorkey by default, might be overridden using config entry */
+              this->props[VO_PROP_AUTOPAINT_COLORKEY].value = 0;
+              xv_prop_conf (this, VO_PROP_AUTOPAINT_COLORKEY,
+                "video.device.xv_autopaint_colorkey", VIDEO_DEVICE_XV_AUTOPAINT_COLORKEY_HELP);
+              break;
+            case XV_PROP_FILTER: {
+              /* This setting is specific to Permedia 2/3 cards. */
+              int xv_filter = config->register_range (config, "video.device.xv_filter", 0,
+                attribute_it.data->min, attribute_it.data->max, VIDEO_DEVICE_XV_FILTER_HELP,
+                20, xv_prop_update, &this->props[XV_PROP_FILTER]);
+              xv_prop_update_int (&this->props[XV_PROP_FILTER], xv_filter);
+              break;
+            }
+            case XV_PROP_DOUBLE_BUFFER: {
+              int xv_double_buffer = config->register_bool (config, "video.device.xv_double_buffer", 1,
+                VIDEO_DEVICE_XV_DOUBLE_BUFFER_HELP, 20, xv_prop_update, &this->props[XV_PROP_DOUBLE_BUFFER]);
+              xv_prop_update_int (&this->props[XV_PROP_DOUBLE_BUFFER], xv_double_buffer);
+              break;
+            }
+            case XV_PROP_SYNC_TO_VBLANK: {
+              int xv_sync_to_vblank = config->register_bool (config, "video.device.xv_sync_to_vblank", 1,
+                _("enable vblank sync"),
+                _("This option will synchronize the update of the video image to the "
+                  "repainting of the entire screen (\"vertical retrace\"). This eliminates "
+                  "flickering and tearing artifacts. On nvidia cards one may also "
+                  "need to run \"nvidia-settings\" and choose which display device to "
+                  "sync to under the XVideo Settings tab"),
+                20, xv_prop_update, &this->props[XV_PROP_SYNC_TO_VBLANK]);
+              xv_prop_update_int (&this->props[XV_PROP_SYNC_TO_VBLANK], xv_sync_to_vblank);
+              break;
+            }
+            case XV_PROP_BICUBIC: {
+              int xv_bicubic = config->register_enum (config, "video.device.xv_bicubic", 2,
+                (char **)bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP, 20, xv_prop_update, &this->props[XV_PROP_BICUBIC]);
+              xv_prop_update_int (&this->props[XV_PROP_BICUBIC], xv_bicubic);
+              break;
+            }
           }
         }
       }
@@ -1523,8 +1649,8 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   if ((this->props[VO_PROP_BRIGHTNESS].atom != XCB_NONE) &&
     (this->props[VO_PROP_CONTRAST].atom != XCB_NONE)) {
     static const char * const xv_fullrange_conf_labels[] = {"Off", "Normal", NULL};
-    this->fullrange_mode = this->xine->config->register_enum (
-      this->xine->config,
+    this->fullrange_mode = config->register_enum (
+      config,
       "video.output.xv_fullrange",
       0,
       (char **)xv_fullrange_conf_labels,
@@ -1540,10 +1666,21 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
     );
     if (this->fullrange_mode)
       this->capabilities |= VO_CAP_FULLRANGE;
-  } else {
-    this->fullrange_mode = 0;
   }
 
+#ifdef DEBUG_EMU
+  this->emu_yuy2 = config->register_bool (
+    config,
+    "video.output.xv_debug_emu",
+    0,
+    _("Force emulating yuy2"),
+    _("Debug."),
+    10,
+    xv_debug_emu_cb_config,
+    this
+  );
+#endif
+
   this->use_pitch_alignment =
     config->register_bool (config, "video.device.xv_pitch_alignment", 0,
 			   VIDEO_DEVICE_XV_PITCH_ALIGNMENT_HELP,
@@ -1569,23 +1706,28 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 /*
  * class functions
  */
-static void *init_class (xine_t *xine, void *visual_gen) {
-  xv_class_t        *this = (xv_class_t *) calloc(1, sizeof(xv_class_t));
+static void *init_class (xine_t *xine, const void *visual_gen) {
+  xv_class_t        *this;
+
+  (void)visual_gen;
+
+  this = calloc(1, sizeof(*this));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = open_plugin;
   this->driver_class.identifier      = "Xv";
   this->driver_class.description     = N_("xine video output plugin using the MIT X video extension");
   this->driver_class.dispose         = default_video_driver_class_dispose;
 
-  this->config                       = xine->config;
   this->xine                         = xine;
 
   return this;
 }
 
 static const vo_info_t vo_info_xv = {
-  9,                      /* priority    */
-  XINE_VISUAL_TYPE_XCB    /* visual type */
+  .priority    = 9,
+  .visual_type = XINE_VISUAL_TYPE_XCB,
 };
 
 /*
@@ -1595,5 +1737,5 @@ static const vo_info_t vo_info_xv = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "xv", XINE_VERSION_CODE, &vo_info_xv, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
