@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -143,7 +143,6 @@ typedef struct {
 typedef struct {
   video_driver_class_t driver_class;
 
-  config_values_t     *config;
   xine_t              *xine;
 } xshm_class_t;
 
@@ -343,10 +342,12 @@ static uint32_t xshm_get_capabilities (vo_driver_t *this_gen) {
 }
 
 static void xshm_compute_ideal_size (xshm_driver_t *this, xshm_frame_t *frame) {
+  (void)this;
   _x_vo_scale_compute_ideal_size( &frame->sc );
 }
 
 static void xshm_compute_rgb_size (xshm_driver_t *this, xshm_frame_t *frame) {
+  (void)this;
   _x_vo_scale_compute_output_size( &frame->sc );
 
   /* avoid problems in yuv2rgb */
@@ -631,11 +632,12 @@ static void xshm_update_frame_format (vo_driver_t *this_gen,
   xshm_frame_t   *frame = (xshm_frame_t *) frame_gen;
   int             j, y_pitch, uv_pitch;
 
+  (void)this_gen;
   flags &= VO_BOTH_FIELDS;
 
   /* (re)allocate yuv buffers */
-  if ((width != frame->sc.delivered_width)
-      || (height != frame->sc.delivered_height)
+  if (((int)width != frame->sc.delivered_width)
+      || ((int)height != frame->sc.delivered_height)
       || (format != frame->format)) {
 
     frame->sc.delivered_width    = width;
@@ -703,6 +705,7 @@ static void xshm_overlay_clut_yuv2rgb(xshm_driver_t  *this, vo_overlay_t *overla
   int i;
   uint32_t *rgb;
 
+  (void)this;
   if (!overlay->rgb_clut) {
     rgb = overlay->color;
     for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
@@ -741,6 +744,7 @@ static void xshm_overlay_begin (vo_driver_t *this_gen,
 static void xshm_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img) {
   xshm_driver_t  *this  = (xshm_driver_t *) this_gen;
 
+  (void)vo_img;
   if( this->ovl_changed && this->xoverlay ) {
     LOCK_DISPLAY(this);
     x11osd_expose(this->xoverlay);
@@ -1022,6 +1026,7 @@ static void xshm_get_property_min_max (vo_driver_t *this_gen,
 				     int property, int *min, int *max) {
   /* xshm_driver_t *this = (xshm_driver_t *) this_gen;  */
 
+  (void)this_gen;
   if (property == VO_PROP_BRIGHTNESS) {
     *min = -128;
     *max = +127;
@@ -1150,6 +1155,7 @@ static void xshm_dispose (vo_driver_t *this_gen) {
   }
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   free (this);
 }
@@ -1212,7 +1218,7 @@ static const char *visual_class_name(Visual *visual) {
 /* expects XINE_VISUAL_TYPE_X11_2 with configurable locking */
 static vo_driver_t *xshm_open_plugin_2 (video_driver_class_t *class_gen, const void *visual_gen) {
   xshm_class_t         *class   = (xshm_class_t *) class_gen;
-  config_values_t      *config  = class->config;
+  config_values_t      *config  = class->xine->config;
   const x11_visual_t   *visual  = (const x11_visual_t *) visual_gen;
   xshm_driver_t        *this;
   XWindowAttributes     attribs;
@@ -1436,36 +1442,43 @@ static vo_driver_t *xshm_open_plugin_old (video_driver_class_t *class_gen, const
 /*
  * class functions
  */
-static void *xshm_init_class (xine_t *xine, void *visual_gen) {
-  xshm_class_t	       *this = (xshm_class_t *) calloc(1, sizeof(xshm_class_t));
+static void *xshm_init_class (xine_t *xine, const void *visual_gen) {
+  xshm_class_t *this;
+
+  (void)visual_gen;
+
+  this = calloc(1, sizeof(*this));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = xshm_open_plugin_old;
   this->driver_class.identifier      = "XShm";
   this->driver_class.description     = N_("xine video output plugin using the MIT X shared memory extension");
   this->driver_class.dispose         = default_video_driver_class_dispose;
-  this->config                       = xine->config;
   this->xine                         = xine;
 
   return this;
 }
 
-static void *xshm_init_class_2 (xine_t *xine, void *visual_gen) {
+static void *xshm_init_class_2 (xine_t *xine, const void *visual_gen) {
   xshm_class_t	       *this;
   this = xshm_init_class (xine, visual_gen);
-  this->driver_class.open_plugin     = xshm_open_plugin_2;
+  if (this) {
+    this->driver_class.open_plugin     = xshm_open_plugin_2;
+  }
   return this;
 }
 
 
 static const vo_info_t vo_info_xshm = {
-  6,                      /* priority    */
-  XINE_VISUAL_TYPE_X11    /* visual type */
+  .priority    = 6,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
 /* visual type with configurable X11 locking */
 static const vo_info_t vo_info_xshm_2 = {
-  6,                      /* priority    */
-  XINE_VISUAL_TYPE_X11_2  /* visual type */
+  .priority    = 6,
+  .visual_type = XINE_VISUAL_TYPE_X11_2,
 };
 
 
@@ -1477,5 +1490,5 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "xshm", XINE_VERSION_CODE, &vo_info_xshm, xshm_init_class },
   { PLUGIN_VIDEO_OUT, 22, "xshm", XINE_VERSION_CODE, &vo_info_xshm_2, xshm_init_class_2 },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
