@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -63,6 +63,8 @@
 #define LOG
 */
 
+#define DEBUG_EMU
+
 #include "xine.h"
 #include <xine/video_out.h>
 #include <xine/xine_internal.h>
@@ -73,10 +75,9 @@
 #define XV_PROPS
 #include "xv_common.h"
 
-#define LOCK_DISPLAY(this) {if(this->lock_display) this->lock_display(this->user_data); \
-                            else XLockDisplay(this->display);}
-#define UNLOCK_DISPLAY(this) {if(this->unlock_display) this->unlock_display(this->user_data); \
-                            else XUnlockDisplay(this->display);}
+#define LOCK_DISPLAY(this) this->lock_display (this->ld_user_data)
+#define UNLOCK_DISPLAY(this) this->unlock_display (this->ud_user_data);
+
 typedef struct xv_driver_s xv_driver_t;
 
 typedef struct {
@@ -94,15 +95,18 @@ typedef struct {
 } xv_property_t;
 
 typedef struct {
+  /* public part */
   vo_frame_t         vo_frame;
-
-  int                width, height, format;
+  /* change detection */
   double             ratio;
-
+  int                req_width, req_height, format;
+  /* xv image */
+  int                width, height, xvformat;
   XvImage           *image;
   XShmSegmentInfo    shminfo;
-
-  int                req_width, req_height;
+  /* yv12 backend for yuy2 emulation */
+  uint8_t           *base[3];
+  int                pitches[3];
 } xv_frame_t;
 
 
@@ -110,8 +114,6 @@ struct xv_driver_s {
 
   vo_driver_t        vo_driver;
 
-  config_values_t   *config;
-
   /* X11 / Xv related stuff */
   Display           *display;
   int                screen;
@@ -143,10 +145,12 @@ struct xv_driver_s {
   alphablend_t       alphablend_extra_data;
 
   void             (*lock_display) (void *);
+  void              *ld_user_data;
 
   void             (*unlock_display) (void *);
+  void              *ud_user_data;
 
-  void              *user_data;
+  int                emu_yuy2;
 
   /* color matrix switching */
   uint8_t            cm_lut[32];
@@ -157,7 +161,6 @@ struct xv_driver_s {
 typedef struct {
   video_driver_class_t driver_class;
 
-  config_values_t     *config;
   xine_t              *xine;
 } xv_class_t;
 
@@ -179,12 +182,18 @@ static uint32_t xv_get_capabilities (vo_driver_t *this_gen) {
 
 static void xv_frame_field (vo_frame_t *vo_img, int which_field) {
   /* not needed for Xv */
+  (void)vo_img;
+  (void)which_field;
 }
 
+static void xv_rem_yuy2_emu (xv_frame_t *f);
+
 static void xv_frame_dispose (vo_frame_t *vo_img) {
   xv_frame_t  *frame = (xv_frame_t *) vo_img ;
   xv_driver_t *this  = (xv_driver_t *) vo_img->driver;
 
+  xv_rem_yuy2_emu (frame);
+
   if (frame->image) {
 
     if (frame->shminfo.shmaddr) {
@@ -216,18 +225,27 @@ static vo_frame_t *xv_alloc_frame (vo_driver_t *this_gen) {
   if (!frame)
     return NULL;
 
+  frame->req_width  = 0;
+  frame->req_height = 0;
+  frame->format     = 0;
+  frame->width      = 0;
+  frame->height     = 0;
+  frame->xvformat   = 0;
+  frame->image      = NULL;
+  frame->base[0]    = NULL;
+  frame->vo_frame.proc_slice = NULL;
+  frame->vo_frame.proc_frame = NULL;
+
   pthread_mutex_init (&frame->vo_frame.mutex, NULL);
 
   /*
    * supply required functions
    */
-  frame->vo_frame.proc_slice = NULL;
-  frame->vo_frame.proc_frame = NULL;
   frame->vo_frame.field      = xv_frame_field;
   frame->vo_frame.dispose    = xv_frame_dispose;
   frame->vo_frame.driver     = this_gen;
 
-  return (vo_frame_t *) frame;
+  return &frame->vo_frame;
 }
 
 static int HandleXError (Display *display, XErrorEvent *xevent) {
@@ -274,7 +292,7 @@ static XvImage *create_ximage (xv_driver_t *this, XShmSegmentInfo *shminfo,
     break;
   default:
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unknown format %08x\n",format);
-    _x_abort();
+    return NULL;
   }
 
   if (xv_format == 0) {
@@ -433,76 +451,158 @@ static void dispose_ximage (xv_driver_t *this,
   }
 }
 
+static void xv_slice_yuy2_emu (vo_frame_t *vo_img, uint8_t **src) {
+  xv_frame_t  *frame = (xv_frame_t *)vo_img;
+  int y = (src[0] - frame->vo_frame.base[0]) / frame->vo_frame.pitches[0];
+  int h;
+
+  if ((y < 0) || (y >= frame->height))
+    return;
+
+  if (!vo_img->proc_called)
+    vo_img->proc_called = 1;
+
+  h = frame->height - y;
+  h = h > 16 ? 16 : h;
+  yuy2_to_yv12 (src[0], frame->vo_frame.pitches[0],
+    frame->base[0] + y * frame->pitches[0], frame->pitches[0],
+    frame->base[1] + (y >> 1) * frame->pitches[1], frame->pitches[1],
+    frame->base[2] + (y >> 1) * frame->pitches[2], frame->pitches[2],
+    frame->width, h);
+}
+
+static int xv_add_yuy2_emu (xv_frame_t *f) {
+  uint8_t *base;
+  int pitch;
+  if (f->base[0])
+    return 0;
+  pitch = f->image->pitches[0] * 2;
+  base = xine_malloc_aligned (pitch * f->image->height);
+  if (!base)
+    return 1;
+  /* save backend */
+  f->base[0] = f->vo_frame.base[0];
+  f->base[1] = f->vo_frame.base[1];
+  f->base[2] = f->vo_frame.base[2];
+  f->pitches[0] = f->vo_frame.pitches[0];
+  f->pitches[1] = f->vo_frame.pitches[1];
+  f->pitches[2] = f->vo_frame.pitches[2];
+  /* new frontend */
+  f->vo_frame.base[0] = base;
+  f->vo_frame.base[1] =
+  f->vo_frame.base[2] = NULL;
+  f->vo_frame.pitches[0] = pitch;
+  f->vo_frame.pitches[1] =
+  f->vo_frame.pitches[2] = 0;
+  f->format = XINE_IMGFMT_YUY2;
+  f->vo_frame.proc_slice = xv_slice_yuy2_emu;
+  {
+    const union {uint8_t bytes[4]; uint32_t word;} black = {{0, 128, 0, 128}};
+    uint32_t *q = (uint32_t *)base;
+    int i;
+    for (i = pitch * f->image->height / 4; i > 0; i--)
+      *q++ = black.word;
+  }
+  return 0;
+}
+
+static void xv_rem_yuy2_emu (xv_frame_t *f) {
+  if (!f->base[0])
+    return;
+  xine_free_aligned (f->vo_frame.base[0]);
+  f->vo_frame.base[0] = f->base[0];
+  f->vo_frame.base[1] = f->base[1];
+  f->vo_frame.base[2] = f->base[2];
+  f->vo_frame.pitches[0] = f->pitches[0];
+  f->vo_frame.pitches[1] = f->pitches[1];
+  f->vo_frame.pitches[2] = f->pitches[2];
+  f->base[0] = NULL;
+  f->format = XINE_IMGFMT_YV12;
+  f->vo_frame.proc_slice = NULL;
+}
+
 static void xv_update_frame_format (vo_driver_t *this_gen,
 				    vo_frame_t *frame_gen,
 				    uint32_t width, uint32_t height,
 				    double ratio, int format, int flags) {
   xv_driver_t  *this  = (xv_driver_t *) this_gen;
   xv_frame_t   *frame = (xv_frame_t *) frame_gen;
+  int resize;
 
+  (void)flags;
   if (this->use_pitch_alignment) {
     width = (width + 7) & ~0x7;
   }
+  resize = (frame->req_width != (int)width) || (frame->req_height != (int)height);
 
-  if ((frame->req_width != width)
-      || (frame->req_height != height)
-      || (frame->format != format)) {
+  if (resize || (frame->format != format)) {
+    int fmt = format;
+    if ((fmt == XINE_IMGFMT_YUY2) && (this->emu_yuy2 || !this->xv_format_yuy2))
+      fmt = XINE_IMGFMT_YV12;
 
-    /* printf (LOG_MODULE ": updating frame to %d x %d (ratio=%d, format=%08x)\n",width,height,ratio_code,format); */
-
-    LOCK_DISPLAY(this);
-
-    /*
-     * (re-) allocate xvimage
-     */
-
-    if (frame->image) {
-      dispose_ximage (this, &frame->shminfo, frame->image);
-      frame->image = NULL;
-    }
-
-    frame->image = create_ximage (this, &frame->shminfo, width, height, format);
-    if (!frame->image) {
+    if (resize || (frame->xvformat != fmt)) {
+      /* get emulation out of the way first if any */
+      xv_rem_yuy2_emu (frame);
+      /* (re-) allocate xvimage */
+      LOCK_DISPLAY (this);
+      if (frame->image)
+        dispose_ximage (this, &frame->shminfo, frame->image);
+      frame->image = create_ximage (this, &frame->shminfo, width, height, fmt);
       UNLOCK_DISPLAY (this);
-      frame->vo_frame.base[0] = NULL;
-      frame->vo_frame.base[1] = NULL;
-      frame->vo_frame.base[2] = NULL;
-      frame->req_width = 0;
-      frame->vo_frame.width = 0;
-      return;
-    }
-
-    if(format == XINE_IMGFMT_YUY2) {
-      frame->vo_frame.pitches[0] = frame->image->pitches[0];
-      frame->vo_frame.base[0] = frame->image->data + frame->image->offsets[0];
-      {
-        const union {uint8_t bytes[4]; uint32_t word;} black = {{0, 128, 0, 128}};
-        uint32_t *q = (uint32_t *)frame->vo_frame.base[0];
-        int i;
-        for (i = frame->vo_frame.pitches[0] * frame->image->height / 4; i > 0; i--)
-          *q++ = black.word;
+      if (!frame->image) {
+        frame->vo_frame.base[0] = NULL;
+        frame->vo_frame.base[1] = NULL;
+        frame->vo_frame.base[2] = NULL;
+        frame->req_width = 0;
+        frame->vo_frame.width = 0;
+        return;
+      }
+      frame->xvformat = fmt;
+      /* get params, blackfill */
+      if (fmt == XINE_IMGFMT_YUY2) {
+        frame->vo_frame.pitches[0] = frame->image->pitches[0];
+        frame->vo_frame.base[0]    = frame->image->data + frame->image->offsets[0];
+        {
+          const union {uint8_t bytes[4]; uint32_t word;} black = {{0, 128, 0, 128}};
+          uint32_t *q = (uint32_t *)frame->vo_frame.base[0];
+          int i;
+          for (i = frame->vo_frame.pitches[0] * frame->image->height / 4; i > 0; i--)
+            *q++ = black.word;
+        }
+      } else { /* XINE_IMGFMT_YV12 */
+        frame->vo_frame.pitches[0] = frame->image->pitches[0];
+        frame->vo_frame.pitches[1] = frame->image->pitches[2];
+        frame->vo_frame.pitches[2] = frame->image->pitches[1];
+        frame->vo_frame.base[0] = frame->image->data + frame->image->offsets[0];
+        frame->vo_frame.base[1] = frame->image->data + frame->image->offsets[2];
+        frame->vo_frame.base[2] = frame->image->data + frame->image->offsets[1];
+        memset (frame->vo_frame.base[0],   0, frame->vo_frame.pitches[0] *  frame->image->height);
+        memset (frame->vo_frame.base[1], 128, frame->vo_frame.pitches[1] * (frame->image->height >> 1));
+        memset (frame->vo_frame.base[2], 128, frame->vo_frame.pitches[2] * (frame->image->height >> 1));
       }
     }
-    else {
-      frame->vo_frame.pitches[0] = frame->image->pitches[0];
-      frame->vo_frame.pitches[1] = frame->image->pitches[2];
-      frame->vo_frame.pitches[2] = frame->image->pitches[1];
-      frame->vo_frame.base[0] = frame->image->data + frame->image->offsets[0];
-      frame->vo_frame.base[1] = frame->image->data + frame->image->offsets[2];
-      frame->vo_frame.base[2] = frame->image->data + frame->image->offsets[1];
-      memset (frame->vo_frame.base[0], 0, frame->vo_frame.pitches[0] * frame->image->height);
-      memset (frame->vo_frame.base[1], 128, frame->vo_frame.pitches[1] * frame->image->height / 2);
-      memset (frame->vo_frame.base[2], 128, frame->vo_frame.pitches[2] * frame->image->height / 2);
-    }
-
+    /* update emulation */
+    frame->format = format;
+    if (fmt != format) {
+      if (xv_add_yuy2_emu (frame)) {
+        LOCK_DISPLAY (this);
+        dispose_ximage (this, &frame->shminfo, frame->image);
+        UNLOCK_DISPLAY (this);
+        frame->image = NULL;
+        frame->vo_frame.base[0] = NULL;
+        frame->vo_frame.base[1] = NULL;
+        frame->vo_frame.base[2] = NULL;
+        frame->req_width = 0;
+        frame->vo_frame.width = 0;
+        return;
+      }
+    } else
+      xv_rem_yuy2_emu (frame);
     /* allocated frame size may not match requested size */
     frame->req_width  = width;
     frame->req_height = height;
     frame->width  = frame->image->width;
     frame->height = frame->image->height;
-    frame->format = format;
-
-    UNLOCK_DISPLAY(this);
   }
 
   if (frame->vo_frame.width > frame->width)
@@ -582,6 +682,7 @@ static void xv_overlay_begin (vo_driver_t *this_gen,
 static void xv_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img) {
   xv_driver_t  *this = (xv_driver_t *) this_gen;
 
+  (void)vo_img;
   if( this->ovl_changed && this->xoverlay ) {
     LOCK_DISPLAY(this);
     x11osd_expose(this->xoverlay);
@@ -1099,6 +1200,7 @@ static void xv_dispose (vo_driver_t *this_gen) {
   }
   UNLOCK_DISPLAY(this);
 
+  /* should do nothing as video_out has sent VO_PROP_DISCARD_FRAMES = -1 before */
   for( i=0; i < VO_NUM_RECENT_FRAMES; i++ ) {
     if( this->recent_frames[i] )
       this->recent_frames[i]->vo_frame.dispose
@@ -1113,9 +1215,13 @@ static void xv_dispose (vo_driver_t *this_gen) {
   }
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   cm_close (this);
 
+  /* cm_close already does this.
+  this->xine->config->unregister_callbacks (this->xine->config, NULL, NULL, this, sizeof (*this));
+  */
   free (this);
 }
 
@@ -1146,37 +1252,39 @@ static void xv_prop_init (xv_driver_t *this, const xv_prop_list_t *l, const XvAt
     LOG_MODULE ": port attribute %s (%d) value is %d\n", l->name, l->index, prop->value);
 }
 
-static void xv_prop_update (void *prop_gen, xine_cfg_entry_t *entry) {
-  xv_property_t *prop = (xv_property_t *)prop_gen;
+static void xv_prop_update_int (xv_property_t *prop, int value) {
   xv_driver_t   *this = prop->this;
 
   LOCK_DISPLAY (this);
-  XvSetPortAttribute (this->display, this->xv_port, prop->atom, entry->num_value);
+  XvSetPortAttribute (this->display, this->xv_port, prop->atom, value);
   UNLOCK_DISPLAY (this);
-  prop->value = entry->num_value;
+  prop->value = value;
 
-  xprintf (this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": %s = %d\n", prop->name, entry->num_value);
+  xprintf (this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": %s = %d\n", prop->name, value);
+}
+
+static void xv_prop_update (void *prop_gen, xine_cfg_entry_t *entry) {
+  xv_prop_update_int ((xv_property_t *)prop_gen, entry->num_value);
 }
 
 static void xv_prop_conf (xv_driver_t *this, int property,
   const char *config_name, const char *config_desc, const char *config_help) {
+  config_values_t *config = this->xine->config;
   xv_property_t *prop = &this->props[property];
   cfg_entry_t *entry;
 
   /* is this a boolean property ? */
   if ((prop->min == 0) && (prop->max == 1))
-    this->config->register_bool (this->config, config_name, prop->value,
+    config->register_bool (config, config_name, prop->value,
       config_desc, config_help, 20, xv_prop_update, prop);
   else
-    this->config->register_range (this->config, config_name, prop->value,
+    config->register_range (config, config_name, prop->value,
       prop->min, prop->max, config_desc, config_help, 20, xv_prop_update, prop);
 
-  entry = this->config->lookup_entry (this->config, config_name);
+  entry = config->lookup_entry (config, config_name);
 
-  if ((entry->num_value < prop->min) || (entry->num_value > prop->max)) {
-    this->config->update_num (this->config, config_name, ((prop->min + prop->max) >> 1));
-    entry = this->config->lookup_entry (this->config, config_name);
-  }
+  if ((entry->num_value < prop->min) || (entry->num_value > prop->max))
+    xv_prop_update_int (prop, (prop->min + prop->max) >> 1);
 
   prop->entry = entry;
 
@@ -1199,6 +1307,13 @@ static void xv_fullrange_cb_config (void *this_gen, xine_cfg_entry_t *entry) {
   this->cm_active = 0;
 }
 
+#ifdef DEBUG_EMU
+static void xv_debug_emu_cb_config (void *this_gen, xine_cfg_entry_t *entry) {
+  xv_driver_t *this = (xv_driver_t *)this_gen;
+  this->emu_yuy2 = entry->num_value;
+}
+#endif
+
 static int xv_open_port (xv_driver_t *this, XvPortID port) {
   {
     int i, formats;
@@ -1238,18 +1353,14 @@ static int xv_open_port (xv_driver_t *this, XvPortID port) {
     UNLOCK_DISPLAY (this);
   }
 
-  /* XXX
-   * Bail out if both YV12 and YUY2 are not supported.
-   * Proper fix would be implementing format conversions.
-   * - decoder should check video output CAP_ flags
-   * - video out could insert automatic conversion filter
-   * - ...
-   */
-  if (!this->xv_format_yuy2 || !this->xv_format_yv12) {
+  /* No yv12: bail out. */
+  if (!this->xv_format_yv12) {
     xprintf (this->xine, XINE_VERBOSITY_LOG,
-      LOG_MODULE ": this adaptor does not support YV12 and YUY2 formats.\n");
+      LOG_MODULE ": this adaptor does not support YV12 format.\n");
     return 0;
   }
+  /* No yuy2: just report. Who can deliver yv12 with little effort shall do that.
+   * Otherwise, accept anyway and emulate. */
 
   {
     int ret;
@@ -1267,8 +1378,8 @@ xv_find_adaptor_by_port (int port, unsigned int adaptors,
   unsigned int an;
   for (an = 0; an < adaptors; an++)
     if (adaptor_info[an].type & XvImageMask)
-      if (port >= adaptor_info[an].base_id &&
-	  port < adaptor_info[an].base_id + adaptor_info[an].num_ports)
+      if (port >= (int)adaptor_info[an].base_id &&
+	  port < (int)(adaptor_info[an].base_id + adaptor_info[an].num_ports))
 	return an;
   return 0; /* shouldn't happen */
 }
@@ -1297,10 +1408,20 @@ static XvPortID xv_autodetect_port(xv_driver_t *this,
   return 0;
 }
 
+static void xv_default_lock_display (void *user_data) {
+  xv_driver_t *this = user_data;
+  XLockDisplay (this->display);
+}
+
+static void xv_default_unlock_display (void *user_data) {
+  xv_driver_t *this = user_data;
+  XUnlockDisplay (this->display);
+}
+
 /* expects XINE_VISUAL_TYPE_X11_2 with configurable locking */
 static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *visual_gen) {
   xv_class_t           *class = (xv_class_t *) class_gen;
-  config_values_t      *config = class->config;
+  config_values_t      *config = class->xine->config;
   xv_driver_t          *this;
   int                   i;
   const x11_visual_t   *visual = (const x11_visual_t *) visual_gen;
@@ -1311,20 +1432,47 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   if (!this)
     return NULL;
 
-  this->use_shm               = 1;
+#ifndef HAVE_ZERO_SAFE_MEM
+  /* 0/NULL inits, for optimizing away. */
+  this->x11_old_error_handler = NULL;
   this->xoverlay              = NULL;
   this->ovl_changed           = 0;
-  this->x11_old_error_handler = NULL;
+  this->xv_format_yv12        = 0;
+  this->xv_format_yuy2        = 0;
+  this->emu_yuy2              = 0;
+  this->fullrange_mode        = 0;
+  for (i = 0; i < XV_NUM_PROPERTIES; i++) {
+    this->props[i].initial_value = 0;
+    this->props[i].value = 0;
+    this->props[i].min   = 0;
+    this->props[i].max   = 0;
+    this->props[i].defer = 0;
+    this->props[i].name  = NULL;
+    this->props[i].entry = NULL;
+  }
+#endif
+
+  this->use_shm               = 1;
   this->xine                  = class->xine;
   this->display               = visual->display;
   this->screen                = visual->screen;
-  this->config                = config;
   this->capabilities          = VO_CAP_CROP | VO_CAP_ZOOM_X | VO_CAP_ZOOM_Y;
 
   /* configurable X11 locking */
-  this->lock_display      = visual->lock_display;
-  this->unlock_display    = visual->unlock_display;
-  this->user_data         = visual->user_data;
+  if (visual->lock_display) {
+    this->lock_display = visual->lock_display;
+    this->ld_user_data = visual->user_data;
+  } else {
+    this->lock_display = xv_default_lock_display;
+    this->ld_user_data = this;
+  }
+  if (visual->unlock_display) {
+    this->unlock_display = visual->unlock_display;
+    this->ud_user_data   = visual->user_data;
+  } else {
+    this->unlock_display = xv_default_unlock_display;
+    this->ud_user_data   = this;
+  }
 
   LOCK_DISPLAY(this);
 
@@ -1440,14 +1588,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
    */
 
   for (i = 0; i < XV_NUM_PROPERTIES; i++) {
-    this->props[i].initial_value = 0;
-    this->props[i].value = 0;
-    this->props[i].min   = 0;
-    this->props[i].max   = 0;
     this->props[i].atom  = None;
-    this->props[i].defer = 0;
-    this->props[i].name  = NULL;
-    this->props[i].entry = NULL;
     this->props[i].this  = this;
   }
 
@@ -1481,52 +1622,55 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
           }
           xv_prop_init (this, l, attr[k]);
 
-          if (l->index == XV_PROP_ITURBT_709) { /* nvidia */
-            this->cm_active = this->props[XV_PROP_ITURBT_709].value ? 2 : 10;
-
-          } else if (l->index == XV_PROP_COLORSPACE) { /* radeonhd */
-            this->cm_active = this->props[XV_PROP_COLORSPACE].value == 2 ? 2 : (this->props[XV_PROP_COLORSPACE].value == 1 ? 10 : 0);
-
-          } else if (l->index == VO_PROP_COLORKEY) {
-            xv_prop_conf (this, VO_PROP_COLORKEY, "video.device.xv_colorkey", VIDEO_DEVICE_XV_COLORKEY_HELP);
-
-          } else if (l->index == VO_PROP_AUTOPAINT_COLORKEY) {
-            /* disable autopaint colorkey by default, might be overridden using config entry */
-            this->props[VO_PROP_AUTOPAINT_COLORKEY].value = 0;
-            xv_prop_conf (this, VO_PROP_AUTOPAINT_COLORKEY,
-              "video.device.xv_autopaint_colorkey", VIDEO_DEVICE_XV_AUTOPAINT_COLORKEY_HELP);
-
-          } else if (l->index == XV_PROP_FILTER) {
-            int xv_filter;
-            /* This setting is specific to Permedia 2/3 cards. */
-            xv_filter = config->register_range (config, "video.device.xv_filter", 0,
-              attr[k].min_value, attr[k].max_value, VIDEO_DEVICE_XV_FILTER_HELP,
-              20, xv_prop_update, &this->props[XV_PROP_FILTER]);
-            config->update_num (config,"video.device.xv_filter",xv_filter);
-
-          } else if (l->index == XV_PROP_DOUBLE_BUFFER) {
-            int xv_double_buffer;
-            xv_double_buffer = config->register_bool (config, "video.device.xv_double_buffer", 1,
-              VIDEO_DEVICE_XV_DOUBLE_BUFFER_HELP, 20, xv_prop_update, &this->props[XV_PROP_DOUBLE_BUFFER]);
-            config->update_num (config,"video.device.xv_double_buffer", xv_double_buffer);
-
-          } else if (l->index == XV_PROP_SYNC_TO_VBLANK) {
-            int xv_sync_to_vblank;
-            xv_sync_to_vblank = config->register_bool (config, "video.device.xv_sync_to_vblank", 1,
-              _("enable vblank sync"),
-              _("This option will synchronize the update of the video image to the "
-                "repainting of the entire screen (\"vertical retrace\"). This eliminates "
-                "flickering and tearing artifacts. On nvidia cards one may also "
-                "need to run \"nvidia-settings\" and choose which display device to "
-                "sync to under the XVideo Settings tab"),
-              20, xv_prop_update, &this->props[XV_PROP_SYNC_TO_VBLANK]);
-            config->update_num(config,"video.device.xv_sync_to_vblank", xv_sync_to_vblank);
-
-          } else if (l->index == XV_PROP_BICUBIC) {
-            int xv_bicubic;
-            xv_bicubic = config->register_enum (config, "video.device.xv_bicubic", 2,
-              (char **)bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP, 20, xv_prop_update, &this->props[XV_PROP_BICUBIC]);
-            config->update_num(config,"video.device.xv_bicubic",xv_bicubic);
+          switch (l->index) {
+            case XV_PROP_ITURBT_709: /* nvidia */
+              this->cm_active = this->props[XV_PROP_ITURBT_709].value ? 2 : 10;
+              break;
+            case XV_PROP_COLORSPACE: /* radeonhd */
+              this->cm_active = this->props[XV_PROP_COLORSPACE].value == 2 ? 2
+                              : (this->props[XV_PROP_COLORSPACE].value == 1 ? 10 : 0);
+              break;
+            case VO_PROP_COLORKEY:
+              xv_prop_conf (this, VO_PROP_COLORKEY, "video.device.xv_colorkey", VIDEO_DEVICE_XV_COLORKEY_HELP);
+              break;
+            case VO_PROP_AUTOPAINT_COLORKEY:
+              /* disable autopaint colorkey by default, might be overridden using config entry */
+              this->props[VO_PROP_AUTOPAINT_COLORKEY].value = 0;
+              xv_prop_conf (this, VO_PROP_AUTOPAINT_COLORKEY,
+                "video.device.xv_autopaint_colorkey", VIDEO_DEVICE_XV_AUTOPAINT_COLORKEY_HELP);
+              break;
+            case XV_PROP_FILTER: {
+              /* This setting is specific to Permedia 2/3 cards. */
+              int xv_filter = config->register_range (config, "video.device.xv_filter", 0,
+                attr[k].min_value, attr[k].max_value, VIDEO_DEVICE_XV_FILTER_HELP,
+                20, xv_prop_update, &this->props[XV_PROP_FILTER]);
+              xv_prop_update_int (&this->props[XV_PROP_FILTER], xv_filter);
+              break;
+            }
+            case XV_PROP_DOUBLE_BUFFER: {
+              int xv_double_buffer = config->register_bool (config, "video.device.xv_double_buffer", 1,
+                VIDEO_DEVICE_XV_DOUBLE_BUFFER_HELP, 20, xv_prop_update, &this->props[XV_PROP_DOUBLE_BUFFER]);
+              xv_prop_update_int (&this->props[XV_PROP_DOUBLE_BUFFER], xv_double_buffer);
+              break;
+            }
+            case XV_PROP_SYNC_TO_VBLANK: {
+              int xv_sync_to_vblank = config->register_bool (config, "video.device.xv_sync_to_vblank", 1,
+                _("enable vblank sync"),
+                _("This option will synchronize the update of the video image to the "
+                  "repainting of the entire screen (\"vertical retrace\"). This eliminates "
+                  "flickering and tearing artifacts. On nvidia cards one may also "
+                  "need to run \"nvidia-settings\" and choose which display device to "
+                  "sync to under the XVideo Settings tab"),
+                20, xv_prop_update, &this->props[XV_PROP_SYNC_TO_VBLANK]);
+              xv_prop_update_int (&this->props[XV_PROP_SYNC_TO_VBLANK], xv_sync_to_vblank);
+              break;
+            }
+            case XV_PROP_BICUBIC: {
+              int xv_bicubic = config->register_enum (config, "video.device.xv_bicubic", 2,
+                (char **)bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP, 20, xv_prop_update, &this->props[XV_PROP_BICUBIC]);
+              xv_prop_update_int (&this->props[XV_PROP_BICUBIC], xv_bicubic);
+              break;
+            }
           }
         }
       }
@@ -1544,8 +1688,8 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   if ((this->props[VO_PROP_BRIGHTNESS].atom != None) &&
     (this->props[VO_PROP_CONTRAST].atom != None)) {
     static const char * const xv_fullrange_conf_labels[] = {"Off", "Normal", NULL};
-    this->fullrange_mode = this->xine->config->register_enum (
-      this->xine->config,
+    this->fullrange_mode = config->register_enum (
+      config,
       "video.output.xv_fullrange",
       0,
       (char **)xv_fullrange_conf_labels,
@@ -1561,10 +1705,21 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
     );
     if (this->fullrange_mode)
       this->capabilities |= VO_CAP_FULLRANGE;
-  } else {
-    this->fullrange_mode = 0;
   }
 
+#ifdef DEBUG_EMU
+  this->emu_yuy2 = config->register_bool (
+    config,
+    "video.output.xv_debug_emu",
+    0,
+    _("Force emulating yuy2"),
+    _("Debug."),
+    10,
+    xv_debug_emu_cb_config,
+    this
+  );
+#endif
+
   /*
    * check max. supported image size
    */
@@ -1575,7 +1730,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
     LOCK_DISPLAY (this);
     XvQueryEncodings (this->display, xv_port, &nencode, &encodings);
     if (encodings) {
-      int n;
+      unsigned int n;
       for (n = 0; n < nencode; n++) {
         if (!strcmp (encodings[n].name, "XV_IMAGE")) {
           xprintf (this->xine, XINE_VERBOSITY_LOG,
@@ -1649,36 +1804,43 @@ static vo_driver_t *open_plugin_old (video_driver_class_t *class_gen, const void
 /*
  * class functions
  */
-static void *init_class (xine_t *xine, void *visual_gen) {
-  xv_class_t        *this = (xv_class_t *) calloc(1, sizeof(xv_class_t));
+static void *init_class (xine_t *xine, const void *visual_gen) {
+  xv_class_t *this;
+
+  (void)visual_gen;
+
+  this = calloc(1, sizeof(*this));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = open_plugin_old;
   this->driver_class.identifier      = "Xv";
   this->driver_class.description     = N_("xine video output plugin using the MIT X video extension");
   this->driver_class.dispose         = default_video_driver_class_dispose;
 
-  this->config                       = xine->config;
   this->xine                         = xine;
 
   return this;
 }
 
-static void *init_class_2 (xine_t *xine, void *visual_gen) {
+static void *init_class_2 (xine_t *xine, const void *visual_gen) {
   xv_class_t	       *this;
   this = init_class (xine, visual_gen);
-  this->driver_class.open_plugin     = open_plugin_2;
+  if (this) {
+    this->driver_class.open_plugin     = open_plugin_2;
+  }
   return this;
 }
 
 static const vo_info_t vo_info_xv = {
-  9,                      /* priority    */
-  XINE_VISUAL_TYPE_X11    /* visual type */
+  .priority    = 9,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
 /* visual type with configurable X11 locking */
 static const vo_info_t vo_info_xv_2 = {
-  9,                      /* priority    */
-  XINE_VISUAL_TYPE_X11_2  /* visual type */
+  .priority    = 9,
+  .visual_type = XINE_VISUAL_TYPE_X11_2,
 };
 
 /*
@@ -1689,5 +1851,6 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "xv", XINE_VERSION_CODE, &vo_info_xv, init_class },
   { PLUGIN_VIDEO_OUT, 22, "xv", XINE_VERSION_CODE, &vo_info_xv_2, init_class_2 },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
+
