@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -217,7 +217,6 @@ typedef struct {
   video_driver_class_t driver_class;
 
   Display             *display;
-  config_values_t     *config;
   XvPortID             xv_port;
   XvAdaptorInfo       *adaptor_info;
   unsigned int         adaptor_num;
@@ -458,6 +457,8 @@ static uint32_t xvmc_get_capabilities (vo_driver_t *this_gen) {
 
 static void xvmc_frame_field (vo_frame_t *vo_img, int which_field) {
   lprintf ("xvmc_frame_field\n");
+  (void)vo_img;
+  (void)which_field;
 }
 
 static void xvmc_frame_dispose (vo_frame_t *vo_img) {
@@ -588,6 +589,7 @@ static cxid_t *xvmc_set_context (xvmc_driver_t *this,
 
   lprintf ("xvmc_set_context %dx%d %04x\n",width,height,format);
 
+  (void)ratio;
   /* initialize block & macro block pointers first time */
   if(macroblocks->blocks == NULL ||  macroblocks->macro_blocks == NULL) {
     macroblocks->blocks       = calloc(1, sizeof(XvMCBlockArray));
@@ -598,8 +600,8 @@ static cxid_t *xvmc_set_context (xvmc_driver_t *this,
   }
 
   if((this->context_id.xid != NULL)   &&
-     (width  == this->surface_width)  &&
-     (height == this->surface_height) &&
+     ((int)width  == this->surface_width)  &&
+     ((int)height == this->surface_height) &&
      (format == this->surface_format) &&
      (flags  == this->surface_flags)) {
 
@@ -763,13 +765,14 @@ static void xvmc_update_frame_format (vo_driver_t *this_gen,
 
   if (format != XINE_IMGFMT_XVMC) {
     xprintf (this->xine, XINE_VERBOSITY_LOG, "xvmc_update_frame_format: frame format %08x not supported\n", format);
-    _x_abort();
+    frame->vo_frame.width = frame->width = 0;
+    return;
   }
 
   lprintf ("xvmc_update_frame_format\n");
 
-  if ((frame->width != width)
-      || (frame->height != height)
+  if ((frame->width != (int)width)
+      || (frame->height != (int)height)
       || (frame->format != format)) {
 
     lprintf ("updating frame to %d x %d (ratio=%f, format=%08x)\n",
@@ -1236,6 +1239,7 @@ static void xvmc_dispose (vo_driver_t *this_gen) {
   }
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   free (this);
 }
@@ -1250,6 +1254,7 @@ static void xvmc_check_capability (xvmc_driver_t *this,
   cfg_entry_t *entry;
   const char  *str_prop = attr.name;
 
+  (void)base_id;
   /*
    * some Xv drivers (Gatos ATI) report some ~0 as max values, this is confusing.
    */
@@ -1313,7 +1318,7 @@ static void xvmc_update_XV_DOUBLE_BUFFER(void *this_gen, xine_cfg_entry_t *entry
 
 static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *visual_gen) {
   xvmc_class_t         *class   = (xvmc_class_t *) class_gen;
-  config_values_t      *config  = class->config;
+  config_values_t      *config  = class->xine->config;
   xvmc_driver_t        *this    = NULL;
   unsigned int          i, formats;
   XvPortID              xv_port = class->xv_port;
@@ -1541,7 +1546,7 @@ static void dispose_class (video_driver_class_t *this_gen) {
   free (this);
 }
 
-static void *init_class (xine_t *xine, void *visual_gen) {
+static void *init_class (xine_t *xine, const void *visual_gen) {
   const x11_visual_t *visual = (const x11_visual_t *) visual_gen;
   xvmc_class_t      *this;
   Display           *display = NULL;
@@ -1708,7 +1713,6 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   this->driver_class.dispose         = dispose_class;
 
   this->display                      = display;
-  this->config                       = xine->config;
   this->xv_port                      = xv_port;
   this->adaptor_info                 = adaptor_info;
   this->adaptor_num                  = adaptor_num;
@@ -1726,8 +1730,8 @@ static void *init_class (xine_t *xine, void *visual_gen) {
 
 static const vo_info_t vo_info_xvmc = {
   /* priority must be low until it supports displaying non-accelerated stuff */
-  0,                    /* priority    */
-  XINE_VISUAL_TYPE_X11  /* visual type */
+  .priority    = 0,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
 /*
@@ -1737,7 +1741,7 @@ static const vo_info_t vo_info_xvmc = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "xvmc", XINE_VERSION_CODE, &vo_info_xvmc, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
 #endif
