@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * Copyright (C) 2004 the Unichrome project
  *
  * This file is part of xine, a free video player.
@@ -36,6 +36,7 @@
 
 #include "xxmc.h"
 #include <unistd.h>
+#include <pthread.h>
 #include "xv_common.h"
 
 
@@ -503,6 +504,8 @@ static uint32_t xxmc_get_capabilities (vo_driver_t *this_gen) {
 static void xxmc_frame_field (vo_frame_t *vo_img, int which_field)
 {
   lprintf ("xvmc_frame_field\n");
+  (void)vo_img;
+  (void)which_field;
 }
 
 static void xxmc_frame_dispose (vo_frame_t *vo_img) {
@@ -621,7 +624,7 @@ static XvImage *create_ximage (xxmc_driver_t *this, XShmSegmentInfo *shminfo,
     break;
   default:
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unknown format %08x\n",format);
-    _x_abort();
+    return NULL;
   }
 
   if (this->use_shm) {
@@ -720,7 +723,7 @@ static XvImage *create_ximage (xxmc_driver_t *this, XShmSegmentInfo *shminfo,
    */
 
   if (!this->use_shm) {
-    char *data;
+    char *data = NULL;
 
     switch (format) {
     case XINE_IMGFMT_YV12:
@@ -734,8 +737,9 @@ static XvImage *create_ximage (xxmc_driver_t *this, XShmSegmentInfo *shminfo,
       data = malloc (width * height);
       break;
     default:
+      /* not reached */
       xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unknown format %08x\n",format);
-      _x_abort();
+      // _x_abort();
     }
 
     image = XvCreateImage (this->display, this->xv_port,
@@ -762,7 +766,7 @@ static void xxmc_dispose_context(xxmc_driver_t *driver)
 
     xprintf(driver->xine, XINE_VERBOSITY_LOG,
 	    LOG_MODULE ": Freeing up XvMC Surfaces and subpictures.\n");
-    if (driver->xvmc_palette) free(driver->xvmc_palette);
+    _x_freep(&driver->xvmc_palette);
     _x_dispose_xx44_palette( &driver->palette );
     xxmc_xvmc_destroy_subpictures( driver );
     xxmc_xvmc_destroy_surfaces( driver );
@@ -792,7 +796,7 @@ static void xxmc_dispose_context(xxmc_driver_t *driver)
 static int xxmc_find_context(xxmc_driver_t *driver, xine_xxmc_t *xxmc,
 			     unsigned width, unsigned height)
 {
-  int i,k,found;
+  unsigned int i,k,found;
   xvmc_capabilities_t *curCap;
   unsigned request_mpeg_flags, request_accel_flags;
 
@@ -900,8 +904,7 @@ static void xxmc_setup_subpictures(xxmc_driver_t *driver, unsigned width, unsign
     if (sp == NULL) return;
 
     _x_init_xx44_palette( &driver->palette, sp->num_palette_entries);
-    driver->xvmc_palette = (char *) xine_xmalloc(sp->num_palette_entries
-						 * sp->entry_bytes);
+    driver->xvmc_palette = (char *)calloc(sp->num_palette_entries, sp->entry_bytes);
     xxmc_xvmc_free_subpicture( driver, sp);
     if (driver->xvmc_palette == NULL) return;
     driver->hwSubpictures = 1;
@@ -1113,7 +1116,7 @@ static void dispose_ximage (xxmc_driver_t *this,
 
   }
   else {
-    if (myimage->data) free(myimage->data);
+    _x_freep(&myimage->data);
     XFree (myimage);
   }
 
@@ -1128,14 +1131,18 @@ static void xxmc_do_update_frame_xv(vo_driver_t *this_gen,
   xxmc_driver_t  *this  = (xxmc_driver_t *) this_gen;
   xxmc_frame_t   *frame = (xxmc_frame_t *) frame_gen;
 
+  (void)flags;
   if (this->use_pitch_alignment) {
     width = (width + 7) & ~0x7;
   }
 
-  if ((frame->width != width)
-      || (frame->height != height)
+  if ((frame->width != (int)width)
+      || (frame->height != (int)height)
       || (frame->last_sw_format != format)) {
 
+    frame->width  = width;
+    frame->height = height;
+    frame->format = format;
     frame->last_sw_format = format;
     XLockDisplay (this->display);
 
@@ -1150,26 +1157,25 @@ static void xxmc_do_update_frame_xv(vo_driver_t *this_gen,
 
     frame->image = create_ximage (this, &frame->shminfo, width, height, format);
 
-    if(format == XINE_IMGFMT_YUY2) {
+    if( frame->image && format == XINE_IMGFMT_YUY2) {
       frame->vo_frame.pitches[0] = frame->image->pitches[0];
       frame->vo_frame.base[0] = frame->image->data + frame->image->offsets[0];
-    }
-    else {
+    } else if (frame->image && format == XINE_IMGFMT_YV12) {
       frame->vo_frame.pitches[0] = frame->image->pitches[0];
       frame->vo_frame.pitches[1] = frame->image->pitches[2];
       frame->vo_frame.pitches[2] = frame->image->pitches[1];
       frame->vo_frame.base[0] = frame->image->data + frame->image->offsets[0];
       frame->vo_frame.base[1] = frame->image->data + frame->image->offsets[2];
       frame->vo_frame.base[2] = frame->image->data + frame->image->offsets[1];
+    } else {
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "alert! unsupported image format %04x\n", format);
+      frame->vo_frame.width = frame->width = 0;
     }
 
     XUnlockDisplay (this->display);
   }
 
   frame->ratio = ratio;
-  frame->width  = width;
-  frame->height = height;
-  frame->format = format;
   frame->vo_frame.format = frame->format;
 }
 
@@ -1182,7 +1188,7 @@ static int xxmc_accel_update(xxmc_driver_t *driver,
 			     uint32_t last_request,
 			     uint32_t new_request)
 {
-  int k;
+  unsigned int k;
 
   /*
    * Same acceleration request. No need to change.
@@ -1264,7 +1270,7 @@ static void xxmc_do_update_frame(vo_driver_t *this_gen,
        * changes on the original vo_frame to all the intercepted frames */
       unsigned char *p0 = (unsigned char *)&orig_frame_content;
       unsigned char *p1 = (unsigned char *)&frame->vo_frame;
-      int i;
+      unsigned int i;
       for (i = 0; i < sizeof (vo_frame_t); i++) {
         if (*p0 != *p1) {
           /* propagate the change */
@@ -1595,6 +1601,19 @@ static void xxmc_add_recent_frame (xxmc_driver_t *this, xxmc_frame_t *frame)
   this->recent_frames[0] = frame;
 }
 
+static int xv_flush_recent_frames (xxmc_driver_t *this) {
+  int i, n = 0;
+
+  for (i = 0; i < VO_NUM_RECENT_FRAMES; i++) {
+    if (this->recent_frames[i]) {
+      this->recent_frames[i]->vo_frame.free (&this->recent_frames[i]->vo_frame);
+      this->recent_frames[i] = NULL;
+      n++;
+    }
+  }
+  return n;
+}
+
 static int xxmc_redraw_needed (vo_driver_t *this_gen)
 {
   xxmc_driver_t  *this = (xxmc_driver_t *) this_gen;
@@ -1879,6 +1898,11 @@ static int xxmc_set_property (vo_driver_t *this_gen,
   else {
     switch (property) {
 
+    case VO_PROP_DISCARD_FRAMES:
+      if (value == -1)
+        value = xv_flush_recent_frames (this);
+      break;
+
     case VO_PROP_INTERLACED:
       this->props[property].value = value;
       xprintf(this->xine, XINE_VERBOSITY_LOG,
@@ -2089,6 +2113,7 @@ static void xxmc_dispose (vo_driver_t *this_gen) {
   free_context_lock(&this->xvmc_lock);
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   free (this);
 }
@@ -2122,6 +2147,7 @@ static void xxmc_check_capability (xxmc_driver_t *this,
   cfg_entry_t *entry;
   const char  *str_prop = attr.name;
 
+  (void)base_id;
   if (VO_PROP_COLORKEY && (attr.max_value == ~0))
     attr.max_value = 2147483615;
 
@@ -2262,8 +2288,8 @@ xxmc_find_adaptor_by_port (int port, unsigned int adaptors,
   unsigned int an;
   for (an = 0; an < adaptors; an++)
     if (adaptor_info[an].type & XvImageMask)
-      if (port >= adaptor_info[an].base_id &&
-	  port < adaptor_info[an].base_id + adaptor_info[an].num_ports)
+      if (port >= (int)adaptor_info[an].base_id &&
+	  port < (int)(adaptor_info[an].base_id + adaptor_info[an].num_ports))
 	return an;
   return 0; /* shouldn't happen */
 }
@@ -2333,9 +2359,12 @@ static void checkXvMCCap( xxmc_driver_t *this, XvPortID xv_port)
     xvmc_context_writer_unlock( &this->xvmc_lock );
     return;
   }
-  this->xvmc_cap = (xvmc_capabilities_t *)
-    xine_xmalloc(numSurf * sizeof(xvmc_capabilities_t));
-  if (NULL == this->xvmc_cap) return;
+  this->xvmc_cap = (xvmc_capabilities_t *)calloc(numSurf, sizeof(xvmc_capabilities_t));
+  if (NULL == this->xvmc_cap) {
+    XVMCUNLOCKDISPLAY( this->display );
+    xvmc_context_writer_unlock( &this->xvmc_lock );
+    return;
+  }
   this->xvmc_num_cap = numSurf;
   curInfo = surfaceInfo;
   curCap = this->xvmc_cap;
@@ -2441,13 +2470,13 @@ static void checkXvMCCap( xxmc_driver_t *this, XvPortID xv_port)
 
 static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *visual_gen) {
   xxmc_class_t           *class = (xxmc_class_t *) class_gen;
-  config_values_t      *config = class->config;
+  config_values_t      *config = class->xine->config;
   xxmc_driver_t          *this;
   int                   i, formats;
   XvAttribute          *attr;
   XvImageFormatValues  *fo;
   int                   nattr;
-  x11_visual_t         *visual = (x11_visual_t *) visual_gen;
+  const x11_visual_t   *visual = (const x11_visual_t *) visual_gen;
   XColor                dummy;
   XvImage              *myimage;
   unsigned int          adaptors;
@@ -2826,15 +2855,20 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 /*
  * class functions
  */
-static void *init_class (xine_t *xine, void *visual_gen) {
-  xxmc_class_t        *this = calloc(1, sizeof (xxmc_class_t));
+static void *init_class (xine_t *xine, const void *visual_gen) {
+  xxmc_class_t *this;
+
+  (void)visual_gen;
+
+  this = calloc(1, sizeof (*this));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = open_plugin;
   this->driver_class.identifier      = "XxMC";
   this->driver_class.description     = N_("xine video output plugin using the MIT X video extension");
   this->driver_class.dispose         = default_video_driver_class_dispose;
 
-  this->config                       = xine->config;
   this->xine                         = xine;
 
   return this;
@@ -2844,8 +2878,8 @@ static const vo_info_t vo_info_xxmc = {
   /* keep priority lower than Xv for now. we may increase this
    * when the xxmc driver is more mature/tested.
    */
-  5,                    /* priority    */
-  XINE_VISUAL_TYPE_X11  /* visual type */
+  .priority    = 5,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
 /*
@@ -2855,6 +2889,6 @@ static const vo_info_t vo_info_xxmc = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "xxmc", XINE_VERSION_CODE, &vo_info_xxmc, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
