@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -135,22 +135,16 @@ static void buffer_pool_free (buf_element_t *element) {
  * allocate a buffer from buffer pool
  */
 
-static buf_element_t *buffer_pool_size_alloc (fifo_buffer_t *this, size_t size) {
+static buf_element_t *buffer_pool_size_alloc_int (fifo_buffer_t *this, int n) {
 
-  int i, n;
+  int i;
   be_ei_t *buf;
 
-  n = size ? ((int)size + this->buffer_pool_buf_size - 1) / this->buffer_pool_buf_size : 1;
-  if (n > (this->buffer_pool_capacity >> 2))
-    n = this->buffer_pool_capacity >> 2;
-  if (n < 1)
-    n = 1;
-
-  pthread_mutex_lock (&this->buffer_pool_mutex);
-
   for (i = 0; this->alloc_cb[i]; i++)
     this->alloc_cb[i] (this, this->alloc_cb_data[i]);
 
+  if (n < 1)
+    n = 1;
   /* we always keep one free buffer for emergency situations like
    * decoder flushes that would need a buffer in buffer_pool_try_alloc() */
   n += 2;
@@ -230,6 +224,15 @@ static buf_element_t *buffer_pool_size_alloc (fifo_buffer_t *this, size_t size)
   return &buf->elem;
 }
 
+static buf_element_t *buffer_pool_size_alloc (fifo_buffer_t *this, size_t size) {
+  int n = size ? ((int)size + this->buffer_pool_buf_size - 1) / this->buffer_pool_buf_size : 1;
+  if (n > (this->buffer_pool_capacity >> 2))
+    n = this->buffer_pool_capacity >> 2;
+  pthread_mutex_lock (&this->buffer_pool_mutex);
+  return buffer_pool_size_alloc_int (this, n);
+}
+
+
 static buf_element_t *buffer_pool_alloc (fifo_buffer_t *this) {
   be_ei_t *buf;
   int i;
@@ -272,6 +275,72 @@ static buf_element_t *buffer_pool_alloc (fifo_buffer_t *this) {
   return &buf->elem;
 }
 
+static buf_element_t *buffer_pool_realloc (buf_element_t *buf, size_t new_size) {
+  fifo_buffer_t *this;
+  buf_element_t **last_buf;
+  be_ei_t *old_buf = (be_ei_t *)buf, *new_buf, *want_buf;
+  int n;
+
+  if (!old_buf)
+    return NULL;
+  if ((int)new_size <= old_buf->elem.max_size)
+    return NULL;
+  if (old_buf->elem.free_buffer != buffer_pool_free)
+    return NULL;
+  this = (fifo_buffer_t *)old_buf->elem.source;
+  if (!this)
+    return NULL;
+
+  n = ((int)new_size + this->buffer_pool_buf_size - 1) / this->buffer_pool_buf_size;
+  /* limit size to keep pool fluent */
+  if (n > (this->buffer_pool_capacity >> 3))
+    n = this->buffer_pool_capacity >> 3;
+  n -= old_buf->nbufs;
+
+  want_buf = old_buf + old_buf->nbufs;
+  last_buf = &this->buffer_pool_top;
+  pthread_mutex_lock (&this->buffer_pool_mutex);
+  while (1) {
+    new_buf = (be_ei_t *)(*last_buf);
+    if (!new_buf)
+      break;
+    if (new_buf == want_buf)
+      break;
+    if (new_buf > want_buf) {
+      new_buf = NULL;
+      break;
+    }
+    new_buf += new_buf->nbufs;
+    last_buf = &(new_buf[-1].elem.next);
+  }
+
+  if (new_buf) do {
+    int s;
+    /* save emergecy buf */
+    if (n > this->buffer_pool_num_free - 1)
+      n = this->buffer_pool_num_free - 1;
+    if (n < 1)
+      break;
+    s = new_buf->nbufs - n;
+    if (s > 0) {
+      new_buf += n;
+      new_buf->nbufs = s;
+      *last_buf = &new_buf->elem;
+    } else {
+      n = new_buf->nbufs;
+      new_buf += n;
+      *last_buf = new_buf[-1].elem.next;
+    }
+    this->buffer_pool_num_free -= n;
+    pthread_mutex_unlock (&this->buffer_pool_mutex);
+    old_buf->nbufs += n;
+    old_buf->elem.max_size = old_buf->nbufs * this->buffer_pool_buf_size;
+    return NULL;
+  } while (0);
+
+  return buffer_pool_size_alloc_int (this, n);
+}
+
 /*
  * allocate a buffer from buffer pool - may fail if none is available
  */
@@ -405,7 +474,7 @@ static void fifo_buffer_insert (fifo_buffer_t *fifo, buf_element_t *element) {
  * insert buffer element to fifo buffer (demuxers MUST NOT call this one)
  */
 static void dummy_fifo_buffer_insert (fifo_buffer_t *fifo, buf_element_t *element) {
-
+  (void)fifo;
   element->free_buffer(element);
 }
 
@@ -448,6 +517,70 @@ static buf_element_t *fifo_buffer_get (fifo_buffer_t *fifo) {
   return buf;
 }
 
+static buf_element_t *fifo_buffer_tget (fifo_buffer_t *fifo, xine_ticket_t *ticket) {
+  /* Optimization: let decoders hold port ticket by default.
+   * Unfortunately, fifo callbacks are 1 big freezer, as they run with fifo locked,
+   * and may try to revoke ticket for pauseing or other stuff.
+   * Always releasing ticket when there are callbacks is safe but inefficient.
+   * Instead, we release ticket when we are going to wait for fifo or a buffer,
+   * and of course, when the ticket has been revoked.
+   * This should melt the "put" side. We could still freeze ourselves directly
+   * at the "get" side, what ticket->revoke () self grant hack shall fix.
+   */
+  buf_element_t *buf;
+  int mode = ticket ? 2 : 0, i;
+
+  if (pthread_mutex_trylock (&fifo->mutex)) {
+    if (mode & 2) {
+      ticket->release (ticket, 0);
+      mode = 1;
+    }
+    pthread_mutex_lock (&fifo->mutex);
+  }
+
+  if (!fifo->first) {
+    if (mode & 2) {
+      ticket->release (ticket, 0);
+      mode = 1;
+    }
+    fifo->fifo_num_waiters++;
+    do {
+      pthread_cond_wait (&fifo->not_empty, &fifo->mutex);
+    } while (!fifo->first);
+    fifo->fifo_num_waiters--;
+  }
+
+  buf = fifo->first;
+
+  fifo->first = fifo->first->next;
+  if (fifo->first==NULL)
+    fifo->last = NULL;
+
+  if (buf->free_buffer == buffer_pool_free) {
+    be_ei_t *beei = (be_ei_t *)buf;
+    fifo->fifo_size -= beei->nbufs;
+  } else {
+    fifo->fifo_size -= 1;
+  }
+  fifo->fifo_data_size -= buf->size;
+
+  if ((mode & 2) && ticket->ticket_revoked) {
+    ticket->release (ticket, 0);
+    mode = 1;
+  }
+
+  for(i = 0; fifo->get_cb[i]; i++)
+    fifo->get_cb[i](fifo, buf, fifo->get_cb_data[i]);
+
+  pthread_mutex_unlock (&fifo->mutex);
+
+  if (mode & 1)
+    ticket->acquire (ticket, 0);
+
+  return buf;
+}
+
+
 /*
  * clear buffer (put all contained buffer elements back into buffer pool)
  */
@@ -590,9 +723,9 @@ static uint32_t fifo_buffer_data_size (fifo_buffer_t *this) {
 static int fifo_buffer_num_free (fifo_buffer_t *this) {
   int buffer_pool_num_free;
 
-  pthread_mutex_lock(&this->mutex);
+  pthread_mutex_lock (&this->buffer_pool_mutex);
   buffer_pool_num_free = this->buffer_pool_num_free;
-  pthread_mutex_unlock(&this->mutex);
+  pthread_mutex_unlock (&this->buffer_pool_mutex);
 
   return buffer_pool_num_free;
 }
@@ -747,7 +880,7 @@ fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) {
   this = calloc(1, sizeof(fifo_buffer_t));
   if (!this)
     return NULL;
-
+#ifndef HAVE_ZERO_SAFE_MEM
   /* Do these first, when compiler still knows "this" is all zeroed.
    * Let it optimize away this on most systems where clear mem
    * interpretes as 0, 0f or NULL safely.
@@ -763,6 +896,7 @@ fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) {
   this->alloc_cb_data[0]        = NULL;
   this->get_cb_data[0]          = NULL;
   this->put_cb_data[0]          = NULL;
+#endif
 
   /* printf ("Allocating %d buffers of %ld bytes in one chunk\n", num_buffers, (long int) buf_size); */
   multi_buffer = xine_mallocz_aligned (num_buffers * (buf_size + sizeof (be_ei_t)));
@@ -774,6 +908,7 @@ fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) {
   this->put                 = fifo_buffer_put;
   this->insert              = fifo_buffer_insert;
   this->get                 = fifo_buffer_get;
+  this->tget                = fifo_buffer_tget;
   this->clear               = fifo_buffer_clear;
   this->size                = fifo_buffer_size;
   this->num_free            = fifo_buffer_num_free;
@@ -799,6 +934,7 @@ fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) {
   this->buffer_pool_alloc      = buffer_pool_alloc;
   this->buffer_pool_try_alloc  = buffer_pool_try_alloc;
   this->buffer_pool_size_alloc = buffer_pool_size_alloc;
+  this->buffer_pool_realloc    = buffer_pool_realloc;
 
   this->buffer_pool_large_wait  = LARGE_NUM;
 
@@ -827,11 +963,25 @@ fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) {
  * allocate and initialize new (empty) fifo buffer
  */
 fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size) {
-
-  fifo_buffer_t *this;
-
-  this = _x_fifo_buffer_new(num_buffers, buf_size);
-  this->put    = dummy_fifo_buffer_put;
-  this->insert = dummy_fifo_buffer_insert;
+  fifo_buffer_t *this = _x_fifo_buffer_new (num_buffers, buf_size);
+  if (this) {
+    this->put    = dummy_fifo_buffer_put;
+    this->insert = dummy_fifo_buffer_insert;
+  }
   return this;
 }
+
+void _x_free_buf_elements(buf_element_t *head) {
+
+  if (head) {
+    buf_element_t  *cur, *next;
+
+    cur = head;
+    while (cur) {
+      next = cur->next;
+      cur->free_buffer(cur);
+      cur = next;
+    }
+  }
+}
+
