@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -39,7 +39,6 @@
 #define LOG
 #define LOG_AUDIO
 */
-#define METRONOM_INTERNAL
 #define METRONOM_CLOCK_INTERNAL
 
 #include <xine/xine_internal.h>
@@ -55,6 +54,7 @@
 #define WRAP_THRESHOLD       120000
 #define MAX_NUM_WRAP_DIFF        10
 #define MAX_SCR_PROVIDERS        10
+#define MAX_SPEED_CHANGE_CALLBACKS 16
 #define VIDEO_DRIFT_TOLERANCE 45000
 #define AUDIO_DRIFT_TOLERANCE 45000
 
@@ -88,6 +88,7 @@ typedef struct unixscr_s {
 } unixscr_t;
 
 static int unixscr_get_priority (scr_plugin_t *scr) {
+  (void)scr;
   return 5; /* low priority */
 }
 
@@ -229,8 +230,48 @@ typedef struct {
     SYNC_THREAD_RUNNING           /* self explaining */
   }                sync_thread_state;
   scr_plugin_t    *providers[MAX_SCR_PROVIDERS + 1];
+  int                     speed_change_used;
+  xine_speed_change_cb_t *speed_change_callbacks[MAX_SPEED_CHANGE_CALLBACKS + 1];
+  void                   *speed_change_data[MAX_SPEED_CHANGE_CALLBACKS + 1];
 } metronom_clock_private_t;
 
+static void metronom_register_speed_change_callback (metronom_clock_t *this,
+  xine_speed_change_cb_t *callback, void *user_data) {
+  if (callback) {
+    metronom_clock_private_t *this_priv = (metronom_clock_private_t *)this;
+    pthread_mutex_lock (&this_priv->mct.lock);
+    if (this_priv->speed_change_used < MAX_SPEED_CHANGE_CALLBACKS) {
+      this_priv->speed_change_callbacks[this_priv->speed_change_used] = callback;
+      this_priv->speed_change_data[this_priv->speed_change_used]      = user_data;
+      this_priv->speed_change_used++;
+      this_priv->speed_change_callbacks[this_priv->speed_change_used] = NULL;
+    }
+    pthread_mutex_unlock (&this_priv->mct.lock);
+  }
+}
+
+static void metronom_unregister_speed_change_callback (metronom_clock_t *this,
+  xine_speed_change_cb_t *callback, void *user_data) {
+  if (callback) {
+    metronom_clock_private_t *this_priv = (metronom_clock_private_t *)this;
+    int i;
+    pthread_mutex_lock (&this_priv->mct.lock);
+    for (i = 0; this_priv->speed_change_callbacks[i]; i++) {
+      if ((this_priv->speed_change_callbacks[i] == callback) &&
+          (this_priv->speed_change_data[i]      == user_data)) {
+        this_priv->speed_change_used--;
+        if (i != this_priv->speed_change_used) {
+          this_priv->speed_change_callbacks[i] = this_priv->speed_change_callbacks[this_priv->speed_change_used];
+          this_priv->speed_change_data[i]      = this_priv->speed_change_data[this_priv->speed_change_used];
+        }
+        this_priv->speed_change_callbacks[this_priv->speed_change_used] = NULL;
+        break;
+      }
+    }
+    pthread_mutex_unlock (&this_priv->mct.lock);
+  }
+}
+
 #define START_PTS 0
 #define STOP_PTS ~0
 #define MASK_PTS (1 << 19) /* 5.825 s */
@@ -275,20 +316,26 @@ static int64_t metronom_get_current_time (metronom_clock_t *this) {
 static void metronom_stop_clock(metronom_clock_t *this) {
   metronom_clock_private_t *this_priv = (metronom_clock_private_t *)this;
   scr_plugin_t **r;
+  int i;
 
   pthread_mutex_lock (&this_priv->mct.lock);
   for (r = this_priv->providers; *r && (r < this_priv->providers + MAX_SCR_PROVIDERS); r++)
     (*r)->set_fine_speed (*r, XINE_SPEED_PAUSE);
+  for (i = 0; this_priv->speed_change_callbacks[i]; i++)
+    this_priv->speed_change_callbacks[i] (this_priv->speed_change_data[i], XINE_SPEED_PAUSE);
   pthread_mutex_unlock (&this_priv->mct.lock);
 }
 
 static void metronom_resume_clock(metronom_clock_t *this) {
   metronom_clock_private_t *this_priv = (metronom_clock_private_t *)this;
   scr_plugin_t **r;
+  int i;
 
   pthread_mutex_lock (&this_priv->mct.lock);
   for (r = this_priv->providers; *r && (r < this_priv->providers + MAX_SCR_PROVIDERS); r++)
     (*r)->set_fine_speed (*r, XINE_FINE_SPEED_NORMAL);
+  for (i = 0; this_priv->speed_change_callbacks[i]; i++)
+    this_priv->speed_change_callbacks[i] (this_priv->speed_change_data[i], XINE_FINE_SPEED_NORMAL);
   pthread_mutex_unlock (&this_priv->mct.lock);
 }
 
@@ -304,7 +351,7 @@ static void metronom_adjust_clock(metronom_clock_t *this, int64_t desired_pts) {
 static int metronom_set_speed (metronom_clock_t *this, int speed) {
   metronom_clock_private_t *this_priv = (metronom_clock_private_t *)this;
   scr_plugin_t **r;
-  int            true_speed;
+  int            true_speed, i;
 
   true_speed = this_priv->mct.scr_master->set_fine_speed (this_priv->mct.scr_master, speed);
 
@@ -313,13 +360,85 @@ static int metronom_set_speed (metronom_clock_t *this, int speed) {
   pthread_mutex_lock (&this_priv->mct.lock);
   for (r = this_priv->providers; *r && (r < this_priv->providers + MAX_SCR_PROVIDERS); r++)
     (*r)->set_fine_speed (*r, true_speed);
+  for (i = 0; this_priv->speed_change_callbacks[i]; i++)
+    this_priv->speed_change_callbacks[i] (this_priv->speed_change_data[i], true_speed);
   pthread_mutex_unlock (&this_priv->mct.lock);
 
   return true_speed;
 }
 
+/*
+ * metronom
+ */
+
+typedef struct {
+
+  metronom_t metronom;
+
+  /*
+   * metronom internal stuff
+   */
+  xine_t         *xine;
+
+  metronom_t     *master;
+
+  int64_t         pts_per_smpls;
+
+  int64_t         video_vpts;
+  int64_t         spu_vpts;
+  int64_t         audio_vpts;
+  int64_t         audio_vpts_rmndr;  /* the remainder for integer division */
+
+  int64_t         vpts_offset;
+
+  int64_t         video_drift;
+  int64_t         video_drift_step;
+
+  int             base_av_offset;
+
+  int             audio_samples;
+  int64_t         audio_drift_step;
+
+  int64_t         prebuffer;
+  int64_t         av_offset;
+  int64_t         spu_offset;
+
+  pthread_mutex_t lock;
+
+  int             have_video;
+  int             have_audio;
+
+  int             bounce_left_audio;
+  int             bounce_left_video;
+  int             bounce_jumped;
+  int64_t         bounce_diff;
+  int64_t         bounce_vpts_offs;
+
+  int64_t         last_discontinuity_offs;
+  int             last_discontinuity_type;
+  int             video_discontinuity_count;
+  int             audio_discontinuity_count;
+  int             discontinuity_handled_count;
+  int             num_video_waiters;
+  int             num_audio_waiters;
+  pthread_cond_t  video_discontinuity_reached;
+  pthread_cond_t  audio_discontinuity_reached;
+
+  int             force_video_jump;
+  int             force_audio_jump;
+
+  int64_t         last_audio_pts;
+
+  int64_t         last_video_pts;
+  int64_t         img_duration;
+  int             img_cpt;
+  int             video_mode;
+
+} metronom_impl_t;
+
+static void metronom_set_audio_rate (metronom_t *this_gen, int64_t pts_per_smpls) {
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
 
-static void metronom_set_audio_rate (metronom_t *this, int64_t pts_per_smpls) {
   pthread_mutex_lock (&this->lock);
 
   this->pts_per_smpls = pts_per_smpls;
@@ -329,7 +448,8 @@ static void metronom_set_audio_rate (metronom_t *this, int64_t pts_per_smpls) {
   lprintf("%" PRId64 " pts per %d samples\n", pts_per_smpls, AUDIO_SAMPLE_NUM);
 }
 
-static int64_t metronom_got_spu_packet (metronom_t *this, int64_t pts) {
+static int64_t metronom_got_spu_packet (metronom_t *this_gen, int64_t pts) {
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
   int64_t vpts;
 
   pthread_mutex_lock (&this->lock);
@@ -357,8 +477,36 @@ static int64_t metronom_got_spu_packet (metronom_t *this, int64_t pts) {
   return vpts;
 }
 
-static void metronom_handle_discontinuity (metronom_t *this, int type,
-                                           int64_t disc_off) {
+/* There are 3 stages of discontinuity handling:
+ * 1. At demux time (entering fifo):
+ *   If pts jumps more than 2 seconds, insert a discontinuity control buf.
+ * 2. At metronom_handle_foo_discontinuity () time (leaving fifo):
+ *   Wait for both fifos to reach this, then bump vpts offset.
+ * 3. At metronom_got_bar () time (after decoding):
+ *   Apply vpts to output buf.
+ *
+ * Issue: Not all decoders work straight forward. Many video decoders
+ * delay and reorder their output. We could defer discontinuity handling
+ * to stage 3, but decoder may swallow frames due to errors.
+ *
+ * Issue: mpeg style cotainers may jump during reorder, and pts will
+ * bounce some 20 times. Lets detect these, and try to stay calm.
+ * Workaround: at 2, remember previous setting, and swap with it when
+ * appropriate. This reduces timeline drift. At 3, do accept a few
+ * frames from both settings.
+ *
+ * Issue: Audio frames are often small (< 500 bytes). Putting 1 such
+ * frame into mpeg-ts packets (188 bytes) will waste a lot of space.
+ * Thats why they usually take 0.5 .. 1 seconds at once, and all video
+ * frames for the same time need to be sent before. That normal
+ * discontinuity wait will then outdate some audio, and yield a gap of
+ * silence.
+ * Workaround: Use the bounce hack above to handle most absolute
+ * discontinuities without wait. */
+#define BOUNCE_MAX 220000
+
+static int metronom_handle_discontinuity (metronom_impl_t *this,
+  int type, int try, int64_t disc_off) {
   int64_t cur_time;
 
   /* video_vpts and audio_vpts adjustements */
@@ -380,33 +528,117 @@ static void metronom_handle_discontinuity (metronom_t *this, int type,
         int64_t t;
         int speed = this->xine->clock->speed;
         if (speed <= 0)
-          break;
+          return 0;
         pthread_mutex_lock (&this->lock);
         t = this->video_vpts > this->audio_vpts ? this->video_vpts : this->audio_vpts;
         t -= this->prebuffer + cur_time;
         pthread_mutex_unlock (&this->lock);
-        if ((t <= 0) || (t > 90000))
-          break;
+        if ((t <= 0) || (t > 135000))
+          return 0;
         xprintf (this->xine, XINE_VERBOSITY_DEBUG,
           "metronom: gapless switch: wait %"PRId64" pts.\n", t);
         xine_usec_sleep ((int)((t * XINE_FINE_SPEED_NORMAL) / (speed * 90)) * 1000);
       }
-      break;
+      return 0;
 
     case DISC_STREAMSTART:
+      lprintf ("DISC_STREAMSTART\n");
+      disc_off = 0;
+      /* fall through */
     case DISC_STREAMSEEK:
-      this->video_vpts = this->prebuffer + cur_time;
-      this->audio_vpts = this->video_vpts;
+      lprintf ("DISC_STREAMSEEK\n");
+      this->video_vpts       = this->prebuffer + cur_time;
+      this->audio_vpts       = this->video_vpts;
+      this->vpts_offset      = this->video_vpts - disc_off;
+      this->bounce_left_audio = -1;
+      this->bounce_left_video = -1;
+      this->bounce_jumped    = 0;
       this->audio_vpts_rmndr = 0;
       this->force_audio_jump = 1;
       this->force_video_jump = 1;
-      this->video_drift = 0;
+      this->video_drift      = 0;
+      this->last_video_pts   = 0;
+      this->last_audio_pts   = 0;
       xprintf (this->xine, XINE_VERBOSITY_DEBUG,
         "metronom: vpts adjusted with prebuffer to %" PRId64 ".\n", this->video_vpts);
-      break;
+      lprintf("video_vpts: %" PRId64 ", audio_vpts: %" PRId64 "\n", this->video_vpts, this->audio_vpts);
+      return 0;
+
+    case DISC_ABSOLUTE: {
+      int64_t d, video_vpts, vpts_offset;
+      int mode;
+      lprintf ("DISC_ABSOLUTE\n");
+      /* calculate but dont set yet */
+      mode = ((this->video_vpts < cur_time) << 1) | (this->audio_vpts < cur_time);
+      video_vpts = (mode == 3) ? this->prebuffer + cur_time
+                 : (mode == 2) ? this->audio_vpts
+                 : this->video_vpts;
+      vpts_offset = video_vpts - disc_off;
+      /* where are we? */
+      d = vpts_offset - this->vpts_offset;
+      if (d < 0)
+        d = -d;
+      if (d < BOUNCE_MAX) {
+        /* small step, keep old previous. */
+        ;
+      } else {
+        /* big step. */
+        d = vpts_offset - this->bounce_vpts_offs;
+        if (d < 0)
+          d = -d;
+        if (d < BOUNCE_MAX) {
+          /* near old previous, swap with it. */
+          d = this->vpts_offset;
+          this->vpts_offset = this->bounce_vpts_offs;
+          this->bounce_vpts_offs = d;
+          d -= this->vpts_offset;
+          this->bounce_diff = d;
+          this->bounce_left_audio = BOUNCE_MAX;
+          this->bounce_left_video = BOUNCE_MAX;
+          this->last_audio_pts = 0;
+          this->last_video_pts = 0;
+          xprintf (this->xine, XINE_VERBOSITY_DEBUG, "metronom: pts bounce by %" PRId64 ".\n", d);
+          return 0;
+        }
+        if (try && (this->bounce_left_audio >= 0))
+          return 1;
+        /* remember current as prev, and set new. */
+        this->bounce_vpts_offs = this->vpts_offset;
+      }
+      this->vpts_offset = vpts_offset;
+      this->bounce_diff = this->bounce_vpts_offs - vpts_offset;
+      this->video_vpts = video_vpts;
+      this->bounce_left_audio = BOUNCE_MAX;
+      this->bounce_left_video = BOUNCE_MAX;
+      this->bounce_jumped = 1;
+      if (mode == 2) {
+        /* still frame with audio */
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+          "metronom: video vpts adjusted to audio vpts %" PRId64 ".\n", this->video_vpts);
+      } else if (mode == 3) {
+        /* still frame, no audio */
+        this->audio_vpts = video_vpts;
+        this->audio_vpts_rmndr = 0;
+        this->force_video_jump = 1;
+        this->force_audio_jump = 1;
+        this->video_drift = 0;
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+          "metronom: vpts adjusted with prebuffer to %" PRId64 ".\n", this->video_vpts);
+      } else if (mode == 1) {
+        /* video, no sound */
+        this->audio_vpts = video_vpts;
+        this->audio_vpts_rmndr = 0;
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+          "metronom: audio vpts adjusted to video vpts %" PRId64 ".\n", this->video_vpts);
+      }
+    }
+    this->last_video_pts = 0;
+    this->last_audio_pts = 0;
+    lprintf ("video_vpts: %" PRId64 ", audio_vpts: %" PRId64 "\n", this->video_vpts, this->audio_vpts);
+    return 0;
 
-    case DISC_ABSOLUTE:
     case DISC_RELATIVE:
+      lprintf ("DISC_RELATIVE\n");
       if (this->video_vpts < cur_time) {
         /* still frame */
         if (this->audio_vpts > cur_time) {
@@ -438,37 +670,59 @@ static void metronom_handle_discontinuity (metronom_t *this, int type,
           /* video + audio */
         }
       }
-      break;
+      this->vpts_offset = this->vpts_offset - disc_off;
+      this->last_video_pts = 0;
+      this->last_audio_pts = 0;
+      lprintf ("video_vpts: %" PRId64 ", audio_vpts: %" PRId64 "\n", this->video_vpts, this->audio_vpts);
+      return 0;
+
+    default:
+      return 0;
   }
-
-  lprintf("video_vpts: %" PRId64 ", audio_vpts: %" PRId64 "\n", this->video_vpts, this->audio_vpts);
-
-  /* vpts_offset adjustements */
-  switch (type) {
-  case DISC_STREAMSTART:
-    lprintf("DISC_STREAMSTART\n");
-    this->vpts_offset = this->video_vpts;
-    break;
-  case DISC_ABSOLUTE:
-    lprintf("DISC_ABSOLUTE\n");
-    this->vpts_offset = this->video_vpts - disc_off;
-    break;
-  case DISC_RELATIVE:
-    lprintf("DISC_RELATIVE\n");
-    this->vpts_offset = this->vpts_offset - disc_off;
-    break;
-  case DISC_STREAMSEEK:
-    lprintf("DISC_STREAMSEEK\n");
-    this->vpts_offset = this->video_vpts - disc_off;
-    break;
-  }
-
-  this->last_video_pts = 0;
-  this->last_audio_pts = 0;
 }
 
-static void metronom_handle_video_discontinuity (metronom_t *this, int type,
-						 int64_t disc_off) {
+static void metronom_handle_vdr_trick_pts (metronom_impl_t *this, int64_t pts) {
+  int64_t cur_time = this->xine->clock->get_current_time (this->xine->clock);
+  if (this->video_vpts < cur_time) {
+    if (this->audio_vpts >= cur_time) {
+      /* still frame with audio */
+      this->video_vpts = this->audio_vpts;
+    } else {
+      /* still frame, no audio */
+      this->audio_vpts =
+      this->video_vpts = this->prebuffer + cur_time;
+      this->audio_vpts_rmndr = 0;
+      this->force_video_jump = 1;
+      this->force_audio_jump = 1;
+      this->video_drift = 0;
+    }
+  } else {
+    if (this->audio_vpts < cur_time) {
+      /* video, no sound */
+      this->audio_vpts = this->video_vpts;
+      this->audio_vpts_rmndr = 0;
+    }
+  }
+  this->vpts_offset = this->video_vpts - pts;
+  this->bounce_diff = this->bounce_vpts_offs - this->vpts_offset;
+  this->bounce_left_audio = -1;
+  this->bounce_left_video = -1;
+  this->bounce_jumped = 0;
+  xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+    "metronom: vdr trick pts %" PRId64 ", vpts %" PRId64 ".\n", pts, this->video_vpts);
+}
+
+static void metronom_handle_video_discontinuity (metronom_t *this_gen, int type,
+                                                 int64_t disc_off) {
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
+  int waited;
+
+  if (type == DISC_GAPLESS) {
+    /* this would cause deadlock in metronom_handle_discontinuity()
+       because of double pthread_mutex_lock(&this->lock) */
+    _x_assert(type != DISC_GAPLESS);
+    return;
+  }
 
   pthread_mutex_lock (&this->lock);
 
@@ -479,12 +733,34 @@ static void metronom_handle_video_discontinuity (metronom_t *this, int type,
   }
 
   this->video_discontinuity_count++;
-  pthread_cond_signal (&this->video_discontinuity_reached);
+  if (this->num_video_waiters && (this->audio_discontinuity_count <= this->video_discontinuity_count))
+    pthread_cond_signal (&this->video_discontinuity_reached);
 
   xprintf (this->xine, XINE_VERBOSITY_DEBUG,
     "metronom: video discontinuity #%d, type is %d, disc_off %" PRId64 ".\n",
     this->video_discontinuity_count, type, disc_off);
 
+  if (this->video_discontinuity_count <= this->discontinuity_handled_count) {
+    pthread_mutex_unlock (&this->lock);
+    return;
+  }
+
+  if (type == DISC_ABSOLUTE) {
+    if (!metronom_handle_discontinuity (this, type, 1, disc_off)) {
+      this->discontinuity_handled_count = this->video_discontinuity_count;
+      pthread_mutex_unlock (&this->lock);
+      return;
+    }
+  }
+
+  /* If both audio and video are there, the video side shall take
+   * effect. Previous code did this by letting audio wait even if
+   * video came first. Lets drop that unnecessary wait, and pass
+   * over params instead. */
+  this->last_discontinuity_type = type;
+  this->last_discontinuity_offs = disc_off;
+
+  waited = 0;
   if (this->have_audio) {
     while (this->audio_discontinuity_count <
 	   this->video_discontinuity_count) {
@@ -493,20 +769,24 @@ static void metronom_handle_video_discontinuity (metronom_t *this, int type,
         "metronom: waiting for audio discontinuity #%d...\n",
         this->video_discontinuity_count);
 
+      this->num_audio_waiters++;
       pthread_cond_wait (&this->audio_discontinuity_reached, &this->lock);
+      this->num_audio_waiters--;
+      waited = 1;
     }
   }
 
-  metronom_handle_discontinuity(this, type, disc_off);
-
-  this->discontinuity_handled_count++;
-  pthread_cond_signal (&this->video_discontinuity_reached);
+  if (!waited) {
+    metronom_handle_discontinuity (this, type, 0, disc_off);
+    this->discontinuity_handled_count++;
+  }
 
   pthread_mutex_unlock (&this->lock);
 }
 
-static void metronom_got_video_frame (metronom_t *this, vo_frame_t *img) {
+static void metronom_got_video_frame (metronom_t *this_gen, vo_frame_t *img) {
 
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
   int64_t pts = img->pts;
 
   pthread_mutex_lock (&this->lock);
@@ -573,6 +853,21 @@ static void metronom_got_video_frame (metronom_t *this, vo_frame_t *img) {
 
     pts += this->vpts_offset;
 
+    if (this->bounce_left_video >= 0) {
+      int64_t diff = this->video_vpts - pts;
+      if ((abs (diff) > BOUNCE_MAX) && (abs (diff - this->bounce_diff) < BOUNCE_MAX)) {
+        pts += this->bounce_diff;
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG, "metronom: bounced video frame with pts %" PRId64 ".\n", img->pts);
+      }
+      this->bounce_left_video -= img->duration;
+      if (this->bounce_left_video < 0) {
+        this->bounce_left_audio = -1;
+        this->bounce_left_video = -1;
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+          "metronom: leaving bounce area at pts %" PRId64 ".\n", img->pts);
+      }
+    }
+
     if (this->video_mode == VIDEO_PREDICTION_MODE) {
 
       int64_t diff = this->video_vpts - pts;
@@ -581,7 +876,8 @@ static void metronom_got_video_frame (metronom_t *this, vo_frame_t *img) {
 
       if ((abs (diff) > VIDEO_DRIFT_TOLERANCE) || (this->force_video_jump)) {
 
-        xprintf (this->xine, XINE_VERBOSITY_DEBUG, "metronom: video jump by %"PRId64" pts.\n", diff);
+
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG, "metronom: video jump by %"PRId64" pts.\n", -diff);
         this->force_video_jump = 0;
         this->video_vpts       = pts;
         this->video_drift      = 0;
@@ -615,7 +911,7 @@ static void metronom_got_video_frame (metronom_t *this, vo_frame_t *img) {
     }
   }
 
-  img->vpts = this->video_vpts + this->av_offset;
+  img->vpts = this->video_vpts + this->av_offset + this->base_av_offset;
 
   /* We need to update this->video_vpts is both modes.
    * this->video_vpts is used as the next frame vpts if next frame pts=0
@@ -653,11 +949,13 @@ static void metronom_got_video_frame (metronom_t *this, vo_frame_t *img) {
       img->pts, (int)img->duration, img->vpts);
 }
 
-static void metronom_handle_audio_discontinuity (metronom_t *this, int type,
-						 int64_t disc_off) {
+static void metronom_handle_audio_discontinuity (metronom_t *this_gen, int type,
+                                                 int64_t disc_off) {
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
+  int waited;
 
   if (type == DISC_GAPLESS) {
-    metronom_handle_discontinuity (this, type, disc_off);
+    metronom_handle_discontinuity (this, type, 0, disc_off);
     return;
   }
 
@@ -670,37 +968,59 @@ static void metronom_handle_audio_discontinuity (metronom_t *this, int type,
   }
 
   this->audio_discontinuity_count++;
-  pthread_cond_signal (&this->audio_discontinuity_reached);
+  if (this->num_audio_waiters && (this->audio_discontinuity_count >= this->video_discontinuity_count))
+    pthread_cond_signal (&this->audio_discontinuity_reached);
 
   xprintf (this->xine, XINE_VERBOSITY_DEBUG,
     "metronom: audio discontinuity #%d, type is %d, disc_off %" PRId64 ".\n",
     this->audio_discontinuity_count, type, disc_off);
 
-  if (this->have_video) {
-
-    /* next_vpts_offset, in_discontinuity is handled in expect_video_discontinuity */
-    while ( this->audio_discontinuity_count >
-            this->discontinuity_handled_count ) {
-
-      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
-        "metronom: waiting for in_discontinuity update #%d...\n",
-        this->audio_discontinuity_count);
-
-      pthread_cond_wait (&this->video_discontinuity_reached, &this->lock);
-    }
-  } else {
-    metronom_handle_discontinuity(this, type, disc_off);
+  if (this->audio_discontinuity_count <= this->discontinuity_handled_count) {
+    pthread_mutex_unlock (&this->lock);
+    return;
   }
 
+  if (type == DISC_ABSOLUTE) {
+    if (!metronom_handle_discontinuity (this, type, 1, disc_off)) {
+      this->discontinuity_handled_count = this->audio_discontinuity_count;
+      pthread_mutex_unlock (&this->lock);
+      return;
+    }
+  }
+
+  waited = 0;
+  if (this->have_video) {
+    while ( this->audio_discontinuity_count >
+            this->video_discontinuity_count ) {
+
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+        "metronom: waiting for video discontinuity #%d...\n",
+        this->audio_discontinuity_count);
+
+      this->num_video_waiters++;
+      pthread_cond_wait (&this->video_discontinuity_reached, &this->lock);
+      this->num_video_waiters--;
+      waited = 1;
+    }
+  } else {
+    this->last_discontinuity_type = type;
+    this->last_discontinuity_offs = disc_off;
+  }
+
+  if (!waited) {
+    metronom_handle_discontinuity (this, this->last_discontinuity_type, 0, this->last_discontinuity_offs);
+    this->discontinuity_handled_count++;
+  }
   this->audio_samples = 0;
   this->audio_drift_step = 0;
 
   pthread_mutex_unlock (&this->lock);
 }
 
-static int64_t metronom_got_audio_samples (metronom_t *this, int64_t pts,
-					   int nsamples) {
+static int64_t metronom_got_audio_samples (metronom_t *this_gen, int64_t pts,
+                                           int nsamples) {
 
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
   int64_t vpts;
 
   lprintf("got %d audio samples, pts is %" PRId64 ", last pts = %" PRId64 "\n", nsamples, pts, this->last_audio_pts);
@@ -733,9 +1053,34 @@ static int64_t metronom_got_audio_samples (metronom_t *this, int64_t pts,
     vpts = pts + this->vpts_offset;
     diff = this->audio_vpts - vpts;
 
+    if (this->bounce_left_audio >= 0) {
+      if ((abs (diff) > BOUNCE_MAX) && (abs (diff - this->bounce_diff) < BOUNCE_MAX)) {
+        vpts += this->bounce_diff;
+        diff = this->audio_vpts - vpts;
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG, "metronom: bounced audio buffer with pts %" PRId64 ".\n", pts);
+      }
+      this->bounce_left_audio -= (nsamples * this->pts_per_smpls) >> AUDIO_SAMPLE_LD;
+      if (this->bounce_left_audio < 0) {
+        this->bounce_left_audio = -1;
+        this->bounce_left_video = -1;
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+          "metronom: leaving bounce area at pts %" PRId64 ".\n", pts);
+      }
+      if (this->bounce_jumped) {
+        if ((diff > 0) /* && (diff < BOUNCE_MAX) */) {
+          vpts += diff;
+          this->vpts_offset += diff;
+          this->bounce_diff = this->bounce_vpts_offs - this->vpts_offset;
+          xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+            "metronom: fixing discontinuity jump by %" PRId64 " pts.\n", diff);
+          diff = 0;
+        }
+      }
+    }
+
     /* compare predicted and given vpts */
     if((abs(diff) > AUDIO_DRIFT_TOLERANCE) || (this->force_audio_jump)) {
-      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "metronom: audio jump by %" PRId64 " pts.\n", diff);
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "metronom: audio jump by %" PRId64 " pts.\n", -diff);
       this->force_audio_jump = 0;
       this->audio_vpts       = vpts;
       this->audio_vpts_rmndr = 0;
@@ -799,7 +1144,9 @@ static int64_t metronom_got_audio_samples (metronom_t *this, int64_t pts,
   return vpts;
 }
 
-static void metronom_set_option (metronom_t *this, int option, int64_t value) {
+static void metronom_set_option (metronom_t *this_gen, int option, int64_t value) {
+
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
 
   if (option == METRONOM_LOCK) {
     if (value) {
@@ -851,6 +1198,9 @@ static void metronom_set_option (metronom_t *this, int option, int64_t value) {
     xprintf (this->xine, XINE_VERBOSITY_LOG,
       "metronom: prebuffer=%" PRId64 " pts.\n", this->prebuffer);
     break;
+  case METRONOM_VDR_TRICK_PTS:
+    metronom_handle_vdr_trick_pts (this, value);
+    break;
   default:
     xprintf(this->xine, XINE_VERBOSITY_NONE,
       "metronom: unknown option in set_option: %d.\n", option);
@@ -876,8 +1226,9 @@ static void metronom_clock_set_option (metronom_clock_t *this,
   pthread_mutex_unlock (&this->lock);
 }
 
-static int64_t metronom_get_option (metronom_t *this, int option) {
+static int64_t metronom_get_option (metronom_t *this_gen, int option) {
 
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
   int64_t result;
   int mutex_locked;
 
@@ -919,6 +1270,12 @@ static int64_t metronom_get_option (metronom_t *this, int option) {
       else
         result = this->audio_vpts;
       break;
+  case METRONOM_WAITING:
+    result = (this->num_audio_waiters ? 1 : 0) | (this->num_video_waiters ? 2 : 0);
+    break;
+  case METRONOM_VDR_TRICK_PTS:
+    result = this->video_vpts;
+    break;
   default:
     result = 0;
     xprintf (this->xine, XINE_VERBOSITY_NONE,
@@ -943,7 +1300,8 @@ static int64_t metronom_clock_get_option (metronom_clock_t *this, int option) {
   return 0;
 }
 
-static void metronom_set_master(metronom_t *this, metronom_t *master) {
+static void metronom_set_master(metronom_t *this_gen, metronom_t *master) {
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
   metronom_t *old_master = this->master;
 
   pthread_mutex_lock(&this->lock);
@@ -985,8 +1343,7 @@ static scr_plugin_t* get_master_scr(metronom_clock_t *this) {
 static void *metronom_sync_loop (void *const this_gen) {
   metronom_clock_private_t *const this_priv = (metronom_clock_private_t *const)this_gen;
 
-  struct timeval tv;
-  struct timespec ts;
+  struct timespec ts = {0, 0};
   scr_plugin_t **r;
   int64_t        pts;
 
@@ -999,9 +1356,8 @@ static void *metronom_sync_loop (void *const this_gen) {
     for (r = this_priv->providers; *r && (r < this_priv->providers + MAX_SCR_PROVIDERS); r++)
       if (*r != this_priv->mct.scr_master) (*r)->adjust (*r, pts);
 
-    gettimeofday(&tv, NULL);
-    ts.tv_sec  = tv.tv_sec + 5;
-    ts.tv_nsec = tv.tv_usec * 1000;
+    xine_gettime (&ts);
+    ts.tv_sec += 5;
     pthread_cond_timedwait (&this_priv->mct.cancel, &this_priv->mct.lock, &ts);
 
     pthread_mutex_unlock (&this_priv->mct.lock);
@@ -1122,7 +1478,11 @@ static void metronom_unregister_scr (metronom_clock_t *this, scr_plugin_t *scr)
     metronom_stop_sync_thread (this_priv);
 }
 
-static void metronom_exit (metronom_t *this) {
+static void metronom_exit (metronom_t *this_gen) {
+
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
+
+  this->xine->config->unregister_callbacks (this->xine->config, NULL, NULL, this, sizeof (*this));
 
   pthread_mutex_destroy (&this->lock);
   pthread_cond_destroy (&this->video_discontinuity_reached);
@@ -1135,7 +1495,7 @@ static void metronom_clock_exit (metronom_clock_t *this) {
   metronom_clock_private_t *this_priv = (metronom_clock_private_t *)this;
   scr_plugin_t **r;
 
-  this_priv->mct.xine->config->unregister_callback (this_priv->mct.xine->config, "engine.use_metronom_sync_thread");
+  this_priv->mct.xine->config->unregister_callbacks (this_priv->mct.xine->config, NULL, NULL, this_priv, sizeof (*this_priv));
 
   metronom_stop_sync_thread (this_priv);
 
@@ -1149,12 +1509,17 @@ static void metronom_clock_exit (metronom_clock_t *this) {
 }
 
 
+static void metronom_base_av_offs_hook (void *this_gen, xine_cfg_entry_t *entry) {
+  metronom_impl_t *this = (metronom_impl_t *)this_gen;
+  this->base_av_offset = entry->num_value;
+}
+
 metronom_t * _x_metronom_init (int have_video, int have_audio, xine_t *xine) {
 
-  metronom_t *this = calloc(1, sizeof (metronom_t));
+  metronom_impl_t *this = calloc(1, sizeof (metronom_impl_t));
   if (!this)
     return NULL;
-
+#ifndef HAVE_ZERO_SAFE_MEM
   /* Do these first, when compiler still knows "this" is all zeroed.
    * Let it optimize away this on most systems where clear mem
    * interpretes as 0, 0f or NULL safely.
@@ -1172,17 +1537,34 @@ metronom_t * _x_metronom_init (int have_video, int have_audio, xine_t *xine) {
   this->last_audio_pts              = 0;
   this->audio_vpts_rmndr            = 0;
   this->audio_discontinuity_count   = 0;
-
-  this->set_audio_rate             = metronom_set_audio_rate;
-  this->got_video_frame            = metronom_got_video_frame;
-  this->got_audio_samples          = metronom_got_audio_samples;
-  this->got_spu_packet             = metronom_got_spu_packet;
-  this->handle_audio_discontinuity = metronom_handle_audio_discontinuity;
-  this->handle_video_discontinuity = metronom_handle_video_discontinuity;
-  this->set_option                 = metronom_set_option;
-  this->get_option                 = metronom_get_option;
-  this->set_master                 = metronom_set_master;
-  this->exit                       = metronom_exit;
+  this->num_audio_waiters           = 0;
+  this->num_video_waiters           = 0;
+  this->pts_per_smpls               = 0;
+  this->spu_vpts                    = 0;
+  this->audio_samples               = 0;
+  this->audio_drift_step            = 0;
+  this->bounce_jumped               = 0;
+  this->bounce_diff                 = 0;
+  this->bounce_vpts_offs            = 0;
+  this->last_discontinuity_offs     = 0;
+  this->last_discontinuity_type     = 0;
+  this->force_video_jump            = 0;
+  this->force_audio_jump            = 0;
+  this->img_duration                = 0;
+  this->video_mode                  = 0;
+#endif
+  this->bounce_left_audio           = -1;
+  this->bounce_left_video           = -1;
+  this->metronom.set_audio_rate             = metronom_set_audio_rate;
+  this->metronom.got_video_frame            = metronom_got_video_frame;
+  this->metronom.got_audio_samples          = metronom_got_audio_samples;
+  this->metronom.got_spu_packet             = metronom_got_spu_packet;
+  this->metronom.handle_audio_discontinuity = metronom_handle_audio_discontinuity;
+  this->metronom.handle_video_discontinuity = metronom_handle_video_discontinuity;
+  this->metronom.set_option                 = metronom_set_option;
+  this->metronom.get_option                 = metronom_get_option;
+  this->metronom.set_master                 = metronom_set_master;
+  this->metronom.exit                       = metronom_exit;
 
   this->xine                       = xine;
 
@@ -1203,7 +1585,16 @@ metronom_t * _x_metronom_init (int have_video, int have_audio, xine_t *xine) {
   this->audio_vpts = this->prebuffer;
   pthread_cond_init (&this->audio_discontinuity_reached, NULL);
 
-  return this;
+  this->base_av_offset = xine->config->register_num (xine->config, "video.output.base_delay", 0,
+    _("basic video to audio delay in pts"),
+    _("Getting in sync picture and sound is a complex story.\n"
+      "Xine will compensate for any delays it knows about.\n"
+      "However, external hardware like flatscreens, sound systems, or simply\n"
+      "the distance between you and the speakers may add in more.\n"
+      "Here you can adjust video timing in steps of 1/90000 seconds manually."),
+    10, metronom_base_av_offs_hook, this);
+
+  return &this->metronom;
 }
 
 
@@ -1233,6 +1624,10 @@ metronom_clock_t *_x_metronom_clock_init(xine_t *xine)
 
   if (!this_priv)
     return NULL;
+#ifndef HAVE_ZERO_SAFE_MEM
+  this_priv->speed_change_used = 0;
+  this_priv->speed_change_callbacks[0] = NULL;
+#endif
 
   this_priv->mct.set_option       = metronom_clock_set_option;
   this_priv->mct.get_option       = metronom_clock_get_option;
@@ -1246,6 +1641,9 @@ metronom_clock_t *_x_metronom_clock_init(xine_t *xine)
   this_priv->mct.unregister_scr   = metronom_unregister_scr;
   this_priv->mct.exit             = metronom_clock_exit;
 
+  this_priv->mct.register_speed_change_callback   = metronom_register_speed_change_callback;
+  this_priv->mct.unregister_speed_change_callback = metronom_unregister_speed_change_callback;
+
   this_priv->mct.xine             = xine;
   this_priv->mct.scr_adjustable   = 1;
   this_priv->mct.scr_list         = this_priv->providers;
@@ -1269,3 +1667,4 @@ metronom_clock_t *_x_metronom_clock_init(xine_t *xine)
 
   return &this_priv->mct;
 }
+
