@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2017 the xine project
+ * Copyright (C) 2007-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -19,6 +19,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <xine/configfile.h>
 #include <xine/xine_internal.h>
 #include <xine/spu.h>
@@ -27,22 +31,18 @@
 #define BLACK_OPACITY   67
 #define COLOUR_OPACITY 100
 
-static void no_op (void *user_data, xine_cfg_entry_t *entry)
-{
-}
-
 void _x_spu_misc_init (xine_t *this)
 {
   this->config->register_range (this->config, "subtitles.bitmap.black_opacity",
                                 BLACK_OPACITY, 0, 100,
                                 _("opacity for the black parts of bitmapped subtitles"),
                                 NULL,
-                                10, no_op, NULL);
+                                10, NULL, NULL);
   this->config->register_range (this->config, "subtitles.bitmap.colour_opacity",
                                 COLOUR_OPACITY, 0, 100,
                                 _("opacity for the colour parts of bitmapped subtitles"),
                                 NULL,
-                                10, no_op, NULL);
+                                10, NULL, NULL);
 }
 
 void _x_spu_get_opacity (xine_t *this, xine_spu_opacity_t *opacity)
