@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2017 the xine project
+ * Copyright (C) 2001-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -29,7 +29,6 @@
  *
  */
 
-#define XINE_ENGINE_INTERNAL
 #define ARCH_WARN
 
 #ifdef HAVE_CONFIG_H
@@ -223,10 +222,6 @@ static __inline__ void *linux_kernel_memcpy_impl (void *to, const void *from, si
   return ret;
 }
 
-#define AVX_MMREG_SIZE 32
-#define SSE_MMREG_SIZE 16
-#define MMX_MMREG_SIZE 8
-
 #define MMX1_MIN_LEN 0x800  /* 2K blocks */
 #define MIN_LEN 0x40  /* 64-byte blocks */
 
@@ -699,7 +694,7 @@ static void update_fast_memcpy (void *user_data, xine_cfg_entry_t *entry) {
   int     method = entry->num_value;
 
   /* check if function is configured and valid for this machine */
-  if ((method > 0) && (method < sizeof (memcpy_method) / sizeof (memcpy_method[0]) - 1) &&
+  if ((method > 0) && ((size_t)method < sizeof (memcpy_method) / sizeof (memcpy_method[0]) - 1) &&
      ((config_flags & memcpy_method[method].cpu_require) == memcpy_method[method].cpu_require)) {
     xprintf (xine, XINE_VERBOSITY_DEBUG, "xine_fast_memcpy (): using \"%s\"\n", memcpy_method[method].name);
     xine_fast_memcpy = memcpy_method[method].function;
