@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -772,7 +772,7 @@ int xine_monotonic_clock(struct timeval *tv, struct timezone *tz)
   return gettimeofday (tv, tz);
 }
 
-char *xine_strcat_realloc (char **dest, char *append)
+char *xine_strcat_realloc (char **dest, const char *append)
 {
   char *newstr = realloc (*dest, (*dest ? strlen (*dest) : 0) + strlen (append) + 1);
   if (newstr)
