@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2017 the xine project
+ *  Copyright (C) 2002-2018 the xine project
  *
  *  This file is part of xine, a free video player.
  *
@@ -106,8 +106,8 @@ enum {
 };
 
 struct lexer *lexer_init_r(const char * buf, int size) {
-  static const char boms[] = { 0xFF, 0xFE, 0, 0, 0xFE, 0xFF },
-		    bom_utf8[] = { 0xEF, 0xBB, 0xBF };
+  static const uint8_t boms[]     = { 0xFF, 0xFE, 0, 0, 0xFE, 0xFF };
+  static const uint8_t bom_utf8[] = { 0xEF, 0xBB, 0xBF };
   struct lexer * lexer = calloc (1, sizeof (*lexer));
 
   if (!lexer) {
@@ -177,7 +177,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
   char c;
 
   if (tok) {
-    while ((tok_pos < tok_size) && (lexer->lexbuf_pos < lexer->lexbuf_size)) {
+    while ((tok_pos + 8 < tok_size) && (lexer->lexbuf_pos < lexer->lexbuf_size)) {
       c = lexer->lexbuf[lexer->lexbuf_pos];
       lprintf("c=%c, state=%d, in_comment=%d\n", c, state, lexer->in_comment);
 
@@ -286,7 +286,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	  case '/':
 	    tok[tok_pos] = c;
 	    lexer->lexbuf_pos++;
-	    tok_pos++; /* FIXME */
+	    tok_pos++;
 	    tok[tok_pos] = '\0';
 	    return T_M_START_2;
 	    break;
@@ -299,7 +299,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	  case '?':
 	    tok[tok_pos] = c;
 	    lexer->lexbuf_pos++;
-	    tok_pos++; /* FIXME */
+	    tok_pos++;
 	    tok[tok_pos] = '\0';
 	    return T_TI_START;
 	    break;
@@ -322,7 +322,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	  if (c == '>') {
 	    tok[tok_pos] = c;
 	    lexer->lexbuf_pos++;
-	    tok_pos++; /* FIXME */
+	    tok_pos++;
 	    tok[tok_pos] = '\0';
 	    if (!lexer->in_comment)
 	      lexer->lex_mode = DATA;
@@ -358,7 +358,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	    if (lexer->lexbuf[lexer->lexbuf_pos] == '-')
 	      {
 		lexer->lexbuf_pos++;
-		tok[tok_pos++] = '-'; /* FIXME */
+		tok[tok_pos++] = '-'; 
 		tok[tok_pos++] = '-';
 		tok[tok_pos] = '\0';
 		lexer->in_comment = 1;
@@ -368,7 +368,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	  case 'D':
 	    lexer->lexbuf_pos++;
 	    if (strncmp(lexer->lexbuf + lexer->lexbuf_pos, "OCTYPE", 6) == 0) {
-	      strncpy(tok + tok_pos, "DOCTYPE", 7); /* FIXME */
+	      strcpy(tok + tok_pos, "DOCTYPE"); /* FIXME */
 	      lexer->lexbuf_pos += 6;
 	      return T_DOCTYPE_START;
 	    } else {
@@ -378,7 +378,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	  case '[':
 	    lexer->lexbuf_pos++;
 	    if (strncmp(lexer->lexbuf + lexer->lexbuf_pos, "CDATA[", 6) == 0) {
-	      strncpy (tok + tok_pos, "[CDATA[", 7); /* FIXME */
+	      strcpy (tok + tok_pos, "[CDATA["); /* FIXME */
 	      lexer->lexbuf_pos += 6;
 	      lexer->lex_mode = CDATA;
 	      return T_CDATA_START;
@@ -397,7 +397,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	  if (c == '>') {
 	    tok[tok_pos] = c;
 	    lexer->lexbuf_pos++;
-	    tok_pos++; /* FIXME */
+	    tok_pos++;
 	    tok[tok_pos] = '\0';
 	    if (!lexer->in_comment)
 	      lexer->lex_mode = DATA;
@@ -432,7 +432,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	    tok[tok_pos] = c;
 	    tok_pos++;
 	    lexer->lexbuf_pos++;
-	    tok[tok_pos] = '\0'; /* FIX ME */
+	    tok[tok_pos] = '\0'; /* FIXME */
 	    if (strlen(tok) != 3) {
 	      tok[tok_pos - 3] = '\0';
 	      lexer->lexbuf_pos -= 3;
@@ -541,7 +541,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	     tok_pos, tok_size, lexer->lexbuf_pos, lexer->lexbuf_size);
 
     /* pb */
-    if (tok_pos >= tok_size) {
+    if (tok_pos + 8 >= tok_size) {
       if (fixed)
         return T_ERROR;
       *_tok_size *= 2;
@@ -652,19 +652,20 @@ char *lexer_decode_entities (const char *tok)
       /* entity is a number
        * (note: strtol() allows "0x" prefix for hexadecimal, but we don't)
        */
+      char *end;
       if (*tp == 'x' && tp[1] && tp[2] != 'x')
-	i = strtol (tp + 1, (char **)&tp, 16);
+	i = strtol (tp + 1, &end, 16);
       else
-	i = strtol (tp, (char **)&tp, 10);
+	i = strtol (tp, &end, 10);
 
-      if (*tp != ';' || i < 1)
+      if (*end != ';' || i < 1)
       {
         /* out of range, or format error */
 	*bp++ = '&';
 	continue;
       }
 
-      tok = tp + 1;
+      tok = end + 1;
 
       if (i < 128)
         /* ASCII - store as-is */
