@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2017 the xine project
+ *  Copyright (C) 2002-2018 the xine project
  *
  *  This file is part of xine, a free video player.
  *
@@ -261,7 +261,6 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
   int parse_res;
   int bypass_get_token = 0;
   int retval = 0; /* used when state==4; non-0 if there are missing </...> */
-  xml_node_t *subtree = NULL;
   xml_node_t *current_subtree = NULL;
   xml_property_t *current_property = NULL;
   xml_property_t *properties = NULL;
@@ -283,6 +282,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  /* do nothing */
 	  break;
 	case (T_EOF):
+          xml_parser_free_props(properties);
 	  return retval; /* normal end */
 	  break;
 	case (T_M_START_1):
@@ -314,6 +314,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -323,6 +324,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
       case STATE_Q_NODE:
 	switch (res) {
 	case (T_IDENT):
+          xml_parser_free_props(properties);
 	  properties = NULL;
 	  current_property = NULL;
 
@@ -347,6 +349,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -358,7 +361,9 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	case (T_SEPAR):
 	  /* nothing */
 	  break;
-	case (T_M_STOP_1):
+        case (T_M_STOP_1): {
+          xml_node_t *subtree;
+
 	  /* new subtree */
 	  subtree = new_xml_node();
 
@@ -367,6 +372,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 
 	  /* set node propertys */
 	  subtree->props = properties;
+          properties = NULL;
 	  lprintf("info: rec %d new subtree %s\n", rec, node_name);
 	  root_names[rec + 1] = strdup (node_name);
 	  parse_res = xml_parser_get_node_internal (xml_parser, token_buffer, token_buffer_size,
@@ -375,12 +381,15 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 						    subtree, root_names, rec + 1, flags);
 	  free (root_names[rec + 1]);
 	  if (parse_res == -1 || parse_res > 0) {
+            xml_parser_free_tree_rec(subtree, 1);
 	    return parse_res;
 	  }
 	  if (current_subtree == NULL) {
+            _x_assert(current_node->child == NULL);
 	    current_node->child = subtree;
 	    current_subtree = subtree;
 	  } else {
+            _x_assert(current_subtree->next == NULL);
 	    current_subtree->next = subtree;
 	    current_subtree = subtree;
 	  }
@@ -390,9 +399,11 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  }
 	  state = STATE_IDLE;
 	  break;
-	case (T_M_STOP_2):
+        }
+        case (T_M_STOP_2): {
 	  /* new leaf */
 	  /* new subtree */
+          xml_node_t *subtree;
 	  new_leaf:
 	  subtree = new_xml_node();
 
@@ -401,18 +412,22 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 
 	  /* set node propertys */
 	  subtree->props = properties;
+          properties = NULL;
 
 	  lprintf("info: rec %d new subtree %s\n", rec, node_name);
 
 	  if (current_subtree == NULL) {
+            _x_assert(current_node->child == NULL);
 	    current_node->child = subtree;
 	    current_subtree = subtree;
 	  } else {
+            _x_assert(current_subtree->next == NULL);
 	    current_subtree->next = subtree;
 	    current_subtree = subtree;
 	  }
 	  state = STATE_IDLE;
 	  break;
+        }
 	case (T_IDENT):
 	  /* save property name */
 	  new_prop:
@@ -431,6 +446,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -448,6 +464,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  goto new_prop;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -480,11 +497,13 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  else
 	  {
 	    lprintf("error: xml struct, tok=%s, waited_tok=%s\n", tok, root_names[rec]);
+            xml_parser_free_props(properties);
 	    return -1;
 	  }
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -498,6 +517,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -533,6 +553,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -568,6 +589,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -598,6 +620,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -609,6 +632,10 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	case (T_C_STOP):
 	  state = STATE_IDLE;
 	  break;
+        case (T_EOF):
+          lprintf("error: unterminated comment, state %d\n", state);
+          xml_parser_free_props(properties);
+          return -1;
 	default:
 	  break;
 	}
@@ -635,6 +662,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
         default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -648,6 +676,7 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 	  break;
 	default:
 	  lprintf("error: unexpected token \"%s\", state %d\n", tok, state);
+          xml_parser_free_props(properties);
 	  return -1;
 	  break;
 	}
@@ -656,15 +685,18 @@ static int xml_parser_get_node_internal (xml_parser_t *xml_parser,
 
       default:
 	lprintf("error: unknown parser state, state=%d\n", state);
+        xml_parser_free_props(properties);
 	return -1;
       }
     }
     /* lex error */
     lprintf("error: lexer error\n");
+    xml_parser_free_props(properties);
     return -1;
   } else {
     /* max recursion */
     lprintf("error: max recursion\n");
+    xml_parser_free_props(properties);
     return -1;
   }
 }
@@ -854,9 +886,13 @@ static void xml_parser_dump_node (const xml_node_t *node, int indent) {
 
   p = node->props;
   while (p) {
-    char *value = xml_escape_string (p->value, XML_ESCAPE_SINGLE_QUOTE);
-    printf ("%s='%s'", p->name, value);
-    free (value);
+    if (p->value) {
+      char *value = xml_escape_string (p->value, XML_ESCAPE_SINGLE_QUOTE);
+      printf ("%s='%s'", p->name, value);
+      free (value);
+    } else {
+      printf ("%s", p->name);
+    }
     p = p->next;
     if (p) {
       printf ("\n%*s", indent+2+(int)l, "");
@@ -864,6 +900,12 @@ static void xml_parser_dump_node (const xml_node_t *node, int indent) {
   }
   printf (">\n");
 
+  if (node->data) {
+    char *value = xml_escape_string (node->data, XML_ESCAPE_SINGLE_QUOTE);
+    printf ("%*s\"%s\"\n", indent + 2, "", value);
+    free(value);
+  }
+
   n = node->child;
   while (n) {
 
@@ -892,6 +934,7 @@ int main (int argc, char **argv)
 {
   xml_parser_t *parser;
   int i, ret = 0;
+  (void)argc;
   for (i = 1; argv[i]; ++i)
   {
     xml_node_t *tree;
