@@ -30,16 +30,22 @@
 
 XINE_VERSION_MAJOR=1
 XINE_VERSION_MINOR=2
-XINE_VERSION_SUB=9
+XINE_VERSION_SUB=10
 XINE_VERSION_PATCH=
 # Release series number (usually $XINE_MAJOR.$XINE_MINOR)
 XINE_VERSION_SERIES=1.2
 
-XINE_LT_CURRENT=9
+XINE_LT_CURRENT=10
 XINE_LT_REVISION=0
-XINE_LT_AGE=7
+XINE_LT_AGE=8
 
-test -f "`dirname $0`/.cvsversion" && XINE_VERSION_SUFFIX="hg"
+if [ -f "`dirname $0`/.cvsversion" ]; then
+    HG_REV="`hg summary | sed -e '1s/^parent: \([0-9]*\):.*$/\1/;1q'`"
+    HG_DATE_UNIX="`hg export -r ${HG_REV} | sed -n '3s/^# Date \([0-9]*\) .*$/\1/p;3q'`"
+    HG_DATE="`date -u +%Y%m%d --date=@${HG_DATE_UNIX}`"
+    HG_DIRTY=`hg out 2>/dev/null | grep -q changeset || hg sum | grep -q 'commit:.*modified' && echo "+dirty"`
+    XINE_VERSION_SUFFIX="-${HG_DATE}hg${HG_REV}${HG_DIRTY}"
+fi
 XINE_VERSION_SPEC="${XINE_VERSION_MAJOR}.${XINE_VERSION_MINOR}.${XINE_VERSION_SUB}${XINE_VERSION_PATCH}${XINE_VERSION_SUFFIX}"
 
 ####
